package applications.clustering;

import java.awt.Color;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Random;

import javax.swing.JFrame;

import utils.visibility.AbstractDrawable;
import utils.visibility.DrawField2D;
import utils.visibility.Drawable;
import utils.visibility.Drawer;
import utils.visibility.SVGDrawer;
import xxl.core.functions.Function;
import applications.clustering.data.ClusterDataGenerator;
import clustering.FuzzyAverages;
import clustering.FuzzyKMeans;
import clustering.FuzzyKMeansState;
import clustering.FuzzyKMeansUtils;
import clustering.FuzzyValidity;
import clustering.KMeansUtils;
import clustering.RelativeFuzzyKMeans;

public class IncFKMTest {


    public static double epsilon = 0.000001;

    public static double fuzzifier = 2;

    public static boolean plot = false;

    public static double d = 3;
    
    public static int dim=25;

    public static int clusterCount = 20;

    public static double noise = 0.0;

    public static FuzzyValidity fv = FuzzyKMeans.MYVF3;

    public static long seed = 132958437l;

    

    public static FuzzyKMeansState[] getClusteredPartition(long seed) {
        FuzzyKMeansState[] data = getPartition(seed);
        return new FuzzyKMeansState[]{performTestFKM(data[0].clusterCount, (double[][]) data[0].data,
                (double[][]) data[0].kMeans),performTestFKM(data[1].clusterCount, (double[][]) data[1].data,
                        (double[][]) data[1].kMeans)};
    }

    public static FuzzyKMeansState[] getPartition(long seed) {
        Random rnd = new Random(seed);
        ClusterDataGenerator gen = new ClusterDataGenerator(dim, rnd);

        double stds = 5;
        int dpc = 1000 / clusterCount;
        double[][] mean = new double[clusterCount][dim];
        double[][] std = new double[clusterCount][dim];
        for (int i = 0; i < clusterCount; i++) {
            boolean ok = true;
            do {
                for (int d = 0; d < dim; d++) {
                    mean[i][d] = rnd.nextDouble() * 200;
                    std[i][d] = stds;
                }
                ok = true;
                for (int c = 0; c < i; c++) {
                    double diff = 0;
                    for (int d = 0; d < dim; d++) {
                        double dd = mean[i][d] - mean[c][d];
                        diff += dd * dd;
                    }
                    if (diff < (36 * stds * stds)) {
                        ok = false;
                        break;
                    }
                }
            } while (!ok);
        }

        for (int i = 0; i < clusterCount-1; i++) {
            gen.addNormalCluster(mean[i], std[i], dpc);
        }
        
        ClusterDataGenerator gen2=(ClusterDataGenerator)gen.clone();
        
        
        gen.addNormalCluster(mean[clusterCount-1], std[clusterCount-1], dpc);
        gen2.addNormalCluster(mean[clusterCount-2], std[clusterCount-2], dpc);
        double[][] data = gen.getData();
        double[][] data2 = gen2.getData();
        
        FuzzyKMeansState partition = new FuzzyKMeansState(data, data.length,
                clusterCount);
        partition.kMeans = mean;
        
        FuzzyKMeansState partition2 = new FuzzyKMeansState(data2, data2.length,
                clusterCount-1);
        partition2.kMeans = mean;

        return new FuzzyKMeansState[]{partition,partition2};
    }

    public static FuzzyKMeansState performTestFKM(int initCN, double[][] data) {

        FuzzyKMeans kmeans = new FuzzyKMeans(KMeansUtils.getDoubleDistance(dim),
                fuzzifier, FuzzyKMeans.OBJECTIVE, fv,
                FuzzyKMeansUtils.INITALIZE_RANDOM, KMeansUtils
                        .getDoubleCopyFunction(dim), FuzzyAverages
                        .getDoubleMeansFunction(dim));
        FuzzyKMeansState result = kmeans.invokeFuzzyKMeans(data, data.length,
                initCN, epsilon);
        if (plot) {
            plotData(data, result.clusterCount, result.kMeans);
            System.out.println(initCN + ", " + result.validity);
        }
        return result;
    }

    public static FuzzyKMeansState performTestFKM(int initCN, double[][] data,
            double[][] initKMeans) {

        FuzzyKMeans kmeans = new FuzzyKMeans(KMeansUtils.getDoubleDistance(dim),
                fuzzifier, FuzzyKMeans.OBJECTIVE, fv, FuzzyKMeansUtils
                        .getKMeansInitialization(initKMeans), KMeansUtils
                        .getDoubleCopyFunction(dim), FuzzyAverages
                        .getDoubleMeansFunction(dim));
        FuzzyKMeansState result = kmeans.invokeFuzzyKMeans(data, data.length,
                initCN, epsilon);
        if (plot) {
            plotData(data, result.clusterCount, result.kMeans);
            System.out.println(initCN + ", " + result.validity);
        }
        return result;
    }

    public static FuzzyKMeansState performTestRFKM(int initCN, double[][] data) {

        RelativeFuzzyKMeans rfkm = new RelativeFuzzyKMeans(new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(dim), fuzzifier,
                FuzzyKMeans.OBJECTIVE, fv,
                // FuzzyKMeansUtils.getKMeansInitialization(result.kMeans),
                FuzzyKMeansUtils.INITALIZE_RANDOM, KMeansUtils
                        .getDoubleCopyFunction(dim), FuzzyAverages
                        .getDoubleMeansFunction(dim)));
        FuzzyKMeansState result = rfkm.invokeRelativeKMeans(data, data.length,
                2, 20, initCN, epsilon, 10);

        if (plot) {
            plotData(data, result.clusterCount, result.kMeans);
            System.out.println(result.clusterCount + ", " + result.validity);
        }

        return result;
    }
    
    public static FuzzyKMeansState performInitTestRFKM(int initCN, double[][] data,Object[] initKMeans) {

        RelativeFuzzyKMeans rfkm = new RelativeFuzzyKMeans(new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(dim), fuzzifier,
                FuzzyKMeans.OBJECTIVE, fv,
                FuzzyKMeansUtils
                .getKMeansInitialization(initKMeans), KMeansUtils
                        .getDoubleCopyFunction(dim), FuzzyAverages
                        .getDoubleMeansFunction(dim)));
        FuzzyKMeansState result = rfkm.invokeRelativeKMeans(data, data.length,
                2, 20, initCN, epsilon, 10);

        if (plot) {
            plotData(data, result.clusterCount, result.kMeans);
            System.out.println(result.clusterCount + ", " + result.validity);
        }

        return result;
    }

    

    public static void plotData(final double[][] data, final int clusterCount,
            final Object[] kMeans) {
        Drawable d = new AbstractDrawable() {
            public void draw(DrawField2D draw) {
                for (int i = 0; i < data.length; i++) {
                    draw.drawPoint(data[i][0], data[i][1], Color.BLACK);
                }
                for (int i = 0; i < clusterCount; i++) {
                    double[] mean = (double[]) kMeans[i];
                    draw.drawCross(mean[0], mean[1], Color.RED, 10);
                }
            }
        };
        // Drawer draw = new Drawer(800, 800, -12,12,-12,12);
        // Drawer draw = new Drawer(800, 800, -0.2,1.2,-0.2,1.2);
        Drawer draw = new Drawer(800, 800, -10, 210, -10, 210);
        draw.add(d);
        JFrame frame = draw.createFrame("Cluster Data");
        frame.setVisible(true);
        try {
            SVGDrawer.drawSVG(draw, new FileWriter("D:/svgImage.svg"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
       
        testPaper();
    }

   
   

    public static FuzzyKMeansState getFullTest(int minC, int maxC, int loop,
            double[][] data) {
        FuzzyKMeansState best = null;
        for (int c = minC; c <= maxC; c++) {
            for (int i = 0; i < loop; i++) {
                FuzzyKMeansState res = performTestFKM(c, data);
                if (best == null || best.validity > res.validity) {
                    best = res;
                }
            }
        }
        return best;
    }

    public static FuzzyKMeansState getSimpleTest(int clusterCount, int loop,
            double[][] data) {
        FuzzyKMeansState best = null;
        for (int i = 0; i < loop; i++) {
            FuzzyKMeansState res = performTestFKM(clusterCount, data);
            if (best == null || best.validity > res.validity) {
                best = res;
            }
        }
        return best;
    }

    
    public static void testPaper() {
        try {
            //Writer out = new FileWriter("D:/incClusterOut"+dim+".txt");
            Writer out = new FileWriter("/vol1/home/jberinge/result/incClusterOut"+dim+".txt");
            Random rnd = new Random();
            long start, end;
            while (true) {
                clusterCount=rnd.nextInt(17)+3;
                long seed = rnd.nextLong();
                FuzzyKMeansState data[] = getClusteredPartition(seed);
                boolean up=rnd.nextBoolean();
                if (!up){
                    int resC=data[1].clusterCount;
                    double resV=data[1].validity;
                    FuzzyKMeansState resultRFKM = performInitTestRFKM(data[0].clusterCount,
                            (double[][]) data[1].data, data[0].kMeans);
                    out.write(seed+","+data[0].clusterCount+","+data[0].validity+","+resC+","+resV+","+resultRFKM.clusterCount+","+resultRFKM.validity+","+!up);
                }
                else{
                    int resC=data[0].clusterCount;
                    double resV=data[0].validity;
                    FuzzyKMeansState resultRFKM = performInitTestRFKM(data[1].clusterCount,
                            (double[][]) data[0].data, data[1].kMeans);
                    out.write(seed+","+data[1].clusterCount+","+data[1].validity+","+resC+","+resV+","+resultRFKM.clusterCount+","+resultRFKM.validity+","+!up);
                }
                
                out.write("\n");
                out.flush();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
}
