package applications.clustering;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Random;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.logging.StreamHandler;

import javax.swing.JFrame;

import utils.visibility.AbstractDrawable;
import utils.visibility.DrawField2D;
import utils.visibility.Drawable;
import utils.visibility.Drawer;
import utils.visibility.SVGDrawer;
import applications.clustering.data.ClusterDataGenerator;
import clustering.FuzzyAverages;
import clustering.FuzzyKMeans;
import clustering.FuzzyKMeansState;
import clustering.FuzzyKMeansUtils;
import clustering.FuzzyValidity;
import clustering.KMeansUtils;
import clustering.RelativeFuzzyKMeans;

public class PerformFKMTest {

    public static int[] size = new int[] { 30, 30, 30, 30, 30, 30 };

    public static double epsilon = 0.000001;

    public static double fuzzifier = 2;

    public static boolean plot = false;

    public static double d = 3;

    public static int dim = 2;

    public static int clusterCount = 6;

    public static double noise = 0.0;

    public static double distC = 6;

    public static double room = 200;

    public static FuzzyValidity fv = FuzzyKMeans.MYVF3;

    public static long seed = 132958437l;

    public static double[][] getData(long seed) {
        return (double[][]) getPartition(seed).data;
    }

    public static FuzzyKMeansState getClusteredPartition(long seed) {
        FuzzyKMeansState data = getPartition(seed);
        return performTestFKM(data.clusterCount, (double[][]) data.data,
                (double[][]) data.kMeans);
    }

    public static FuzzyKMeansState getPartition(long seed) {
        Random rnd = new Random(seed);
        ClusterDataGenerator gen = new ClusterDataGenerator(dim, rnd);

        double stds = 5;
        int dpc = 1000 / clusterCount;
        double[][] mean = new double[clusterCount][dim];
        double[][] std = new double[clusterCount][dim];
        for (int i = 0; i < clusterCount; i++) {
            boolean ok = true;
            do {
                for (int d = 0; d < dim; d++) {
                    mean[i][d] = rnd.nextDouble() * room;
                    std[i][d] = stds;
                }
                ok = true;
                for (int c = 0; c < i; c++) {
                    double diff = 0;
                    for (int d = 0; d < dim; d++) {
                        double dd = mean[i][d] - mean[c][d];
                        diff += dd * dd;
                    }
                    if (diff < (distC * distC * stds * stds)) {
                        ok = false;
                        break;
                    }
                }
            } while (!ok);
        }

        for (int i = 0; i < clusterCount; i++) {
            gen.addNormalCluster(mean[i], std[i], dpc);
        }
        if (noise > 0)
            gen.addNoise((int) (gen.getDataCount() * noise / (1 - noise)));

        double[][] data = gen.getData();

        FuzzyKMeansState partition = new FuzzyKMeansState(data, data.length,
                clusterCount);
        partition.kMeans = mean;

        return partition;
    }

    public static FuzzyKMeansState performTestFKM(int initCN, double[][] data) {

        FuzzyKMeans kmeans = new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(dim), fuzzifier,
                FuzzyKMeans.OBJECTIVE, fv, FuzzyKMeansUtils.INITALIZE_RANDOM,
                KMeansUtils.getDoubleCopyFunction(dim), FuzzyAverages
                        .getDoubleMeansFunction(dim));
        FuzzyKMeansState result = kmeans.invokeFuzzyKMeans(data, data.length,
                initCN, epsilon);
        if (plot) {
            plotData(data, result.clusterCount, result.kMeans);
            System.out.println(initCN + ", " + result.validity);
        }
        return result;
    }

    public static FuzzyKMeansState performTestFKM(int initCN, double[][] data,
            double[][] initKMeans) {

        FuzzyKMeans kmeans = new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(dim), fuzzifier,
                FuzzyKMeans.OBJECTIVE, fv, FuzzyKMeansUtils
                        .getKMeansInitialization(initKMeans), KMeansUtils
                        .getDoubleCopyFunction(dim), FuzzyAverages
                        .getDoubleMeansFunction(dim));
        FuzzyKMeansState result = kmeans.invokeFuzzyKMeans(data, data.length,
                initCN, epsilon);
        if (plot) {
            plotData(data, result.clusterCount, result.kMeans);
            System.out.println(initCN + ", " + result.validity);
        }
        return result;
    }

    public static FuzzyKMeansState performTestRFKM(int initCN, double[][] data) {

        RelativeFuzzyKMeans rfkm = new RelativeFuzzyKMeans(new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(dim), fuzzifier,
                FuzzyKMeans.OBJECTIVE, fv,
                // FuzzyKMeansUtils.getKMeansInitialization(result.kMeans),
                FuzzyKMeansUtils.INITALIZE_RANDOM, KMeansUtils
                        .getDoubleCopyFunction(dim), FuzzyAverages
                        .getDoubleMeansFunction(dim)));
        FuzzyKMeansState result = rfkm.invokeRelativeKMeans(data, data.length,
                2, 20, initCN, epsilon, 10);

        if (plot) {
            plotData(data, result.clusterCount, result.kMeans);
            System.out.println(result.clusterCount + ", " + result.validity);
        }

        return result;
    }

    public static void performTestFKM() {
        for (double val2 = 3; val2 <= 4; val2 += 1) {
            Writer write = null;
            try {
                write = new BufferedWriter(
                        new FileWriter("D:/work/Research/FKM/resultMy"
                                + (int) (val2) + ".txt"));

                for (int i = 0; i < 3; i++) {
                    Random rnd = new Random();
                    double[][] data = getData(rnd.nextLong());
                    for (int c = 2; c <= 12; c++) {
                        for (int l = 0; l < 20; l++) {
                            FuzzyKMeansState result = performTestRFKM(c, data);
                            write.write(result.validity + "   ");
                        }
                        write.write("\n");

                        write.flush();
                    }
                }
                write.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void plotData(final double[][] data, final int clusterCount,
            final Object[] kMeans) {
        Drawable d = new AbstractDrawable() {
            public void draw(DrawField2D draw) {
                for (int i = 0; i < data.length; i++) {
                    draw.drawPoint(data[i][0], data[i][1], Color.BLACK);
                }
                for (int i = 0; i < clusterCount; i++) {
                    double[] mean = (double[]) kMeans[i];
                    draw.drawCross(mean[0], mean[1], Color.RED, 10);
                }
            }
        };
        // Drawer draw = new Drawer(800, 800, -12,12,-12,12);
        // Drawer draw = new Drawer(800, 800, -0.2,1.2,-0.2,1.2);
        Drawer draw = new Drawer(800, 800, -10, 210, -10, 210);
        draw.add(d);
        JFrame frame = draw.createFrame("Cluster Data");
        frame.setVisible(true);
        try {
            SVGDrawer.drawSVG(draw, new FileWriter("D:/svgImage.svg"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        testTime();
        //testGraph();
        // test1();
        // test2();
        // test3();
        // testPlot();
        // testAllPaper();
        // exampleVF();
    }

    public static void test3() {
        Logger logger = Logger.getLogger("Test");
        logger.setLevel(Level.FINE);
        Handler handler = new StreamHandler(System.out, new SimpleFormatter());
        handler.setLevel(Level.FINE);
        logger.addHandler(handler);

        plot = true;

        long start = System.nanoTime();
        long seed = new Random().nextLong();
        System.out.println("seed: " + seed);
        double[][] data = getData(seed);
        // double[][] data = getData(new Random(132958437l));
        FuzzyKMeansState result = performTestRFKM(10, data);
        long end = System.nanoTime();
        logger.log(Level.FINE, "RESULT: " + result.clusterCount + ","
                + result.validity + "," + result.objective + "\n\n");
        System.out.println("RESULT: " + result.clusterCount + ","
                + result.validity + "," + result.objective + "");
    }

    public static void test2() {

        double[][] data = getData(-9001481805036199140l);
        // double[][] data = getData(new Random(132958437l));
        for (int c = 2; c <= 10; c++) {
            double min = Double.MAX_VALUE;
            double sum = 0;
            double sum2 = 0;
            for (int i = 0; i < 100; i++) {
                FuzzyKMeansState result = performTestFKM(c, data);
                sum += result.validity;
                sum2 += result.validity * result.validity;
                if (result.validity < min) {
                    min = result.validity;
                }
            }
            double mean = sum / 100.0;
            double std = Math.sqrt(sum2 / 100.0 - mean * mean);
            System.out.println(c + "   " + min + "  " + mean + "   " + std);
        }
    }

    public static void test1() {
        Logger logger = Logger.getLogger("Test");
        logger.setLevel(Level.FINE);
        Handler handler = new StreamHandler(System.out, new SimpleFormatter());
        handler.setLevel(Level.FINE);
        logger.addHandler(handler);
        double[][] data = getData(132958437l);
        // performTestFKM();
        int wrong = 0;
        double time1 = 0;
        double time2 = 0;
        int loops = 50;
        for (int i = 0; i < loops; i++) {
            long start = System.nanoTime();
            FuzzyKMeansState result = performTestRFKM(4, data);
            long end = System.nanoTime();
            time1 += (end - start) / 1000000.0;
            logger.log(Level.FINE, "RESULT: " + result.clusterCount + ","
                    + result.validity + "," + result.objective + "\n\n");
            System.out.println("RESULT: " + result.clusterCount + ","
                    + result.validity + "," + result.objective + "");
            if (result.clusterCount != clusterCount || result.validity > 9.5) {
                wrong++;
                // System.exit(1);
            }
        }
        System.out.println("Anteil nicht optimaler Lsungen:"
                + (wrong / (double) loops) + "\n\n");
        logger.log(Level.FINE, "Anteil nicht optimaler Lsungen:"
                + (wrong / (double) loops) + "\n\n");
        int wrong2 = 0;
        for (int i = 0; i < loops; i++) {
            long start = System.nanoTime();
            // FuzzyKMeansState result = getFullTest(2,12,10,data);
            FuzzyKMeansState result = performTestFKM(10, data);
            long end = System.nanoTime();
            time2 += (end - start) / 1000000.0;
            logger.log(Level.FINE, i + ".RESULT: " + result.clusterCount + ","
                    + result.validity + "," + result.objective + "\n\n");
            System.out.println("RESULT: " + result.clusterCount + ","
                    + result.validity + "," + result.objective + "");
            if (result.clusterCount != clusterCount || result.validity > 9.5)
                wrong2++;
        }
        System.out.println("\n\nAnteil nicht optimaler Lsungen (My Way):"
                + (wrong / (double) loops));
        System.out.println("Anteil nicht optimaler Lsungen (Standard):"
                + (wrong2 / (double) loops));

        System.out.println("\n\nLaufzeit (My Way):" + (time1 / (double) loops)
                + "ms");
        System.out.println("Laufzeit (Standard):" + (time2 / (double) loops)
                + "ms");
    }

    public static FuzzyKMeansState getFullTest(int minC, int maxC, int loop,
            double[][] data) {
        FuzzyKMeansState best = null;
        for (int c = minC; c <= maxC; c++) {
            for (int i = 0; i < loop; i++) {
                FuzzyKMeansState res = performTestFKM(c, data);
                if (best == null || best.validity > res.validity) {
                    best = res;
                }
            }
        }
        return best;
    }

    public static FuzzyKMeansState getLoopedTest(int cc, int loop,
            double[][] data) {
        FuzzyKMeansState best = null;
        for (int i = 0; i < loop; i++) {
            FuzzyKMeansState res = performTestFKM(cc, data);
            if (best == null || best.validity > res.validity) {
                best = res;
            }
        }

        return best;
    }

    public static FuzzyKMeansState getSimpleTest(int clusterCount, int loop,
            double[][] data) {
        FuzzyKMeansState best = null;
        for (int i = 0; i < loop; i++) {
            FuzzyKMeansState res = performTestFKM(clusterCount, data);
            if (best == null || best.validity > res.validity) {
                best = res;
            }
        }
        return best;
    }

    public static void exampleVF() {
        distC = 6;
        room = 200;
        dim = 5;

        Random rnd = new Random();
        clusterCount = 5;
        long seed = rnd.nextLong();
        System.out.println(seed);
        FuzzyKMeansState data = getClusteredPartition(seed);

        fv = FuzzyKMeans.SEPARATION;
        try {
            Writer out = new FileWriter("D:/exampleXB" + dim + ".txt");

            for (int c = 2; c <= 19; c++) {
                double best = Double.MAX_VALUE;
                for (int i = 0; i < 10; i++) {
                    FuzzyKMeansState res = performTestFKM(c,
                            (double[][]) data.data);
                    if (best > res.validity) {
                        best = res.validity;
                    }
                }
                out.write(c + "  " + best + "\n");
                out.flush();
            }
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        fv = FuzzyKMeans.MYVF3;
        try {
            Writer out = new FileWriter("D:/exampleM" + dim + ".txt");

            for (int c = 2; c <= 19; c++) {
                double best = Double.MAX_VALUE;
                for (int i = 0; i < 10; i++) {
                    FuzzyKMeansState res = performTestFKM(c,
                            (double[][]) data.data);
                    if (best > res.validity) {
                        best = res.validity;
                    }
                }
                out.write(c + "  " + best + "\n");
                out.flush();
            }
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void testPaper() {
        fv = FuzzyKMeans.MYVF3;
        try {
            Writer out = new FileWriter(
                    "/vol1/home/jberinge/result/clusterOutN" + dim + ".txt");

            // Writer out = new FileWriter("D:/clusterOutN"
            // + dim + ".txt");
            BufferedReader seeder = new BufferedReader(new FileReader(
                    "/vol1/home/jberinge/result/clusterOut" + dim + ".txt"));
            // Random rnd = new Random();
            long start, end;
            for (clusterCount = 2; clusterCount < 20; clusterCount++) {
                for (int i = 0; i < 50; i++) {
                    String line = seeder.readLine();
                    int p = line.indexOf(",");
                    long seed = Long.parseLong(line.substring(0, p));
                    // long seed = rnd.nextLong();
                    FuzzyKMeansState data = getClusteredPartition(seed);
                    start = System.nanoTime();
                    FuzzyKMeansState resultRFKM = performTestRFKM(10,
                            (double[][]) data.data);
                    end = System.nanoTime();
                    double timeRFKM = (end - start) / 1000000.0;

                    // start = System.nanoTime();
                    // FuzzyKMeansState resultCFKM = performTestRFKM(
                    // data.clusterCount, (double[][]) data.data);
                    // end = System.nanoTime();
                    // double timeCFKM = (end - start) / 1000000.0;
                    //
                    // start = System.nanoTime();
                    // FuzzyKMeansState resultFFKM = getFullTest(2, 20, 10,
                    // (double[][]) data.data);
                    // end = System.nanoTime();
                    // double timeFFKM = (end - start) / 1000000.0;
                    //
                    // start = System.nanoTime();
                    // FuzzyKMeansState resultFKM = getSimpleTest(
                    // data.clusterCount, 10, (double[][]) data.data);
                    // end = System.nanoTime();
                    // double timeFKM = (end - start) / 1000000.0;

                    out.write(seed + "," + clusterCount + "," + data.validity);
                    out.write("," + resultRFKM.clusterCount + ","
                            + resultRFKM.validity + "," + timeRFKM);
                    // out.write("," + resultCFKM.clusterCount + ","
                    // + resultCFKM.validity + "," + timeCFKM);
                    // out.write("," + resultFFKM.clusterCount + ","
                    // + resultFFKM.validity + "," + timeFFKM);
                    // out.write("," + resultFKM.clusterCount + ","
                    // + resultFKM.validity + "," + timeFKM);
                    out.write("\n");
                    out.flush();
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void testGraph() {

        try {
            Writer out = new FileWriter("D:/graphtest" + dim + "b.txt");
            Random rnd = new Random();

            clusterCount = 5;
            for (int cc = 2; cc < 15; cc++) {
                long seed = rnd.nextLong();
                FuzzyKMeansState data = getClusteredPartition(seed);

                fv = FuzzyKMeans.MYVF3;
                FuzzyKMeansState resultRFKM = getLoopedTest(cc, 10,
                        (double[][]) data.data);

                fv = FuzzyKMeans.SEPARATION;

                FuzzyKMeansState resultXBFKM = getLoopedTest(cc, 10,
                        (double[][]) data.data);

                out.write(cc + "," + resultRFKM.validity + ","
                        + resultXBFKM.validity);

                out.write("\n");
                out.flush();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void testTime() {

        Random rnd = new Random();
        long start, end;
        long seed = rnd.nextLong();
        
        FuzzyKMeansState resultFFKM;
        
        FuzzyKMeansState data;
        
        double timeFFKM;
        
        fv = FuzzyKMeans.MYVF3;
        
        
//        clusterCount = 3;
//        dim = 5;
//        data = getClusteredPartition(seed);
//        start = System.nanoTime();
//        resultFFKM = getFullTest(2, 19, 10,
//                (double[][]) data.data);
//        end = System.nanoTime();
//        timeFFKM = (end - start) / 1000000.0;
//        System.out.println("Dim="+dim+"  CC="+clusterCount+"  Time="+timeFFKM);
//        
//        clusterCount = 7;
//        dim = 5;      
//        data = getClusteredPartition(seed);
//        start = System.nanoTime();
//        resultFFKM = getFullTest(2, 19, 10,
//                (double[][]) data.data);
//        end = System.nanoTime();
//        timeFFKM = (end - start) / 1000000.0;    
//        System.out.println("Dim="+dim+"  CC="+clusterCount+"  Time="+timeFFKM);
        
        clusterCount = 17;
        dim = 5;      
        data = getClusteredPartition(seed);
        start = System.nanoTime();
        resultFFKM = getFullTest(2, 19, 10,
                (double[][]) data.data);
        end = System.nanoTime();
        timeFFKM = (end - start) / 1000000.0;    
        System.out.println("Dim="+dim+"  CC="+clusterCount+"  Time="+timeFFKM);
        
        

    }

    public static void testPaperXB() {
        fv = FuzzyKMeans.SEPARATION;
        try {
            Writer out = new FileWriter(
                    "/vol1/home/jberinge/result/clusterOutXBN" + dim + ".txt");
            Random rnd = new Random();
            long start, end;
            for (clusterCount = 2; clusterCount < 20; clusterCount++) {
                for (int i = 0; i < 50; i++) {
                    long seed = rnd.nextLong();
                    FuzzyKMeansState data = getClusteredPartition(seed);
                    start = System.nanoTime();
                    FuzzyKMeansState resultRFKM = performTestRFKM(10,
                            (double[][]) data.data);
                    end = System.nanoTime();
                    double timeRFKM = (end - start) / 1000000.0;

                    // start = System.nanoTime();
                    // FuzzyKMeansState resultCFKM = performTestRFKM(
                    // data.clusterCount, (double[][]) data.data);
                    // end = System.nanoTime();
                    // double timeCFKM = (end - start) / 1000000.0;
                    //
                    // start = System.nanoTime();
                    // FuzzyKMeansState resultFFKM = getFullTest(2, 20, 10,
                    // (double[][]) data.data);
                    // end = System.nanoTime();
                    // double timeFFKM = (end - start) / 1000000.0;
                    //
                    // start = System.nanoTime();
                    // FuzzyKMeansState resultFKM = getSimpleTest(
                    // data.clusterCount, 10, (double[][]) data.data);
                    // end = System.nanoTime();
                    // double timeFKM = (end - start) / 1000000.0;

                    out.write(seed + "," + clusterCount + "," + data.validity);
                    out.write("," + resultRFKM.clusterCount + ","
                            + resultRFKM.validity + "," + timeRFKM);
                    // out.write("," + resultCFKM.clusterCount + ","
                    // + resultCFKM.validity + "," + timeCFKM);
                    // out.write("," + resultFFKM.clusterCount + ","
                    // + resultFFKM.validity + "," + timeFFKM);
                    // out.write("," + resultFKM.clusterCount + ","
                    // + resultFKM.validity + "," + timeFKM);
                    out.write("\n");
                    out.flush();
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void testAllPaper() {
        dim = 2;
        testPaper();
        // testPaperXB();
        dim = 5;
        testPaper();
        // testPaperXB();
        dim = 10;
        testPaper();
        // testPaperXB();
        dim = 25;
        testPaper();
        // testPaperXB();
    }
}
