package applications.clustering;

import java.awt.Color;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Random;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.logging.StreamHandler;

import javax.swing.JFrame;

import utils.visibility.AbstractDrawable;
import utils.visibility.DrawField2D;
import utils.visibility.Drawable;
import utils.visibility.Drawer;
import applications.clustering.data.ClusterDataGenerator;
import clustering.KMeans;
import clustering.KMeansState;
import clustering.KMeansUtils;
import clustering.RelativeKMeans;

public class PerformKMeansTest {

    public static double[][] mean = new double[][] { { 0, 0 }, { 0, 1 },
            { 8, -8 }, { 8, 0 },{ -8, -8 }, { -8, 0 } };

    public static double[][] std = new double[][] { { 1, 1 }, { 1, 1 },
            { 1, 1 }, { 1, 1 },{ 1, 1 }, { 1, 1 } };

    public static int[] size = new int[] { 250, 250, 250, 250, 250, 250 };

    public static double epsilon = 0.0001;

    public static double[][] getData(int clusterCount, Random rnd) {
        ClusterDataGenerator gen = new ClusterDataGenerator(2, rnd);
        for (int i = 0; i < clusterCount; i++) {
            gen.addNormalCluster(mean[i], std[i], size[i]);
        }
        double[][] data = gen.getData();

        return data;
    }

    public static double performTestFKM(int initCN, double val2, Random rnd) {
        mean[1][1] = val2;
        double[][] data = getData(6, rnd);
        KMeans kmeans = new KMeans(KMeansUtils.getDoubleDistance(2),
                 KMeans.OBJECTIVE, KMeans.MYVF2,
                KMeansUtils.INITALIZE_RANDOM, KMeansUtils
                        .getDoubleCopyFunction(2), KMeansUtils
                        .getDoubleMeansFunction(2));
        KMeansState result=kmeans.invokeKMeans(data, data.length, initCN);
        //plotData(data);
        System.out.println(initCN + ", " + result.validity);
        return result.validity;
    }
    
    public static double performTestRFKM(int initCN, double val2, Random rnd) {
        mean[1][1] = val2;
        double[][] data = getData(6, rnd);
        KMeans kmeans = new KMeans(KMeansUtils.getDoubleDistance(2),
                KMeans.OBJECTIVE, KMeans.MYVF2,
               KMeansUtils.INITALIZE_RANDOM, KMeansUtils
                       .getDoubleCopyFunction(2), KMeansUtils
                       .getDoubleMeansFunction(2));
        KMeansState result=kmeans.invokeKMeans(data, data.length, initCN);
        RelativeKMeans rkm=new RelativeKMeans(new KMeans(KMeansUtils.getDoubleDistance(2),
                 KMeans.OBJECTIVE, KMeans.MYVF2,
                KMeansUtils.INITALIZE_RANDOM, KMeansUtils
                        .getDoubleCopyFunction(2), KMeansUtils
                        .getDoubleMeansFunction(2)));
        result=rkm.invokeRelativeKMeans(data, data.length, 2,100,initCN,10);
        
        plotData(data,result.clusterCount,result.kMeans);
        System.out.println(result.clusterCount + ", " + result.validity);
        return result.validity;
    }

    public static void performTestFKM() {
        for (double val2 = 3; val2 <= 4; val2 += 1) {
            Writer write = null;
            try {
                write = new BufferedWriter(
                        new FileWriter("D:/work/Research/FKM/resultMy"
                                + (int) (val2) + ".txt"));

                for (int i = 0; i < 3; i++) {
                    Random rnd = new Random();
                    double minVal = Double.MAX_VALUE;
                    int minID = 0;
                    for (int c = 2; c <= 12; c++) {
                        double val = Double.MAX_VALUE;
                        for (int l = 0; l < 20; l++) {
                            double res = performTestRFKM(c, val2, rnd);
                            if (res < val) {
                                val = res;
                            }
                        }
                        if (val < minVal) {
                            minVal = val;
                            minID = c;
                        }
                        write.write(val + ", ");
                        if (c >= 8)
                            write.write(minID + "\n");
                        write.flush();
                    }
                }
                write.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void plotData(final double[][] data,final int clusterCount, final Object[] kMeans) {
        Drawable d = new AbstractDrawable() {
            public void draw(DrawField2D draw) {
                for (int i = 0; i < data.length; i++) {
                    draw.drawPoint(data[i][0], data[i][1], Color.BLACK);
                }
                for (int i = 0; i < clusterCount; i++) {
                    double[] mean=(double[])kMeans[i];
                    draw.drawCross(mean[0], mean[1], Color.RED,10);
                }
            }
        };
        Drawer draw = new Drawer(800, 800, -12, 12, -12, 12);
        draw.add(d);
        JFrame frame = draw.createFrame("Cluster Data");
        frame.setVisible(true);
    }

    public static void main(String[] args) {
        Logger logger=Logger.getLogger("Test");
        logger.setLevel(Level.ALL);
        Handler handler=new StreamHandler(System.err,new SimpleFormatter());
        handler.setLevel(Level.ALL);
        logger.addHandler(handler);
        
        
        //performTestFKM();
        performTestRFKM(3,4,new Random());//21543267));
        
    }
}
