package applications.clustering;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import utils.visibility.SVGDrawer;
import utils.visibility.View2DCluster;
import utils.visibility.ViewFunction;
import clustering.FuzzyAverages;
import clustering.FuzzyKMeans;
import clustering.FuzzyKMeansState;
import clustering.FuzzyKMeansUtils;
import clustering.KMeansUtils;
import clustering.RelativeFuzzyMeasure;
import clustering.RelativeFuzzyMeasures;

public class Vis2D {

    public static double[][] getData() {
        int n = 1000;
        double[][] data = new double[n][];
        for (int i = 0; i < n; i++) {
            double rad = Math.random() * Math.PI * 2;
            double len = Math.random() + 2;
            data[i] = getData(rad, len);
        }
        return data;
    }

    public static double[] getData(double rad, double len) {
        return new double[] { Math.sin(rad) * len, Math.cos(rad) * len };
    }

    public static FuzzyKMeansState getCluster(double[][] meandata,
            double[][] data) {
        return getCluster(meandata, meandata.length, data);
    }

    public static FuzzyKMeansState getCluster(double[][] meandata,
            int clusterCount, double[][] data) {
        FuzzyKMeans fkm = new FuzzyKMeans(KMeansUtils.getDoubleDistance(2), 2,
                FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3, FuzzyKMeansUtils
                        .getKMeansInitialization(meandata), KMeansUtils
                        .getDoubleCopyFunction(2), FuzzyAverages
                        .getDoubleMeansFunction(2));

        int dataCount = data.length;
        Object[] kmeans = new Object[clusterCount];
        double[][] clusterDist = new double[dataCount][clusterCount];
        double[] clusterCounts = new double[clusterCount];
        double[][] cluster = new double[dataCount][clusterCount];
        FuzzyKMeansState state = new FuzzyKMeansState(data, dataCount,
                clusterCount);
        state.epsilon = 0.000001;
        state.clusterSquaredDist = clusterDist;
        state.clusterCounts = clusterCounts;
        state.fuzzyCluster = cluster;
        state.maxClusterCount = clusterCount;
        state.minClusterCount = clusterCount;
        state.fkmc = fkm;
        state.kMeans = kmeans;
        fkm.initialize(state);
        fkm.computeMemberships(state);
        double lastObjective = fkm.computeObjective(state);

        fkm.computeValidity(state);
        return state;
    }

    public static FuzzyKMeansState performTestFKM(int initCN, double[][] data) {

        FuzzyKMeans kmeans = new FuzzyKMeans(KMeansUtils.getDoubleDistance(2),
                2, FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3,
                FuzzyKMeansUtils.INITALIZE_RANDOM, KMeansUtils
                        .getDoubleCopyFunction(2), FuzzyAverages
                        .getDoubleMeansFunction(2));
        FuzzyKMeansState result = kmeans.invokeFuzzyKMeans(data, data.length,
                initCN, 0.000001);
        return result;
    }

    public static FuzzyKMeansState get1(double[][] data) {
        int k = 3;
        double[][] km = new double[k][];
        double len = 2;
        double rad1 = 0;
        double rad2 = 2 * Math.PI / 3;
        double rad3 = 4 * Math.PI / 3;
        km[0] = getData(rad1, len);
        km[1] = getData(rad2, len);
        km[2] = getData(rad3, len);
        return getCluster(km, data);
    }

    public static FuzzyKMeansState get10(double[][] data, int k, double off) {
        double[][] km = new double[k][];
        double len = 2.5;
        for (int i = 0; i < k; i++) {
            double rad = i * 2 * Math.PI / k + off;
            km[i] = getData(rad, len);
        }

        return getCluster(km, data);
    }

    public static FuzzyKMeansState get20(double[][] data, int k, double off,
            int usek) {
        double[][] km = new double[k][];
        double len = 2;
        for (int i = 0; i < k; i++) {
            double rad = i * 2 * Math.PI / k + off;
            km[i] = getData(rad, len);
        }

        return getCluster(km, usek, data);
    }

    public static FuzzyKMeansState get10b(double[][] data, int k, double off,
            double rand) {
        double[][] km = new double[k][];
        double len = 2;
        for (int i = 0; i < k; i++) {
            double rad = i * 2 * Math.PI / k + off + rand * Math.random();
            km[i] = getData(rad, len);
        }

        return getCluster(km, data);
    }

    public static FuzzyKMeansState get11(double[][] data) {
        int k = 6;
        double[][] km = new double[k][];
        double len = 2;
        double rad1 = Math.PI / 6;
        double rad2 = 3 * Math.PI / 6;
        double rad3 = 5 * Math.PI / 6;
        double rad4 = 7 * Math.PI / 6;
        double rad5 = 9 * Math.PI / 6;
        double rad6 = 11 * Math.PI / 6;
        km[0] = getData(rad1, len);
        km[1] = getData(rad2, len);
        km[2] = getData(rad3, len);
        km[3] = getData(rad4, len);
        km[4] = getData(rad5, len);
        km[5] = getData(rad6, len);
        return getCluster(km, data);
    }

    public static FuzzyKMeansState get12(double[][] data) {
        int k = 12;
        double[][] km = new double[k][];
        double len = 2;
        for (int i = 0; i < k; i++) {
            double rad = i * 2 * Math.PI / k;
            km[i] = getData(rad, len);
        }

        return getCluster(km, data);
    }

    public static FuzzyKMeansState get2(double[][] data) {
        int k = 3;
        double[][] km = new double[k][];
        double len = 2;
        double rad1 = Math.PI / 3;
        double rad2 = 2 * Math.PI / 3 + Math.PI / 3;
        double rad3 = 4 * Math.PI / 3 + Math.PI / 3;
        km[0] = getData(rad1, len);
        km[1] = getData(rad2, len);
        km[2] = getData(rad3, len);
        return getCluster(km, data);
    }

    public static FuzzyKMeansState get3(double[][] data) {
        int k = 3;
        double[][] km = new double[k][];
        double len = 2;
        double rad1 = Math.PI / 12;
        double rad2 = 2 * Math.PI / 3 + Math.PI / 12;
        double rad3 = 4 * Math.PI / 3 + Math.PI / 12;
        km[0] = getData(rad1, len);
        km[1] = getData(rad2, len);
        km[2] = getData(rad3, len);
        return getCluster(km, data);
    }

    public static void test1() {
        double[][] data = getData();

        FuzzyKMeansState fks1 = get1(data);
        new View2DCluster(fks1, 600, 600, -4, 4, -4, 4, "Partition 1");

        FuzzyKMeansState fks2 = get2(data);
        new View2DCluster(fks2, 600, 600, -4, 4, -4, 4, "Partition 2");

        FuzzyKMeansState fks3 = get3(data);
        new View2DCluster(fks3, 600, 600, -4, 4, -4, 4, "Partition 3");

        double v1 = rfm.distance(fks1, fks2);
        double v2 = rfm.distance(fks1, fks3);
        double v3 = rfm.distance(fks2, fks3);

        System.out.println(v1);
        System.out.println(v2);
        System.out.println(v3);
    }

    public static void test1b() {
        double[][] data = getData();
        int k = 3;
        int steps = 128;
        FuzzyKMeansState fks1 = get10(data, k, 0);
        double[][] func = new double[steps + 1][2];

        for (int i = 0; i <= steps; i++) {
            double rad = (i * 2 * Math.PI) / (k * steps);
            FuzzyKMeansState fks2 = get10(data, k, rad);
            double v1 = rfm.distance(fks1, fks2);

            func[i][0] = rad;
            func[i][1] = v1;

            System.out.println(rad + "  " + v1);
        }

        new ViewFunction(func, 600, 600, 0, 2 * Math.PI / k, 0, 1,
                "Relative Measure");

    }

    public static void test1c() {
        double[][] data = getData();
        int steps = 4;
        FuzzyKMeansState fks1 = get10(data, 2, 0);
        double[][] func = new double[steps + 1][2];

 

        for (int i = 1; i <= steps; i++) {
            int k = (int) Math.pow(2, i);
            FuzzyKMeansState fks2 = get10(data, k, 0);
            double v1 = rfm.distance(fks1, fks2);

            func[i][0] = i;
            func[i][1] = v1;

            System.out.println(i + "  " + v1);

        }

        new ViewFunction(func, 600, 600, 0, steps, 0, 1,
                "Relative Measure");

    }

    public static void test2() {
        double[][] data = getData();

        FuzzyKMeansState fks1 = get1(data);
        new View2DCluster(fks1, 600, 600, -4, 4, -4, 4, "Partition 1");

        FuzzyKMeansState fks2 = get11(data);
        new View2DCluster(fks2, 600, 600, -4, 4, -4, 4, "Partition 2");

        FuzzyKMeansState fks3 = get2(data);
        new View2DCluster(fks3, 600, 600, -4, 4, -4, 4, "Partition 3");

        double v1 = rfm.distance(fks1, fks2);
        double v2 = rfm.distance(fks1, fks3);
        double v3 = rfm.distance(fks2, fks3);

        System.out.println(v1);
        System.out.println(v2);
        System.out.println(v3);

    }

    public static void test3() {
        double[][] data = getData();

        int k1 = 2;

        FuzzyKMeansState fks1 = get10(data, k1, 0);
        FuzzyKMeansState fks2 = get10(data, k1, Math.PI / k1);
        // new View2DCluster(fks2, 600, 600, -4, 4, -4, 4,"Partition 2");

        int k = 48;

        double[][][] func = new double[2][k - k1 + 1][2];

        for (int i = k1; i <= k; i++) {

            FuzzyKMeansState fks3 = get10(data, i, 0);
            double v1 = rfm.distance(fks1, fks3);
            func[0][i - k1][0] = i;
            func[0][i - k1][1] = v1;

            double v2 = rfm.distance(fks2, fks3);
            func[1][i - k1][0] = i;
            func[1][i - k1][1] = v2;

            System.out.println(v1 + "   " + v2);
        }

        new ViewFunction(func, 600, 600, 0, k, 0, 1, "Relative Measure");
    }

    public static void test4() {
        double[][] data = getData();

        int k1 = 12;

        FuzzyKMeansState fks1 = get10(data, k1, 0);

        int k = 2;

        double[][] func = new double[k1 - k + 1][2];

        for (int i = k1; i >= k; i--) {

            FuzzyKMeansState fks3 = get20(data, k1, 0, i);
            double v1 = rfm.distance(fks1, fks3);
            func[k1 - i][0] = i;
            func[k1 - i][1] = v1;

            System.out.println(v1);
        }

        new ViewFunction(func, 600, 600, 0, k1, 0, 1, "Relative Measure");
    }

    public static void test5() {
        FuzzyKMeansState data = PerformFKMTest
                .getClusteredPartition(23542455673l);

        int steps = 12;
        double[][] func = new double[steps + 1][2];

        for (int i = 0; i <= steps; i++) {

            FuzzyKMeansState res = PerformFKMTest.performTestFKM(
                    data.clusterCount + i, (double[][]) data.data);
            double v1 = rfm.distance(data, res);

            func[i][0] = i + data.clusterCount;
            func[i][1] = v1;

            System.out.println(v1);
        }

        new ViewFunction(func, 600, 600, 0, data.clusterCount + steps, 0, 1,
                "Relative Measure");

    }

    public static void viewTestData() {
        double[][] data = getData();
        int k = 3;
        int steps = 4;

        for (int i = 0; i <= steps; i++) {
            double rad = (i * 2 * Math.PI) / (k * steps);
            FuzzyKMeansState fks2 = get10(data, k, rad);
            View2DCluster v = new View2DCluster(fks2, 600, 600, -4, 4, -4, 4,
                    "Partition " + (i + 1));
            try {
                Writer write = new FileWriter("D:/work/Research/FKM/ring" + i
                        + ".svg");
                SVGDrawer.drawSVG(v.getDraw(), write);
                write.flush();
                write.close();
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
    }

    public static void viewTestData2() {
        double[][] data = getData();
        int steps = 4;

        for (int i = 1; i <= steps; i++) {
            int k = (int) Math.pow(2, i);
            FuzzyKMeansState fks2 = get10(data, k, 0);
            View2DCluster v = new View2DCluster(fks2, 600, 600, -4, 4, -4, 4,
                    "Partition " + (i + 1));
            try {
                Writer write = new FileWriter("D:/work/Research/FKM/ringCC" + i
                        + ".svg");
                SVGDrawer.drawSVG(v.getDraw(), write);
                write.flush();
                write.close();
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
    }

    public static RelativeFuzzyMeasure rfm = RelativeFuzzyMeasures.MY_RFM;

    public static void main(String[] args) {
        //viewTestData2();
        test1c();
    }
}
