package applications.clustering;

import java.io.File;
import java.io.IOException;
import java.util.Random;

import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ArffLoader;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.NormalNormalize;
import clustering.FuzzyAverages;
import clustering.FuzzyKMeans;
import clustering.FuzzyKMeansState;
import clustering.FuzzyKMeansUtils;
import clustering.KMeansUtils;
import clustering.RelativeFuzzyKMeans;

public class WekaFKMTest {

    public static double epsilon = 0.0001;

    public static FuzzyKMeansState performTestFKM(int initCN, double[][] data) {

        FuzzyKMeans kmeans = new FuzzyKMeans(KMeansUtils
                .getDoubleDistance(data[0].length), 2.0, FuzzyKMeans.OBJECTIVE,
                FuzzyKMeans.MYVF4, FuzzyKMeansUtils.INITALIZE_RANDOM,
                KMeansUtils.getDoubleCopyFunction(data[0].length),
                FuzzyAverages.getDoubleMeansFunction(data[0].length));
        FuzzyKMeansState result = kmeans.invokeFuzzyKMeans(data, data.length,
                initCN, epsilon);
        // plotData(data,result.clusterCount,result.kMeans);
        // System.out.println(initCN + ", " + result.validity);
        return result;
    }

    public static FuzzyKMeansState performTestRFKM(int initCN, double[][] data) {
        FuzzyKMeans kmeans = new FuzzyKMeans(KMeansUtils
                .getDoubleDistance(data[0].length), 2.0, FuzzyKMeans.OBJECTIVE,
                FuzzyKMeans.MYVF3, FuzzyKMeansUtils.INITALIZE_RANDOM,
                KMeansUtils.getDoubleCopyFunction(data[0].length),
                FuzzyAverages.getDoubleMeansFunction(data[0].length));
        FuzzyKMeansState result = kmeans.invokeFuzzyKMeans(data, data.length,
                initCN, epsilon);
        RelativeFuzzyKMeans rfkm = new RelativeFuzzyKMeans(new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(data[0].length), 2.0,
                FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3, FuzzyKMeansUtils
                        .getKMeansInitialization(result.kMeans), KMeansUtils
                        .getDoubleCopyFunction(data[0].length), FuzzyAverages
                        .getDoubleMeansFunction(data[0].length)));
        result = rfkm.invokeRelativeKMeans(data, data.length, 2, 10, initCN,
                epsilon, 10);

        System.out.println(result.clusterCount + ", " + result.validity);

        return result;
    }

    public static Instances getInstances(String file) {
        ArffLoader loader = new ArffLoader();
        try {
            loader.setFile(new File(file));
            Instances inst = loader.getDataSet();
            //inst.setClassIndex(0);
            // inst.setClassIndex(inst.numAttributes() - 1);
            NormalNormalize nNorm = new NormalNormalize();
            nNorm.setInputFormat(inst);
            inst = Filter.useFilter(inst, nNorm);
            // inst.randomize(new Random(123142373));
            return inst;

        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static double[][] getData(Instances inst) {
        int attCount = inst.numAttributes();
        int instCount = inst.numInstances();
        double[][] data = new double[instCount][attCount - 1];
        for (int i = 0; i < instCount; i++) {
            Instance in = inst.instance(i);
            int p = 0;
            for (int k = 0; k < attCount; k++) {
                if (inst.classIndex() != k && !inst.attribute(k).isNominal()) {
                    data[i][p] = in.value(k);
                    p++;
                }
            }
        }
        return data;

    }

    public static void testWeka(String file) {
        Instances insts = getInstances(file);
        double[][] data = getData(insts);
        for (int c = 2; c <= 10; c++) {
            double min = Double.MAX_VALUE;
            for (int i = 0; i < 100; i++) {
                FuzzyKMeansState result = performTestFKM(c, data);
                if (result.validity < min) {
                    min = result.validity;
                }
            }
            System.out.println(c + "   " + min);
        }
    }

    public static void test() {
        testWeka("D:/work/datasets/UCI/wine.arff");
    }

    public static void main(String[] args) {
        test();
    }

}
