package applications.clustering.data;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

import cern.jet.random.Normal;
import edu.cornell.lassp.houle.RngPack.RandomElement;

public class ClusterDataGenerator implements Cloneable {
    
    protected int dimension;
    protected List<double[]> data;
    protected RandomElement ran;
    
    protected double[] min;
    protected double[] max;
    
    public Object clone(){
        ClusterDataGenerator cl;
        try {
            cl = (ClusterDataGenerator)super.clone();
            cl.data=new ArrayList(data);
            
            return cl;
        } catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        return null;
    }
    
    public ClusterDataGenerator(int dimension,final Random random){
        this.dimension=dimension;
        min=new double[dimension];
        max=new double[dimension];
        for (int d=0;d<dimension;d++){
            min[d]=Double.MAX_VALUE;
            max[d]=-Double.MAX_VALUE;
        }
        data=new ArrayList();
        ran=new RandomElement(){
            public double raw(){
                return random.nextDouble();
            }
        };
    }
    
    public ClusterDataGenerator(int dimension){
        this(dimension,new Random());
    }
    
    public void addNormalCluster(double[]mean,double[]std, int datacount){
        Normal[] normal=new Normal[dimension];
        for (int i=0;i<dimension;i++){
            normal[i]=new Normal(mean[i],std[i],ran);
        }
        for (int i=0;i<datacount;i++){
            double[] val=new double[dimension];
            for(int d=0;d<dimension;d++){
                val[d]=normal[d].nextDouble();
                if (val[d]<min[d]){
                    min[d]=val[d];
                }
                if (val[d]>max[d]){
                    max[d]=val[d];
                }
            }
            data.add(val);
        }
    }
    
    public void addAreaCluster(double[]min,double[]max, int datacount){
        
        for (int i=0;i<datacount;i++){
            double[] val=new double[dimension];
            for(int d=0;d<dimension;d++){
                val[d]=ran.raw()*(max[d]-min[d])+min[d];
                if (val[d]<min[d]){
                    min[d]=val[d];
                }
                if (val[d]>max[d]){
                    max[d]=val[d];
                }
            }
            data.add(val);
        }
    }
    
    
    
    public void addNoise(double[]min, double[]max, int datacount){
        for (int i=0;i<datacount;i++){
            double[] val=new double[dimension];
            for(int d=0;d<dimension;d++){
                val[d]=ran.uniform(min[d],max[d]);
            }
            data.add(val);
        }
    }
    
    public void addNoise(int datacount){
        for (int i=0;i<datacount;i++){
            double[] val=new double[dimension];
            for(int d=0;d<dimension;d++){
                val[d]=ran.uniform(min[d],max[d]);
            }
            data.add(val);
        }
    }
    
    public int getDataCount(){
        return data.size();
    }
    
    public double[][] getData(){
        double[][]resdata=new double[data.size()][];
        Collections.shuffle(data);
        resdata=data.toArray(resdata);
        return resdata;
    }

}
