package applications.pipes;

import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Random;

import javax.swing.JFrame;

import pipes.Blocker2;
import pipes.CompareStream;
import pipes.DFTNormWeightedStreams;
import pipes.FunctionStream;
import pipes.FuzzyStreamCluster;
import pipes.FuzzyStreamingKMeans;
import pipes.MultiSource;
import pipes.MultipleSource;
import pipes.NormWeightedStreams;
import pipes.NullSink;
import pipes.SaveObjectStream;
import pipes.SaveTextStream;
import pipes.Verifier;
import pipes.data.SyntheticRealIterators.FactoryFunction;
import utils.visibility.clustering.VisualizeFKM;
import xxl.core.collections.queues.ArrayQueue;
import xxl.core.functions.Function;
import xxl.core.pipes.operators.identities.BufferPipe;
import xxl.core.pipes.processors.Processor;
import xxl.core.pipes.scheduler.Controllable;
import xxl.core.pipes.scheduler.RoundRobinStrategy;
import xxl.core.pipes.scheduler.StrategyProcessor;
import xxl.core.pipes.sources.Source;
import clustering.FuzzyAverages;
import clustering.FuzzyKMeans;
import clustering.FuzzyKMeansState;
import clustering.FuzzyKMeansUtils;
import clustering.KMeansUtils;
import clustering.RelativeFuzzyMeasures;

public class PaperClustering {

    protected static String dir = "/vol1/home/jberinge/result/streamFKM/";

    protected static String fileNonDFT = dir + "clusteringNonDFT";

    protected static String fileResultDFT = dir + "compareDFT";

    protected static String fileResultNonDFT = dir + "compareNonDFT";
    
    protected static String fileTimeDFT = dir + "timeDFT";
    protected static String fileTimeNonDFT = dir + "timeNonDFT";

    protected static int[] dfts = new int[] { 500, 250, 100, 50, 25, 10, 5, 2,
            1, 1023, 750 };

    protected static int d = -1;

    protected static int v = 0;

    public static PaperClustering wakeUp;

    public static void main(String[] args) {
        // viewDFTClustering(25);

//        int a = Integer.parseInt(args[0]);
//        // d = a % 11;
//        // v = a / 11;
//        d = a;
//        wakeUp = new PaperClustering();
//        wakeUp.run();

        timeNonDFTClustering();
        
        // rerun3();
        // generateNonDFTClustering(1);

        // verifyData();
    }

    public synchronized void run() {
        // while (true)
        {
            rerunTime();

            try {
                wakeUp = this;
                this.wait();
            } catch (InterruptedException e) {
                System.err.println("ERROR : " + e.getMessage());
                e.printStackTrace(System.err);
            }
            System.exit(0);
        }
    }

    public synchronized void wakeUp() {
        this.notify();
    }

    public static void rerun() {

        System.gc();
        System.out.println("d=" + d + "(" + dfts[d] + ")  v=" + v);
        compareDFTClustering(dfts[d], v);

    }
    
    public static void rerunTime() {

        System.gc();
        System.out.println("d=" + d + "(" + dfts[d] + ") ");
        timeDFTClustering(dfts[d]);

    }

    public static void rerun2() {

        System.gc();
        System.out.println("  v=" + v);
        compareNonDFTClustering(v);

    }

    public static void rerun3() {

        System.out.println("  v=" + v);
        generateNonDFTClustering(v);

    }

    public static MultipleSource getSource(int period) {
        int count = 100;
        int blockSize = 128;

        Random random = new Random(161752783);
        FactoryFunction generator = new FactoryFunction(6, count, blockSize,
                random);
        final MultiSource ms = new MultiSource(generator, count, period,
                blockSize);
        ms.setCloseMode(Source.AUTOMATIC_CLOSE);
        return ms;
    }

    public static void generateNonDFTClustering(int v) {
        int count = 100;
        int blockSize = 128;
        int blockCount = 16;
        int maxClasses = 12;
        double lastWeight = 0.2;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        String[] obsStreams = new String[count];
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        for (int i = 0; i < count; i++) {
            obsStreams[i] = "Stream Nr." + i;
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(1500);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);
        }
        BufferPipe dc = new BufferPipe(bl, 2, new ArrayQueue(), 1, 1);
        StrategyProcessor proc = new StrategyProcessor(new RoundRobinStrategy(
                new Controllable[] { dc }), 0, 2500);
        proc.start();

        NormWeightedStreams nws = new NormWeightedStreams(obsStreams, count,
                blockSize, blockCount, weight, bl.getFloatPool());
        nws.addSource(dc, 0);
        dc.subscribe(nws, 0);

        FuzzyStreamCluster wcs2 = new FuzzyStreamCluster(new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(windowSize), 2.0,
                FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3,
                FuzzyKMeansUtils.INITALIZE, KMeansUtils
                        .getDoubleCopyFunction(windowSize), FuzzyAverages
                        .getDoubleMeansFunction(windowSize)), obsStreams,
                count, blockCount, maxClasses, 3, 6);
        nws.subscribe(wcs2, 0);
        wcs2.addSource(nws, 0);

        wcs2.setBufferPipe(dc);

        SaveObjectStream save2 = null;
        try {
            save2 = new SaveObjectStream(fileNonDFT + "v" + v + ".bin");
            save2.setMaxCount(500);
            save2.setFunction(new Function() {
                public Object invoke() {
                    rerun3();
                    return null;
                }
            });
        } catch (IOException e) {
            e.printStackTrace();
        }

        wcs2.subscribe(save2, 0);
        save2.addSource(wcs2, 0);

        ms.open();
    }

    public static void compareDFTClustering(int dftCount, int t) {
        int count = 100;
        int blockSize = 128;
        int blockCount = 16;
        int maxClasses = 12;
        double lastWeight = 0.2;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        String[] obsStreams = new String[count];
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        double[][] cluster = new double[count][6];
        int cl = 0;
        for (int i = 0; i < count; i++) {
            cluster[i][cl] = 1.0;
            cl = (cl + 1) % 6;
            obsStreams[i] = "Stream Nr." + i;
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(200);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);
        }

        DFTNormWeightedStreams dft = new DFTNormWeightedStreams(obsStreams,
                count, blockSize, blockCount, dftCount, weight, bl
                        .getFloatPool());
        dft.addSource(bl, 0);
        bl.subscribe(dft, 0);

        FuzzyStreamCluster wcs = new FuzzyStreamCluster(obsStreams, count,
                blockCount, dftCount, maxClasses, 3, 6, 2.0, 0.0);

        FuzzyStreamingKMeans.QUIET = false;

        dft.subscribe(wcs, 0);
        wcs.addSource(dft, 0);

        final Processor proc = ms.getProcessor();

        try {
            String[] cfiles = new String[10];
            String[] tfiles = new String[10];
            for (int v = 0; v < 10; v++) {
                cfiles[v] = fileNonDFT + "v" + v + ".bin";
                tfiles[v] = fileResultDFT + dftCount + "t" + t + "v" + v
                        + ".txt";
            }

            CompareStream cs = new CompareStream<FuzzyKMeansState>(cfiles,
                    RelativeFuzzyMeasures.MY_RFM);

            cs.setFunction(new Function() {
                public Object invoke() {
                    proc.terminate();
                    wakeUp.wakeUp();
                    return null;
                }
            });

            wcs.subscribe(cs, 0);
            cs.addSource(wcs, 0);

            SaveTextStream saveResult = new SaveTextStream(tfiles);
            cs.subscribe(saveResult, 0);
            saveResult.addSource(cs, 0);

            ms.open();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void compareNonDFTClustering(int t) {
        int count = 100;
        int blockSize = 128;
        int blockCount = 16;
        int maxClasses = 12;
        double lastWeight = 0.2;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        String[] obsStreams = new String[count];
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        double[][] cluster = new double[count][6];
        int cl = 0;
        for (int i = 0; i < count; i++) {
            cluster[i][cl] = 1.0;
            cl = (cl + 1) % 6;
            obsStreams[i] = "Stream Nr." + i;
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(200);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);
        }

        NormWeightedStreams nws = new NormWeightedStreams(obsStreams, count,
                blockSize, blockCount, weight, bl.getFloatPool());
        nws.addSource(bl, 0);
        bl.subscribe(nws, 0);

        FuzzyStreamCluster wcs2 = new FuzzyStreamCluster(new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(windowSize), 2.0,
                FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3,
                FuzzyKMeansUtils.INITALIZE, KMeansUtils
                        .getDoubleCopyFunction(windowSize), FuzzyAverages
                        .getDoubleMeansFunction(windowSize)), obsStreams,
                count, blockCount, maxClasses, 3, 6);
        nws.subscribe(wcs2, 0);
        wcs2.addSource(nws, 0);

        final Processor proc = ms.getProcessor();
        try {
            String[] cfiles = new String[10];
            String[] tfiles = new String[10];
            for (int v = 0; v < 10; v++) {
                cfiles[v] = fileNonDFT + "v" + v + ".bin";
                tfiles[v] = fileResultNonDFT + "t" + t + "v" + v + ".txt";
            }

            CompareStream cs = new CompareStream<FuzzyKMeansState>(cfiles,
                    RelativeFuzzyMeasures.MY_RFM);
            cs.setFunction(new Function() {
                public Object invoke() {
                    proc.terminate();
                    wakeUp.wakeUp();
                    return null;
                }
            });

            wcs2.subscribe(cs, 0);
            cs.addSource(wcs2, 0);

            SaveTextStream saveResult = new SaveTextStream(tfiles);
            cs.subscribe(saveResult, 0);
            saveResult.addSource(cs, 0);

            ms.open();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void verifyData() {
        int count = 100;
        int blockSize = 128;
        int blockCount = 16;
        double lastWeight = 0.2;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        String[] obsStreams = new String[count];
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        for (int i = 0; i < count; i++) {
            obsStreams[i] = "Stream Nr." + i;
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(1500);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);
        }
        BufferPipe dc = new BufferPipe(bl, 2, new ArrayQueue(), 1, 1);
        StrategyProcessor proc = new StrategyProcessor(new RoundRobinStrategy(
                new Controllable[] { dc }), 0, 2500);
        proc.start();

        NormWeightedStreams nws = new NormWeightedStreams(obsStreams, count,
                blockSize, blockCount, weight, bl.getFloatPool());
        nws.addSource(dc, 0);
        dc.subscribe(nws, 0);

        String[] obsStreams2 = new String[count];
        int[] lengths2 = new int[count];
        boolean[] autosize2 = new boolean[count];
        double[] min2 = new double[count];
        double[] max2 = new double[count];
        for (int i = 0; i < count; i++) {
            obsStreams2[i] = "Stream Nr." + i;
            lengths2[i] = windowSize;
            autosize2[i] = true;
            min2[i] = 0;
            max2[i] = 1;
        }

        final MultipleSource ms2 = getSource(1500);

        Blocker2 bl2 = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms2.subscribe(bl2, i, i);
            bl2.addSource(ms2, i);
        }
        BufferPipe dc2 = new BufferPipe(bl2, 2, new ArrayQueue(), 1, 1);
        StrategyProcessor proc2 = new StrategyProcessor(new RoundRobinStrategy(
                new Controllable[] { dc2 }), 0, 2500);
        proc2.start();

        NormWeightedStreams nws2 = new NormWeightedStreams(obsStreams2, count,
                blockSize, blockCount, weight, bl2.getFloatPool());
        nws2.addSource(dc2, 0);
        dc2.subscribe(nws2, 0);

        Verifier ver = new Verifier();
        ver.addSource(nws, 0);
        nws.subscribe(ver, 0);

        ver.addSource(nws2, 1);
        nws2.subscribe(ver, 1);

        ms.open();
        ms2.open();
    }

    public static void timeDFTClustering(int dftCount) {
        int count = 100;
        int blockSize = 128;
        int blockCount = 16;
        int maxClasses = 12;
        double lastWeight = 0.2;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        String[] obsStreams = new String[count];
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        double[][] cluster = new double[count][6];
        int cl = 0;
        for (int i = 0; i < count; i++) {
            cluster[i][cl] = 1.0;
            cl = (cl + 1) % 6;
            obsStreams[i] = "Stream Nr." + i;
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(200);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);
        }

        final DFTNormWeightedStreams dft = new DFTNormWeightedStreams(obsStreams,
                count, blockSize, blockCount, dftCount, weight, bl
                        .getFloatPool());
        dft.addSource(bl, 0);
        bl.subscribe(dft, 0);

        FuzzyStreamCluster wcs = new FuzzyStreamCluster(obsStreams, count,
                blockCount, dftCount, maxClasses, 3, 6, 2.0, 0.0);

        FuzzyStreamingKMeans.QUIET = false;

        dft.subscribe(wcs, 0);
        wcs.addSource(dft, 0);
        
        try {
            dft.timeWriter=new FileWriter(fileTimeDFT+dftCount+".txt");
        } catch (IOException e) {
            e.printStackTrace();
        }

        NullSink saveResult = new NullSink();
        wcs.subscribe(saveResult, 0);
        saveResult.addSource(wcs, 0);
        
        final Processor proc = ms.getProcessor();
        
        saveResult.setFunction(new Function() {
            public Object invoke() {
                try {
                    dft.timeWriter.flush();
                    dft.timeWriter.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                proc.terminate();
                wakeUp.wakeUp();
                return null;
            }
        });
        
        saveResult.setMaxCount(500);

        ms.open();
    }
    
    public static void timeNonDFTClustering() {
        int count = 100;
        int blockSize = 128;
        int blockCount = 16;
        int maxClasses = 12;
        double lastWeight = 0.2;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        String[] obsStreams = new String[count];
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        double[][] cluster = new double[count][6];
        int cl = 0;
        for (int i = 0; i < count; i++) {
            cluster[i][cl] = 1.0;
            cl = (cl + 1) % 6;
            obsStreams[i] = "Stream Nr." + i;
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(200);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);
        }

        final NormWeightedStreams nws = new NormWeightedStreams(obsStreams, count,
                blockSize, blockCount, weight, bl.getFloatPool());
        nws.addSource(bl, 0);
        bl.subscribe(nws, 0);

        FuzzyStreamCluster wcs2 = new FuzzyStreamCluster(new FuzzyKMeans(
                KMeansUtils.getDoubleDistance(windowSize), 2.0,
                FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3,
                FuzzyKMeansUtils.INITALIZE, KMeansUtils
                        .getDoubleCopyFunction(windowSize), FuzzyAverages
                        .getDoubleMeansFunction(windowSize)), obsStreams,
                count, blockCount, maxClasses, 3, 6);
        nws.subscribe(wcs2, 0);
        wcs2.addSource(nws, 0);

        try {
            nws.timeWriter=new FileWriter(fileTimeNonDFT+".txt");
        } catch (IOException e) {
            e.printStackTrace();
        }

        NullSink saveResult = new NullSink();
        wcs2.subscribe(saveResult, 0);
        saveResult.addSource(wcs2, 0);
        
        final Processor proc = ms.getProcessor();
        
        saveResult.setFunction(new Function() {
            public Object invoke() {
                try {
                    nws.timeWriter.flush();
                    nws.timeWriter.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                proc.terminate();
                wakeUp.wakeUp();
                return null;
            }
        });
        
        saveResult.setMaxCount(500);

        ms.open();
    }


    public static void viewDFTClustering(final int dftCount) {
        int count = 100;
        int blockSize = 128;
        int blockCount = 16;
        int maxClasses = 12;
        double lastWeight = 0.2;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        final int windowSize = blockSize * blockCount;
        final String[] obsStreams = new String[count];
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        double[][] cluster = new double[count][6];
        int cl = 0;
        for (int i = 0; i < count; i++) {
            cluster[i][cl] = 1.0;
            cl = (cl + 1) % 6;
            obsStreams[i] = "Stream Nr." + i;
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(200);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);
        }

        DFTNormWeightedStreams dft = new DFTNormWeightedStreams(obsStreams,
                count, blockSize, blockCount, dftCount, weight, bl
                        .getFloatPool());
        dft.addSource(bl, 0);
        bl.subscribe(dft, 0);

        FuzzyStreamCluster wcs = new FuzzyStreamCluster(obsStreams, count,
                blockCount, dftCount, maxClasses, 3, 6, 2.0, 0.0);

        FuzzyStreamingKMeans.QUIET = false;

        dft.subscribe(wcs, 0);
        wcs.addSource(dft, 0);

        final int[] select = new int[6];
        for (int i = 0; i < 6; i++) {
            select[i] = (int) (Math.random() * count);
        }

        FunctionStream<FuzzyKMeansState> fs = new FunctionStream();
        fs.setFunction(new Function<FuzzyKMeansState, Object>() {
            JFrame fr = null;

            VisualizeFKM vfkm = null;

            public Object invoke(FuzzyKMeansState state) {
                if (vfkm == null) {
                    vfkm = new VisualizeFKM(state, obsStreams, select, 6, 800,
                            800);
                    fr = vfkm.createFrame("vis");
                    fr.setVisible(true);
                } else {
                    vfkm.setDFTState(state);
                    fr.repaint(50);
                }

                return null;
            }
        });
        // fs.setStartStop(5,100);

        wcs.subscribe(fs, 0);
        fs.addSource(wcs, 0);

        ms.open();
    }

}
