package applications.pipes;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.net.MalformedURLException;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Iterator;

import javax.swing.JFrame;

import pipes.Blocker2;
import pipes.DFTNormWeightedStockStreams;
import pipes.FunctionStream;
import pipes.FuzzyStreamCluster;
import pipes.MultiSource;
import pipes.MultipleSource;
import pipes.ReadObjectStream;
import pipes.SaveObjectStream;
import utils.visibility.SVGDrawer;
import utils.visibility.clustering.VisualizeMeanStreams;
import weka.core.Utils;
import xxl.core.functions.Function;
import xxl.core.relational.cursors.InputStreamMetaDataCursor;
import xxl.core.relational.metaData.ResultSetMetaDatas;
import xxl.core.relational.tuples.ArrayTuple;
import xxl.core.util.CompositeMetaData;
import clustering.FuzzyKMeansState;

public class StockClustering {

    protected static String dir = "D:/work/Research/AFKM/";

    // "/vol1/home/jberinge/result/streamFKM/";

    // "D:/work/Research/AFKM/";

    protected static String data = "D:/work/datasets/Aktien/HDAX2605-090704.tbl";

    // "/vol1/home/jberinge/data/stocks/HDAX2605-240804.tbl";
    // "D:/work/datasets/Aktien/HDAX2605-090704.tbl";

    protected static String[] names;

    public static void main(String[] args) {
        // clusterSavedVisStock(25);
        // clusterVisStock(25);
        // clusterStock(25);
        // clusterStock(50);
        // analyse();
        results();
    }

    public static String[] getNames() {

        InputStreamMetaDataCursor cursor = null;
        try {
            cursor = new InputStreamMetaDataCursor(new File(data).toURL(),
                    new String[] { "\t" }, new String[] { "\n" }, "#",
                    ArrayTuple.FACTORY_METHOD);
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }

        CompositeMetaData cmeta = cursor.getMetaData();
        ResultSetMetaData meta = (ResultSetMetaData) cmeta
                .get(ResultSetMetaDatas.RESULTSET_METADATA_TYPE);
        names = new String[111];
        for (int i = 0; i < 111; i++) {
            try {
                names[i] = meta.getColumnName(i + 3);
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        return names;

    }

    public static MultipleSource getSource(int period, final int blocksize) {
        try {
            InputStreamMetaDataCursor cursor = new InputStreamMetaDataCursor(
                    new File(data).toURL(), new String[] { "\t" },
                    new String[] { "\n" }, "#", ArrayTuple.FACTORY_METHOD);

            CompositeMetaData cmeta = cursor.getMetaData();
            ResultSetMetaData meta = (ResultSetMetaData) cmeta
                    .get(ResultSetMetaDatas.RESULTSET_METADATA_TYPE);
            names = new String[111];
            for (int i = 0; i < 111; i++) {
                try {
                    names[i] = meta.getColumnName(i + 3);
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }

            final Iterator it = MultiSource.getMultiSourceFileIterator(cursor,
                    111, 3);
            return new MultiSource(new Function() {
                public Object invoke() {
                    double[][] data = new double[111][blocksize];
                    for (int i = 0; i < blocksize; i++) {
                        Object[] val = (Object[]) it.next();
                        for (int j = 0; j < 111; j++) {
                            data[j][i] = ((Double) val[j]).doubleValue();
                        }
                    }
                    return data;
                }
            }, 111, period, blocksize);
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;

    }

    public static void clusterStock(int dftCount) {
        int count = 111;
        int blockSize = 30;
        int blockCount = 16;

        int maxClasses = 20;
        double lastWeight = 0.4;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        getNames();
        String[] obsStreams = names;
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        for (int i = 0; i < count; i++) {
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(500, blockSize);
        // VisualizeStreams visStreams = new VisualizeStreams("Streams", count,
        // obsStreams, lengths, autosize, min, max, 0, null, null);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);

            // ms.subscribe(visStreams, i);
            // visStreams.addSource(ms, i);
        }

        DFTNormWeightedStockStreams dft = new DFTNormWeightedStockStreams(
                obsStreams, count, blockSize, blockCount, dftCount, weight, bl
                        .getFloatPool());
        dft.addSource(bl, 0);
        bl.subscribe(dft, 0);

        FuzzyStreamCluster wcs =
        // new FuzzyStreamCluster(obsStreams, count, blockCount, dftCount,
        // maxClasses, 6, 2.0);
        new FuzzyStreamCluster(obsStreams, count, blockCount, dftCount,
                maxClasses, 2, 6, 2.0, 0.0);
        dft.subscribe(wcs, 0);
        wcs.addSource(dft, 0);

        SaveObjectStream<FuzzyKMeansState> save2 = null;
        try {
            save2 = new SaveObjectStream(dir + "Stock2Cluster" + dftCount
                    + ".bin");

        } catch (IOException e) {
            e.printStackTrace();
        }

        wcs.subscribe(save2, 0);
        save2.addSource(wcs, 0);

        ms.open();
    }

    public static void clusterSavedVisStock(int dftCount) {
        String file = dir + "Stock2Cluster" + dftCount + ".bin";
        ReadObjectStream<FuzzyKMeansState> cluster = null;
        try {
            cluster = new ReadObjectStream(new FileInputStream(file), 150);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        FunctionStream<FuzzyKMeansState> fs = new FunctionStream();
        fs.setFunction(new Function<FuzzyKMeansState, Object>() {
            JFrame fr = null;

            int t = 0;

            VisualizeMeanStreams vfkm = null;

            public Object invoke(FuzzyKMeansState state) {
                t++;
                System.out.println(t);
                if (vfkm == null) {
                    vfkm = new VisualizeMeanStreams(state, 900, 600);
                    fr = vfkm.createFrame("vis");
                    fr.setVisible(true);
                } else {
                    vfkm.setDFTState(state);
                    fr.repaint(50);

                }

                // try {
                // String file = dir + "Image" + t + ".svg";
                // Writer write = new BufferedWriter(new FileWriter(file));
                // SVGDrawer.drawSVG(vfkm, write);
                // write.flush();
                // write.close();
                // } catch (IOException e) {
                // e.printStackTrace();
                // }

                try {
                    String file = dir + "Image" + t + ".jpg";
                    OutputStream write = new BufferedOutputStream(
                            new FileOutputStream(file));
                    vfkm.saveImage(write);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                return null;
            }
        });
        // fs.setStartStop(5,100);

        cluster.subscribe(fs, 0);
        fs.addSource(cluster, 0);
        cluster.open();
    }

    public static void clusterVisStock(int dftCount) {
        int count = 111;
        int blockSize = 30;
        int blockCount = 16;

        int maxClasses = 20;
        double lastWeight = 0.4;
        double weight = Math.pow(lastWeight, 1.0 / (blockSize * blockCount));

        int windowSize = blockSize * blockCount;
        getNames();
        String[] obsStreams = names;
        int[] lengths = new int[count];
        boolean[] autosize = new boolean[count];
        double[] min = new double[count];
        double[] max = new double[count];
        for (int i = 0; i < count; i++) {
            lengths[i] = windowSize;
            autosize[i] = true;
            min[i] = 0;
            max[i] = 1;
        }

        final MultipleSource ms = getSource(500, blockSize);
        // VisualizeStreams visStreams = new VisualizeStreams("Streams", count,
        // obsStreams, lengths, autosize, min, max, 0, null, null);

        Blocker2 bl = new Blocker2(count, 1);
        for (int i = 0; i < count; i++) {
            ms.subscribe(bl, i, i);
            bl.addSource(ms, i);

            // ms.subscribe(visStreams, i);
            // visStreams.addSource(ms, i);
        }

        DFTNormWeightedStockStreams dft = new DFTNormWeightedStockStreams(
                obsStreams, count, blockSize, blockCount, dftCount, weight, bl
                        .getFloatPool());
        dft.addSource(bl, 0);
        bl.subscribe(dft, 0);

        FuzzyStreamCluster wcs =
        // new FuzzyStreamCluster(obsStreams, count, blockCount, dftCount,
        // maxClasses, 6, 2.0);
        new FuzzyStreamCluster(obsStreams, count, blockCount, dftCount,
                maxClasses, 2, 6, 2.0, 0.0);
        dft.subscribe(wcs, 0);
        wcs.addSource(dft, 0);
        FunctionStream<FuzzyKMeansState> fs = new FunctionStream();
        fs.setFunction(new Function<FuzzyKMeansState, Object>() {
            JFrame fr = null;

            VisualizeMeanStreams vfkm = null;

            public Object invoke(FuzzyKMeansState state) {
                if (vfkm == null) {
                    vfkm = new VisualizeMeanStreams(state, 950, 700);
                    fr = vfkm.createFrame("vis");
                    fr.setVisible(true);
                } else {
                    vfkm.setDFTState(state);
                    fr.repaint(50);
                }

                return null;
            }
        });
        // fs.setStartStop(5,100);

        wcs.subscribe(fs, 0);
        fs.addSource(wcs, 0);
        ms.open();
    }

    public static void analyse() {
        int dftCount = 25;
        String file = dir + "Stock2Cluster" + dftCount + ".bin";
        try {
            InputStream is = new FileInputStream(file);
            ObjectInputStream ois = new ObjectInputStream(
                    new BufferedInputStream(is));
            FuzzyKMeansState state = (FuzzyKMeansState) read(ois);

            int count = state.dataCount;
            double[][] comp = new double[count][count];
            while (state != null) {
                final int cc = state.clusterCount;
                for (int i = 0; i < count; i++) {
                    for (int j = i + 1; j < count; j++) {
                        double d = 0;
                        for (int c = 0; c < cc; c++) {
                            d += Math.abs(state.fuzzyCluster[i][c]
                                    - state.fuzzyCluster[j][c]);
                        }
                        comp[i][j] += d;
                        comp[j][i] += d;
                    }
                }
                state = (FuzzyKMeansState) read(ois);
            }
            String fileOut = dir + "stockMatrix" + dftCount + ".bin";
            ObjectOutputStream oos = new ObjectOutputStream(
                    new BufferedOutputStream(new FileOutputStream(fileOut)));
            oos.writeObject(comp);
            oos.flush();
            oos.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void analyse2() {
        int dftCount = 25;
        String file = dir + "Stock2Cluster" + dftCount + ".bin";
        String fileout = dir + "StockCC" + dftCount + ".txt";
        try {
            InputStream is = new FileInputStream(file);
            ObjectInputStream ois = new ObjectInputStream(
                    new BufferedInputStream(is));

            FileWriter write = new FileWriter(fileout);

            FuzzyKMeansState state = (FuzzyKMeansState) read(ois);
            while (state != null) {
                final int cc = state.clusterCount;
                write.write("" + cc + "\n");
                state = (FuzzyKMeansState) read(ois);
            }
            write.flush();
            write.close();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static Object read(ObjectInputStream ois) {
        try {
            Object r = ois.readObject();
            return r;
        } catch (IOException e) {
            // e.printStackTrace();
            return null;
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void results() {
        int dftCount = 25;
        String file = dir + "stockMatrix" + dftCount + ".bin";
        String[] names = getNames();
        try {
            InputStream is = new FileInputStream(file);
            ObjectInputStream ois = new ObjectInputStream(
                    new BufferedInputStream(is));
            double[][] comp = (double[][]) read(ois);
            int n = comp.length;
            int[][] inds = new int[n][];
            for (int i = 0; i < n; i++) {
                comp[i][i] = Double.MAX_VALUE;
                inds[i] = Utils.sort(comp[i]);
                System.out.println(names[i] + " <--> " + names[inds[i][0]]
                        + "," + names[inds[i][1]] + "," + names[inds[i][2]]
                        + "," + names[inds[i][3]]);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
