/*
 * Created on 20.08.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package clustering;

import java.util.List;

import xxl.core.functions.Function;

/**
 * @author beringer
 */
public class KMeansUtils {

    public static Average getComplexMeansFunction(final int length) {
        return new Average() {
            public Object average(KMeansClustering kmc, Object[] obs, int len,
                    Object mean) {

                double[][] help = (double[][]) mean;
                if (help == null)
                    help = new double[length][2];
                for (int i = 0; i < length; i++) {
                    help[i][0] = 0f;
                    help[i][1] = 0f;
                }
                for (int j = 0; j < len; j++) {
                    double[][] object = (double[][]) obs[j];
                    for (int i = 0; i < length; i++) {
                        help[i][0] += object[i][0];
                        help[i][1] += object[i][1];
                    }
                }
                for (int i = 0; i < length; i++) {
                    help[i][0] /= len;
                    help[i][1] /= len;
                }
                return help;
            }
        };
    }

    static public Initialization INITALIZE_RANDOM = new Initialization() {
        public void invoke(KMeansState state) {
            Function copyFunction = ((KMeansState) state).kmc.getCopyFunction();
            for (int i = 0; i < state.clusterCount; i++) {
                state.kMeans[i] = copyFunction.invoke(state.data[(int) (Math
                        .random() * state.dataCount)], state.kMeans[i]);
            }

        }
    };

    public static Function getComplexCopyFunction(final int length) {
        return new Function() {
            public Object invoke(Object ob1, Object ob2) {
                double[][] d1 = (double[][]) ob1;
                double[][] d2 = (double[][]) ob2;
                if (d2 == null)
                    d2 = new double[length][2];
                for (int i = 0; i < length; i++) {
                    d2[i][0] = d1[i][0];
                    d2[i][1] = d1[i][1];
                }
                return d2;
            }
        };
    }

    public static DimDistance getComplexDistance(final int length) {
        return new AbstractDimDistance(length) {
            public double squaredDistance(Object ob1, Object ob2) {
                double[][] d1 = (double[][]) ob1;
                double[][] d2 = (double[][]) ob2;
                double sum2 = 0.0f;
                double dif = 0;
                double dif0 = 0;
                double dif1 = 0;
                for (int i = 0; i < dimCount; i++) {
                    dif0 = d1[i][0] - d2[i][0];
                    dif1 = d1[i][1] - d2[i][1];
                    dif = dif0 * dif0 + dif1 * dif1;
                    // dif=((d1[i][0]*d1[i][0]+d1[i][1]*d1[i][1])-(d2[i][0]*d2[i][0]+d2[i][1]*d2[i][1]));
                    sum2 += dif;
                }
                return sum2;
            }

            public double squaredDistance(Object ob1, Object ob2, int dim) {
                double[][] d1 = (double[][]) ob1;
                double[][] d2 = (double[][]) ob2;
                double dif = 0;
                double dif0 = 0;
                double dif1 = 0;
                dif0 = d1[dim][0] - d2[dim][0];
                dif1 = d1[dim][1] - d2[dim][1];
                dif = dif0 * dif0 + dif1 * dif1;
                return dif;
            }
        };
    }

    /**
     * @param fuzzifier
     */
    public static KMeansClustering getComplexKMeans(int length) {
        return new KMeans(getComplexDistance(length), KMeans.OBJECTIVE,
                KMeans.SEPARATION, KMeans.INITALIZE,
                getComplexCopyFunction(length), getComplexMeansFunction(length));
    }

    public static Average getDoubleMeansFunction(final int length) {
        return new Average() {
            public Object average(KMeansClustering kmc, Object[] obs, int len,
                    Object mean) {

                double[] help = (double[]) mean;
                if (help == null)
                    help = new double[length];
                for (int i = 0; i < length; i++) {
                    help[i] = 0d;
                }
                for (int j = 0; j < len; j++) {
                    double[] object = (double[]) obs[j];
                    for (int i = 0; i < length; i++) {
                        help[i] += object[i];
                    }
                }
                for (int i = 0; i < length; i++) {
                    help[i] /= len;
                }
                return help;
            }
        };
    }

    public static Function getDoubleCopyFunction(final int length) {
        return new Function() {
            public Object invoke(Object ob1){
                return invoke(ob1,null);
            }
            
            public Object invoke(Object ob1, Object ob2) {
                double[] d1 = (double[]) ob1;
                double[] d2 = (double[]) ob2;
                if (d2 == null)
                    d2 = new double[length];
                for (int i = 0; i < length; i++) {
                    d2[i] = d1[i];
                }
                return d2;
            }
        };

    }

    public static DimDistance getDoubleDistance(final int length) {
        return new AbstractDimDistance(length) {
            public double squaredDistance(Object ob1, Object ob2) {
                double[] d1 = (double[]) ob1;
                double[] d2 = (double[]) ob2;
                double sum2 = 0.0f;
                double dif = 0;
                for (int i = 0; i < dimCount; i++) {
                    dif = d1[i] - d2[i];
                    dif = dif * dif;
                    // dif=((d1[i][0]*d1[i][0]+d1[i][1]*d1[i][1])-(d2[i][0]*d2[i][0]+d2[i][1]*d2[i][1]));
                    sum2 += dif;
                }
                return sum2;
            }

            public double squaredDistance(Object ob1, Object ob2, int dim) {
                double[] d1 = (double[]) ob1;
                double[] d2 = (double[]) ob2;
                double dif = 0;
                dif = d1[dim] - d2[dim];
                dif = dif * dif;
                return dif;
            }
        };
    }

    /**
     * @param fuzzifier
     */
    public static KMeans getDoubleKMeans(int length) {
        return new KMeans(getDoubleDistance(length), KMeans.OBJECTIVE,
                KMeans.SEPARATION, KMeans.INITALIZE,
                getDoubleCopyFunction(length), getDoubleMeansFunction(length));
    }

    public static KMeans getDoubleKMeansLerman(int length) {
        return new KMeans(getDoubleDistance(length), KMeans.OBJECTIVE,
                KMeans.LERMAN, KMeans.INITALIZE, getDoubleCopyFunction(length),
                getDoubleMeansFunction(length));
    }

}
