/*
 * Created on 20.08.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package clustering;

import xxl.core.functions.Function;

/**
 * @author beringer
 */
public class PossibilisticKMeans extends AbstractFuzzyKMeans {

	static public FuzzyObjective OBJECTIVE= new FuzzyObjective() {
        public double invoke(FuzzyKMeansState state){
            PossibilisticKMeans pkm=(PossibilisticKMeans)state.fkmc;

			double result= 0d;
			for (int i= 0; i < state.dataCount; i++) {
				double sum= 0.0;
				for (int j= 0; j < state.clusterCount; j++) {
					double val= state.fuzzyCluster[i][j];
					double val2= 1 - val;
					sum += val;
					double dist= state.clusterSquaredDist[i][j];
					if (pkm.fuzzifier == 2.0) {
						val= val * val;
						val2= val2 * val2;
					} else {
						val= Math.pow(val, pkm.fuzzifier);
						val2= Math.pow(val2, pkm.fuzzifier);
					}
					result += val * dist+pkm.thick[j]*val2;
				}
			}
            state.objective=result;
			return result;
		}

	};

	//protected boolean initialized;
	double[] thick;

	public PossibilisticKMeans(
		Distance distance,
		double fuzzifier,
        FuzzyValidity validity,
		Initialization initialize,
		Function copyObject,
		FuzzyAverage meansFunction) {
		super(distance, fuzzifier, OBJECTIVE, validity, initialize, copyObject, meansFunction);
	}

	public void computeInitialMemberships(
            FuzzyKMeansState state) {

		int full= 0;
		for (int k= 0; k < state.clusterCount; k++) {
            state.clusterCounts[k]= 0d;
		}
		for (int i= 0; i < state.dataCount; i++) {
			full= 0;
			double minDist= Double.MAX_VALUE;
			for (int k= 0; k < state.clusterCount; k++) {
				double d= distance.squaredDistance(state.data[i], state.kMeans[k]);
                state.clusterSquaredDist[i][k]= d;
				if (d < minDist)
					minDist= d;
				if (d == 0)
					full++;
			}
			for (int k= 0; k < state.clusterCount; k++) {
				if (full > 0) {
					if (state.clusterSquaredDist[i][k] == 0)
                        state.fuzzyCluster[i][k]= 1 / full;
					else
                        state.fuzzyCluster[i][k]= 0;
					continue;
				}
				double sum= 0f;
				double d2= state.clusterSquaredDist[i][k];
				for (int k2= 0; k2 < state.clusterCount; k2++) {
					//clusterFuzzyDist[k][k2]=0f;
					if (fuzzifier == 2.0)
						sum += (d2 / (state.clusterSquaredDist[i][k2]));
					else
						sum
							+= Math.pow((d2 / state.clusterSquaredDist[i][k2]), (1.0 / (fuzzifier - 1.0)));
				}

                state.fuzzyCluster[i][k]= 1 / sum;
                state.clusterCounts[k] += state.fuzzyCluster[i][k];
			}
		}
	}

	public void computeMemberships(
		FuzzyKMeansState state) {

		if (state.clusterCounts[0] >= 0.0) {
			computeInitialMemberships(
				state);
		}
		thick= new double[state.clusterCount];
		for (int k= 0; k < state.clusterCount; k++) {
			double denom= 0.0;
			double numer= 0.0;
			for (int i= 0; i < state.dataCount; i++) {
				double dist= distance.squaredDistance(state.data[i], state.kMeans[k]);
                state.clusterSquaredDist[i][k]= dist;
				double val= state.fuzzyCluster[i][k];
				if (fuzzifier == 2.0)
					val *= val;
				else
					val= Math.pow(val, fuzzifier);
				denom += val;
				numer += val * dist;
			}
			thick[k]= numer / denom;
		}
		for (int i= 0; i < state.dataCount; i++) {
            double sum=0;
			for (int k= 0; k < state.clusterCount; k++) {
				double val= state.clusterSquaredDist[i][k] / thick[k];
				if (fuzzifier != 2.0)
					val= Math.pow(val, (1.0 / (fuzzifier - 1.0)));
                state.fuzzyCluster[i][k]= 1.0 / (1.0 + val);
                sum+=state.fuzzyCluster[i][k];
			}
            if (sum>1){
                for (int k= 0; k < state.clusterCount; k++) {
                    state.fuzzyCluster[i][k]/=sum;
                }
            }
		}

	}

}
