/*
 * Created on 03.06.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package clustering;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author beringer
 */
public class RelativeFuzzyKMeans {

    static Logger logger = Logger.getLogger("Test");

    static protected boolean QUIET = true;

    static public int OBJECTIVE_CHOOSE = 0;

    static public int VALIDITY_CHOOSE = 1;

    // public int mode=OBJECTIVE_CHOOSE;
    protected int mode = VALIDITY_CHOOSE;

    protected FuzzyKMeansClustering fkmAlg;

    protected int testCount;

    protected int[] testInd;

    protected int[] kCounter;

    protected Object[][] bestMeans;

    protected double[] bestValidity;

    public RelativeFuzzyKMeans(FuzzyKMeansClustering fkmAlg) {
        this.fkmAlg = fkmAlg;
    }

    public void setOptimizationMode(int mode) {
        this.mode = mode;
    }

    public int getMode() {
        return mode;
    }

    public FuzzyKMeansState invokeRelativeKMeans(Object[] data, int dataCount,
            int minClusterCount, int maxClusterCount, int initClusterCount,
            double epsilon, double testpart) {
        if (minClusterCount > maxClusterCount)
            throw new IllegalArgumentException(
                    "minClusterCount>maxClusterCount !!!");
        if (testpart >= 1)
            testCount = (int) Math.round(testpart);
        else
            testCount = (int) Math.ceil(dataCount * testpart);
        int maxClusterBorder = maxClusterCount + 1;
        testInd = new int[testCount];
        int clusterCount = initClusterCount;
        Object[] kmeans = new Object[maxClusterBorder];

        double[][] cluster = new double[dataCount][maxClusterBorder];
        double[] clusterCounts = new double[dataCount];
        double[][] clusterDist = new double[dataCount][maxClusterBorder];

        kCounter = new int[maxClusterBorder + 1];
        bestMeans = new Object[maxClusterBorder + 1][];
        bestValidity = new double[maxClusterBorder + 1];

        FuzzyKMeansState state = new FuzzyKMeansState(data, dataCount,
                clusterCount);
        state.epsilon = epsilon;
        state.clusterSquaredDist = clusterDist;
        state.clusterCounts = clusterCounts;
        state.fuzzyCluster = cluster;
        state.maxClusterCount = maxClusterBorder;
        state.minClusterCount = Math.max(2, minClusterCount - 1);
        state.fkmc = fkmAlg;
        state.kMeans = kmeans;
        fkmAlg.initialize(state);
        fkmAlg.computeMemberships(state);
        boolean hasChanged = false;
        int oldClusterCount;

        state = removeEqualCluster(state);

        // falls notwendig clustercount in [minCC,maxCC] bringen
        while (clusterCount > maxClusterCount) {
            state = testRemoveClusterCount(state, true);
            clusterCount = state.clusterCount;
        }

        while (clusterCount < minClusterCount) {
            state = testAddClusterCount(state, true);
            clusterCount = state.clusterCount;
        }

        state = removeEqualCluster(state);

        // FKM mit vierfachem epsilon
        double newVal, oldVal;
        newVal = fkmAlg.computeObjective(state);
        do {
            oldVal = newVal;
            fkmAlg.computeClusterCenters(state);
            fkmAlg.computeMemberships(state);
            newVal = fkmAlg.computeObjective(state);
        } while ((oldVal - newVal) > epsilon * 4);

        // Am Anfang mglichst viele schlechte Klassen lschen
        do {
            oldClusterCount = clusterCount;
            if (clusterCount > 2) {
                state = testRemoveClusterCount(state, false);
                clusterCount = state.clusterCount;
            }
        } while (oldClusterCount != clusterCount);

        // Beginn der Hauptschleife
        int loop = 0;
        do {
            loop++;
            hasChanged = false;

            newVal = fkmAlg.computeObjective(state);
            do {
                oldVal = newVal;
                fkmAlg.computeClusterCenters(state);
                fkmAlg.computeMemberships(state);
                newVal = fkmAlg.computeObjective(state);
            } while ((oldVal - newVal) > epsilon * 4);

            // Klassen lschen, hinzufgen
            oldClusterCount = clusterCount;
            state = testClusterCount(state);
            clusterCount = state.clusterCount;
            if (oldClusterCount != clusterCount) {
                hasChanged = true;
                kCounter[state.clusterCount]++;
                if (kCounter[state.clusterCount] >= 2) {
                    if (kCounter[state.clusterCount] == 2) {
                        bestMeans[state.clusterCount] = new Object[state.clusterCount];
                    }
                    if (kCounter[state.clusterCount] == 2
                            || bestValidity[state.clusterCount] > state.validity) {
                        copyMeans(state.kMeans, bestMeans[state.clusterCount],
                                state.clusterCount);
                        bestValidity[state.clusterCount] = state.validity;
                    }
                    if (kCounter[state.clusterCount] > state.clusterCount) {
                        int bestInd = 0;
                        double bestVal = Double.MAX_VALUE;
                        for (int i = state.minClusterCount; i < state.maxClusterCount; i++) {
                            if (kCounter[i] >= 2 && bestValidity[i] < bestVal) {
                                bestInd = i;
                                bestVal = bestValidity[i];
                            }
                        }
                        logger.log(Level.INFO, "No Absolut: " + (bestInd) + " "
                                + bestVal);
                        state.clusterCount = bestInd;
                        copyMeans(bestMeans[bestInd], state.kMeans, bestInd);
                        fkmAlg.computeMemberships(state);
                        hasChanged = false;
                    }
                }
            }

        } while (hasChanged && (loop < 20));

        // falls notwendig clustercount in [minCC,maxCC] bringen
        while (clusterCount > maxClusterCount) {
            state = testRemoveClusterCount(state, true);
            clusterCount = state.clusterCount;
        }

        while (clusterCount < minClusterCount) {
            state = testAddClusterCount(state, true);
            clusterCount = state.clusterCount;
        }

        // zum Abschluss Fuzzy K-Means mit korrektem epsilon fr genaue Lsung
        do {
            oldVal = newVal;
            fkmAlg.computeClusterCenters(state);
            fkmAlg.computeMemberships(state);
            newVal = fkmAlg.computeObjective(state);
        } while ((oldVal - newVal) > epsilon);
        fkmAlg.computeValidity(state);
        return state;
    }

    public int getStream(int dataCount, double[][] cluster, int clusterCount,
            double[][] clusterSquaredDist, double fuzzifier, double value,
            double limit) {
        double allSum = 0f;
        double val;
        for (int i = 0; i < dataCount; i++) {
            double res = 0f;
            for (int c = 0; c < clusterCount; c++) {
                if (fuzzifier == 2.0)
                    val = cluster[i][c] * cluster[i][c];
                else
                    val = Math.pow(cluster[i][c], fuzzifier);

                res += val * clusterSquaredDist[i][c];
            }
            // if (res > limit) {
            allSum += res;
            if (allSum > value)
                return i;
            // }
        }
        return -1;
    }

    protected void copyMeans(Object[] means1, Object[] means2, int len) {
        for (int i = 0; i < len; i++) {
            means2[i] = fkmAlg.getCopyFunction().invoke(means1[i], means2[i]);
        }
    }

    public Object testAddCluster(FuzzyKMeansState state, Object bestConf) {

        FuzzyKMeansState bestS;
        int best;
        if (bestConf != null) {
            best = ((Number) ((Object[]) bestConf)[0]).intValue();
            bestS = (FuzzyKMeansState) ((Object[]) bestConf)[1];
        } else {
            best = 0;
            bestS = null;
        }
        double allSum = 0d;
        double val;
        double fuzzifier = state.fkmc.getFuzzifier();
        if (state.sumDist <= 0) {
            for (int i = 0; i < state.dataCount; i++) {
                for (int c = 0; c < state.clusterCount; c++) {
                    if (fuzzifier == 2.0)
                        val = state.fuzzyCluster[i][c]
                                * state.fuzzyCluster[i][c];
                    else
                        val = Math.pow(state.fuzzyCluster[i][c], fuzzifier);

                    allSum += val * state.clusterSquaredDist[i][c];
                }
            }
        } else
            allSum = state.sumDist;

        double sum = allSum / state.dataCount;
        double limit = sum;

        Object[] kmeansP = new Object[state.dataCount + 1];
        double[] clusterCountsP = new double[state.dataCount + 1];
        double[][] clusterP = new double[state.dataCount + 1][state.maxClusterCount];
        double[][] clusterDistP = new double[state.dataCount + 1][state.maxClusterCount];

        // allSum = 0f;
        //
        // for (int i = 0; i < state.dataCount; i++) {
        // double res = 0f;
        // for (int j = 0; j < state.clusterCount; j++) {
        // // int clust = cluster[i][j];
        // double dist = state.clusterSquaredDist[i][j];
        // res += dist * state.fuzzyCluster[i][j]
        // * state.fuzzyCluster[i][j];
        // }
        // if (res > limit)
        // allSum += res;
        // }
        for (int i = 0; i < testCount; i++) {
            testInd[i] = getStream(state.dataCount, state.fuzzyCluster,
                    state.clusterCount, state.clusterSquaredDist, fuzzifier, (double) Math
                            .random()
                            * allSum, limit);
        }

        double objective = Double.MAX_VALUE;
        double bestObjective = Double.MAX_VALUE;
        double bestV = Double.MAX_VALUE;
        double validity;
        FuzzyKMeansState bestState = null;
        int bestInd = -1;

        for (int i = 0; i < testCount; i++) {
            if (state.clusterCount == state.maxClusterCount)
                break;
            int ind = testInd[i];
            if (ind >= 0) {
                copyMeans(state.kMeans, kmeansP, state.clusterCount);
                kmeansP[state.clusterCount] = fkmAlg.getCopyFunction().invoke(
                        state.data[ind], kmeansP[state.clusterCount]);
                FuzzyKMeansState stateP = new FuzzyKMeansState(state.data,
                        state.dataCount, state.clusterCount + 1);
                stateP.epsilon = state.epsilon;
                stateP.clusterSquaredDist = clusterDistP;
                stateP.clusterCounts = clusterCountsP;
                stateP.fuzzyCluster = clusterP;
                stateP.maxClusterCount = state.maxClusterCount;
                stateP.minClusterCount = state.minClusterCount;
                stateP.fkmc = fkmAlg;
                stateP.kMeans = kmeansP;
                fkmAlg.computeMemberships(stateP);
                fkmAlg.computeClusterCenters(stateP);
                fkmAlg.computeMemberships(stateP);
                fkmAlg.computeClusterCenters(stateP);
                fkmAlg.computeMemberships(stateP);
                fkmAlg.computeClusterCenters(stateP);
                fkmAlg.computeMemberships(stateP);
                fkmAlg.computeClusterCenters(stateP);
                fkmAlg.computeMemberships(stateP);
                fkmAlg.computeClusterCenters(stateP);
                fkmAlg.computeMemberships(stateP);
                
                objective = fkmAlg.computeObjective(stateP);
                validity = fkmAlg.computeValidity(stateP);
                if (mode == OBJECTIVE_CHOOSE) {
                    if (bestObjective > objective) {
                        bestObjective = objective;
                        bestInd = i;
                        bestV = validity;
                        bestState = (FuzzyKMeansState) stateP.clone();
                    }
                } else if (mode == VALIDITY_CHOOSE) {
                    if (bestV > validity) {
                        bestObjective = objective;
                        bestInd = i;
                        bestV = validity;
                        bestState = (FuzzyKMeansState) stateP.clone();
                        if (bestS == null || bestS.validity > bestV)
                            logger.log(Level.FINE, "Better Add: "
                                    + (state.clusterCount + 1) + " " + bestInd
                                    + " " + bestV + " " + bestObjective);
                    }
                }
            }
        }

        if (bestS == null || bestS.validity > bestV) {
            bestS = bestState;
            best = bestInd + 1;
        } else
            logger.log(Level.FINE, "Best Looser Add: "
                    + (state.clusterCount + 1) + " " + bestInd + " " + bestV);

        return new Object[] { new Integer(best), bestS };
    }

    protected void switchElement(Object[] ob, int e1, int e2) {
        Object h = ob[e1];
        ob[e1] = ob[e2];
        ob[e2] = h;
    }

    public Object testRemoveCluster(FuzzyKMeansState state, Object bestConf) {
        if (state.clusterCount == 2) {
            System.out.println("HM!!!");
        }
        int best;
        FuzzyKMeansState bestS;
        if (bestConf != null) {
            best = ((Number) ((Object[]) bestConf)[0]).intValue();
            bestS = (FuzzyKMeansState) ((Object[]) bestConf)[1];
        } else {
            best = 0;
            bestS = null;
        }
        Object[] kmeansM = new Object[state.dataCount];
        double[] clusterCountsM = new double[state.dataCount];
        double[][] clusterM = new double[state.dataCount][state.maxClusterCount];
        double[][] clusterDistM = new double[state.dataCount][state.maxClusterCount];
        double objective = Double.MAX_VALUE;
        double bestObjective = Double.MAX_VALUE;
        double bestV = Double.MAX_VALUE;
        FuzzyKMeansState bestState = null;
        double validity;
        int bestInd = -1;
        for (int i = 0; i < state.clusterCount; i++) {
            /* if (clusterCounts[i]<supportLimit) */{
                FuzzyKMeansState stateM = null;
                if (state.clusterCounts[i] <= 0.2) {
                    validity = 0;
                    objective = 0;
                } else {
                    copyMeans(state.kMeans, kmeansM, state.clusterCount);
                    switchElement(kmeansM, i, state.clusterCount - 1);
                    stateM = new FuzzyKMeansState(state.data, state.dataCount,
                            state.clusterCount - 1);
                    stateM.epsilon = state.epsilon;
                    stateM.clusterSquaredDist = clusterDistM;
                    stateM.clusterCounts = clusterCountsM;
                    stateM.fuzzyCluster = clusterM;
                    stateM.maxClusterCount = state.maxClusterCount;
                    stateM.minClusterCount = state.minClusterCount;
                    stateM.fkmc = fkmAlg;
                    stateM.kMeans = kmeansM;
                    fkmAlg.computeMemberships(stateM);
                    fkmAlg.computeClusterCenters(stateM);
                    fkmAlg.computeMemberships(stateM);
                    fkmAlg.computeClusterCenters(stateM);
                    fkmAlg.computeMemberships(stateM);
                    fkmAlg.computeClusterCenters(stateM);
                    fkmAlg.computeMemberships(stateM);
                    fkmAlg.computeClusterCenters(stateM);
                    fkmAlg.computeMemberships(stateM);
                    fkmAlg.computeClusterCenters(stateM);
                    fkmAlg.computeMemberships(stateM);
                    objective = fkmAlg.computeObjective(stateM);
                    validity = fkmAlg.computeValidity(stateM);
                }
                if (mode == OBJECTIVE_CHOOSE) {
                    if (bestObjective > objective) {
                        bestObjective = objective;
                        bestInd = i;
                        bestV = validity;
                        bestState = (FuzzyKMeansState) stateM.clone();
                    }
                } else if (mode == VALIDITY_CHOOSE) {
                    if (bestV > validity) {
                        bestObjective = objective;
                        bestInd = i;
                        bestV = validity;
                        bestState = (FuzzyKMeansState) stateM.clone();
                        if (bestS == null || bestS.validity > bestV)
                            logger.log(Level.FINE, "Better Remove: "
                                    + (state.clusterCount - 1) + " " + bestInd
                                    + " " + bestV + " " + bestObjective);
                    }
                }
            }
        }

        if ((bestS == null || bestS.validity > bestV)
                && (state.clusterCount > 2)) {
            bestS = bestState;
            best = -(bestInd + 1);
        } else
            logger.log(Level.FINE, "Best Looser Remove: "
                    + (state.clusterCount - 1) + " " + bestInd + " " + bestV);

        return new Object[] { new Integer(best), bestS };
    }

    protected void addCluster(FuzzyKMeansState state, int nr) {
        state.kMeans[state.clusterCount] = fkmAlg.getCopyFunction().invoke(
                state.data[nr], state.kMeans[state.clusterCount]);
        state.clusterCount++;
        fkmAlg.computeMemberships(state);
        if (!QUIET)
            System.out.println("ADD:" + nr + "    ClusterCount:"
                    + (state.clusterCount + 1));
    }

    protected void removeCluster(FuzzyKMeansState state, int nr) {
        switchElement(state.kMeans, nr, state.clusterCount - 1);
        state.clusterCount--;
        for (int i = 0; i < state.dataCount; i++) {
            state.fuzzyCluster[i][state.clusterCount] = 0d;
        }
        fkmAlg.computeMemberships(state);
        if (!QUIET)
            System.out.println("REMOVE:" + nr + "    ClusterCount:"
                    + (state.clusterCount - 1));
    }

    public FuzzyKMeansState removeEqualCluster(FuzzyKMeansState state) {
        Distance distance = fkmAlg.getDistance();
        boolean del = false;
        do {
            del = false;
            Object bestConf = null;
            for (int c = 0; c < state.clusterCount - 1 && !del; c++) {
                for (int c2 = c + 1; c2 < state.clusterCount && !del; c2++) {
                    double distc = distance.squaredDistance(state.kMeans[c],
                            state.kMeans[c2]);
                    if (distc < 0.00000001) {
                        bestConf = new Object[] { new Integer(-(c2 + 1)),
                                new Double(0.0) };
                        del = true;
                        break;
                    }
                }
            }
            if (del) {
                doClusterCountOld(state, bestConf);
                if (state.clusterCount == 1) {
                    bestConf = new Object[] { new Integer(0), null };

                    bestConf = testAddCluster(state, bestConf);
                    state = doClusterCount(state, bestConf);

                }

            }
        } while (del);
        return state;
    }

    public int doClusterCountOld(FuzzyKMeansState state, Object bestConf) {
        int best;
        double bestV = 0;
        if (bestConf != null) {
            best = ((Number) ((Object[]) bestConf)[0]).intValue();
            bestV = ((Number) ((Object[]) bestConf)[1]).doubleValue();
        } else {
            best = 0;
        }
        if (best > 0) {
            addCluster(state, best - 1);
        } else if (best < 0) {
            removeCluster(state, (-best) - 1);
        } else
            return state.clusterCount;
        fkmAlg.computeMemberships(state);
        fkmAlg.computeClusterCenters(state);
        fkmAlg.computeMemberships(state);
        fkmAlg.computeClusterCenters(state);
        fkmAlg.computeMemberships(state);
        state.validity = fkmAlg.computeValidity(state);
        state.objective = fkmAlg.computeObjective(state);
        logger.log(Level.FINE, "NEW Best: " + (state.clusterCount) + " "
                + state.validity + " " + state.objective);
        if (state.validity > 2 * bestV) {
            logger.log(Level.INFO, "HM!!!! " + state.validity + " "
                    + bestV);
            System.exit(1);
        }
        return state.clusterCount;
    }

    public FuzzyKMeansState doClusterCount(FuzzyKMeansState state,
            Object bestConf) {
        int best;
        FuzzyKMeansState bestState = null;
        if (bestConf != null) {
            best = ((Number) ((Object[]) bestConf)[0]).intValue();
            bestState = (FuzzyKMeansState) ((Object[]) bestConf)[1];
        } else {
            best = 0;
        }
        if (best != 0) {
            state = bestState;
        } else
            return state;
        state.validity = fkmAlg.computeValidity(state);
        state.objective = fkmAlg.computeObjective(state);
        logger.log(Level.FINE, "NEW Best: " + (state.clusterCount) + " "
                + state.validity + " " + state.objective);
        if (state.clusterCount == 1) {
            logger.log(Level.INFO, "HM!!!! " + state.validity);
            System.exit(1);
        }
        return state;
    }

    public FuzzyKMeansState testClusterCount(FuzzyKMeansState state) {

        double bestValidity = fkmAlg.computeValidity(state);
        state.validity = bestValidity;
        // System.out.println("minDist: "+fkmAlg.mind+" midDist: "+fkmAlg.sumd);
        Object bestConf = new Object[] { new Integer(0), state };
        logger.log(Level.FINE, "Best: " + (state.clusterCount) + " "
                + bestValidity);
        double min = computeMinimalClusterDist(state.kMeans, state.clusterCount);
        if (min == 0) {
            System.out.println(" min==0!!!!");
            bestConf = new Object[] { new Integer(0), null };
        }
        if (state.clusterCount > state.minClusterCount || min == 0)
            bestConf = testRemoveCluster(state, bestConf);

        if ((((Integer) ((Object[]) bestConf)[0]).intValue() == 0) && min > 0
                || state.clusterCount <= 2)
            bestConf = testAddCluster(state, bestConf);
        state = doClusterCount(state, bestConf);
        return state;
    }

    public double computeMinimalClusterDist(Object[] kmeans, int clusterCount) {
        double min = Double.POSITIVE_INFINITY;
        Distance dist = fkmAlg.getDistance();
        for (int c = 0; c < clusterCount - 1; c++) {
            for (int c2 = c + 1; c2 < clusterCount; c2++) {
                double distc = dist.distance(kmeans[c], kmeans[c2]);
                if (distc < min) {
                    min = distc;
                }
            }
        }
        return min;
    }

    public FuzzyKMeansState testAddClusterCount(FuzzyKMeansState state,
            boolean force) {
        Object bestConf = null;
        if (!force) {
            state.validity = fkmAlg.computeValidity(state);
            bestConf = new Object[] { new Integer(0), state };
        } else {
            bestConf = new Object[] { new Integer(0), null };
        }
        bestConf = testAddCluster(state, bestConf);
        state = doClusterCount(state, bestConf);
        return state;
    }

    public FuzzyKMeansState testRemoveClusterCount(FuzzyKMeansState state,
            boolean force) {
        Object bestConf = null;
        if (!force) {
            state.validity = fkmAlg.computeValidity(state);
            bestConf = new Object[] { new Integer(0), state };
        } else {
            bestConf = new Object[] { new Integer(0), null };
        }
        bestConf = testRemoveCluster(state, bestConf);
        state = doClusterCount(state, bestConf);
        return state;
    }

    public FuzzyKMeansClustering getFuzzyKMeansClustering() {
        return fkmAlg;
    }

    public void setTestCount(int testCount) {
        this.testCount = testCount;
        testInd = new int[testCount];
    }

    public int getTestCount() {
        return testCount;
    }

}
