package clustering;

public class RelativeFuzzyMeasures {
    
    
    public static RelativeFuzzyMeasure DIST=new RelativeFuzzyMeasure(){
        public double distance(FuzzyKMeansState state1,FuzzyKMeansState state2){
            int cc1=state1.clusterCount;
            int cc2=state2.clusterCount;
            int n=state1.dataCount;
            Distance distance=state2.fkmc.getDistance();
            double[] minD1=new double[cc1];
            double[] minD2=new double[cc2];
            for (int c1=0;c1<cc1;c1++){
                for (int c2=0;c2<cc2;c2++){
                    double val=distance.distance(state1.kMeans[c1],state2.kMeans[c2]);
                    if (val<minD1[c1] || c2==0){
                        minD1[c1]=val;
                    }
                    if (val<minD2[c2] || c1==0){
                        minD2[c2]=val;
                    }
                }
            }
            double s1=0;
            for (int c1=0;c1<cc1;c1++){
                s1+=minD1[c1]*state1.clusterCounts[c1]/n;
            }
            double s2=0;
            for (int c2=0;c2<cc2;c2++){
                s2+=minD2[c2]*state2.clusterCounts[c2]/n;
            }
            return 0.125*(s1+s2);
        }
    };
    
    
    public static RelativeFuzzyMeasure MY_RFM=new RelativeFuzzyMeasure(){
        public double distance(FuzzyKMeansState state1,FuzzyKMeansState state2){
            int cc1=state1.clusterCount;
            int cc2=state2.clusterCount;
            double[][] error12=new double[cc1][cc2];
            double[][] error21=new double[cc2][cc1];
            
            int n=state1.dataCount;
            for (int i=0;i<n;i++){
                for (int c1=0;c1<cc1;c1++){
                    for (int c2=0;c2<cc2;c2++){
                        error12[c1][c2]+=1-Math.min(1,1-state1.fuzzyCluster[i][c1]+state2.fuzzyCluster[i][c2]);
                        error21[c2][c1]+=1-Math.min(1,1-state2.fuzzyCluster[i][c2]+state1.fuzzyCluster[i][c1]);
                    }
                }
            }
            double s1=0;
            for (int c1=0;c1<cc1;c1++){
                double min=Double.MAX_VALUE;                
                for (int c2=0;c2<cc2;c2++){
                    if (error12[c1][c2]<min){
                        min=error12[c1][c2];
                    }
                }
                s1+=min;
            }
            s1/=n;
            double s2=0;
            for (int c2=0;c2<cc2;c2++){
                double min=Double.MAX_VALUE;                
                for (int c1=0;c1<cc1;c1++){
                    if (error21[c2][c1]<min){
                        min=error21[c2][c1];
                    }
                }
                s2+=min;
            }
            s2/=n;
            return (s1+s2-s1*s2);
        }
    };
    
    public static RelativeFuzzyMeasure EYKE1=new RelativeFuzzyMeasure(){
        public double distance(FuzzyKMeansState state1,FuzzyKMeansState state2){
            int cc1=state1.clusterCount;
            int cc2=state2.clusterCount;
            int n=state1.dataCount;
            double[][] union=new double[cc1][cc2];
            double[][] inter=new double[cc1][cc2];
            
            for (int i=0;i<n;i++){
                for (int c1=0;c1<cc1;c1++){
                    for (int c2=0;c2<cc2;c2++){
                        inter[c1][c2]+=Math.min(state1.fuzzyCluster[i][c1],state2.fuzzyCluster[i][c2]);
                        union[c1][c2]+=Math.max(state1.fuzzyCluster[i][c1],state2.fuzzyCluster[i][c2]);
                    }
                }
            }
            double[] sc1=new double[cc1];
            for (int c1=0;c1<cc1;c1++){
                sc1[c1]=0;
                for (int c2=0;c2<cc2;c2++){
                    sc1[c1]=Math.max(sc1[c1],inter[c1][c2]/union[c1][c2]);    
                }
            }
            double s1=1;
            for (int c1=0;c1<cc1;c1++){
                double w=state1.clusterCounts[c1]/n;
                double m=Math.max(1-w,sc1[c1]);
                s1=Math.min(s1,m);
            }
            
             
            
            double[] sc2=new double[cc2];
            for (int c2=0;c2<cc2;c2++){
                sc2[c2]=0;
                for (int c1=0;c1<cc1;c1++){
                    sc2[c2]=Math.max(sc2[c2],inter[c1][c2]/union[c1][c2]);    
                }
            }
            double s2=1;
            for (int c2=0;c2<cc2;c2++){
                double w=state2.clusterCounts[c2]/n;
                double m=Math.max(1-w,sc2[c2]);
                s2=Math.min(s2,m);
            }
            return 1-Math.min(s1,s2);
        }
    };
    
    
    public static RelativeFuzzyMeasure EYKE2=new RelativeFuzzyMeasure(){
        public double distance(FuzzyKMeansState state1,FuzzyKMeansState state2){
            int cc1=state1.clusterCount;
            int cc2=state2.clusterCount;
            int n=state1.dataCount;
            double[][] union=new double[cc1][cc2];
            double[][] inter=new double[cc1][cc2];
            
            for (int i=0;i<n;i++){
                for (int c1=0;c1<cc1;c1++){
                    for (int c2=0;c2<cc2;c2++){
                        inter[c1][c2]+=Math.min(state1.fuzzyCluster[i][c1],state2.fuzzyCluster[i][c2]);
                        union[c1][c2]+=Math.max(state1.fuzzyCluster[i][c1],state2.fuzzyCluster[i][c2]);
                    }
                }
            }
            double[] sc1=new double[cc1];
            for (int c1=0;c1<cc1;c1++){
                sc1[c1]=0;
                for (int c2=0;c2<cc2;c2++){
                    double v=inter[c1][c2]/union[c1][c2];
                    sc1[c1]=sc1[c1]+v-sc1[c1]*v;    
                }
            }
            double s1=1;
            for (int c1=0;c1<cc1;c1++){
                double w=state1.clusterCounts[c1]/n;
                double m=Math.pow(sc1[c1],w);
                s1=s1*m;
            }
            
             
            
            double[] sc2=new double[cc2];
            for (int c2=0;c2<cc2;c2++){
                sc2[c2]=0;
                for (int c1=0;c1<cc1;c1++){
                    double v=inter[c1][c2]/union[c1][c2];
                    sc2[c2]=sc2[c2]+v-sc2[c2]*v;    
                    
                }
            }
            double s2=1;
            for (int c2=0;c2<cc2;c2++){
                double w=state2.clusterCounts[c2]/n;
                double m=Math.pow(sc2[c2],w);
                s2=s2*m;
            }
            return 1-s1*s2;
        }
    };


}
