package pipes;

import clustering.Distance;

public class BlockDistance implements Distance<Object[]> {

    protected double[] weights;
    protected int blockCount;
    protected Distance distance;
    
    public BlockDistance(Distance distance, int blockCount, double weight){
        this.blockCount=blockCount;
        this.distance=distance;
        weights=new double[blockCount];
        weights[0]=1.0;
        for (int i=1;i<blockCount;i++){
            weights[i]=weights[i-1]*weight;
        }
    }
    public BlockDistance(Distance distance, int blockCount, double[] weights){
        this.blockCount=blockCount;
        this.distance=distance;
        this.weights=weights;
    }

    public double squaredDistance(Object[] d1, Object[] d2) {
        double dist=0;
        for (int i=0;i<blockCount;i++){
            dist+=weights[i]*distance.squaredDistance(d1[i],d2[i]);
        }
        return dist;
    }

    public double distance(Object[] d1, Object[] d2) {
        double dist=0;
        for (int i=0;i<blockCount;i++){
            dist+=weights[i]*distance.distance(d1[i],d2[i]);
        }
        return dist;
    }

}
