/*
 * StreamCluster.java
 *
 * Created on 30. Januar 2003, 08:55
 */
package pipes;
import xxl.core.collections.queues.ArrayQueue;
import xxl.core.collections.queues.Queue;
import xxl.core.pipes.operators.AbstractPipe;

/**
 * Synchronized multiple Sources to Blocks of a specified size.
 * If all Sources have sent the size Elements, the processQueues Method is called
 * to process the Elements.
 *
 * @author  Jrgen Beringer
 */
public abstract class BlockSynchronizer extends AbstractPipe{
	
	protected int size;
	protected Queue[] queues;
	protected int min;
	protected int minCount;
	String shorties;
	/** Creates a new instance of ClusterPipes */
	public BlockSynchronizer(int size, int minCount) {
		this.size=size;
		this.minCount=minCount;
		queues=new ArrayQueue[size];
		for (int i=0;i<size;i++){
			queues[i]=new ArrayQueue();
		}
		min=0;
	}
	
	public void processObject(Object o, int ID) throws IllegalArgumentException {
		//System.out.println("new "+ID);
		synchronized(queues[ID]){
			queues[ID].enqueue(o);
			if (queues[ID].size()==(min+1)) getMin();
		}
	}
	
	public abstract void processQueues(Queue[] queues);
	
	public void getMin(){
		int newmin=Integer.MAX_VALUE;
		for (int i=0;i<size;i++){
			if (newmin>queues[i].size()){
				newmin=queues[i].size();
				if (newmin==0){
					min=0;
					return;
				}
			}
		}
		//if (min!=newmin) System.out.println("new min:"+newmin);
		min=newmin;
		if (min>=minCount) {
			processQueues(queues);
			getMin();
		}
	}
	
}
