/*
 * Created on 18.03.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package pipes;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;

import xxl.core.pipes.sinks.AbstractSink;
import clustering.FuzzyKMeansState;
import clustering.KMeansState;

/**
 * @author beringer
 */
public class ClusterVisualizer extends AbstractSink {
    protected Image back;

    protected Graphics graphics;

    protected int count;

    protected int clusterCount;

    protected int height;

    protected Font font;

    protected Object[] streamsObjects;

    protected double[][] values;

    protected Frame frame;

    protected Panel panel;

    protected ScrollPane scrollPane;

    protected boolean disposed = false;

    protected Color[] colors = new Color[] { Color.red, new Color(0, 255, 0),
            Color.blue, Color.yellow, Color.magenta, Color.cyan, Color.orange,
            Color.pink, new Color(127, 127, 0), new Color(127, 0, 127),
            new Color(127, 63, 63), new Color(0, 127, 127),
            new Color(127, 127, 255), new Color(127, 255, 0),
            new Color(255, 127, 0), new Color(255, 127, 127),
            new Color(127, 255, 127), new Color(127, 0, 255),
            new Color(255, 0, 127), new Color(127, 0, 0), new Color(0, 127, 0),
            new Color(0, 0, 127), new Color(255, 127, 255),
            new Color(63, 196, 255), new Color(255, 255, 127),
            new Color(127, 196, 63), Color.gray, Color.black };

    static boolean DEBUG = false;

    protected static final int BLOCKWIDTH = 20;

    protected static final int BLOCKOFFSET = 5;

    protected static int NAMEWIDTH = 110;

    protected boolean fuzzy;

    public ClusterVisualizer(Object[] streamsObjects, int clusterCount) {
        this(streamsObjects, clusterCount, false);
    }

    public ClusterVisualizer(Object[] streamsObjects, int clusterCount,
            boolean fuzzy) {
        this(streamsObjects, streamsObjects.length, clusterCount, BLOCKWIDTH
                + BLOCKOFFSET, null, fuzzy);
    }

    public ClusterVisualizer(Object[] streamsObjects, int count,
            int clusterCount, int height, Font font) {
        this(streamsObjects, count, clusterCount, height, font, false);
    }

    public ClusterVisualizer(Object[] streamsObjects, int count,
            int clusterCount, int height, Font font, boolean fuzzy) {
        this.count = count;
        this.clusterCount = clusterCount;
        this.height = height;
        this.font = font;
        this.streamsObjects = streamsObjects;
        this.fuzzy = fuzzy;
        back = new BufferedImage(clusterCount * (BLOCKWIDTH + BLOCKOFFSET)
                + NAMEWIDTH + 4 * BLOCKOFFSET, height * (count + 1) + 2
                * BLOCKOFFSET, BufferedImage.TYPE_INT_ARGB);
        graphics = back.getGraphics();
        drawNames(graphics);

        panel = new Panel() {
            public void update(Graphics g) {
                paint(g);
            }

            public void paint(Graphics g) {
                if (DEBUG)
                    System.out.println("repaint");
                g.drawImage(back, 0, 0, frame);
            }

            public Dimension getPreferredSize() {
                return new Dimension(back.getWidth(null), back.getHeight(null));
            }

        };
        // panel.setSize(theWidth, theHeight);
    }

    public Component getComponent() {
        return panel;
    }

    public ScrollPane getScrollPane() {
        ScrollPane sp = new ScrollPane();
        sp.add(panel);
        return sp;
    }

    public Frame createFrame(String name) {
        frame = new Frame(name);
        frame.setSize(back.getWidth(frame) + 40, Math.min(500, back
                .getHeight(frame) + 40));
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                frame.setVisible(false);
                stop();
                disposed = true;
                frame.dispose();
            }
        });
        frame.setLayout(new BorderLayout());
        frame.add(getScrollPane(), "Center");
        frame.show();
        return frame;
    }

    public void setBounds(int x, int y, int width, int height) {
        if (frame != null)
            frame.setBounds(x, y, width, height);
    }

    public void drawNames(Graphics g) {
        if (font != null)
            g.setFont(font);
        g.setColor(Color.black);
        for (int i = 0; i < count; i++) {
            g.drawString(streamsObjects[i].toString(), BLOCKOFFSET, (i + 1)
                    * height + BLOCKOFFSET + height / 2);
        }
        for (int j = 0; j < clusterCount; j++) {
            Color col1 = colors[j % colors.length];
            int c = j / colors.length;
            for (int ii = 0; ii < c; ii++)
                col1 = col1.brighter();
            g.setColor(col1);
            g.fillRect(2 * BLOCKOFFSET + NAMEWIDTH + j
                    * (BLOCKWIDTH + BLOCKOFFSET), BLOCKOFFSET, height
                    - BLOCKOFFSET, height - BLOCKOFFSET);
            g.setColor(Color.black);
            g.drawString("" + j, 2 * BLOCKOFFSET + NAMEWIDTH + j
                    * (BLOCKWIDTH + BLOCKOFFSET) + 3, BLOCKOFFSET + height
                    - BLOCKOFFSET - 3);
        }
    }

    public void drawValues(Graphics g) {
        for (int i = 0; i < count; i++) {
            for (int j = 0; j < clusterCount; j++) {
                // col = 255 - (int) (values[i][j] * 255);
                // g.setColor(new Color(col, col, col));
                Color col1 = null;
                Color text = Color.black;
                if (values[i][j] > 0.001) {
                    /*
                     * if (j>=colors.length) col1=colors[colors.length-1]; else
                     * col1=colors[j];
                     */
                    if (!fuzzy) {
                        col1 = colors[j % colors.length];
                    } else {
                        double val = values[i][j];
                        val = 1 - ((1 - val) * (1 - val));
                        int cval = (int) (255 - (255 * val / 1.0));
                        col1 = new Color(cval, cval, cval);
                        text = Color.green;
                    }
                    int c = j / colors.length;
                    for (int ii = 0; ii < c; ii++)
                        col1 = col1.brighter();
                    g.setColor(col1);
                    g.fillRect(2 * BLOCKOFFSET + NAMEWIDTH + j
                            * (BLOCKWIDTH + BLOCKOFFSET), (i + 1) * height
                            + BLOCKOFFSET, height - BLOCKOFFSET, height
                            - BLOCKOFFSET);
                    g.setColor(text);
                    g.drawString("" + j, 2 * BLOCKOFFSET + NAMEWIDTH + j
                            * (BLOCKWIDTH + BLOCKOFFSET) + 3, (i + 2) * height
                            + BLOCKOFFSET - BLOCKOFFSET - 3);
                } else {
                    col1 = Color.white;
                    g.setColor(col1);
                    g.fillRect(2 * BLOCKOFFSET + NAMEWIDTH + j
                            * (BLOCKWIDTH + BLOCKOFFSET), (i + 1) * height
                            + BLOCKOFFSET, height - BLOCKOFFSET, height
                            - BLOCKOFFSET);
                }

            }
        }
    }

    public void processObject(Object ob, int id) {
        if (id != 0)
            return;
        if (ob instanceof FuzzyKMeansState) {
            ob = ((FuzzyKMeansState) ob).fuzzyCluster;
        } else if (ob instanceof KMeansState) {
            ob = ((KMeansState) ob).cluster;
        }
            
        if (DEBUG)
            System.out.println("Input");
        if (ob instanceof int[]) {
            int[] aktValues = (int[]) ob;
            if (values == null) {
                values = new double[count][clusterCount];
            }
            for (int i = 0; i < count; i++) {
                for (int j = 0; j < clusterCount; j++) {
                    values[i][j] = 0;
                }
                values[i][aktValues[i]] = 1;
            }
        } else  {
            values = (double[][]) ob;
        }
        drawValues(graphics);
        panel.repaint();
        if (disposed) {
            new Thread() {
                public void run() {
                    if (frame != null)
                        frame.dispose();
                }
            }.start();
        } else if (frame != null)
            frame.setVisible(true);
    }

    public void done(int ID) {
        frame.setVisible(false);
        frame.dispose();
    }

}
