package pipes;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import xxl.core.pipes.sinks.AbstractSink;
import clustering.FuzzyKMeansState;

public class ClusteringSaver extends AbstractSink<FuzzyKMeansState> {

    protected ObjectOutputStream dout;

    protected boolean complex;

    public ClusteringSaver(ObjectOutputStream dout, boolean complex) {
        this.dout = dout;
        this.complex = complex;
    }

    public ClusteringSaver(String file, boolean complex) {
        try {
            dout = new ObjectOutputStream(new BufferedOutputStream(
                    new FileOutputStream(file)));
            this.complex = complex;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void processObject(FuzzyKMeansState state, int ID)
            throws IllegalArgumentException {
        try {
            writeRat(state.fuzzyCluster, state.dataCount, state.clusterCount);
            if (complex)
                writeComplex(state.kMeans, state.clusterCount);
            else
                writeRat(state.kMeans, state.clusterCount,
                        ((double[]) state.kMeans[0]).length);
            dout.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    protected void writeComplex(Object[] complex, int size1) {
        try {
            dout.writeInt(size1);
            int size2 = ((double[][]) complex[0]).length;
            dout.writeInt(size2);
            for (int i = 0; i < size1; i++) {
                for (int j = 0; j < size2; j++) {
                    dout.writeDouble(((double[][]) complex[i])[j][0]);
                    dout.writeDouble(((double[][]) complex[i])[j][1]);
                }
            }
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    protected void writeRat(Object[] rat, int size1, int size2) {
        try {
            dout.writeInt(size1);
            dout.writeInt(size2);
            for (int i = 0; i < size1; i++) {
                for (int j = 0; j < size2; j++) {
                    dout.writeDouble(((double[]) rat[i])[j]);
                }
            }
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

}
