package pipes;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;

import xxl.core.pipes.operators.AbstractPipe;
import xxl.core.util.CompositeMetaData;
import clustering.FuzzyKMeansState;

public class ClusteringStateLoader extends AbstractPipe<Object,FuzzyKMeansState> {

    protected ObjectInputStream din;

    protected boolean complex;

    double[][] cluster;

    int kValue;
    
    int initCount;
    
    int init;

    Object[] kMeans;

    public ClusteringStateLoader(ObjectInputStream din, boolean complex, int initCount) {
        super();
        this.din = din;
        this.complex = complex;
        this.initCount=initCount;
        init=0;
    }

    public ClusteringStateLoader(String file, boolean complex, int initCount) {
        super();
        try {
            din = new ObjectInputStream(new BufferedInputStream(
                    new FileInputStream(file)));
            this.complex = complex;
            this.initCount=initCount;
            init=0;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void processObject(Object o, int ID) throws IllegalArgumentException {
        if (init<initCount) {
            init++;
            if (init<initCount) return;
        }
        
        cluster = (double[][]) readRat();
        if (complex)
            kMeans = readComplex();
        else
            kMeans = readRat();
        kValue = kMeans.length;
        FuzzyKMeansState state=new FuzzyKMeansState();
        state.fuzzyCluster=cluster;
        state.clusterCount=kValue;
        state.kMeans=kMeans;
        if (o instanceof Object[]) state.data=(Object[])o;

        transfer(state);
    }

    protected double[][][] readComplex() {
        try {
            int size1 = din.readInt();
            int size2 = din.readInt();
            double[][][] complex = new double[size1][size2][2];
            for (int i = 0; i < size1; i++) {
                for (int j = 0; j < size2; j++) {
                    complex[i][j][0] = din.readDouble();
                    complex[i][j][1] = din.readDouble();
                }
            }
            return complex;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }

    protected double[][] readRat() {
        try {
            int size1 = din.readInt();
            int size2 = din.readInt();
            double[][] complex = new double[size1][size2];
            for (int i = 0; i < size1; i++) {
                for (int j = 0; j < size2; j++) {
                    complex[i][j] = din.readDouble();
                }
            }
            return complex;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }

    public CompositeMetaData getMetaData() {
        // TODO Auto-generated method stub
        return null;
    }

}
