package pipes;

import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import xxl.core.pipes.operators.AbstractPipe;
import clustering.Distance;
import clustering.FuzzyAverages;
import clustering.FuzzyKMeans;
import clustering.FuzzyKMeansClustering;
import clustering.FuzzyKMeansState;
import clustering.FuzzyKMeansUtils;
import clustering.KMeansUtils;

public class CompareFuzzyClustering extends AbstractPipe {

    protected Writer cc;
    protected Writer cc2;

    protected Writer c1;

    protected Writer c2;

    protected Writer c3;

    protected Object[] dataIDFT;

    protected Object[] dataOldDFT;

    protected int dataCount;

    protected int dftCount;

    protected int windowSize;

    protected FuzzyKMeansClustering fkmc;

    protected FuzzyKMeansClustering fkmcOld;

    protected double[][] values1;

    protected Distance distance1;

    protected Distance distance2;

    protected boolean set1 = false;

    protected boolean set2 = false;

    protected FuzzyKMeansState state;

    protected FuzzyKMeansState state2;
    
    protected boolean end=false;

    protected CompareFuzzyClustering(FuzzyKMeansClustering fkmc, int dataCount,
            int dftCount, int windowSize) {
        this.dataCount = dataCount;
        this.dftCount = dftCount;
        this.windowSize = windowSize;
        this.fkmc = fkmc;
        distance1 = StreamCluster.getDFTDistance(dftCount);
        distance2 = KMeansUtils.getDoubleDistance(windowSize);
        values1 = new double[windowSize][2];
        fkmcOld = new FuzzyKMeans(KMeansUtils.getDoubleDistance(windowSize),
                2.0, FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3,
                FuzzyKMeansUtils.INITALIZE, KMeansUtils
                        .getDoubleCopyFunction(windowSize), FuzzyAverages
                        .getDoubleMeansFunction(windowSize));
    }

    public CompareFuzzyClustering(Writer cc,Writer cc2, Writer c1, Writer c2, Writer c3,
            FuzzyKMeansClustering fkmc, int dataCount, int dftCount,
            int windowSize) {
        this(fkmc, dataCount, dftCount, windowSize);
        this.cc = cc;
        this.cc2=cc2;
        this.c1 = c1;
        this.c2 = c2;
        this.c3 = c3;

        
    }

    int count = 0;

    public void processObject(Object o, int ID) throws IllegalArgumentException {
        if(end) return;
        if (ID == 0) {
            state = (FuzzyKMeansState) o;
            set1 = true;
        } else if (ID == 1) {
            state2 = (FuzzyKMeansState) o;
            set2 = true;
        }
        if (set1 && set2) {
            set1 = false;
            set2 = false;
            if (!end) compareNext();
            count++;
            if (count == 204){
                try {
                    cc.write('\n');
                    cc.flush();
                    cc2.write('\n');
                    cc2.flush();
                    c1.write('\n');
                    c1.flush();
                    c2.write('\n');
                    c2.flush();
                    c3.write('\n');
                    c3.flush();
                    cc.close();
                    cc2.close();
                    c1.close();
                    c2.close();
                    c3.close();
                    end=true;
                } catch (IOException e) {
                    e.printStackTrace();
                }
                //System.exit(0);
            }
        }

    }

    protected void compareNext() {
        Object[] kMeansOldDFT = transform();
        Object[] kMeansNorm = retransform();
        double val1 = compare(state.fuzzyCluster, state.clusterCount,
                state.kMeans, state2.fuzzyCluster, state2.clusterCount,
                kMeansOldDFT, dataCount, distance1);
        double val2 = compare(state.fuzzyCluster, state.clusterCount,
                kMeansNorm, state2.fuzzyCluster, state2.clusterCount,
                state2.kMeans, dataCount, distance2);

        // dataOldDFT=transformOld(dataOld);
        // dataIDFT=retransformDFT(dataDFT);
        //        
        // double val5=distance(dataOldDFT,dataDFT,dataCount,distance1);
        // double val6=distance(dataOld,dataIDFT,dataCount,distance2);

        // fkmc.computeClusterCenters(kMeansOldDFT,kValueOld,dataDFT,dataCount,clusterOld,null);
        // fkmc.computeClusterCenters(kMeans,kValue,dataDFT,dataCount,cluster,null);
        // double
        // val3=compare(cluster,kValue,kMeans,clusterOld,kValueOld,kMeansOldDFT,distance1);
        FuzzyKMeansState myState = new FuzzyKMeansState();
        myState.fuzzyCluster = state.fuzzyCluster;
        myState.clusterCount = state.clusterCount;
        myState.data = state2.data;
        myState.dataCount = dataCount;
        myState.kMeans = new Object[state.clusterCount];
        fkmcOld.computeClusterCenters(myState);
        // fkmcOld.computeClusterCenters(null);
        double val4 = compare(state.fuzzyCluster, state.clusterCount,
                myState.kMeans, state2.fuzzyCluster, state2.clusterCount,
                state2.kMeans, dataCount, distance2);

        try {

            cc.write(state.clusterCount + "  ");
            cc.flush();
            cc2.write(state2.clusterCount + "  ");
            cc2.flush();
            c1.write(val1 + "  ");
            c1.flush();
            c2.write(val2 + "  ");
            c2.flush();
            c3.write(val4 + "  ");
            c3.flush();

        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    protected static double compare(double[][] cluster1, int kValue1,
            Object[] kMeans1, double[][] cluster2, int kValue2,
            Object[] kMeans2, int dataCount, Distance distance) {
        double result1 = 0;
        for (int c = 0; c < kValue1; c++) {
            double sum = 0;
            for (int i = 0; i < dataCount; i++) {
                sum += cluster1[i][c];
            }
            double dist = nearestClusterDistance(kMeans1[c], kMeans2, kValue2,
                    distance);
            result1 += dist * sum;
        }
        double result2 = 0;
        for (int c = 0; c < kValue2; c++) {
            double sum = 0;
            for (int i = 0; i < dataCount; i++) {
                sum += cluster2[i][c];
            }
            double dist = nearestClusterDistance(kMeans2[c], kMeans1, kValue1,
                    distance);
            result2 += dist * sum;
        }
        return result1 + result2;
    }

    // protected static double compare(double[][] cluster1,int kValue1,
    // Object[]kMeans1,double[][] cluster2, int kValue2, Object[]kMeans2, int
    // dataCount, Distance distance){
    // double[] sum1=new double[kValue1];
    // double[] used1=new double[kValue1];
    // for(int c=0; c<kValue1; c++){
    // for (int i=0;i<dataCount;i++){
    // sum1[c]+=cluster1[i][c];
    // }
    // }
    // double[] sum2=new double[kValue2];
    // double[] used2=new double[kValue2];
    // for(int c=0; c<kValue2; c++){
    // for (int i=0;i<dataCount;i++){
    // sum2[c]+=cluster2[i][c];
    // }
    // }
    // double[][] distC1=new double[kValue1][kValue2];
    // double[][] distC2=new double[kValue2][kValue1];
    // for(int i=0;i<kValue1;i++){
    // for(int c=0; c<kValue2; c++){
    // distC1[i][c]=distance.distance(kMeans1[i],kMeans2[c]);
    // distC2[c][i]=distC1[i][c];
    // }
    // }
    // double result1=0;
    // int[] sortC=Utils.sort(sum1);
    // for(int i=0;i<kValue1;i++){
    // double deg=sum1[sortC[i]];
    // int[] sort=Utils.sort(distC1[sortC[i]]);
    // int p=0;
    // while (deg>0 && p<kValue2){
    // double d=Math.min(deg,sum2[sortC[sort[p]]]-used2[sortC[sort[p]]]);
    // deg-=d;
    // used2[sortC[sort[p]]]+=d;
    // result1+=d*distC1[sortC[i]][sortC[sort[p]]];
    // p++;
    // }
    // }
    // double result2=0;
    // sortC=Utils.sort(sum2);
    // for(int c=0;c<kValue2;c++){
    // double deg=sum2[sortC[c]];
    // int[] sort=Utils.sort(distC2[sortC[c]]);
    // int p=0;
    // while (deg>0 && p<kValue1){
    // double d=Math.min(deg,sum1[sortC[sort[p]]]-used1[sortC[sort[p]]]);
    // deg-=d;
    // used1[sortC[sort[p]]]+=d;
    // result2+=d*distC2[sortC[c]][sortC[sort[p]]];
    // p++;
    // }
    // }
    //        
    // return result1+result2;
    // }

    protected double distance(Object[] data1, Object[] data2, int dataCount,
            Distance distance) {
        double res = 0;
        for (int i = 0; i < dataCount; i++) {
            res += distance.distance(data1[i], data2[i]);
        }
        return res;
    }

    protected static double nearestClusterDistance(Object mean,
            Object[] kMeans, int kValue, Distance distance) {
        int near = -1;
        double minDist = Double.MAX_VALUE;
        for (int i = 0; i < kValue; i++) {
            double dist = distance.distance(mean, kMeans[i]);
            if (dist < minDist) {
                minDist = dist;
                near = i;
            }
        }
        return minDist;
    }

    protected Object[] retransform() {
        Object[] output = new Object[state.clusterCount];
        for (int i = 0; i < state.clusterCount; i++) {
            double[][] dfts = (double[][]) state.kMeans[i];
            for (int m = dftCount + 1; m < windowSize - dftCount; m++) {
                values1[m][0] = 0;
                values1[m][1] = 0;
            }
            for (int m = 1; m <= dftCount; m++) {
                values1[m][0] = dfts[m - 1][0];
                values1[m][1] = dfts[m - 1][1];
                values1[windowSize - m][0] = dfts[m - 1][0];
                values1[windowSize - m][1] = -dfts[m - 1][1];
            }
            values1[0][0] = 0;
            values1[0][1] = 0;

            for (int m = dftCount + 1; m <= windowSize / 2; m++) {
                values1[m][0] = 0;
                values1[m][1] = 0;
                values1[windowSize - m][0] = 0;
                values1[windowSize - m][1] = 0;
            }

            fft(values1);

            double[] res = new double[windowSize];

            for (int m = 1; m < windowSize; m++) {
                double val = Math.sqrt(values1[windowSize - m][0]
                        * values1[windowSize - m][0]
                        + values1[windowSize - m][1]
                        * values1[windowSize - m][1]);
                if (values1[windowSize - m][0] < 0.0)
                    val = -val;
                // val+=mid[i];
                res[m] = (double) val;
            }
            res[0] = values1[0][0];
            output[i] = res;
        }
        return output;
    }

    public Object[] transform() {
        double[][][] dftValues = new double[state2.clusterCount][dftCount][2];
        for (int i = 0; i < state2.clusterCount; i++) {
            double[] values = (double[]) state2.kMeans[i];
            for (int j = 0; j < windowSize; j++) {
                values1[j][0] = values[j];
                values1[j][1] = 0.0;
            }
            fft(values1);

            for (int j = 0; j < dftCount; j++) {
                dftValues[i][j][0] = values1[j + 1][0];
                dftValues[i][j][1] = values1[j + 1][1];
            }
        }
        return dftValues;
    }

    protected Object[] retransformDFT(Object[] data) {
        Object[] output = new Object[dataCount];
        for (int i = 0; i < dataCount; i++) {
            double[][] dfts = (double[][]) data[i];
            for (int m = dftCount + 1; m < windowSize - dftCount; m++) {
                values1[m][0] = 0;
                values1[m][1] = 0;
            }
            for (int m = 1; m <= dftCount; m++) {
                values1[m][0] = dfts[m - 1][0];
                values1[m][1] = dfts[m - 1][1];
                values1[windowSize - m][0] = dfts[m - 1][0];
                values1[windowSize - m][1] = -dfts[m - 1][1];
            }
            values1[0][0] = 0;
            values1[0][1] = 0;

            // for (int m=dftCount+1;m<=windowSize/2;m++){
            // values1[m][0]=0;
            // values1[m][1]=0;
            // values1[windowSize-m][0]=0;
            // values1[windowSize-m][1]=0;
            // }

            fft(values1);

            double[] res = new double[windowSize];

            for (int m = 1; m < windowSize; m++) {
                // double
                // val=Math.sqrt(values1[windowSize-m][0]*values1[windowSize-m][0]+values1[windowSize-m][1]*values1[windowSize-m][1]);
                // if (values1[windowSize-m][0]<0.0) val=-val;
                // val+=mid[i];
                double val = values1[windowSize - m][0];
                res[m] = (double) val;
            }
            res[0] = values1[0][0];
            output[i] = res;
        }
        return output;
    }

    public Object[] transformOld(Object[] data) {
        double[][][] dftValues = new double[dataCount][dftCount][2];
        for (int i = 0; i < dataCount; i++) {
            double[] values = (double[]) data[i];
            for (int j = 0; j < windowSize; j++) {
                values1[j][0] = values[j];
                values1[j][1] = 0.0;
            }
            fft(values1);

            for (int j = 0; j < dftCount; j++) {
                dftValues[i][j][0] = values1[j + 1][0];
                dftValues[i][j][1] = values1[j + 1][1];
            }
            fft(values1);
            i = i;
        }
        return dftValues;
    }

    static public void fft(double[][] values) {
        fftMain(values);
        int n = values.length;
        double sq = Math.sqrt(n);
        for (int i = 0; i < n; i++) {
            values[i][0] /= sq;
            values[i][1] /= sq;
        }
    }

    static public void fftMain(double[][] values) {
        int n = values.length;
        int nd2 = (n >> 1);
        int n2, n3;
        int pos2 = 0;
        double swap0, swap1;
        int k;

        for (int i = 0; i < n; i++) {
            n2 = nd2;
            n3 = 1;
            k = 0;
            while (n2 >= n3) {
                if ((i & n2) > 0)
                    k = k | n3;
                if ((i & n3) > 0)
                    k = k | n2;
                n2 = n2 >> 1;
                n3 = n3 << 1;
            }
            // System.out.println(k);
            if (i < k) {
                // System.out.println("swap :"+i+","+k);
                swap0 = values[i][0];
                swap1 = values[i][1];
                values[i][0] = values[k][0];
                values[i][1] = values[k][1];
                values[k][0] = swap0;
                values[k][1] = swap1;
            }
        }

        int dist = 1;
        for (int pot2 = 2; pot2 <= n; pot2 <<= 1, dist <<= 1) {
            double u0 = 1.0;
            double u1 = 0.0;
            double w0 = Math.cos(Math.PI / (double) dist);
            double w1 = -Math.sin(Math.PI / (double) dist);
            for (int j = 0; j < dist; j++) {
                for (int i = j; i < n; i += pot2) {
                    pos2 = i + dist;
                    // System.out.println("pos:"+(i)+" pos2:"+(pos2)+"
                    // "+u0+","+u1);
                    double val0 = values[pos2][0] * u0 - u1 * values[pos2][1];
                    double val1 = values[pos2][1] * u0 + u1 * values[pos2][0];
                    values[pos2][0] = values[i][0] - val0;
                    values[pos2][1] = values[i][1] - val1;
                    values[i][0] = values[i][0] + val0;
                    values[i][1] = values[i][1] + val1;
                }
                double h = u0 * w0 - w1 * u1;
                u1 = w0 * u1 + w1 * u0;
                u0 = h;
            }
        }
    }

    public void close() {
    }

}
