package pipes;

import java.io.IOException;
import java.io.Writer;

import xxl.core.pipes.operators.AbstractPipe;
import clustering.FuzzyKMeansClustering;
import clustering.FuzzyKMeansState;

public class CompareFuzzyClustering6 extends AbstractPipe {

    protected Writer cc;

    protected Writer cc2;

    protected Writer c1;

    protected int dataCount;

    protected FuzzyKMeansClustering fkmc;

    protected boolean set1 = false;

    protected boolean set2 = false;

    protected FuzzyKMeansState state;

    protected FuzzyKMeansState state2;

    protected boolean end = false;

    protected CompareFuzzyClustering6(int dataCount) {
        this.dataCount=dataCount;
    }

    public CompareFuzzyClustering6(Writer cc,Writer cc2,Writer c1, 
            int dataCount) {
        this(dataCount);
        this.c1 = c1;
        this.cc=cc;
        this.cc2=cc2;
    }

    int count = 0;

    public void processObject(Object o, int ID) throws IllegalArgumentException {
        if (end)
            return;
        if (ID == 0) {
            state = (FuzzyKMeansState) o;
            set1 = true;
        } else if (ID == 1) {
            state2 = (FuzzyKMeansState) o;
            set2 = true;
        }
        if (set1 && set2) {
            set1 = false;
            set2 = false;
            if (!end)
                compareNext();
            count++;
            if (count == 204) {
                try {
                    cc.write('\n');
                    cc.flush();
                    cc2.write('\n');
                    cc2.flush();
                    c1.write('\n');
                    c1.flush();

                    cc.close();
                    cc2.close();
                    c1.close();

                    end = true;
                } catch (IOException e) {
                    e.printStackTrace();
                }
                // System.exit(0);
            }
        }

    }

    protected void compareNext() {
        int k1 = state.clusterCount;
        int k2 = state2.clusterCount;
        double[][] s = new double[k1][k2];
        double[] w1 = new double[k1];
        double[] w2 = new double[k2];
        for (int c1 = 0; c1 < k1; c1++)
            for (int c2 = 0; c2 < k2; c2++) {
                double v1 = 0;
                double v2 = 0;
                for (int i = 0; i < dataCount; i++) {
                    v1 += Math.min(state.fuzzyCluster[i][c1],
                            state2.fuzzyCluster[i][c2]);
                    v2 += Math.max(state.fuzzyCluster[i][c1],
                            state2.fuzzyCluster[i][c2]);
                }
                if (v2>0) s[c1][c2] = v1 / v2;
                else s[c1][c2] =0;
            }
        for (int i = 0; i < dataCount; i++) {
            for (int c1 = 0; c1 < k1; c1++) {
                w1[c1] += state.fuzzyCluster[i][c1];
            }
            for (int c2 = 0; c2 < k2; c2++) {
                w2[c2] += state2.fuzzyCluster[i][c2];
            }
        }
        double sim1=1;
        for (int c1 = 0; c1 < k1; c1++) {
            double w=w1[c1]/dataCount;
            double cv=0;
            for (int c2 = 0; c2 < k2; c2++) {
                double f=Math.pow(s[c1][c2],w);
                cv=cv+f-cv*f;
            }
            sim1=sim1*cv;
        }
        double sim2=1;
        for (int c2 = 0; c2 < k2; c2++) {
            double w=w2[c2]/dataCount;
            double cv=0;
            for (int c1 = 0; c1 < k1; c1++) {
                double f=Math.pow(s[c1][c2],w);
                cv=cv+f-cv*f;
            }
            sim2=sim2*cv;
        }
        double sim=sim1*sim2;
        try {
            cc.write(state.clusterCount + "  ");
            cc.flush();
            cc2.write(state2.clusterCount + "  ");
            cc2.flush();
            
            c1.write(sim+"  ");
            c1.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
        
    }

    public void close() {
    }

}
