package pipes;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.util.NoSuchElementException;

import xxl.core.functions.Function;
import xxl.core.pipes.operators.AbstractPipe;
import xxl.core.util.CompositeMetaData;
import xxl.core.util.Distance;

public class CompareStream<E> extends AbstractPipe<E, double[]> {

    protected static InputStream[] getStreams(String[] files)
            throws FileNotFoundException {
        int len = files.length;
        InputStream[] streams = new InputStream[len];
        for (int i = 0; i < len; i++) {
            streams[i] = new FileInputStream(files[i]);
        }
        return streams;
    }

    protected ObjectInputStream[] read;

    protected int rCount;

    protected Object next;

    protected Distance<E> distance;

    protected Function func = null;

    public CompareStream(String[] file, Distance<E> dist)
            throws FileNotFoundException {
        this(getStreams(file), dist);
    }

    public CompareStream(InputStream[] is, Distance<E> dist) {
        super();

        rCount = is.length;
        read = new ObjectInputStream[rCount];

        for (int i = 0; i < rCount; i++) {
            try {
                read[i] = new ObjectInputStream(new BufferedInputStream(is[i]));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        this.closeMode = NEXT_CLOSE;
        this.distance = dist;
    }

    public void setFunction(Function func) {
        this.func = func;
    }

    public Object read(int i) throws NoSuchElementException {
        try {
            Object r = read[i].readObject();
            return r;
        } catch (IOException e) {
            //e.printStackTrace();
            return null;
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return null;
    }

    public void processObject(E o, int ID) throws IllegalArgumentException {
        E o2 = null;
        double[] d = new double[rCount];
        for (int i = 0; i < rCount; i++) {
            try {
                o2 = (E) read(i);
            } catch (NoSuchElementException e) {
                o2 = null;
            }
            if (o2 == null) {
                if (func != null)
                    func.invoke();
                this.close();
                
                return;
            }
            d[i] = distance.distance(o2, o);
        }
        this.transfer(d);
    }

    public CompositeMetaData<Object, Object> getMetaData() {
        // TODO Auto-generated method stub
        return null;
    }

}
