/*
 * StreamCluster.java
 *
 * Created on 3. Februar 2003, 11:17
 */

package pipes;

import java.io.IOException;
import java.io.Writer;

import xxl.core.collections.queues.ArrayQueue;
import xxl.core.pipes.operators.AbstractPipe;
import xxl.core.pipes.operators.identities.BufferPipe;
import xxl.core.util.ObjectPool;
import clustering.AbstractDistance;
import clustering.Distance;
import clustering.KMeansUtils;

/**
 * 
 * @author Jrgen Beringer
 */
public class DFTNormWeightedStreams extends
        AbstractPipe<double[][], double[][][]> {

    static public Distance getDFTDistance(final int dftCount) {
        return new AbstractDistance() {
            Distance internal = KMeansUtils.getComplexDistance(dftCount);

            public double squaredDistance(Object ob1, Object ob2) {
                return internal.squaredDistance(ob1, ob2) * 2;
            }
        };
    }

    class Block {

        double[] objects;

        double[][] zetas;

        double sum;

        double sum2;

        double wsum;

        double wsum2;

        Block(int dftCount) {
            zetas = new double[dftCount][2];
        }
    }

    static boolean DEBUG = false;

    boolean bounded = true;

    // Anzahl von Streams
    int size;

    // Grsse eines Blocks
    int blockSize;

    // Anzahl von Blcken
    int blockCount;

    // Anzahl von zu berechnenden dft-Werten
    int dftCount;

    // Gewichtungskonstante
    double weight;

    // ObjectPool fr double-Arrays
    ObjectPool doublePool;

    // Grsse des Gesamten Fensters
    int windowSize;

    // Wurzel der Fenstergrsse (oft bentigt)
    double sqrtWS;

    // Queues fr die Blcke
    ArrayQueue[] blockQueues;

    // Block-Array der aktuellen Blcke
    Block[] blocks;

    // Gewichte fr den letzten Block (oft bentigt, teuer zu berechnen)
    double[] weights;

    // Anfangsgewicht des 2.Blocks
    double blockweight;

    // Gewicht des ersten Elements nach dem Fenster.
    double lastblockweight;

    // DFT-Werte der Streams
    double[][][] dfts;

    // normierte DFT-Werte der Streams
    double[][][] normdfts;

    // Hifswerte(komplexe Einheitswurzeln) zur Berechnung der DFT-Werte des
    // 1.Blocks
    double[][][] arcs;

    // Hilfswerte zur Berechnung der Nomierung
    double[][] dftweights;

    // double[][] dftweights2;
    // Summe aller gewichteten Werte der Strme
    double[] sum;

    // qadratische Summe aller Strme
    double[] sum2;

    // gewichtete Mittelwerte
    double[] mid;

    double[] lastMid;

    // gewichtete Standartabweichung
    double[] derive;

    double[] lastDerive;

    // Summe aller Gewichte ber das gesamte Fenster
    double sumWeights;

    // Decoupler-Object
    BufferPipe bufferPipe;

    // initialisiert? also ob die ersten Cluster schon berechnet wurden
    boolean initialized;

    // anzahl von verarbeiteten Blcken
    int init;

    // Anzahl von Strmen der Cluster
    int[] clusterCounts;

    // Anzahl von unvernderten Neuklassifizierungen in Folge
    int zerolength;

    // hilfsvariablen
    // double[][] values1;

    /*
     * double[][] values2; double[] valuesf1;
     * 
     * double[][] values11; double[][] values12; double[] valuesf11;
     */

    // Beschriftung(Namen) der Streams
    Object[] streamsObjects;

    /** Creates a new instance of StreamCluster */
    public DFTNormWeightedStreams(Object[] streamsObjects, int size,
            int blockSize, int blockCount, int dftCount, double weight,
            ObjectPool doublePool) {
        this.streamsObjects = streamsObjects;
        this.size = size;
        this.blockSize = blockSize;
        this.blockCount = blockCount;
        this.dftCount = dftCount;
        this.weight = weight;
        this.doublePool = doublePool;
        windowSize = blockSize * blockCount;
        sqrtWS = (double) Math.sqrt(windowSize);
        dftweights = new double[dftCount][2];
        // dftweights2=new double[windowSize][2];
        dfts = new double[size][dftCount][2];
        normdfts = new double[size][dftCount][2];
        sum = new double[size];
        sum2 = new double[size];
        mid = new double[size];
        derive = new double[size];
        lastMid = new double[size];
        lastDerive = new double[size];
        computeArcs();
        if (bounded)
            blockQueues = new ArrayQueue[size];
        blocks = new Block[size];
        for (int i = 0; i < size; i++) {
            blocks[i] = new Block(dftCount);
            if (bounded) {
                blockQueues[i] = new ArrayQueue();
                blockQueues[i].open();
                for (int j = 0; j < blockCount; j++) {
                    blockQueues[i].enqueue(new Block(dftCount));
                }
            }
        }
        initialized = false;
        init = 0;

        // values1=new double[windowSize][2];
        /*
         * values2=new double[windowSize][2]; valuesf1=new double[windowSize];
         * values11=new double[windowSize][2]; values12=new
         * double[windowSize][2]; valuesf11=new double[windowSize];
         */
    }

    public void setBufferPipe(BufferPipe decoupler) {
        this.bufferPipe = decoupler;
    }

    public Writer timeWriter = null;

    public void processObject(double[][] input, int id) {
        if (DEBUG)
            System.out.println("Block " + (init + 1));
        init++;
        long start = System.nanoTime();

        for (int i = 0; i < size; i++) {
            blocks[i].objects = input[i];
        }

        /*
         * for (int p=blockSize;p<windowSize;p++){
         * valuesf1[p-blockSize]=valuesf1[p]; } for (int p=0;p<blockSize;p++){
         * valuesf1[p+windowSize-blockSize]=blocks[0].objects[p]; } for (int
         * p=blockSize;p<windowSize;p++){ valuesf11[p-blockSize]=valuesf11[p]; }
         * for (int p=0;p<blockSize;p++){
         * valuesf11[p+windowSize-blockSize]=blocks[1].objects[p]; }
         */
        // compute zeta for all new blocks, and release Object-Memory
        for (int i = 0; i < size; i++) {
            computeZeta(blocks[i]);
            if (doublePool != null)
                doublePool.releaseObject(blocks[i].objects);
            blocks[i].objects = null;
        }
        // weight the old dft-Values
        for (int i = 0; i < size; i++) {
            for (int m = 1; m <= dftCount; m++) {
                dfts[i][m - 1][0] = dfts[i][m - 1][0] * blockweight;
                dfts[i][m - 1][1] = dfts[i][m - 1][1] * blockweight;
            }
        }
        // compute new dft-Values and new weighted middle and derivation
        for (int i = 0; i < size; i++) {
            Block newBlock = blocks[i];
            Block last = null;
            if (bounded)
                last = (Block) blockQueues[i].dequeue();
            for (int m = 1; m <= dftCount; m++) {
                double wR = arcs[m - 1][0][0];
                double wI = arcs[m - 1][0][1];
                double h = wR * dfts[i][m - 1][0] - wI * dfts[i][m - 1][1];
                wI = wI * dfts[i][m - 1][0] + wR * dfts[i][m - 1][1];
                wR = h;
                if (bounded) {
                    dfts[i][m - 1][0] = wR
                            + (newBlock.zetas[m - 1][0] - (last.zetas[m - 1][0] * lastblockweight))
                            / sqrtWS;
                    dfts[i][m - 1][1] = wI
                            + (newBlock.zetas[m - 1][1] - (last.zetas[m - 1][1] * lastblockweight))
                            / sqrtWS;
                } else {
                    dfts[i][m - 1][0] = wR + newBlock.zetas[m - 1][0] / sqrtWS;
                    dfts[i][m - 1][1] = wI + newBlock.zetas[m - 1][1] / sqrtWS;
                }
            }
            sum[i] = sum[i] * blockweight;
            sum2[i] = sum2[i] * blockweight;
            sum[i] += newBlock.wsum;
            sum2[i] += newBlock.wsum2;
            if (bounded)
                sum[i] -= last.wsum * lastblockweight;
            if (bounded)
                sum2[i] -= last.wsum2 * lastblockweight;
            mid[i] = sum[i] / sumWeights;
            derive[i] = sum2[i] / sumWeights - mid[i] * mid[i];
            if (derive[i] <= 0.0000001)
                derive[i] = Double.MAX_VALUE;
            else
                derive[i] = Math.sqrt(derive[i]);

            // System.out.print(" derive:"+i+"
            // ="+windowSize+","+sum[i]+","+sum2[i]+","+mid[i]+","+derive[i]+";
            // ");
            if (bounded) {
                blockQueues[i].enqueue(newBlock);
                blocks[i] = last;
            }
            // blockPool.releaseObject(last);
        }
        // norm the computet dft-Values
        for (int i = 0; i < size; i++) {
            for (int m = 1; m <= dftCount; m++) {
                normdfts[i][m - 1][0] = (double) ((dfts[i][m - 1][0] - dftweights[m - 1][0]
                        * mid[i]) / derive[i]);
                normdfts[i][m - 1][1] = (double) ((dfts[i][m - 1][1] - dftweights[m - 1][1]
                        * mid[i]) / derive[i]);
            }
        }
        /*
         * double mysum=0.0; for (int p=0;p<windowSize;p++){
         * mysum+=valuesf1[p]*Math.pow(weight,(windowSize-1-p));
         * values1[p][0]=((valuesf1[p]-mid[0])/derive[0])*Math.pow(weight,(windowSize-1-p));
         * values1[p][1]=0;
         * values2[p][0]=valuesf1[p]*Math.pow(weight,(windowSize-1-p));
         * values2[p][1]=0; } for (int p=0;p<windowSize;p++){
         * values11[p][0]=((valuesf11[p]-mid[1])/derive[1])*Math.pow(weight,(windowSize-1-p));
         * values11[p][1]=0; } System.out.println(mysum+" : "+sum[0]);
         * fft(values1); fft(values2);
         * 
         * fft(values11);
         * 
         * for (int p=0;p<windowSize;p++){
         * values2[p][0]=(values2[p][0]-dftweights2[p][0]*mid[0])/derive[0];
         * values2[p][1]=(values2[p][1]-dftweights2[p][1]*mid[0])/derive[0]; }
         * System.out.print("fft: "); for (int j=0;j<dftCount+1;j++){
         * System.out.print("("+(double)(values1[j][0])+","+(double)(values1[j][1])+")"); }
         * System.out.println(); for (int j=0;j<dftCount+1;j++){
         * System.out.print("("+(double)(values2[j][0])+","+(double)(values2[j][1])+")"); }
         * System.out.println(); System.out.print("inc: "); for (int j=0;j<dftCount;j++){
         * System.out.print("("+normdfts[0][j][0]+","+normdfts[0][j][1]+")"); }
         * System.out.println();
         * 
         * System.out.println("distance:
         * "+computeDistance(normdfts[0],normdfts[1]));
         * System.out.println("distanceAll:
         * "+computeDistanceAll(values1,values11));
         * 
         */
        double[] help = lastMid;
        lastMid = mid;
        mid = help;

        help = lastDerive;
        lastDerive = derive;
        derive = help;

        /*
         * System.out.print("DFTn "); for (int i=0;i<size;i++){
         * System.out.print(lastDerive[i]+" "); } System.out.println();
         */

        transfer(normdfts);

        long end = System.nanoTime();

        if (timeWriter != null) {
            try {
                timeWriter.write("" + (end - start) / 1000000.0 + "\n");
                timeWriter.flush();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        // System.out.println(init+" "+((end-start)/1000000.0));

    }

    public double getStdDerive(int nr) {
        return lastDerive[nr];
    }

    public double getMiddle(int nr) {
        return lastMid[nr];
    }

    // berechnet exp(j*2*pi*(b-i)*m)
    public void computeArcs() {
        arcs = new double[dftCount][blockSize][2];
        for (int m = 1; m <= dftCount; m++) {
            for (int i = 0; i < blockSize; i++) {
                arcs[m - 1][i][0] = (double) Math.cos(2 * Math.PI * m
                        * (blockSize - i) / (double) windowSize);
                arcs[m - 1][i][1] = (double) Math.sin(2 * Math.PI * m
                        * (blockSize - i) / (double) windowSize);
            }
        }
        weights = new double[blockSize];
        weights[blockSize - 1] = 1.0f;
        for (int i = 1; i < blockSize; i++) {
            weights[blockSize - 1 - i] = (double) Math.pow(weight, i);
        }
        lastblockweight = (double) Math.pow(weight, blockSize * (blockCount));
        blockweight = (double) Math.pow(weight, blockSize);
        System.out.println("weigth:" + weight + " windowsize:"
                + (blockSize * (blockCount)) + " lastblockweight:"
                + lastblockweight);
        sumWeights = 0.0;
        double w;
        double[] helpWeights = new double[windowSize];
        for (int p = 0; p < windowSize; p++) {
            w = Math.pow(weight, (windowSize - 1 - p));
            sumWeights += w;
            /*
             * dftweights2[p][0]=w; dftweights2[p][1]=0.0;
             */
            helpWeights[p] = (double) w;
        }
        for (int m = 1; m <= dftCount; m++) {
            double resR = 0.0f;
            double resI = 0.0f;
            for (int i = 0; i < windowSize; i++) {
                resR += (double) ((double) Math.cos(2 * Math.PI * m
                        * (windowSize - i) / (double) windowSize) * helpWeights[i]);
                resI += (double) ((double) Math.sin(2 * Math.PI * m
                        * (windowSize - i) / (double) windowSize) * helpWeights[i]);
            }
            dftweights[m - 1][0] = resR / sqrtWS;
            dftweights[m - 1][1] = resI / sqrtWS;
        }
        // fft(dftweights2);
    }

    // berechnet zeta-Zahl fr einen Block
    public void computeZeta(Block block) {
        double[][] res;
        if (block.zetas == null) {
            res = new double[dftCount][2];
            block.zetas = res;
        } else
            res = block.zetas;
        double sum = 0.0;
        double sum2 = 0.0;
        double wsum = 0.0;
        double wsum2 = 0.0;
        double[] objects = block.objects;
        for (int i = 0; i < blockSize; i++) {
            sum += objects[i];
            sum2 += objects[i] * objects[i];
            wsum += objects[i] * weights[i];
            wsum2 += objects[i] * objects[i] * weights[i];
        }
        for (int m = 1; m <= dftCount; m++) {
            double resR = 0.0f;
            double resI = 0.0f;
            for (int i = 0; i < blockSize; i++) {
                resR += arcs[m - 1][i][0] * objects[i] * weights[i];
                resI += arcs[m - 1][i][1] * objects[i] * weights[i];
            }
            res[m - 1][0] = resR;
            res[m - 1][1] = resI;
        }
        block.sum = sum;
        block.sum2 = sum2;
        block.wsum = wsum;
        block.wsum2 = wsum2;
    }

}
