/*
 * MultiSource.java
 *
 * Created on 27. Januar 2003, 11:47
 */

package pipes;
import java.sql.ResultSetMetaData;
import java.util.Iterator;
import java.util.NoSuchElementException;

import xxl.core.cursors.AbstractCursor;
import xxl.core.cursors.sources.Repeater;
import xxl.core.functions.Function;
import xxl.core.pipes.sources.AbstractSource;
import xxl.core.relational.cursors.InputStreamMetaDataCursor;
import xxl.core.relational.tuples.Tuple;
import xxl.core.util.ObjectPool;
/**
 *
 * @author  Jrgen Beringer
 */
public class FastMultiSource extends AbstractSource {
	protected int count;
	protected int windowsize;
	protected int period;
	protected Function nextFactory;
	protected boolean begin=true;
	protected float[] list;
	protected float[][] output;
	protected int aktSize;
	protected int blockSize;
	protected int blockCount;
	
	protected int number;
	
	protected ObjectPool floatPool;
	/** Creates a new instance of MultiSource */
	public FastMultiSource(Function nextFactory, int count, int period, final int blockSize, int blockCount) {
		super(1);
		this.processor.setPriority(Thread.MIN_PRIORITY);
		this.count = count;
		this.period=period*blockSize;
		this.windowsize=blockCount*blockSize;
		this.blockSize=blockSize;
		this.blockCount=blockCount;
		this.nextFactory = nextFactory;
		number=0;
		floatPool=new ObjectPool(new Function(){
			public Object invoke(){
				return new float[blockSize];
			}
		},
		count*10);
	}
	
	public void makeOutput(){
		aktSize=0;
		output=new float[count][];
		for (int i=0;i<count;i++){
			output[i]=(float[])floatPool.getObject();
		}
	}

	public ObjectPool getFloatPool(){
		return floatPool;
	}

	public Object next() throws NoSuchElementException {
		if (begin&&(number>=blockCount)) {
			begin=false;
			((Repeater)this.processor.getDelayManager()).update(new Long(period));
			this.processor.setPriority(Thread.MAX_PRIORITY);
			System.out.println("!!!!  SET  PERIOD  !!!");
		}
		if (begin) number++;
		makeOutput();
		for (int j=0;j<blockSize;j++){
			list = (float[])nextFactory.invoke();
			for(int i=0;i<count;i++){
				try{
					output[i][j]=list[i];
				}catch(Exception e){e.printStackTrace();}
			}
		}
		return output;
	}

	

	public int getCount() {
		return count;
	}

	public static Function getFactory(final int co, final int windowsize) {
		final float[] periods = new float[co];
		for (int i = 0; i < co; i++) {
			periods[i] = (float) ((Math.random() * 10) - 5);
			//periods[i]=(i%5)+1;
			//periods[i]=2;
			//periods[i]=(int)((Math.random()*5)+1);
		}
		return new Function() {
			int nr = 128;
			public Object invoke() {
				nr++;
				float[] list = new float[co];
				for (int i = 0; i < co; i++) {
					list[i] =
						((float)Math.sin(2 * Math.PI * nr * periods[i] / windowsize));
				}
				return list;
			}
		};
	}

	public static Function getTestFactory(final int co, final int windowsize) {
		final float[] periods = new float[co];
		final int ind = 2;
		final float per = 1.0f;
		final int change=windowsize*4;
		periods[0] = per;
		periods[1] = -per;
		periods[2] = per;
		/*periods[3]=-per;
		periods[4]=per;*/
		/*periods[5]=-1f;
		periods[6]=1f;*/

		return new Function() {
			int nr = 0;
			public Object invoke() {
				nr++;
				//if (nr %16==0) System.out.print("Ich schlaf net!");
				if (nr % change == 0) {
					periods[ind] = per;
					System.out.println("!!!!!!!!!!! ch 0 !!!!!!!!!");
				}
				if (nr % change == (change>>1)) {
					periods[ind] = -per;
					System.out.println("!!!!!!!!!!! ch 1 !!!!!!!!!");
				}

				float[] list = new float[co];
				for (int i = 0; i < co; i++) {
					list[i] =
						((float)(Math.sin(2 * Math.PI * nr * periods[i] / windowsize)/0.7063178));
						//(nr%4<2)?new Float(1f*periods[i]):new Float(-1f*periods[i]);
				}
				return list;
			}
		};
	}
	
	public static Function getTest2Factory(final int co,final int diffs, final int cluster, final int windowsize) {
		final float[] periods = new float[co];
		final float[] list = new float[co];
		for (int i=0;i<co;i++){
			periods[i]=(float)((2*((int)(Math.random()*diffs)-(diffs/2))+1));
			System.out.println("Period "+periods[i]);
		}
		for (int i=0;i<cluster;i++){
			periods[i]=2*((int)(i)-(cluster/2))+1;
		}
		
		return new Function() {
			int nr = 0;
			public Object invoke() {
				nr++;
				//if (nr %16==0) System.out.print("Ich schlaf net!");
				/*if (nr %2048==0){
					for (int i=0;i<co;i++){
						System.out.println("Period "+periods[i]);
					}
				}*/
				/*for (int i=0;i<co;i++){
					if (Math.random()<0.00002){
						int j=(int)(Math.random()*co);
						//System.out.println("!!! CHANGE !!!!  "+i+"<->"+j);
						float h=periods[i];
						periods[i]=periods[j];
						periods[j]=h;
					}
				}*/

				
				for (int i = 0; i < co; i++) {
					list[i] =//nr*(i-3);
						(float)((0.8*Math.sin(2 * Math.PI * nr * periods[i] / windowsize))+Math.random()/2.5-(1/5));
				}
				return list;
			}
		};
	}

	public static Function getCursorFactory(final java.util.Iterator it) {
		return new Function() {
			public Object invoke() {
				return it.next();
			}
		};
	}

	public static Object[] getStreamNames(
		final InputStreamMetaDataCursor fmdc,
		final int count,
		final int offset) {
		Object[] obs = new Object[count];
		ResultSetMetaData rsmd = (ResultSetMetaData)fmdc.getMetaData();
		try {
			for (int i = 0; i < count; i++) {
				obs[i] = rsmd.getColumnName(i + offset);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return obs;
	}

	public static Iterator getMultiSourceFileIterator(
		final InputStreamMetaDataCursor fmdc,
		final int count,
		final int offset) {

		return new AbstractCursor() {
			public Object nextObject() {
				if (!fmdc.hasNext()) {
					System.out.println(
						"\n\n\n\n\n\n\n\n\n!!!!!!!!!  E N D E !!!!!!!!!!!!\n\n\n\n\n\n\n\n\n");
					return null;
				}
				Tuple tup = (Tuple)fmdc.next();
				Object[] obs = new Object[count];
				for (int i = 0; i < count; i++) {
					obs[i] = tup.getObject(i + offset);
				}
				return obs;
			}
            public boolean hasNextObject(){
                if (!fmdc.hasNext()) {
                    System.out.println(
                        "\n\n\n\n\n\n\n\n\n!!!!!!!!!  E N D E !!!!!!!!!!!!\n\n\n\n\n\n\n\n\n");
                    return false;
                }
                else return true;
            }
		};
	}

}
