package pipes;

import lift.haar;
import clustering.Distance;

public class HaarWaveletCompressor implements Distance<double[]>,
        Compressor<double[]> {

    protected int blocksize;
    protected int compsize;
    protected double[] block;
    protected haar haar;

    public HaarWaveletCompressor(int blocksize, int compsize) {
        this.blocksize = blocksize;
        this.compsize = compsize;
        block = new double[blocksize];
        haar = new haar();
    }

    public double squaredDistance(double[] v1, double[] v2) {
        int len = v1.length;
        double sum = 0;
        double d = (v1[0] - v2[0]);
        sum += d * d * 2;
        int cl = 1;
        int pos = 0;
        for (int i = 1; i < blocksize; i++) {
            if (pos == cl) {
                sum *= 2;
                cl *= 2;
                pos = 0;
            }

            if (i<len) d = (v1[i] - v2[i]);
            else d=0;
            sum += d * d * 0.5;

            pos += 1;
        }
        return sum;
    }

    public double distance(double[] v1, double[] v2) {
        return Math.sqrt(squaredDistance(v1, v2));
    }

    public double[] compress(double[] data) {
        System.arraycopy(data, 0, block, 0, blocksize);
        haar.forwardTrans(block);
        double[] res = new double[compsize];
        System.arraycopy(block, 0, res, 0, compsize);
        return res;
    }

    public double[] decompress(double[] data) {
        double[] res = new double[blocksize];
        System.arraycopy(data, 0, res, 0, compsize);
        haar.inverseTrans(res);

        return res;
    }

}
