package pipes;

import java.util.NoSuchElementException;

import xxl.core.pipes.operators.AbstractPipe;
import xxl.core.pipes.operators.Pipe;
import xxl.core.pipes.sinks.Sink;
import xxl.core.pipes.sources.Source;
import xxl.core.pipes.sources.SourceIsClosedException;
import xxl.core.util.CompositeMetaData;

public abstract class MultiplePipe<I,O> extends AbstractPipe<I,O[]> {

    protected class DummyPipe extends AbstractPipe<I,O> {
        public DummyPipe() {
        }
        public void open() throws SourceIsClosedException {
            MultiplePipe.this.open();
        }
        public void close() {
            MultiplePipe.this.close();
        }
        public void processObject(I o, int ID) throws IllegalArgumentException {
            MultiplePipe.this.processObject(o,ID);
        }
        public CompositeMetaData<Object, Object> getMetaData() throws UnsupportedOperationException {
            return MultiplePipe.this.getMetaData();
        }
    }

    protected int[] outputIDs;
    protected Pipe[] pipes;
    protected int count;

    public MultiplePipe(int[] outputIDs) {
        this.outputIDs= outputIDs;
        count= outputIDs.length;
        pipes= new Pipe[count];
        for (int i= 0; i < count; i++) {
            pipes[i]= new DummyPipe();
        }
    }


    public boolean subscribe(Sink sink, int inputID) throws SourceIsClosedException {
        boolean val= true;
        for (int i= 0; i < count; i++) {
            val= val && pipes[i].subscribe(sink, inputID);
        }
        return val;
    }

    public boolean subscribe(Sink sink, int inputID, int outputID) throws SourceIsClosedException {
        boolean val= true;
        //super.subscribe(sink, inputID);
        for (int i= 0; i < count; i++) {
            if (outputID == outputIDs[i])
                val= val && pipes[i].subscribe(sink, inputID);
        }
        return val;
    }

    public boolean unsubscribe(Sink sink, int inputID) throws SourceIsClosedException {
        boolean val= true;
        for (int i= 0; i < count; i++) {
            val= val || pipes[i].unsubscribe(sink, inputID);
        }
        return val;
    }

    public boolean unsubscribe(Sink sink, int inputID, int outputID)
        throws SourceIsClosedException {
        boolean val= false;
        //super.subscribe(sink, inputID);
        for (int i= 0; i < count; i++) {
            if (outputID == outputIDs[i])
                val= val || pipes[i].unsubscribe(sink, inputID);
        }
        return val;
    }

    public void pause(Sink sink, int inputID) throws SourceIsClosedException {
        for (int i= 0; i < count; i++) {
            pipes[i].pause(sink, inputID);
        }
    }

    public void pause(Sink sink, int inputID, int outputID) throws SourceIsClosedException {
        for (int i= 0; i < count; i++) {
            if (outputID == outputIDs[i])
                pipes[i].pause(sink, inputID);
        }
    }

    public void resume(Sink sink, int inputID) throws SourceIsClosedException {
        for (int i= 0; i < count; i++) {
            pipes[i].resume(sink, inputID);
        }
    }

    public void resume(Sink sink, int inputID, int outputID) throws SourceIsClosedException {
        for (int i= 0; i < count; i++) {
            if (outputID == outputIDs[i])
                pipes[i].resume(sink, inputID);
        }
    }

    public int transfer(O[] obs) {
        int c= 0;
        for (int i= 0; i < count; i++) {
            c += pipes[i].transfer(obs[i]);
        }
        return c;
    }

    public int transfer(O ob, int outputID) {
        int c= 0;
        for (int i= 0; i < count; i++) {
            if (outputID == outputIDs[i])
                c += pipes[i].transfer(ob);
        }
        return c;
    }

}
