package pipes;

import java.io.BufferedInputStream;
import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.util.NoSuchElementException;

import xxl.core.pipes.sources.AbstractSource;

public class ReadObjectStream<E> extends AbstractSource<E> {
    
    protected ObjectInputStream read;

    protected Object next;
    
    protected boolean hasToWait;

    public ReadObjectStream(String file,int period)
            throws FileNotFoundException {
        this(new FileInputStream(file),period);
    }

    public ReadObjectStream(InputStream is, int period) {
        super(period);
        hasToWait=(period==0);
        if (hasToWait) processor.hasToPauseUntilWakeUp();
        if (read instanceof ObjectInputStream)
            this.read = (ObjectInputStream) is;
        else
            try {
                this.read = new ObjectInputStream(new BufferedInputStream(is));
            } catch (IOException e) {
                e.printStackTrace();
            }
        this.closeMode=NEXT_CLOSE;
        
        try {
            next = read();
        } catch (NoSuchElementException e1) {
            next = null;
        }
    }

  

    public Object read() throws NoSuchElementException {
        try {
            Object r=read.readObject();
            return r;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        throw new NoSuchElementException();
    }
    
    public void doNext(){
        processor.wakeUp();
    }
    
    public void setHasToWait(boolean wait){
        hasToWait=wait;
        if (hasToWait) processor.hasToPauseUntilWakeUp();
    }
    
    public boolean getHasToWait(){
        return hasToWait;
    }

    public E next() throws NoSuchElementException {
        if (next == null) {
            //SimpleStreaming.rerun();
            //System.exit(0);
            this.close();
            throw new NoSuchElementException();
        }
        E ret = (E)next;
        try {
            next = read();
        } catch (NoSuchElementException e1) {
            next = null;
        }
        if (hasToWait)processor.hasToPauseUntilWakeUp();
        
        return ret;
    }

}
