package pipes;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import xxl.core.functions.Function;
import xxl.core.pipes.sinks.AbstractSink;

public class SaveObjectStream<E> extends AbstractSink<E> {

    protected int datacount;

    protected ObjectOutputStream write;

    protected int step = 0;
    
    protected int max=-1;
    
    protected Function func;

    public SaveObjectStream(String file)
            throws IOException {
        this(new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(file))));
    }

    public SaveObjectStream(ObjectOutputStream write) {
        this.write = write;
        step=0;
    }
    
    /**
     * Set number of stored stream elements. For infinity storage, use -1.
     * @param max number of storing elements
     */
    public void setMaxCount(int max){
        this.max=max;
    }
    
    public void setFunction(Function func){
        this.func=func;
    }

    /**
     * Store next element. Close sink if maximal number of elements is reached.
     * @param o stream element to store
     * @param ID Default input ID
     * @throws IllegalArgumentException
     */
    public void processObject(E o, int ID)
            throws IllegalArgumentException {
        step++;
        if (max>0 && step > max){
            stop();
            return;
        }
        try{
            write.writeUnshared(o);
            write.flush();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        if (max>0 && step >= max){
            try {
                write.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            if (func!=null){
                func.invoke();
                stop();
            }else{
                stop();
                System.exit(0);
            }
                
        }
        
    }

    

}
