package pipes;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import xxl.core.pipes.sinks.AbstractSink;

public class SaveTextStream extends AbstractSink {

    protected static Writer[] getStreams(String[] files) throws IOException {
        int len = files.length;
        Writer[] streams = new Writer[len];
        for (int i = 0; i < len; i++) {
            streams[i] = new FileWriter(files[i]);
        }
        return streams;
    }

    protected int datacount;

    protected Writer[] write;

    protected int wCount;

    protected int step = 0;

    protected int max = -1;

    public SaveTextStream(String[] file) throws IOException {
        this(getStreams(file));
    }

    public SaveTextStream(Writer[] write) {
        this.write = write;
        this.wCount = write.length;
    }

    /**
     * Set number of stored stream elements. For infinity storage, use -1.
     * 
     * @param max
     *            number of storing elements
     */
    public void setMaxCount(int max) {
        this.max = max;
    }

    /**
     * Store next element. Close sink if maximal number of elements is reached.
     * 
     * @param o
     *            stream element to store
     * @param ID
     *            Default input ID
     * @throws IllegalArgumentException
     */
    public void processObject(Object o, int ID) throws IllegalArgumentException {
        step++;
        double[] data = (double[]) o;
        for (int i = 0; i < wCount; i++) {
            try {
                write[i].write(data[i] + "\n");
                write[i].flush();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        if (max > 0 && step >= max)
            System.exit(0);

    }

    public void close() {
        try {
            for (int i = 0; i < wCount; i++) {
                write[i].flush();
                write[i].close();
            }
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

}
