package pipes.classifier;

import java.util.Enumeration;

import pipes.MultiplePipe;
import weka.classifiers.CaseBasedAlgorithm;
import weka.classifiers.OnLineClassifier;
import weka.classifiers.RuleBasedAlgorithm;
import weka.core.Instance;
import weka.core.Instances;
import xxl.core.util.CompositeMetaData;

public class ClassifierPipe extends MultiplePipe<Object, Object> {

    protected static int[] outputIDs = new int[] { 0, 1, 2, 3, 4, 5, 6};

    protected OnLineClassifier classifier;

    protected boolean isCaseBased;

    protected boolean isRuleBased;

    protected int transferPeriod;

    protected int periodCount;

    public ClassifierPipe(OnLineClassifier classifier, boolean isCaseBased,
            boolean isRuleBased, int transferPeriod) {
        super(outputIDs);
        this.classifier = classifier;
        this.isCaseBased = isCaseBased;
        this.isRuleBased = isRuleBased;
        this.transferPeriod = transferPeriod;
        periodCount = 0;
    }

    public ClassifierPipe(OnLineClassifier classifier, int transferPeriod) {
        super(outputIDs);
        this.classifier = classifier;
        this.isCaseBased = classifier instanceof CaseBasedAlgorithm;
        this.isRuleBased = classifier instanceof RuleBasedAlgorithm;
        this.transferPeriod = transferPeriod;
        periodCount = 0;
    }

    public void processObject(Object input, int ID)
            throws IllegalArgumentException {
        if (ID == 4) {
            Instance instance = (Instance) input;
            classifier.updateClassifier(instance);
            periodCount++;
            if ((transferPeriod <= 0 && classifier.modelHasChanged())
                    || (transferPeriod > 0 && periodCount >= transferPeriod)) {
                if (transferPeriod > 0)
                    periodCount -= transferPeriod;
                transfer(classifier, outputIDs[4]);
                if (isCaseBased) {
                    transfer(((CaseBasedAlgorithm) classifier)
                            .getTrainInstances(), outputIDs[5]);
                }
                if (isRuleBased) {
                    transfer(((RuleBasedAlgorithm) classifier).getRules(),
                            outputIDs[6]);
                }
            }
        } else if (ID == 0) {
            Instance instance = (Instance) input;
            try {
                instance.setClassValue(classifier.classifyInstance(instance));
            } catch (Exception e) {
                e.printStackTrace();
            }
            transfer(instance, outputIDs[0]);
        } else if (ID == 1) {
            Instances instances = (Instances) input;
            for (Enumeration<Instance> en = instances.enumerateInstances(); en
                    .hasMoreElements();) {
                Instance instance = en.nextElement();
                try {
                    instance.setClassValue(classifier
                            .classifyInstance(instance));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            transfer(instances, outputIDs[1]);
        }else if (ID == 2) {
            Instance instance = (Instance) input;
            int cls=0;
            try {
                cls=(int)classifier.classifyInstance(instance);
            } catch (Exception e) {
                e.printStackTrace();
            }
            transfer(cls==(int)instance.classValue(),outputIDs[2]);
        } else if (ID == 3) {
            Instances instances = (Instances) input;
            int correct=0;
            for (Enumeration<Instance> en = instances.enumerateInstances(); en
                    .hasMoreElements();) {
                Instance instance = en.nextElement();
                try {
                    if ((int)classifier
                            .classifyInstance(instance)==(int)instance.classValue()) correct++;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            transfer(correct, outputIDs[3]);
        }
    }

    public CompositeMetaData getMetaData() {
        return null;
    }

}
