package pipes.classifier;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import utils.visibility.StreamVisualizer;
import xxl.core.functions.Function;
import xxl.core.pipes.sinks.AbstractSink;

public class VisualizeStreams extends AbstractSink {

    protected int streamCount;

    protected StreamVisualizer[] streams;

    public VisualizeStreams(String name, int streamCount, String[] streamTitle,
            int[] length, boolean[]autosize,double min[], double[] max) {
        this(name, streamCount, streamTitle, length,autosize,min,max, 0, null, null);
    }

    public VisualizeStreams(String name, int streamCount, String[] streamTitle,
            int[] length, boolean[]autosize,double min[], double[] max, int butCount, String[] butTitle, Function[] butActions) {
        this.streamCount = streamCount;
        streams = new StreamVisualizer[streamCount];
        final JFrame frame = new JFrame(name);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                frame.setVisible(false);
                stop();
                frame.dispose();
            }
        });
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        JPanel pan = new JPanel();
        JPanel west = new JPanel();
        JPanel center = new JPanel();
        JPanel south = new JPanel();
        pan.setLayout(new BorderLayout());
        west.setLayout(new GridLayout(streamCount, 1,0,5));
        center.setLayout(new GridLayout(streamCount, 1,0,5));
        for (int i = 0; i < streamCount; i++) {
            JLabel lab = new JLabel(streamTitle[i]);
            // lab.setHorizontalAlignment(SwingConstants.RIGHT);
            west.add(lab);
            streams[i] = new StreamVisualizer(length[i], 400, 50,min[i],max[i],autosize[i]);
            if (autosize[i])streams[i].setAutoMinMax(0,0.25,0.5,true); 
            center.add(streams[i]);
        }
        pan.add(west, BorderLayout.WEST);
        pan.add(center, BorderLayout.CENTER);
        if (butCount > 0) {
            south.setLayout(new FlowLayout());
            for (int i = 0; i < butCount; i++) {
                JButton but = new JButton(butTitle[i]);
                final Function action = butActions[i];
                but.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        action.invoke();
                    }
                });
                south.add(but);
            }
            pan.add(south, BorderLayout.SOUTH);
        }
        frame.setContentPane(pan);
        frame.pack();
        frame.setVisible(true);
    }

    @Override
    public void processObject(Object o, int ID) throws IllegalArgumentException {
        if (ID >= 0 && ID < streamCount) {
            if (o instanceof Number)
                streams[ID].add(((Number) o).doubleValue());
            else if (o instanceof double[])
                streams[ID].add(((double[]) o));
        }
    }

}
