/*
 * Created on 14.10.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package pipes.data;

import java.util.NoSuchElementException;

import pipes.MultipleSource;
import xxl.core.cursors.sources.Repeater;
import xxl.core.pipes.processors.SourceProcessor;

/**
 * @author beringer
 */
public class IndexBlockSource extends MultipleSource {
	
	private static int[] getIdArray(int channels){
		int[] result=new int[channels+1];
		for (int i=0;i<=channels;i++){
			result[i]=i;
		}
		return result;
	}		

	protected DoubleIterator it;
	protected int arraysize;
	protected int windowsize;
	protected int blocksize;
	protected double[][] array;
	protected Repeater repeater;
	protected int pos;
	protected int first;
	protected int last;
	protected int channels;

	public IndexBlockSource(
		long period,
		DoubleIterator it,
		int arraysize,
		int windowsize,
		int blocksize,
		int channels) {

		super(getIdArray(channels), new SourceProcessor(new Repeater(new Long(period))));
		
		repeater= (Repeater) processor.getDelayManager();
		this.channels=channels;
		this.it= it;
		this.arraysize= arraysize;
		this.windowsize= windowsize;
		this.blocksize= blocksize;
		array= new double[channels][arraysize];
		first= -arraysize + 1;
		last= 0;
		pos= 0;
	}

	public void halt() {
		processor.pauseUntilWakeUp();
	}

	public void wakeUp() {
		processor.wakeUp();
	}

	public void setPeriod(long period) {
		repeater.update(new Long(period));
	}

	public void setIterator(DoubleIterator it) {
		this.it= it;
		for (int i= 0; i < arraysize; i++) {
			for (int j=0;j<channels;j++){
				array[j][i]= 0d;
			}
		}
		first= -arraysize + 1;
		last= 0;
		pos= 0;
		double[][] block= new double[channels][windowsize];
		int[] inds= new int[windowsize];
		for (int i= 0; i < windowsize; i++) {
			for (int j=0;j<channels;j++){
				block[j][i]= 0d;
			}
			inds[i]= 0;
		}
		transfer(makeTransfer(inds,block));
	}
	
	public Object[] makeTransfer(int[] ids, double[][] block){
		Object[] result=new Object[channels+1];
		result[0]=ids;
		for (int i=0;i<channels;i++){
			result[i+1]=block[i];
		}
		return result;
	}

	public void setBlocksize(int blocksize) {
		if (blocksize <= windowsize) {
			this.blocksize= blocksize;
		}
	}

	public void update() {
		if (pos <= last)
			return;
		int step= pos - last;
		if (step < arraysize)
			for (int j=0;j<channels;j++){
				System.arraycopy(array[j], step, array[j], 0, arraysize - step);
			}
		else {
			while (step > arraysize) {
				for (int j=0;j<channels;j++){
					it.nextDouble();
				}
				step--;
			}
		}
		for (int i= 0; i < step; i++) {
			for (int j=0;j<channels;j++){
				array[j][arraysize - step + i]= it.nextDouble();
			}
		}
		last= pos;
		first= pos - arraysize + 1;
	}

	public Object next() throws NoSuchElementException {
		if (it==null){
			return new Object[]{new double[0],new int[0]};
		}
		pos += blocksize;
		if (pos > last)
			update();
		double[][] block= new double[channels][blocksize];
		for (int j=0;j<channels;j++){
			System.arraycopy(array[j], pos - first + 1 - blocksize, block[j], 0, blocksize);
		}
		int[] inds= new int[blocksize];
		for (int i= 0; i < blocksize; i++) {
			inds[i]= pos + 1 - blocksize + i;
		}
		return makeTransfer(inds,block);
	}

	public void back() {
		if (pos - blocksize < first + windowsize - 1)
			return;
		pos -= blocksize;
		double[][] block= new double[channels][windowsize];
		for (int j=0;j<channels;j++){
			System.arraycopy(array[j], pos - first + 1 - windowsize, block[j], 0, windowsize);
		}
		int[] inds= new int[windowsize];
		for (int i= 0; i < windowsize; i++) {
			inds[i]= pos + 1 - windowsize + i;
		}
		transfer(makeTransfer(inds,block));
	}

}
