/*
 * Created on 25.09.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package pipes.data;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Random;

import xxl.core.functions.Function;
import xxl.core.math.functions.RealFunction;
import xxl.core.pipes.processors.SourceProcessor;
import xxl.core.pipes.sources.AbstractSource;
import xxl.core.pipes.sources.Source;

/**
 * @author beringer
 */
public class SyntheticRealIterators {

    public static class FactoryFunction extends Function {
        int classCount;

        int streamCount;

        int blockSize;

        ErrorFunction[] itsFunc;

        Iterator[] its;

        public FactoryFunction(final int classCount, final int streamCount,
                final int blockSize, final Random random) {
            this.classCount = classCount;
            this.streamCount = streamCount;
            this.blockSize = blockSize;

            RealFunction[] func = new RealFunction[classCount];
            for (int i = 0; i < classCount; i++) {
                func[i] = getIteratorWindowFunction(getDiff2RandomWalk(0d, 6,
                        0.1, -Double.MAX_VALUE, Double.MAX_VALUE, random), 1600);
            }
            itsFunc = new ErrorFunction[streamCount];
            its = new Iterator[streamCount];
            for (int i = 0; i < streamCount; i++) {
                RealFunction vert = getIteratorWindowFunction(
                        getDiff2RandomWalk(0d, 2, 0.3, -20, +20, random), 100);
                RealFunction hori = getIteratorWindowFunction(
                        getDiff2RandomWalk(0d, 5, 1, -20.0, 20.0, random),
                        100);
                itsFunc[i] = new ErrorFunction(func[i % classCount], vert,
                        hori, 300);

                its[i] = getBlockIterator((DoubleIterator) itsFunc[i],
                        blockSize);
            }
        }

        public FactoryFunction(final int streamCount,
                final int blockSize,final int blockCount, final Random random, int version) {
            this.classCount = classCount;
            this.streamCount = streamCount;
            this.blockSize = blockSize;

            RealFunction[] func = new RealFunction[3];
            
//            func[0] = getIteratorWindowFunction(getDiff2RandomWalk(0d, 1.0,
//                    0.4, -1,1, random), 1600);
//            func[1] = getIteratorWindowFunction(getDiff2RandomWalk(0d, 1.0,
//                    0.4, -1,1, random), 1600);
//            func[2] = getIteratorWindowFunction(getDiff2RandomWalk(0d, 1.0,
//                    0.4, -1,1, random), 1600);
//            
            func[0] = sinus(blockSize*blockCount/4,60,0);
            func[1] = sinus(blockSize*blockCount/4,60,Math.PI/2);
            func[2] = sinus(blockSize*blockCount/4,60,Math.PI/2);
            
            itsFunc = new ErrorFunction[streamCount];
            its = new Iterator[streamCount];
            for (int i = 0; i < streamCount; i++) {
                if (version == 2) {
                    RealFunction vert = getIteratorWindowFunction(
                            getDiff2RandomWalk(0d, 1.0, 0.3, -10, +10, random),
                            100);
                    RealFunction hori = getIteratorWindowFunction(
                            getDiff2RandomWalk(0d, 15, 0.5, -90.0, 90.0, random),
                            100);
                    itsFunc[i] = new ErrorFunction(func[i / (int)Math.ceil(streamCount/3.0)], vert,
                            hori, 300);

                    its[i] = getBlockIterator((DoubleIterator) itsFunc[i],
                            blockSize);
                }
            }
        }

        public Object invoke() {
            Object[] obs = new Object[streamCount];
            for (int i = 0; i < streamCount; i++) {
                obs[i] = its[i].next();
            }
            return obs;
        }

        public void exchangeFunction(int stream, RealFunction func) {
            itsFunc[stream].exchangeFunction(func);
            its[stream] = getBlockIterator((DoubleIterator) itsFunc[stream],
                    blockSize);

        }
        
        public void exchangeFunction(int stream, RealFunction func, int offset) {
            itsFunc[stream].exchangeFunction(func, offset);
            its[stream] = getBlockIterator((DoubleIterator) itsFunc[stream],
                    blockSize);

        }

        public RealFunction getFunction(int stream) {
            return itsFunc[stream].func;
        }

        public ErrorFunction[] getFunctions() {
            return itsFunc;
        }

    }

    public static class ErrorFunction extends AbstractDoubleIterator {

        RealFunction func;

        RealFunction vertical;

        RealFunction horizontal;

        long nr;

        double offset = 0;

        double last = 0;

        public ErrorFunction(final RealFunction func,
                final RealFunction vertical, final RealFunction horizontal,
                final long init) {
            this.func = func;
            this.vertical = vertical;
            this.horizontal = horizontal;
            nr = init;

        }

        public double nextDouble() {
            nr++;
            // return func.eval(nr + horizontal.eval(nr)) * (1 +
            // vertical.eval(nr));
            double funcV=func.eval(nr + horizontal.eval(nr));
            double vert=vertical.eval(nr);
            
            last =  funcV+ offset
                    + vert;
            return last;
        }

        public void exchangeFunction(RealFunction func2) {
            offset = func.eval(nr + horizontal.eval(nr))
                    - func2.eval(nr + horizontal.eval(nr));
            func = func2;
        }
        
        public void exchangeFunction(RealFunction func2, double offset) {
            this.offset = offset; 
            func = func2;
        }

    }

    public static RealFunction ZERO = new RealFunction() {
        public double eval(double val) {
            return 0.0;
        }
    };

    public static RealFunction sinus(final double looplength,
            final double factor, final double offset) {
        return new RealFunction() {
            public double eval(double val) {
                return Math.sin((val * 2 * Math.PI / looplength) + offset)
                        * factor;
            }
        };
    }

    public static RealFunction linear(final double looplength,
            final double min, final double max) {
        return new RealFunction() {
            public double eval(double val) {
                val = val + looplength / 4;
                int c = (int) Math.floor(val / looplength);
                val = val - c * looplength;
                if (val < looplength / 2) {
                    val = val / (looplength / 2);
                } else {
                    val = (looplength - val) / (looplength / 2);
                }
                return (max - min) * val + min;
            }
        };
    }

    public static RealFunction linear(final double changeLength,
            final double staticLength, final double min, final double max) {
        return new RealFunction() {
            double halfLength = changeLength + staticLength;

            double looplength = 2 * halfLength;

            public double eval(double val) {
                val = val % looplength;
                if (val < halfLength) {
                    if (val >= changeLength)
                        return 1.0;
                    else
                        return ((max - min) * val / changeLength) + min;
                } else {
                    val = val - halfLength;
                    if (val >= changeLength)
                        return 0.0;
                    else
                        return ((max - min) * (1 - val / changeLength)) + min;
                }
            }
        };
    }

    public static AbstractSource getIteratorSource(SourceProcessor processor,
            final Iterator it) {
        AbstractSource as = new AbstractSource(processor) {
            public Object next() {
                if (it.hasNext())
                    try {
                        return it.next();
                    } catch (Exception e) {
                        e.printStackTrace();
                        throw new NoSuchElementException();
                    }
                else
                    throw new NoSuchElementException();
            }
        };
        as.setCloseMode(Source.AUTOMATIC_CLOSE);
        return as;
    }

    public static Iterator getBlockIterator(final DoubleIterator dit,
            final int blockSize) {
        return new Iterator() {
            public void remove() {
                dit.remove();
            }

            public boolean hasNext() {
                return dit.hasNext();
            }

            public Object next() {
                double[] result = new double[blockSize];
                for (int i = 0; i < blockSize; i++) {
                    result[i] = dit.nextDouble();
                }
                return result;
            }
        };
    }

    public static DoubleIterator getRandomWalk(final double startValue,
            final double maxWalk, final double minValue, final double maxValue,
            final Random random) {
        return new AbstractDoubleIterator() {
            double last = startValue;

            final double maxdist = maxWalk + maxWalk;

            final double maxdist2 = maxWalk;

            public double nextDouble() {
                last += random.nextDouble() * maxdist - maxdist2;
                if (last < minValue)
                    last = minValue + (minValue - last);
                if (last > maxValue)
                    last = maxValue - (last - maxValue);
                return last;
            }
        };
    }

    public static AbstractDoubleIterator getDiff2RandomWalk(
            final double startValue, final double maxWalk,
            final double maxDiffWalk, final double minValue,
            final double maxValue, final Random random) {
        return new AbstractDoubleIterator() {
            double last = startValue;

            double lastD = 0.0;

            final double maxdist = maxDiffWalk + maxDiffWalk;

            final double maxdist2 = maxDiffWalk;

            public double nextDouble() {
                lastD += random.nextDouble() * maxdist - maxdist2;
                if (lastD < -maxWalk)
                    lastD = -maxWalk + (-maxWalk - lastD);
                if (lastD > maxWalk)
                    lastD = maxWalk - (lastD - maxWalk);
                last += lastD;
                if (last < minValue) {
                    // last= minValue + (minValue - last);
                    last = minValue;
                    lastD = 0;
                }
                if (last > maxValue) {
                    // last= maxValue - (last - maxValue);
                    last = maxValue;
                    lastD = 0;
                }
                return last;
            }
        };
    }

    public static DoubleIterator getRandomStepWalk(final double startValue,
            final int blockSize, final double maxWalk, final double minValue,
            final double maxValue, final Random random) {
        return new AbstractDoubleIterator() {
            double last = startValue;

            final double maxdist = maxWalk + maxWalk;

            final double maxdist2 = maxWalk;

            int pos = blockSize;

            double[] values = new double[blockSize];

            public void init(int begin, int end, double startValue,
                    double endValue, double maxdist) {
                int mid = begin + ((end - begin) / 2);
                double maxdist2 = maxdist * 0.5;
                double val = startValue + ((endValue - startValue) * 0.5);
                val += random.nextDouble() * maxdist - maxdist2;
                if (val < minValue)
                    val = minValue;
                if (val > maxValue)
                    val = maxValue;
                values[mid] = val;
                if (mid > begin)
                    init(begin, mid - 1, startValue, val, maxdist2);
                if (mid < end)
                    init(mid + 1, end, val, endValue, maxdist2);
            }

            public double nextDouble() {
                if (pos >= blockSize) {
                    double val = last + random.nextDouble() * maxdist
                            - maxdist2;
                    if (val < minValue)
                        val = minValue;
                    if (val > maxValue)
                        val = maxValue;
                    values[blockSize - 1] = val;
                    if (blockSize > 1)
                        init(0, blockSize - 2, last, val, maxdist2);
                    last = val;
                    pos = 0;
                }
                pos++;
                return values[pos - 1];
            }
        };
    }

    public static RealFunction getIteratorWindowFunction(
            final AbstractDoubleIterator dit, final int windowsize) {
        return new RealFunction() {
            long min = 0;

            long max = -1;

            double[] window = new double[windowsize];

            public synchronized double eval(double val) {
                if (max < 0) {
                    for (int i = 0; i < windowsize; i++) {
                        window[i] = dit.nextDouble();
                    }
                    min = 1;
                    max = windowsize;
                }
                if (val < min) {
                    throw new IllegalArgumentException("val:" + val + " < "
                            + min);
                }
                long vu = (long) Math.ceil(val);
                long vd = (long) Math.floor(val);
                if (val > max) {
                    load(vu);
                }
                if (val == vu)
                    return window[(int) (vu - min)];
                else if (val == vd)
                    return window[(int) (vd - min)];
                else
                    return (window[(int) (vd - min)] + (window[(int) (vu - min)] - window[(int) (vd - min)])
                            * (val - vd));
            }

            protected synchronized void load(long newMax) {
                long newMin = newMax - windowsize + 1;
                long pos = max + 1;
                while (pos < newMin) {
                    dit.nextDouble();
                    pos++;
                }
                if (newMin <= max)
                    System.arraycopy(window,
                            (int) (windowsize - (max - newMin + 1)), window, 0,
                            (int) (max - newMin + 1));
                while (pos <= newMax) {
                    window[(int) (pos - newMin)] = dit.nextDouble();
                    pos++;
                }
                min = newMin;
                max = newMax;
            }

        };
    }

    public static AbstractDoubleIterator getErroredFunctionIterator(
            final RealFunction func, final RealFunction vertical,
            final RealFunction horizontal, final long init) {

        return new AbstractDoubleIterator() {
            long nr = init;

            public double nextDouble() {
                nr++;
                // return func.eval(nr + horizontal.eval(nr)) * (1 +
                // vertical.eval(nr));
                return func.eval(nr + horizontal.eval(nr))
                        + (vertical.eval(nr));
            }
        };
    }

    public static AbstractDoubleIterator getFunctionIterator(
            final RealFunction func, final double start, final double step) {
        return new AbstractDoubleIterator() {
            double nr = start;

            public double nextDouble() {
                nr += step;
                return func.eval(nr);
            }
        };
    }

    public static Function getFactory(final int classCount,
            final int streamCount, final int blockSize, final Random random) {
        RealFunction[] func = new RealFunction[classCount];
        for (int i = 0; i < classCount; i++) {
            func[i] = getIteratorWindowFunction(getDiff2RandomWalk(0d, 1.0,
                    0.3, -Double.MAX_VALUE, Double.MAX_VALUE, random), 1600);
        }
        final Iterator[] its = new Iterator[streamCount];
        for (int i = 0; i < streamCount; i++) {
            if (i < classCount) {
                its[i] = getFunctionIterator(func[i % classCount], 300, 1);
            } else {
                RealFunction vert = getIteratorWindowFunction(
                        getDiff2RandomWalk(0d, 1.5, 0.5, -15, +15, random), 100);
                RealFunction hori = getIteratorWindowFunction(
                        getDiff2RandomWalk(0d, 5, 0.5, -30.0, 30.0, random),
                        100);
                its[i] = getErroredFunctionIterator(func[i % classCount], vert,
                        hori, 300);
            }
            its[i] = getBlockIterator((DoubleIterator) its[i], blockSize);
        }
        return new Function() {
            public Object invoke() {
                Object[] obs = new Object[streamCount];
                for (int i = 0; i < streamCount; i++) {
                    obs[i] = its[i].next();
                }
                return obs;
            }
        };

    }

    private SyntheticRealIterators() {
    }

}
