package utils.stocks;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.GregorianCalendar;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;

public class FileStreamDataSource implements StreamData {
    
    protected BufferedReader read=null;
    
    protected int attCount;
    
    protected int clsCount;
    
    protected double[] next;
    
    protected Instances header;
    
    public FileStreamDataSource(String file, int attCount, int clsCount){
        try {
            read=new BufferedReader(new FileReader(file));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        init(attCount,clsCount);
    }
    
    public FileStreamDataSource(BufferedReader read, int attCount, int clsCount){
       this.read=read;
       init(attCount,clsCount);
    }
    
    protected void init(int attCount, int clsCount){
        try {
            read.readLine();
        } catch (IOException e) {
            e.printStackTrace();
        }
        this.attCount=attCount;
        this.clsCount=clsCount;
        
        header = new Instances("Means", getAttsInfo(), 0);
        header.setClassIndex(attCount-1);
    }
    
    protected double[] readNext(){
        try {
            String line=read.readLine();
            if (line==null) return null;
            StringTokenizer toks=new StringTokenizer(line," ");
            String date=toks.nextToken();
            int d=Integer.parseInt(date.substring(0,2));
            int m=Integer.parseInt(date.substring(3,5));
            int y=Integer.parseInt(date.substring(6,10));
            double time=new GregorianCalendar(y,m,d).getTime().getTime();
            
            double[] next=new double[attCount];
            next[0]=time;
            
            for (int i=1;i<attCount;i++){
                next[i]=Double.parseDouble(toks.nextToken());
            }
            return next;
            
        } catch (IOException e) {
            return null;
        } catch (NoSuchElementException e) {
            return null;
        }
    }

    public boolean hasNext() {
        if (next==null) {
            next=readNext();
            if (next==null) return false;
        }
        return true;
    }

    public Instance next() {
        if (next==null && !hasNext()) throw new NoSuchElementException();
        Instance n=new Instance(1.0,next);;
        next=null;
        n.setDataset(header);
        return n;
    }

    public void remove() {
        throw new UnsupportedOperationException();
    }
    
    protected FastVector getAttsInfo() {
        FastVector inf = new FastVector(4);
        Properties prop=new Properties();
        prop.setProperty("range","[0,inf]");
        
        inf.add(new Attribute("Time",new ProtectedProperties(prop)));
        
        for (int i = 1; i < attCount-1; i++) {
            prop.setProperty("range","[-inf,inf]");
            inf.add(new Attribute("X" + (i) + "-val",
                    new ProtectedProperties(prop)));
        }
        FastVector clVal = new FastVector(clsCount);
        clVal.add("Decrease");
        clVal.add("Equal");
        clVal.add("Increase");
//        for (int i = 0; i < clsCount; i++) {
//            clVal.add("Class " + (i + 1));
//        }
        inf.add(new Attribute("class", clVal));
        return inf;
    }
    
    public Instances getInitData(int initDataCount){
        Instances data=new Instances(header);
        for(int i=0;i<initDataCount;i++){
            data.add(next());
        }
        return data;
    }
    
    public Instances dataFormat(){
        return header;
    }

}
