package utils.stocks;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Writer;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class ReadWiWoStocks {

    public static String dir = "D:/data/stocks/";

    protected static DecimalFormat format2 = new DecimalFormat("00",new DecimalFormatSymbols(Locale.US));
    protected static DecimalFormat format1 = new DecimalFormat("#0",new DecimalFormatSymbols(Locale.US));
    protected static DecimalFormat formatP = new DecimalFormat("#####0.00",new DecimalFormatSymbols(Locale.US));

    protected static void getStock(String shorty) {
        try {
            int d = 30;
            int m = 9;
            int y = 2006;

            Writer write = new BufferedWriter(new FileWriter(dir + shorty
                    + ".txt"));

            while (y >= 1990) {
                double[][] data = parseSource(getSource(shorty, d, m, y));
                output(data, write);
                int l = data.length;
                d = (int) data[l - 1][0];
                m = (int) data[l - 1][1];
                y = (int) data[l - 1][2];
                d--;
                if (d <= 0) {
                    d = 31;
                    m--;
                    if (m <= 0) {
                        m = 12;
                        y--;
                    }
                }
            }
            write.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    protected static void output(double[][] data, Writer write) {
        try {
            for (int i = 0; i < data.length; i++) {
                if (data[i][2] >= 1990) {
                    write.write(i + ": " + format2.format(data[i][0]) + "."
                            + format2.format(data[i][1]) + "."
                            + format2.format(data[i][2]) + "  " + data[i][3]
                            + "\n");
                }
            }
            write.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    protected static double[][] parseSource(String source) {
        int s = source.indexOf("Historische Daten");
        int s2;
        if (s < 0)
            return null;
        source = source.substring(s);
        int d, m, y;
        double k;
        List<double[]> data = new ArrayList();
        while ((s = source.indexOf("myl=")) >= 0) {
            source = source.substring(s + 5);
            d = Integer.parseInt(source.substring(0, 2));
            m = Integer.parseInt(source.substring(3, 5));
            y = Integer.parseInt(source.substring(6, 10));
            s = source.indexOf("myl+=");
            s = source.indexOf("myl+=", s + 5);
            s = source.indexOf("myl+=", s + 5);
            s = source.indexOf("myl+=", s + 5);
            s = source.indexOf("\n", s + 5);

            if (s < 0)
                return null;

            s2 = source.indexOf("\n", s + 1);
            k = Double.parseDouble(source.substring(s, s2));
            data.add(new double[] { d, m, y, k });
        }
        int l = data.size();
        double[][] result = new double[l][];
        int i = 0;
        for (double[] v : data) {
            result[i] = v;
            i++;
        }
        return result;
    }

    protected static String getSource(String shorty, int day, int month,
            int year) {
        try {
            URL stockside = new URL(
                    "http://www.wiwo.de/pswiwo/fn/ww2/sfn/buildww/cn/cn_wp1_history/id/91/fl/40/fm/0/SH/0/depot/0/wp1_pat/"
                            + shorty
                            + "/wp1_step/2/wp1_factor/0/wp1_day/"
                            + format2.format(day)
                            + "/wp1_month/"
                            + format2.format(month)
                            + "/wp1_year/"
                            + year
                            + "/wp1_time/22:03/wp1_paging/1/wp1_maxhl/100/index.html");
            System.out.println(stockside);
            InputStreamReader reader = new InputStreamReader(stockside
                    .openStream());
            StringBuffer sb = new StringBuffer();
            try {
                int val = 0;
                char[] cbuf = new char[100];
                while (val >= 0) {
                    val = reader.read(cbuf);
                    if (val >= 0)
                        sb.append(cbuf, 0, val);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            reader.close();
            String side = sb.toString();
            // System.out.println(side);
            return side;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void main(String[] args) {
        String[] shorts = new String[] { "DCX.FSE", "BMW.FSE",
                "GM.NYS", "TOM.FSE", "PEU.FSE", "POR3.FSE", "VOW.FSE" };
        // for (String s : shorts) {
        // getStock(s);
        // }
        combine(shorts);
    }

    public static double[] getValues(String line) {
        int d = Integer.parseInt(line.substring(0, 2));
        int m = Integer.parseInt(line.substring(3, 5));
        int y = Integer.parseInt(line.substring(6, 10));
        double k = Double.parseDouble(line.substring(12));
        return new double[] { d, m, y, k };
    }

    public static void combine(String[] shorts) {
        int n = shorts.length;
        BufferedReader[] read = new BufferedReader[n];
        double[][] values = new double[n][];
        try {

            for (int i = 0; i < n; i++) {
                read[i] = new BufferedReader(new FileReader(dir + shorts[i]
                        + ".txt"));
                String line = read[i].readLine();
                values[i] = getValues(line);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        int minD = Integer.MAX_VALUE;
        int minM = Integer.MAX_VALUE;
        int minY = Integer.MAX_VALUE;
        List<double[]> data = new ArrayList();
        boolean end = false;

        do {
            for (int i = 0; i < n; i++) {
                if (values[i][2] < minY) {
                    minY = (int) values[i][2];
                    minM = (int) values[i][1];
                    minD = (int) values[i][0];
                } else if (values[i][2] == minY) {
                    if (values[i][1] < minM) {
                        minY = (int) values[i][2];
                        minM = (int) values[i][1];
                        minD = (int) values[i][0];
                    } else if (values[i][1] == minM) {
                        if (values[i][0] < minD) {
                            minY = (int) values[i][2];
                            minM = (int) values[i][1];
                            minD = (int) values[i][0];
                        }
                    }
                }
            }
            boolean allEq = true;
            for (int i = 0; i < n; i++) {
                if ((values[i][2] != minY) || (values[i][1] != minM)
                        || (values[i][0] != minD)) {
                    allEq = false;
                }
            }
            if (allEq) {
                double[] dat = new double[n + 3];
                dat[0] = minD;
                dat[1] = minM;
                dat[2] = minY;
                for (int i = 0; i < n; i++) {
                    dat[i + 3] = values[i][3];
                }
                data.add(dat);

                for (int i = 0; i < n; i++) {
                    String line;
                    try {
                        line = read[i].readLine();
                        if (line == null) {
                            end = true;
                            break;
                        }
                        values[i] = getValues(line);
                    } catch (IOException e) {
                        end = true;
                        break;
                    }
                }
            } else {
                for (int i = 0; i < n; i++) {
                    while ((values[i][2] > minY)
                            || (values[i][2] == minY && ((values[i][1] > minM) || (values[i][1] == minM && values[i][0] > minD)))) {
                        String line;
                        try {
                            line = read[i].readLine();
                            if (line == null) {
                                end = true;
                                break;
                            }
                            values[i] = getValues(line);
                        } catch (IOException e) {
                            end = true;
                            break;
                        }
                    }
                    if (end)
                        break;
                }
            }
        } while (!end);

        try {
            Writer write = new BufferedWriter(
                    new FileWriter(dir + "stream.txt"));

            write.write("DATE");
            for (int i = 0; i < n; i++) {
                write.write("  " + shorts[i]);
            }
            write.write("\n");

            int c = data.size();
            double[] old=data.get(c-1);
            for (int p = c - 2; p >= 0; p--) {
                double[] dat = data.get(p);
                write
                        .write(format2.format(dat[0]) + "."
                                + format2.format(dat[1]) + "."
                                + format2.format(dat[2]));
                for (int i = 0; i < n; i++) {
                    if (dat[3 + i]<0)dat[3 + i]=old[3+i];
                    double val=100*((dat[3 + i]-old[3+i])/old[3+i]);
                    if (i==n-1){
                        if (val<-0.5) val=0;
                        else if (val<0.5) val=1;
                        else val=2;
                        write.write("  " + format1.format(val));
                    }
                    else write.write("  " + formatP.format(val));
                }
                write.write("\n");
                old=dat;
            }
            write.flush();
            write.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

}
