/*
 * Created on 07.07.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package utils.visibility;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import utils.image.JpegEncoder;

/**
 * GUI-Element zum Zeichnen. Zur Verfgung steht ein 2 dimensionales Feld mit
 * reelen Koordinaten (Ursprung ist links unten). Dieses wird auf die Grsse des Gui-Elementes projeziert.
 * 
 * Objekte knnen wie jedes andere von java.awt.Component erbende Objekt in
 * einen GUI-Container (wie Panel, Frame...) mittels add() eingefgt werden.
 */
public class DrawField2D extends Canvas {

    static public final int POS_CENTER = 0;

    static public final int POS_LEFT_UP = 1;

    static public final int POS_LEFT = 2;

    static public final int POS_LEFT_DOWN = 3;

    static public final int POS_DOWN = 4;

    static public final int POS_RIGHT_DOWN = 5;

    static public final int POS_RIGHT = 6;

    static public final int POS_RIGHT_UP = 7;

    static public final int POS_UP = 8;

    static public final int POINT = 0;

    static public final int CROSS = 1;

    static public final int CIRCLE = 2;

    static public final int RECTANGLE = 3;

    protected double minX, maxX, minY, maxY;

    protected int sizeX, sizeY;

    protected int inX, inY;

    protected Image image;

    protected Graphics ig;

    protected Color back = Color.WHITE;

    public int pointWidth = 10;

    /**
     * 
     * @param sizeX
     *            Breite der Zeichnung
     * @param sizeY
     *            Hhe der Zeichnung
     */
    public DrawField2D(int sizeX, int sizeY) {
        this.sizeX = sizeX;
        this.sizeY = sizeY;
        inX = sizeX - 1;
        inY = sizeY - 1;
        setSize(sizeX, sizeY);
        image = new BufferedImage(sizeX, sizeY, BufferedImage.TYPE_3BYTE_BGR);
        ig = image.getGraphics();
        clear();
        this.addComponentListener(new ComponentAdapter() {

            public void componentResized(ComponentEvent e) {
                resize();
                redraw();
            }
        });
    }

    /**
     * 
     * @param sizeX
     *            Breite der Zeichnung
     * @param sizeY
     *            Hhe der Zeichnung
     * @param minX
     *            X Bereich (min)
     * @param maxX
     *            X Bereich (max)
     * @param minY
     *            Y Bereich (min)
     * @param maxY
     *            Y Bereich (max)
     */
    public DrawField2D(int sizeX, int sizeY, double minX, double maxX,
            double minY, double maxY) {
        this.sizeX = sizeX;
        this.sizeY = sizeY;
        inX = sizeX - 1;
        inY = sizeY - 1;
        setSize(sizeX, sizeY);
        image = new BufferedImage(sizeX, sizeY, BufferedImage.TYPE_3BYTE_BGR);
        ig = image.getGraphics();
        clear();
        setMinMaxX(minX, maxX);
        setMinMaxY(minY, maxY);
        this.addComponentListener(new ComponentAdapter() {

            public void componentResized(ComponentEvent e) {
                resize();
                redraw();
            }
        });
    }

    /**
     * Inhalt lschen.
     * 
     */
    public void clear() {
        sizeX = getWidth();
        sizeY = getHeight();
        inX = sizeX - 1;
        inY = sizeY - 1;
        ig.setColor(back);
        ig.fillRect(0, 0, sizeX, sizeY);
    }

    /**
     * Inhalt im Bereich lschen.
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     */
    public void clear(double x1, double y1, double x2, double y2) {
        sizeX = getWidth();
        sizeY = getHeight();
        inX = sizeX - 1;
        inY = sizeY - 1;
        int xx1 = (int) ((inX * (x1 - minX) / (maxX - minX)) + 0.5);
        int yy1 = inY - (int) ((inY * (y1 - minY) / (maxY - minY)) + 0.5);
        int xx2 = (int) ((inX * (x2 - minX) / (maxX - minX)) + 0.5);
        int yy2 = inY - (int) ((inY * (y2 - minY) / (maxY - minY)) + 0.5);
        ig.setColor(back);
        ig.fillRect(xx1, yy2, xx2 - xx1 + 1, yy1 - yy2 + 1);
    }

    /**
     * Ausschnitt innerhalb des Objektes verschieben
     * 
     * @param sx1
     * @param sy1
     * @param sx2
     * @param sy2
     * @param tx1
     * @param ty1
     * @param tx2
     * @param ty2
     */
    public void moveClip(double sx1, double sy1, double sx2, double sy2,
            double tx1, double ty1, double tx2, double ty2) {
        sizeX = getWidth();
        sizeY = getHeight();
        inX = sizeX - 1;
        inY = sizeY - 1;
        int sxx1 = (int) ((inX * (sx1 - minX) / (maxX - minX)) + 0.5);
        int syy1 = inY - (int) ((inY * (sy1 - minY) / (maxY - minY)) + 0.5);
        int sxx2 = (int) ((inX * (sx2 - minX) / (maxX - minX)) + 0.5);
        int syy2 = inY - (int) ((inY * (sy2 - minY) / (maxY - minY)) + 0.5);
        int txx1 = (int) ((inX * (tx1 - minX) / (maxX - minX)) + 0.5);
        int tyy1 = inY - (int) ((inY * (ty1 - minY) / (maxY - minY)) + 0.5);
        int txx2 = 0, tyy2 = 0;
        if (Math.abs((tx2 - tx1) - (sx2 - sx1)) < 0.00001)
            txx2 = txx1 + (sxx2 - sxx1);
        else
            txx2 = (int) (inX * (tx2 - minX) / (maxX - minX));
        if (Math.abs((ty2 - ty1) - (sy2 - sy1)) < 0.00001)
            tyy2 = tyy1 + (syy2 - syy1);
        else
            tyy2 = sizeY - (int) (inY * (ty2 - minY) / (maxY - minY));
        ig.drawImage(image, txx1, tyy2, txx2, tyy1, sxx1, syy2, sxx2, syy1,
                this);
    }

    /**
     * reelle Grenzen setzen fr die X-Koordinate
     * 
     * @param minX
     * @param maxX
     */
    public void setMinMaxX(double minX, double maxX) {
        this.minX = minX;
        this.maxX = maxX;
    }

    /**
     * reelle Grenzen setzen fr die Y-Koordinate
     * 
     * @param minY
     * @param maxY
     */
    public void setMinMaxY(double minY, double maxY) {
        this.minY = minY;
        this.maxY = maxY;
    }

    public double getMinX() {
        return minX;
    }

    public double getMaxX() {
        return maxX;
    }

    public double getMinY() {
        return minY;
    }

    public double getMaxY() {
        return maxY;
    }

    /**
     * Text zeichnen
     * 
     * @param text
     * @param x
     * @param y
     * @param col
     * @param font
     */
    public void drawString(String text, double x, double y, Color col, Font font) {
        int xx = (int) ((inX * (x - minX) / (maxX - minX)) + 0.5);
        int yy = inY - (int) ((inY * (y - minY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        ig.setFont(font);
        int width = ig.getFontMetrics().stringWidth(text);
        width /= 2;
        ig.drawString(text, xx - width, yy);
    }

    /**
     * Text zeichnen
     * 
     * @param text
     * @param x
     * @param y
     * @param pos
     * @param col
     * @param font
     */
    public void drawString(String text, double x, double y, int pos, Color col,
            Font font) {
        int xx = (int) ((inX * (x - minX) / (maxX - minX)) + 0.5);
        int yy = inY - (int) ((inY * (y - minY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        ig.setFont(font);
        FontMetrics fm = ig.getFontMetrics();
        int width = fm.stringWidth(text);
        int desc = fm.getDescent();
        int asc = fm.getAscent();
        if (pos == POS_CENTER) {
            ig.drawString(text, xx - width / 2, yy + asc - (desc + asc) / 2);
        } else if (pos == POS_LEFT) {
            ig.drawString(text, xx, yy + asc - (desc + asc) / 2);
        } else if (pos == POS_RIGHT) {
            ig.drawString(text, xx - width, yy + asc - (desc + asc) / 2);
        } else if (pos == POS_UP) {
            ig.drawString(text, xx - width / 2, yy - desc);
        } else if (pos == POS_DOWN) {
            ig.drawString(text, xx - width / 2, yy + asc);
        } else if (pos == POS_LEFT_UP) {
            ig.drawString(text, xx, yy - desc);
        } else if (pos == POS_LEFT_DOWN) {
            ig.drawString(text, xx, yy + asc);
        } else if (pos == POS_RIGHT_UP) {
            ig.drawString(text, xx - width, yy - desc);
        } else if (pos == POS_RIGHT_DOWN) {
            ig.drawString(text, xx - width, yy + asc);
        }
    }

    /**
     * Punkt zeichnen.
     * 
     * @param x
     * @param y
     * @param col
     */
    public void drawPoint(double x, double y, Color col) {
        drawPoint(x, y, col, pointWidth);
    }

    /**
     * Dicken Punkt zeichnen.
     * 
     * @param x
     * @param y
     * @param col
     * @param pointWidth
     */
    public void drawPoint(double x, double y, Color col, int pointWidth) {
        int del = pointWidth / 2;
        int xx = (int) ((inX * (x - minX) / (maxX - minX)) + 0.5) - del;
        int yy = inY - (int) ((inY * (y - minY) / (maxY - minY)) + 0.5) - del;
        ig.setColor(col);
        ig.fillOval(xx, yy, pointWidth, pointWidth);
        // ig.setColor(Color.BLACK);
        // ig.drawOval(xx,yy,pointWidth,pointWidth);
    }

    public void markPoint(double x, double y, Color col, int pixelWidth,
            int type) {
        int del = pixelWidth / 2;
        int xx = (int) ((inX * (x - minX) / (maxX - minX)) + 0.5) - del;
        int yy = inY - (int) ((inY * (y - minY) / (maxY - minY)) + 0.5) - del;
        ig.setColor(col);
        switch (type) {
        case CROSS:
            ig.drawLine(xx + 1, yy, xx + pixelWidth, yy + pixelWidth - 1);
            ig.drawLine(xx, yy, xx + pixelWidth, yy + pixelWidth);
            ig.drawLine(xx, yy + 1, xx + pixelWidth - 1, yy + pixelWidth);
            ig.drawLine(xx + pixelWidth - 1, yy, xx, yy + pixelWidth - 1);
            ig.drawLine(xx + pixelWidth, yy + 1, xx + 1, yy + pixelWidth);
            ig.drawLine(xx + pixelWidth, yy, xx, yy + pixelWidth);
            break;
        case RECTANGLE:
            ig.drawRect(xx, yy, pixelWidth, pixelWidth);
            break;
        case CIRCLE:
            ig.drawOval(xx, yy, pixelWidth, pixelWidth);
            break;
        default:
            ig.fillOval(xx, yy, pixelWidth, pixelWidth);
        }
    }

    /**
     * Kreuz zeichnen.
     * 
     * @param x
     * @param y
     * @param col
     * @param pointWidth
     */
    public void drawCross(double x, double y, Color col, int pointWidth) {
        int del = pointWidth / 2;
        int xx = (int) ((inX * (x - minX) / (maxX - minX)) + 0.5) - del;
        int yy = inY - (int) ((inY * (y - minY) / (maxY - minY)) + 0.5) - del;
        ig.setColor(col);
        ig.drawLine(xx + 1, yy, xx + pointWidth, yy + pointWidth - 1);
        ig.drawLine(xx, yy, xx + pointWidth, yy + pointWidth);
        ig.drawLine(xx, yy + 1, xx + pointWidth - 1, yy + pointWidth);
        ig.drawLine(xx + pointWidth - 1, yy, xx, yy + pointWidth - 1);
        ig.drawLine(xx + pointWidth, yy + 1, xx + 1, yy + pointWidth);
        ig.drawLine(xx + pointWidth, yy, xx, yy + pointWidth);
        // ig.fillOval(xx,yy,pointWidth,pointWidth);
        // ig.setColor(Color.BLACK);
        // ig.drawOval(xx,yy,pointWidth,pointWidth);
    }

    /**
     * Linie zeichnen.
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param col
     */
    public void drawLine(double x1, double y1, double x2, double y2, Color col) {
        int xx1 = (int) ((inX * (x1 - minX) / (maxX - minX)) + 0.5);
        int yy1 = inY - (int) ((inY * (y1 - minY) / (maxY - minY)) + 0.5);
        int xx2 = (int) ((inX * (x2 - minX) / (maxX - minX)) + 0.5);
        int yy2 = inY - (int) ((inY * (y2 - minY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        ig.drawLine(xx1, yy1, xx2, yy2);
    }

    /**
     * Dicke Linie zeichnen.
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param col
     * @param widthX
     * @param widthY
     */
    public void drawLine(double x1, double y1, double x2, double y2, Color col,
            int widthX, int widthY) {
        int xx1 = (int) ((inX * (x1 - minX) / (maxX - minX)) + 0.5);
        int yy1 = inY - (int) ((inY * (y1 - minY) / (maxY - minY)) + 0.5);
        int xx2 = (int) ((inX * (x2 - minX) / (maxX - minX)) + 0.5);
        int yy2 = inY - (int) ((inY * (y2 - minY) / (maxY - minY)) + 0.5);
        int posY = (widthY / 2) ;
        int posX = (widthX / 2) ;
        ig.setColor(col);
        
        if (widthX*widthY>4){
        
        int dir=-1;
        if (xx2>=xx1 && yy2>=yy1) dir=0;
        else if (xx2>=xx1 && yy2<yy1) dir=1;
        else if (xx2<xx1 && yy2>=yy1) dir=2;
        else if (xx2<xx1 && yy2<yy1) dir=3;
        
        int[] xpoints=new int[6];
        int[] ypoints=new int[6];
        
        if (dir==0){
            xpoints[0]=xx1+posX;
            ypoints[0]=yy1-posY;
            
            xpoints[1]=xx1-posX;
            ypoints[1]=yy1-posY;
            
            xpoints[2]=xx1-posX;
            ypoints[2]=yy1+posY;
            
            xpoints[3]=xx2-posX;
            ypoints[3]=yy2+posY;
            
            xpoints[4]=xx2+posX;
            ypoints[4]=yy2+posY;
            
            xpoints[5]=xx2+posX;
            ypoints[5]=yy2-posY;
        }else if (dir==1){
            xpoints[0]=xx1-posX;
            ypoints[0]=yy1-posY;
            
            xpoints[1]=xx1-posX;
            ypoints[1]=yy1+posY;
            
            xpoints[2]=xx1+posX;
            ypoints[2]=yy1+posY;
            
            xpoints[3]=xx2+posX;
            ypoints[3]=yy2+posY;
            
            xpoints[4]=xx2+posX;
            ypoints[4]=yy2-posY;
            
            xpoints[5]=xx2-posX;
            ypoints[5]=yy2-posY;
        }else if (dir==2){
            xpoints[0]=xx1+posX;
            ypoints[0]=yy1+posY;
            
            xpoints[1]=xx1+posX;
            ypoints[1]=yy1-posY;
            
            xpoints[2]=xx1-posX;
            ypoints[2]=yy1-posY;
            
            xpoints[3]=xx2-posX;
            ypoints[3]=yy2-posY;
            
            xpoints[4]=xx2-posX;
            ypoints[4]=yy2+posY;
            
            xpoints[5]=xx2+posX;
            ypoints[5]=yy2+posY;
        }else if (dir==3){
            xpoints[0]=xx1-posX;
            ypoints[0]=yy1+posY;
            
            xpoints[1]=xx1+posX;
            ypoints[1]=yy1+posY;
            
            xpoints[2]=xx1+posX;
            ypoints[2]=yy1-posY;
            
            xpoints[3]=xx2+posX;
            ypoints[3]=yy2-posY;
            
            xpoints[4]=xx2-posX;
            ypoints[4]=yy2-posY;
            
            xpoints[5]=xx2-posX;
            ypoints[5]=yy2+posY;
        }
        
        ig.fillPolygon(xpoints,ypoints,6);
        }
        else{
        for (int x = 0; x < widthX; x++) {
                for (int y = 0; y < widthY; y++) {
                    ig.drawLine(xx1 + posX + x, yy1 + posY + y, xx2 + posX + x,
                            yy2 + posY + y);
                }
            }
        }
    }

    /**
     * Rechteck zeichnen.
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param col
     */
    public void drawRect(double x1, double y1, double x2, double y2, Color col) {
        int xx1 = (int) ((inX * (x1 - minX) / (maxX - minX)) + 0.5);
        int yy1 = inY - (int) ((inY * (y1 - minY) / (maxY - minY)) + 0.5);
        int xx2 = (int) ((inX * (x2 - minX) / (maxX - minX)) + 0.5);
        int yy2 = inY - (int) ((inY * (y2 - minY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        ig.drawRect(xx1, yy2, (xx2 - xx1), (yy1 - yy2));
    }

    /**
     * Rechteck zeichnen.
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param col
     */
    public void drawRect(double x1, double y1, double x2, double y2,
            int linewidth, Color col) {
        int xx1 = (int) ((inX * (x1 - minX) / (maxX - minX)) + 0.5);
        int yy1 = inY - (int) ((inY * (y1 - minY) / (maxY - minY)) + 0.5);
        int xx2 = (int) ((inX * (x2 - minX) / (maxX - minX)) + 0.5);
        int yy2 = inY - (int) ((inY * (y2 - minY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        for (int i = 0; i < linewidth; i++)
            ig.drawRect(xx1 + i, yy2 + i, (xx2 - xx1) - 2 * i, (yy1 - yy2) - 2
                    * i);
    }

    /**
     * Rechteck fllen.
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param col
     */
    public void fillRect(double x1, double y1, double x2, double y2, Color col) {
        int xx1 = (int) ((inX * (x1 - minX) / (maxX - minX)) + 0.5);
        int yy1 = inY - (int) ((inY * (y1 - minY) / (maxY - minY)) + 0.5);
        int xx2 = (int) ((inX * (x2 - minX) / (maxX - minX)) + 0.5);
        int yy2 = inY - (int) ((inY * (y2 - minY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        ig.fillRect(xx1, yy2, (xx2 - xx1), (yy1 - yy2));
    }

    /**
     * Zeichnet eine Ellipse mit Mittelpunkt und Radien.
     * 
     * @param mx
     *            x-Mittelpunkt
     * @param my
     *            y-Mittelpunkt
     * @param radX
     *            x-Radius
     * @param radY
     *            y-Radius
     * @param col
     *            Farbe
     */
    public void drawEllipse(double mx, double my, double radX, double radY,
            Color col) {
        int mxx = (int) ((inX * (mx - minX) / (maxX - minX)) + 0.5);
        int myy = inY - (int) ((inY * (my - minY) / (maxY - minY)) + 0.5);
        int rxx = (int) ((inX * (radX) / (maxX - minX)) + 0.5);
        int ryy = (int) ((inY * (radY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        ig.drawOval(mxx - rxx, myy - ryy, rxx * 2, ryy * 2);
    }

    /**
     * Zeichnet eine ausgefllte Ellipse mit Mittelpunkt und Radien.
     * 
     * @param mx
     *            x-Mittelpunkt
     * @param my
     *            y-Mittelpunkt
     * @param radX
     *            x-Radius
     * @param radY
     *            y-Radius
     * @param col
     *            Farbe
     */
    public void fillEllipse(double mx, double my, double radX, double radY,
            Color col) {
        int mxx = (int) ((inX * (mx - minX) / (maxX - minX)) + 0.5);
        int myy = inY - (int) ((inY * (my - minY) / (maxY - minY)) + 0.5);
        int rxx = (int) ((inX * (radX) / (maxX - minX)) + 0.5);
        int ryy = (int) ((inY * (radY) / (maxY - minY)) + 0.5);
        ig.setColor(col);
        ig.fillOval(mxx - rxx, myy - ryy, rxx * 2, ryy * 2);
    }

    /**
     * Zeichnet ein Bild an die Position
     * 
     * @param image
     * @param x
     * @param y
     */
    public void drawImage(Image image, double x, double y) {
        int xx = (int) ((inX * (x - minX) / (maxX - minX)) + 0.5);
        int yy = inY - (int) ((inY * (y - minY) / (maxY - minY)) + 0.5);
        ig.drawImage(image, xx, yy, this);
    }

    /**
     * Zeichnet ein Bild in den angegebenen Bereich
     * 
     * @param image
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     */
    public void drawImage(Image image, double x1, double y1, double x2,
            double y2) {
        int xx1 = (int) ((inX * (x1 - minX) / (maxX - minX)) + 0.5);
        int yy1 = inY - (int) ((inY * (y1 - minY) / (maxY - minY)) + 0.5);
        int xx2 = (int) ((inX * (x2 - minX) / (maxX - minX)) + 0.5);
        int yy2 = inY - (int) ((inY * (y2 - minY) / (maxY - minY)) + 0.5);
        ig.drawImage(image, xx1, yy1, xx2 - xx1, yy2 - yy1, this);
    }

    /**
     * Zeichnung als JPG-Bild abspeichern.
     * 
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param col
     */
    public void saveImage(OutputStream out) {
        JpegEncoder jp = new JpegEncoder(image, 100, out);
        jp.compress();
        try {
            out.flush();
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
        g.drawImage(image, 0, 0, image.getWidth(null), image.getHeight(null),
                0, 0, sizeX, sizeY, this);
    }

    /**
     * Grsse anpassen.
     */
    public void resize() {
        sizeX = getWidth();
        sizeY = getHeight();
        inX = sizeX - 1;
        inY = sizeY - 1;
        if ((inX > 0) && (inY > 0)) {
            image = new BufferedImage(sizeX, sizeY,
                    BufferedImage.TYPE_3BYTE_BGR);
            ig = image.getGraphics();
            clear();
        }
    }

    public Dimension getMinimumSize() {
        return new Dimension(sizeX, sizeY);
    }

    public Dimension getPreferedSize() {
        return new Dimension(sizeX, sizeY);
    }

    public void redraw() {
    }
    
    public JFrame createFrame(String name) {
        final JFrame frame= new JFrame(name);
        
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                frame.setVisible(false);
                stop();
                frame.dispose();
            }
        });
        JButton but=new JButton("redraw");
        but.addActionListener(new ActionListener(){

            public void actionPerformed(ActionEvent e) {
                redraw();
            }
            
        });
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        JPanel pan = new JPanel();
        pan.setLayout(new BorderLayout());
        pan.add(this, "Center");
        pan.add(but, "South");
        frame.setContentPane(pan);
        frame.setSize(getWidth()+20, getHeight()+60);
        return frame;
    }
    
    public ScrollPane getScrollPane() {
        ScrollPane sp= new ScrollPane();
        sp.add(this);
        return sp;
    }
    

    public void stop(){
        
    }

    /*
     * public Dimension getSize() { return new Dimension(sizeX, sizeY); }
     * 
     * 
     * 
     * public int getHeight(){ return sizeY; }
     * 
     * public int getWidth(){ return sizeX; }
     * 
     * public Dimension getMaximumSize(){ return new Dimension(sizeX,sizeY); }
     */
}