/*
 * Created on 22.07.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package utils.visibility;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DecimalFormat;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * @author beringer
 */
public class StreamVisualizer extends DrawField2D {
    
    protected static DecimalFormat[] formats;

    protected int width;

    protected int height;

    protected int length;

    protected double min;

    protected double max;

    protected boolean autoMinMax;

    protected double minDiff = 0.0;

    protected double border = 0.05;

    protected double resize = 1.0;

    protected boolean text=true;

    protected double[] values;

    protected boolean started = false;

    protected DecimalFormat df =formats[3];

    protected Font font = new Font("Dialog", Font.BOLD, 10);
    
    static{
        formats=new DecimalFormat[11];
        formats[0]=new DecimalFormat("0");
        String format="0.";
        for (int i=1;i<=10;i++){
            format=format+"0";
            formats[i]=new DecimalFormat(format);
        }
    }

    public StreamVisualizer(int length, int width, int height) {
        this(length, width, height, 0, 0.00001, true);
    }

    public StreamVisualizer(int length, int width, int height, double min,
            double max) {
        this(length, width, height, min, max, false);
    }

    public StreamVisualizer(int length, int width, int height, double min,
            double max, boolean autoMinMax) {
        super(width, height);
        this.width = width;
        this.height = height;
        this.length = length;
        if (max<=min) max=min+0.000001;
        this.min = min;
        this.max = max;
        
        this.autoMinMax = autoMinMax;

        values = new double[length];
        setMinMaxX(0, length - 1);
        setMinMaxY(min, max);
        setAutoFormat();
        redraw();
        repaint(50);
    }

    public synchronized void resize(int length) {
        if (length != this.length) {
            double[] newVal = new double[length];
            int coplen = Math.min(length, this.length);
            System.arraycopy(values, this.length - coplen, newVal, length
                    - coplen, coplen);
            values = newVal;
            this.length = length;
            setMinMaxX(0, length - 1);
            redraw();
            repaint(50);
        }
    }

    public synchronized void setZero() {
        for (int i = 0; i < length; i++) {
            values[i] = 0.0;
        }
    }

    public Frame createFrame(String name) {
        final JFrame frame = new JFrame(name);
        frame.setSize(getWidth() + 40, getHeight() + 40);
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                frame.setVisible(false);
                frame.dispose();
            }
        });
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        JPanel pan = new JPanel();
        pan.setLayout(new BorderLayout());
        pan.add(this, "Center");
        frame.setContentPane(pan);
        frame.setVisible(true);
        return frame;
    }

    public void add(double[] arr) {
        if (!started) {
            setAll(arr[0]);
            started = true;
        }
        int newlen = arr.length;
        if (newlen >= length) {
            System.arraycopy(arr, newlen - length, values, 0, length);
        } else {
            System.arraycopy(values, newlen, values, 0, length - newlen);
            System.arraycopy(arr, 0, values, length - newlen, newlen);
        }
        redraw();
        repaint(50);
    }

    public void add(double value) {
        if (!started) {
            setAll(value);
            started = true;
        }
        System.arraycopy(values, 1, values, 0, length - 1);
        values[length - 1] = value;
        redraw();
        repaint(50);
    }

    public void setAll(double value) {
        for (int i = 0; i < length; i++) {
            values[i] = value;
        }
    }

    public void setMinMax(double min, double max) {
        this.min = min;
        this.max = max;
        setMinMaxY(min, max);
        setAutoFormat();
    }

    public void setAutoMinMax(double minDiff, double border, double resize,
            boolean text) {
        autoMinMax = true;
        this.minDiff = minDiff;
        this.border = border;
        this.resize = resize;
        this.text = text;
    }

    public void setMinMax() {
        boolean doResize = false;
        double newMin = values[0];
        double newMax = values[0];
        for (int i = 1; i < length; i++) {
            if (newMin > values[i])
                newMin = values[i];
            if (newMax < values[i])
                newMax = values[i];
        }

        if (newMin < min)
            doResize = true;
        else if (newMax > max)
            doResize = true;
        else if (((max - min) > (minDiff * 1.001))
                && (((newMax - newMin) / (max - min)) < resize))
            doResize = true;
        else
            return;

        double delta = (newMax - newMin) * border;
        newMax += delta;
        newMin -= delta;
        if ((newMax - newMin) < minDiff) {
            delta = (minDiff - (newMax - newMin)) / 2;
            newMax += delta;
            newMin = newMax - minDiff;
        }
        min = newMin;
        max = newMax;
        setMinMaxY(min, max);
        setAutoFormat();
    }
    
    public void setAutoFormat(){
        double diff=max-min;
        diff/=50.0;
        int dig=-(int)Math.floor(Math.log10(diff));
        if (dig>10) dig=10;
        else if (dig<0) dig=0;
        df=formats[dig];
    }

    public void redraw() {
        if (autoMinMax)
            setMinMax();
        clear();
        for (int i = 0; i < length - 1; i++) {
            drawLine(i, values[i], i + 1, values[i + 1], Color.RED);
        }
        if (text) {
            drawString(df.format(min), 0d, min, DrawField2D.POS_LEFT_UP,
                    Color.BLACK, font);
            drawString(df.format(max), 0d, max, DrawField2D.POS_LEFT_DOWN,
                    Color.BLACK, font);
            drawString(df.format(values[length-1]), length-1, 0.5*(max+min), DrawField2D.POS_RIGHT,
                    Color.BLUE, font);
        }
    }

}
