package utils.visibility;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import clustering.FuzzyKMeansState;
import clustering.KMeansState;

public class View2DCluster {
    
    public static Color[] colors=new Color[]{Color.RED, Color.BLUE,Color.GREEN, Color.YELLOW, Color.CYAN, Color.MAGENTA, Color.PINK, Color.ORANGE, new Color(127,255,255),new Color(255,127,255), new Color(255,255,127), Color.BLACK};;
    
    KMeansState state;
    FuzzyKMeansState fstate;
    protected boolean fuzzy=false;
    DrawField2D draw;
    double minX;
    double maxX;
    double minY;
    double maxY;
    
    public View2DCluster(KMeansState state, int width, int height,double minX, double maxX, double minY, double maxY, String title){
        this.state=state;
        fuzzy=false;
        this.minX=minX;
        this.maxX=maxX;
        this.minY=minY;
        this.maxY=maxY;
        
        if (state instanceof FuzzyKMeansState){
            fstate=(FuzzyKMeansState)state;
            fuzzy=true;
        }
        
        final JFrame frame=new JFrame("View 2D Clustering: "+title);
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                frame.setVisible(false);
                frame.dispose();
            }
        });
        frame.setSize(width,height);
        frame.setVisible(true);
        view(frame.getContentPane());
        draw();
        
    }
    
    public DrawField2D getDraw(){
        return draw;
    }
    
   
    
    public void view(final Container con) {

        draw = new DrawField2D(con.getWidth(), con.getHeight()) {
            
            public void redraw() {
                draw();
            }
        };
        draw.setMinMaxX(minX, maxX);
        draw.setMinMaxY(minY, maxY);
        con.add(draw);
    }
    
   
    
    public void draw(){
        int n=state.dataCount;
        int k=state.clusterCount;
        for (int i=0;i<n;i++){
            Color col=null;
            if (!fuzzy) col=colors[state.cluster[i]];
            else{
                double r=0;
                double g=0;
                double b=0;
                for(int j=0;j<k;j++){
                    r+=colors[j%12].getRed()*fstate.fuzzyCluster[i][j];
                    g+=colors[j%12].getGreen()*fstate.fuzzyCluster[i][j];
                    b+=colors[j%12].getBlue()*fstate.fuzzyCluster[i][j];
                }
                
                col=new Color((int)r,(int)g,(int)b);
            }
            double[] p=(double[]) state.data[i];
            draw.drawPoint(p[0],p[1],col,8);
        }
        for(int j=0;j<k;j++){
            double[] p=(double[]) state.kMeans[j];
            draw.drawCross(p[0],p[1],colors[j%12].darker(),8);
        }
    }
}
