package utils.visibility.clustering;

import java.awt.Color;
import java.awt.Font;

import utils.math.DFT;
import utils.visibility.DrawField2D;
import clustering.FuzzyKMeansState;

public class VisualizeFKM extends DrawField2D {

    int width;

    int height;

    int[] select;

    String[] names;

    FuzzyKMeansState state;

    int w;

    int h;

    int k;

    int maxy = 1;

    int miny = -1;
    
    protected Font font = new Font("Dialog", Font.PLAIN, 10);

    public VisualizeFKM(FuzzyKMeansState ostate, String[] names, int[] select, int maxClusterCount,
            int width, int height) {
        super(width, height);

        this.width = width;
        this.height = height;
        this.select = select;
        this.names = names;

        w = width / (maxClusterCount + 1);
        h = height / (select.length + 1);
        k = w / 40;
        state = inversDFT(ostate, 64, 2048);

            
    }
    
    public void setDFTState(FuzzyKMeansState ostate){
        state = inversDFT(ostate, 64, 2048);
        redraw();
        this.repaint(50);
    }

    public void redraw() {
        this.setMinMaxX(0, width-1);
        this.setMinMaxY(0, height-1);
        this.fillRect(0,0,width-1,height-1,Color.WHITE);
        for (int c = 0; c < state.clusterCount; c++) {
            this.drawRect((c + 1) * w + k, height - h + k, (c + 1) * w + w - k,
                    height - k, Color.BLACK);
            drawFunction(this, (double[]) (state.kMeans[c]), (c + 1) * w + k,
                    height - h + k, w - 2 * k, h - 2 * k, miny, maxy,
                    Color.BLUE);
        }
        for (int i = 0; i < select.length; i++) {
            int p = select[i];
            this.drawString(names[p], k, height - (i + 1) * h - (h / 2),
                    this.POS_LEFT, Color.RED, font);
            this.drawRect(k, height - (i + 2) * h + k, w - k, height - (i + 1)
                    * h - k, Color.BLACK);
            drawFunction(this, (double[]) (state.data[p]), k, height - (i + 2)
                    * h + k, w - 2 * k, h - 2 * k, miny, maxy, Color.BLUE);
            for (int c = 0; c < state.clusterCount; c++) {
                float v = (float) (1 - state.fuzzyCluster[p][c]);
                v = v * v;
                Color col = new Color(v, v, v);
                this.fillRect((c + 1) * w + w / 4,
                        height - (i + 2) * h + h / 4, (c + 2) * w - w / 4,
                        height - (i + 1) * h - h / 4, col);
                this.drawRect((c + 1) * w + w / 4,
                        height - (i + 2) * h + h / 4, (c + 2) * w - w / 4,
                        height - (i + 1) * h - h / 4, Color.BLACK);
            }
        }
    }

    public static void drawFunction(DrawField2D draw, double[] values,
            double x, double y, double w, double h, double min, double max,
            Color color) {
        int len = values.length;
        double lx = x;
        double v = values[0];
        if (v > max)
            v = max;
        if (v < min)
            v = min;
        double ly = y + (h * (v - min) / (max - min));
        for (int i = 1; i < len; i++) {
            double xx = x + ((w * i) / (len - 1));
            v = values[i];
            if (v > max)
                v = max;
            if (v < min)
                v = min;
            double yy = y + (h * (v - min) / (max - min));

            draw.drawLine(lx, ly, xx, yy, color);

            lx = xx;
            ly = yy;
        }
    }

    public static FuzzyKMeansState inversDFT(FuzzyKMeansState state, int olen,
            int wlen) {

        double fakt = olen / (double) wlen;

        FuzzyKMeansState nstate = (FuzzyKMeansState) state.clone();
        double[][] ndata = new double[state.dataCount][olen];

        double[][] dft = new double[olen][2];
        int dftCount = ((double[][]) state.data[0]).length;
        for (int i = 0; i < state.dataCount; i++) {
            double[][] odata = (double[][]) state.data[i];
            for (int m = dftCount + 1; m < olen - dftCount; m++) {
                dft[m][0] = 0;
                dft[m][1] = 0;
            }
            for (int m = 1; m <= dftCount; m++) {
                dft[m][0] = odata[m - 1][0];
                dft[m][1] = odata[m - 1][1];
                dft[olen - m][0] = odata[m - 1][0];
                dft[olen - m][1] = -odata[m - 1][1];
            }
            dft[0][0] = 0;
            dft[0][1] = 0;

            DFT.fft(dft);

            for (int m = 1; m < olen; m++) {
                double val = Math.sqrt(dft[olen - m][0] * dft[olen - m][0]
                        + dft[olen - m][1] * dft[olen - m][1]);
                if (dft[olen - m][0] < 0.0)
                    val = -val;
                // val+=mid[i];
                ndata[i][m] = (double) val * fakt;
            }
            ndata[i][0] = dft[0][0] * fakt;
        }
        nstate.data = ndata;

        double[][] kMeans = new double[nstate.clusterCount][olen];

        for (int c = 0; c < nstate.clusterCount; c++) {
            double sum = 0;
            for (int i = 0; i < nstate.dataCount; i++) {
                double m = nstate.fuzzyCluster[i][c];
                m = m * m;
                sum += m;
                for (int p = 0; p < olen; p++) {
                    kMeans[c][p] += ndata[i][p] * m;
                }
            }
            for (int p = 0; p < olen; p++) {
                kMeans[c][p] /= sum;
            }
        }

        nstate.kMeans = kMeans;

        return nstate;
    }
}
