package utils.visibility.clustering;

import java.awt.Color;
import java.awt.Font;

import utils.math.DFT;
import utils.visibility.DrawField2D;
import weka.core.Utils;
import clustering.FuzzyKMeansState;

public class VisualizeMeanStreams extends DrawField2D {

    int width;

    int height;

    int[] select;

    String[] names;

    FuzzyKMeansState state;

    double maxy = 0.003;

    double miny = -0.003;
    
    Color[] colors=new Color[]{Color.BLUE, Color.RED, Color.GREEN,Color.YELLOW, Color.CYAN, Color.MAGENTA, Color.ORANGE, Color.PINK, Color.GRAY, Color.BLACK };

    protected Font font = new Font("Dialog", Font.PLAIN, 10);

    public VisualizeMeanStreams(FuzzyKMeansState ostate, int width, int height) {
        super(width, height);

        this.width = width;
        this.height = height;
        this.select = select;
        this.names = names;

        state = inversDFT(ostate, 64, 512);
        redraw();
        this.repaint(50);
    }

    public void setDFTState(FuzzyKMeansState ostate) {
        state = inversDFT(ostate, 64, 512);
        redraw();
        this.repaint(50);
    }

    public void redraw() {
        this.setMinMaxX(0, width - 1);
        this.setMinMaxY(0, height - 1);
        this.fillRect(0, 0, width - 1, height - 1, Color.WHITE);
        double min=miny;
        double max=maxy;
        for (int c = 0; c < state.clusterCount; c++) {
            double[] vals=(double[]) (state.kMeans[c]);
            int ind=Utils.minIndex(vals);
            min=Math.min(vals[ind],min);
            ind=Utils.maxIndex(vals);
            max=Math.max(vals[ind],max);
        }
        
        
        for (int c = 0; c < state.clusterCount; c++) {
            double tt=state.clusterCounts[c]/state.dataCount;
            
            int t=(int)(tt*20*state.clusterCount)+1;
            Color col=null;
            if (c>=colors.length) col=colors[colors.length-1];
            else col=colors[c];
            drawFunction(this, (double[]) (state.kMeans[c]), 0, 0,
                    width - 1, height - 1, min, max, col,t);
        }
    }

    public static void drawFunction(DrawField2D draw, double[] values,
            double x, double y, double w, double h, double min, double max,
            Color color, int thick) {
        int len = values.length;
        double lx = x;
        double v = values[0];
        if (v > max)
            v = max;
        if (v < min)
            v = min;
        double ly = y + (h * (v - min) / (max - min));
        for (int i = 1; i < len; i++) {
            double xx = x + ((w * i) / (len - 1));
            v = values[i];
            if (v > max)
                v = max;
            if (v < min)
                v = min;
            double yy = y + (h * (v - min) / (max - min));

            draw.drawLine(lx, ly, xx, yy, color,thick,thick);

            lx = xx;
            ly = yy;
        }
    }

    public static FuzzyKMeansState inversDFT(FuzzyKMeansState state, int olen,
            int wlen) {

        double fakt = olen / (double) wlen;

        FuzzyKMeansState nstate = (FuzzyKMeansState) state.clone();
        double[][] ndata = new double[state.dataCount][olen];

        double[][] dft = new double[olen][2];
        int dftCount = ((double[][]) state.data[0]).length;
        for (int i = 0; i < state.dataCount; i++) {
            double[][] odata = (double[][]) state.data[i];
            for (int m = dftCount + 1; m < olen - dftCount; m++) {
                dft[m][0] = 0;
                dft[m][1] = 0;
            }
            for (int m = 1; m <= dftCount; m++) {
                dft[m][0] = odata[m - 1][0];
                dft[m][1] = odata[m - 1][1];
                dft[olen - m][0] = odata[m - 1][0];
                dft[olen - m][1] = -odata[m - 1][1];
            }
            dft[0][0] = 0;
            dft[0][1] = 0;

            DFT.fft(dft);

            for (int m = 1; m < olen; m++) {
                double val = Math.sqrt(dft[olen - m][0] * dft[olen - m][0]
                        + dft[olen - m][1] * dft[olen - m][1]);
                if (dft[olen - m][0] < 0.0)
                    val = -val;
                // val+=mid[i];
                ndata[i][m] = (double) val * fakt;
            }
            ndata[i][0] = dft[0][0] * fakt;
        }
        nstate.data = ndata;

        double[][] kMeans = new double[nstate.clusterCount][olen];

        for (int c = 0; c < nstate.clusterCount; c++) {
            double sum = 0;
            for (int i = 0; i < nstate.dataCount; i++) {
                double m = nstate.fuzzyCluster[i][c];
                m = m * m;
                sum += m;
                for (int p = 0; p < olen; p++) {
                    kMeans[c][p] += ndata[i][p] * m;
                }
            }
            for (int p = 0; p < olen; p++) {
                kMeans[c][p] /= sum;
            }
        }

        nstate.kMeans = kMeans;

        return nstate;
    }
}
