package weka.classifiers.lazy;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import utils.indexstructure.Distance;
import utils.indexstructure.DoubleFunction;
import utils.indexstructure.MTree;
import weka.classifiers.CaseBasedAlgorithm;
import weka.classifiers.OnLineClassifier;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NormDistance;
import weka.core.Utils;
import xxl.core.cursors.mappers.Mapper;
import xxl.core.functions.Function;

/**
 * @author beringer
 * 
 */
// Best
// quality()-Function verbessert, Klasse mit eingebracht
public class IBLDS extends OnLineClassifier implements
        CaseBasedAlgorithm {

    /**
     * 
     */
    private static final long serialVersionUID = 2803962004658943062L;

    private static boolean DEBUG = false;

    protected class InstanceInfo implements Serializable {

        public Instance inst;

        public int queryID = Integer.MIN_VALUE;

        public double queryValue = 0;

        public boolean deleted = false;

        public double maxDist = Double.POSITIVE_INFINITY;

        public InstanceInfo(Instance inst) {
            this.inst = inst;
        }

        public InstanceInfo(InstanceInfo inf) {
            this.inst = inf.inst;
            // quality=inf.quality;
        }

        public InstanceInfo(int attrCount, Instances header) {
            this.inst = new Instance(1.0, new double[attrCount]);
            this.inst.setDataset(header);
            if (header == null)
                System.out.println("Error null!!!");
        }

        public String toString() {
            return inst.toString();
        }
    }

    public static int NOMINAL = 2;

    public static int NUMERIC = 1;

    protected Instances header;

    protected int kValue = 1;

    protected InstanceInfo[] kEnvironment;

    protected MTree<InstanceInfo> mtree;

    protected int count;

    protected int timeindex;

    protected int kValue1 = 5;

    protected int kValue2 = 50;

    protected int pos;

    protected List toDelete = new ArrayList();

    protected int toDel = 0;

    protected double pmin = Double.MAX_VALUE;

    protected double smin = Double.MAX_VALUE;

    protected int histlen = 100;

    protected int lastHist = 20;

    protected double lastSum = 0;

    protected int lastPos = 0;

    protected int warning;

    protected double[] lastCor;
    
    protected double[] lastEnv;
    
    protected double sumEnv;
    
    protected double meanEnv;

    protected double p, s;

    protected int classifyCount;

    protected boolean full;

    protected double step = 1.5;

    protected int minRateCount = 40;

    protected int minBetter = 10;

    protected int rateLength = 100;

    public int maxSize = 400;

    public long maxTimeMicro = 100000000;

    protected double fastpart = 0.9;

    protected long[] lastTimes;

    protected long sumTime;

    protected int old = kValue2 / 2;

    protected int young = kValue1;

    protected double minProbDiff = 0.2;;

    protected double significant = 4.0;
    
    protected boolean redundance=true;

    protected int better0 = 0;

    protected int better2 = 0;

    protected int attrCount;

    protected int classIndex;

    protected int timeIndex = 0;

    protected int classCount;

    protected double lastCls;

    protected NormDistance distance;

    protected NormDistance distance2;

    protected int[] usedAttribute;

    protected List usedAttributeList;

    protected boolean normalize = false;

    protected boolean useVDM = true;

    protected int lastQueryID;

    protected InstanceInfo queryInstance;

    // protected QueryComparator queryComparator;

    public Distance distanceXXL = new Distance() {

        public double distance(Object object1, Object object2) {
            InstanceInfo inf1 = (InstanceInfo) object1;
            InstanceInfo inf2 = (InstanceInfo) object2;
            return IBLDS.this.distance(inf1.inst, inf2.inst);
        }
    };

    protected DoubleFunction<InstanceInfo> timeFunc = new DoubleFunction<InstanceInfo>() {
        public double invoke(InstanceInfo inf) {
            return inf.inst.value(timeIndex);
        }
    };

    public IBLDS() {
    }

    public IBLDS(int maxSize) {
        this.maxSize = maxSize;
    }

    public IBLDS(int maxSize, int kValue2, int old, int young,
            double minProbDiff, double significant, int histlen, int lastHist, boolean red) {
        this.maxSize = maxSize;
        this.old = old;
        this.kValue2 = kValue2;
        this.young = young;
        this.minProbDiff = minProbDiff;
        this.significant = significant;
        this.histlen = histlen;
        this.lastHist = lastHist;
        this.redundance=red;
    }

    public void buildClassifier(Instances data) throws Exception {

        lastTimes = new long[10];
        sumTime = 0;
        header = new Instances(data, 0);
        attrCount = data.numAttributes();
        classIndex = data.classIndex();
        classCount = data.numClasses();
        computeUsedAttributes();
        kEnvironment = new InstanceInfo[kValue];
        distance = new NormDistance(data, usedAttribute, normalize, useVDM);
        distance2 = (NormDistance) distance.clone();
        if (mtree == null) {
            mtree = new MTree(distanceXXL, 6, 15);
        } else {
            mtree.clear();
        }

        count = 0;

        pos = -1;
        classifyCount = 0;
        full = false;
        resetStats();
        toDelete.clear();
        lastCor = new double[histlen];
        lastPos = 0;
        lastSum = 0;
        lastEnv= new double[histlen];
        sumEnv=0;
        meanEnv=0;

        lastQueryID = Integer.MIN_VALUE;
        queryInstance = new InstanceInfo((Instance) null);
        Enumeration enumer = data.enumerateInstances();
        while (enumer.hasMoreElements()) {
            Instance inst = (Instance) enumer.nextElement();
            Instance cp = (Instance)inst.copy();
            cp.setDataset(header);
            insert(new InstanceInfo(cp));

        }
    }

    public boolean isSignificant() {
        return full && p + s > pmin + significant * smin
                && p >= this.minProbDiff;
    }

    public int getSinceWarning() {
        return warning;
    }

    public double getProbDiff() {
        int len = lastHist;
        int newLen = Math.max(0, len);
        // if (warning>histlen-len){
        // return p-pmin;
        // }
        int sumOld = 0;
        int sumNew = 0;
        for (int i = 0; i < len; i++) {
            sumOld += lastCor[(lastPos + i + 1) % histlen];
        }
        for (int i = 0; i < newLen; i++) {
            sumNew += lastCor[(lastPos - i + histlen) % histlen];
        }
        return (1.0 - pmin) - ((sumNew+1) / (double) (newLen+classCount));
    }

    public void resetStats() {
        pmin = Double.MAX_VALUE;
        smin = Double.MAX_VALUE;
        // classifyCount=1;
        classifyCount = (int) (histlen * 0.5);
        warning = 0;
    }

    public void insert(InstanceInfo inst) {
        if (inst == null)
            System.out.println("ERROR null inserted");
        mtree.add(inst);
        count++;
    }

    public void remove(InstanceInfo inst) {
        
        if (mtree.remove(inst)) {
            count--;
            distance2.removeInstance(inst.inst);
            if (mtree.remove(inst)) {
                System.out.println("hm...");
                mtree.remove(inst);
            }
        }
    }

    public void addClassify(double correct) {
        lastSum -= lastCor[lastPos];
        lastCor[lastPos] = correct;
        lastSum += correct;
        lastPos = (lastPos + 1) % histlen;
        if (classifyCount < histlen) {
            classifyCount++;
            if (classifyCount >= histlen)
                full = true;
        }
        if (!full)
            return;

        p = (histlen - lastSum) / histlen;
        s = Math.sqrt(p * (1 - p) / classifyCount);
        if (p + s < pmin + smin) {
            pmin = p;
            smin = s;
            warning = 0;
        } else {
            if (p + s > pmin + 1.96 * smin && (p > this.minProbDiff / 2)) {
                warning++;
            } else
                warning = 0;
        }
    }

    public void updateClassifier(Instance newInst) {
        long startM = System.currentTimeMillis();
        long startN = System.nanoTime();
        Instance inst = (Instance)newInst.copy();
        inst.setDataset(header);

        distance2.addInstance(inst);

        update(inst);

        if ((1 - p) <= 1.0 / classCount
                ||(isSignificant() && getSinceWarning() >= lastHist - 2)) {
            double pDiff = getProbDiff();

            if ((p - pmin) > minProbDiff && pDiff > minProbDiff) {
                double pp=pDiff;
                if ((1 - p) <= 1.0 / classCount
                        || ((1 - pmin) - pDiff) <= 1.0 / classCount)
                    pDiff = 1.0;
                int toDel = Math.min((int) (count * pDiff), count - kValue2);
                if (toDel > 0) {
                    doDelete();
                    List removed = mtree.remove(toDel, timeFunc);
                    int rem = removed.size();
                    Function map = new Function<InstanceInfo, Instance>() {
                        public Instance invoke(InstanceInfo ob) {
                            return ob.inst;
                        }
                    };
                    distance2.removeInstances(new Mapper(map, removed
                            .iterator()));
                    count -= rem;
                    if (DEBUG) System.out.println("STATS: " + rem + "    " + pDiff+"  "+pp+"  "+p+"   "+pmin);
                    pDiff = pp;
                    resetStats();
                }
            }
        }

        long endN = System.nanoTime();
        long endM = System.currentTimeMillis();
        long time = 0;
        if (endM - startM > 1000)
            time = (endM - startM) * 1000;
        else
            time = (endN - startN) / 1000;
        sumTime -= lastTimes[0];
        sumTime += time;
        System.arraycopy(lastTimes, 1, lastTimes, 0, 9);
        lastTimes[9] = time;
        time = sumTime / 10;

        if (time > maxTimeMicro) {
            sumTime -= lastTimes[9];
            lastTimes[9] = 0;
            System.out.println("Time!!!");
        }

        if (time > maxTimeMicro) {
            List cc = mtree.remove(2 * kValue1, timeFunc);
            count -= cc.size();
        }
        if (time > fastpart * maxTimeMicro) {

            toDel = kValue1;
        }
        if (NormDistance.maxDistance(distance, distance2) >= 0.1) {
            distance = (NormDistance) distance2.clone();
            mtree.updateDistance(distanceXXL);
        }

    }

    protected void update(Instance inst) {
        int cls = -1;
        try {
            cls = (int) classifyInstance(inst, kEnvironment);
            addClassify(cls == (int) inst.classValue() ? 1 : 0);
            // addClassify(lastCls);
        } catch (Exception e) {
            e.printStackTrace();
        }

        int mSize = count;
        doDelete();

        updateData(inst);

        doDelete();
        insert(new InstanceInfo(inst));

        if (count > maxSize) {
            if (count - maxSize > kValue1) {
                List c = mtree.remove(count - maxSize, timeFunc);
                count -= c.size();
            }
            toDel = count - maxSize;
        }

    }

    /**
     * Insert Instance in the sorted list for search of nearest instances.
     * 
     * @param list
     * @param inf
     * @param maxSize
     * @return
     */
    protected double insert(List list, InstanceInfo inf, int maxSize) {
        double val = inf.queryValue;
        double max = Double.MAX_VALUE;
        if (list.size() >= maxSize) {
            max = ((InstanceInfo) list.get(maxSize - 1)).queryValue;
        }
        if (max < val)
            return max;
        int p = 0;
        while (p < list.size()
                && ((InstanceInfo) list.get(p)).queryValue <= val) {
            p++;
        }
        list.add(p, inf);
        if (list.size() >= maxSize) {
            max = ((InstanceInfo) list.get(maxSize - 1)).queryValue;
            while (((InstanceInfo) list.get(list.size() - 1)).queryValue > max) {
                list.remove(list.size() - 1);
            }
        }
        return max;
    }

    protected void updateData(Instance inst) {
        int instcls = (int) inst.classValue();

        queryInstance.inst = inst;
        lastQueryID++;
        queryInstance.queryID = lastQueryID;
        // queryComparator.queryID = lastQueryID;
        // mtree.checkRadius();
        Iterator nearest = mtree.getNearestNeighbours(queryInstance);
        InstanceInfo[] near = new InstanceInfo[kValue2];
        double time[] = new double[kValue2];
        int cls[] = new int[kValue2];
        for (int i = 0; i < kValue2 && nearest.hasNext(); i++) {
            InstanceInfo inf = (InstanceInfo) nearest.next();
            if (inf==null) return;
            if (inf.deleted) {
                System.out.println("DEL");
            }
            if (inf.queryID != lastQueryID) {
                inf.queryValue = distance(inst, inf.inst);
                inf.queryID = lastQueryID;
            }
            near[i] = inf;
            time[i] = inf.inst.value(timeIndex);
            cls[i] = (int) inf.inst.classValue();

        }
        if (near[kValue2-1]==null) return;
        sumEnv-=lastEnv[lastPos];
        lastEnv[lastPos]=distance.distance(inst, near[kValue1-1].inst);
        sumEnv+=lastEnv[lastPos];
        meanEnv=sumEnv/histlen;
        boolean del = false;
        do {
            del = false;
            int[] timesort = Utils.sort(time);

            if (near[kValue2 - 1] != null)
                del = forget(inst, instcls, near, cls, time, timesort);

            /*
             * OnLineVisualizeTestk2.drawData=getTrainInstances();
             * OnLineVisualizeTestk2.draw.redraw();
             * OnLineVisualizeTestk2.draw.repaint();
             */
            if (del) {
                // System.out.println("ERROR");
                for (int i = 0; i < kValue2; i++) {
                    if (near[i] == null)
                        break;
                    if (near[i].deleted) {
                        System.arraycopy(near, i + 1, near, i, kValue2 - i - 1);
                        System.arraycopy(time, i + 1, time, i, kValue2 - i - 1);
                        System.arraycopy(cls, i + 1, cls, i, kValue2 - i - 1);
                        if (nearest.hasNext()) {
                            InstanceInfo inf = (InstanceInfo) nearest.next();
                            if (inf.queryID != lastQueryID) {
                                inf.queryValue = distance(inst, inf.inst);
                                inf.queryID = lastQueryID;
                            }
                            near[kValue2 - 1] = inf;
                            time[kValue2 - 1] = inf.inst.value(timeIndex);
                            cls[kValue2 - 1] = (int) inf.inst.classValue();
                        } else
                            near[kValue2 - 1] = null;
                        i--;
                    }
                }
            }
        } while (del);
    }

    protected boolean forget(Instance newInst, int cl, InstanceInfo[] kInst,
            int[] kcls, double[] time, int[] timesort) {

        boolean isDeleted = false;

        double youngT = time[timesort[kValue2 - young]];
        int[] classYounger = new int[classCount];

        int classcl = 0;

        int countY = 1;
        classYounger[cl]++;
        int oldId = -1;
        double oldest = Double.MAX_VALUE;
        for (int i = 1; i <= kValue1; i++) {
            if (time[i - 1] < oldest) {
                oldest = time[i - 1];
                oldId = i - 1;
            }
        }
        int nearestYoung = kValue2;
        for (int i = 1; i <= young; i++) {
            if (time[timesort[kValue2 - i]] >= youngT) {
                if (timesort[kValue2 - i] < nearestYoung) {
                    nearestYoung = timesort[kValue2 - i];
                }
                classYounger[kcls[timesort[kValue2 - i]]]++;
                countY++;
            } else
                break;
        }
        //if (nearestYoung >= kValue2/2)            return isDeleted;
        classcl = classYounger[cl];
        int ca = countY;
        for (int i = young + 1; i <= kValue2; i++) {
            ca++;
            if (kcls[timesort[kValue2 - i]] == cl) {
                classcl++;
            } else
                break;
        }
        boolean isRedundance = (classcl / (double) ca >= 0.98 && classYounger[cl] == countY) && (meanEnv>distance.distance(newInst, kInst[kValue1-1].inst));

        int[] sortedYounger = Utils.sort(classYounger);

        if (sortedYounger[classCount - 1] == cl
                && (classYounger[cl] - classYounger[sortedYounger[classCount - 2]])
                        / ((double) countY) > 1.0 / (classCount+1 )
         //&& classYounger[cl] / ((double) countY) > 2.0 / (classCount + 1.0)
        ) {
            for (int i = 0; i < old; i++) {
                if (timesort[i]<kValue1 && (kcls[timesort[i]] != cl)
                // || (kValue1 * fast) > timesort[i]
                ) {
                    isDeleted = true;
                    toDelete.add(kInst[timesort[i]]);
                    kInst[timesort[i]].deleted = true;
                }
            }
        }
        if (redundance && isRedundance) {
            
            for (int i = 0; i < old; i++) {
                if (!kInst[timesort[i]].deleted && timesort[i]<kValue1) {
                    isDeleted = true;
                    toDelete.add(kInst[timesort[i]]);
                    kInst[timesort[i]].deleted = true;
                }
            }
        }
        if (!isDeleted && (toDel > 0 || maxSize <= count - toDelete.size())) {
            // isDeleted = true;
            toDelete.add(kInst[oldId]);
            kInst[oldId].deleted = true;
            if (toDel > 0)
                toDel--;
        }

        return isDeleted;
    }

    protected void doDelete() {
        if (toDelete.size() > 0) {
            if (DEBUG)
                System.out.print(toDelete.size() + " Instanzen gelscht.     ");
        } else
            return;
        Iterator it = toDelete.iterator();
        while (it.hasNext()) {
            InstanceInfo inf = (InstanceInfo) it.next();
            remove(inf);
        }
        // hasDeleted(mtree);
        toDelete.clear();

    }

    public List getTrainInstances() {
        LinkedList list = new LinkedList();
        Iterator it = mtree.iterator();
        while (it.hasNext()) {
            InstanceInfo instinf = (InstanceInfo) it.next();
            if (!instinf.deleted)
                list.add(instinf.inst);
        }
        return list;
    }

    public int getTrainInstancesCount() {
        return count;
    }

    public double classifyIB(Instance instance, int k) {
        try {
            int h = kValue;
            kValue = k;
            double res = classifyInstance(instance);
            kValue = h;
            return res;
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return -1;
    }

    protected double classifyInstance(Instance instance,
            InstanceInfo[] kEnvironment) throws Exception {
        int cls = (int) instance.classValue();
        double[] dist = distributionForInstance(instance, kEnvironment);
        if (dist == null) {
            throw new Exception("Null distribution predicted");
        }
        switch (instance.classAttribute().type()) {
        case Attribute.NOMINAL:
            double max = 0;
            int maxIndex = 0;
            double max2 = 0;

            for (int i = 0; i < dist.length; i++) {
                if (dist[i] > max) {
                    maxIndex = i;
                    max = dist[i];
                }
                if (dist[i] > max2 && i != cls) {
                    max2 = dist[i];
                }
            }
            lastCls = 2 * dist[cls] - max2;
            if (max > 0) {
                return maxIndex;
            } else {
                return Instance.missingValue();
            }
        case Attribute.NUMERIC:
            return dist[0];
        default:
            return Instance.missingValue();
        }
    }

    /**
     * Predicts the class memberships for a given instance. If an instance is
     * unclassified, the returned array elements must be all zero.
     * 
     * @param instance
     *            the instance to be classified
     * @return an array containing the estimated membership probabilities of the
     *         test instance in each class
     * @exception Exception
     *                if distribution could not be computed successfully
     */
    public double[] distributionForInstance(Instance instance) {
        return distributionForInstance(instance, null);
    }

    protected double[] distributionForInstance(Instance instance,
            InstanceInfo[] data) {
        queryInstance.inst = instance;
        lastQueryID++;
        queryInstance.queryID = lastQueryID;
        // queryComparator.queryID = lastQueryID;
        Iterator nearest = mtree.getNearestNeighbours(queryInstance);
        double[] res = new double[classCount];
        int count = 0;
        while (nearest.hasNext() && count < kValue) {
            InstanceInfo inf = (InstanceInfo) nearest.next();
            if (inf.deleted) {
                System.out.println("DELETE ERROR");
                toDelete.add(inf);
                continue;
            }
            if (data != null)
                data[count] = inf;
            res[(int) inf.inst.classValue()] += 1;// inf.inst.weight();
            count += 1;// inf.inst.weight();
        }
        for (int i = 0; i < classCount; i++) {
            res[i] /= count;
        }
        return res;
    }

    /**
     * Compute distance of two Instances
     * 
     * @param instance1
     *            first instance
     * @param instance2
     *            second instance
     * @return distance of instances
     */
    protected double distance(Instance instance1, Instance instance2) {
        double dist = 0;
        for (int i = 0; i < attrCount; i++) {
            if (usedAttribute[i] > 0) {
                double val1 = 0, val2 = 0;
                if (instance1.isMissing(i))
                    val1 = Double.NaN;
                else
                    val1 = instance1.value(i);
                if (instance2.isMissing(i))
                    val2 = Double.NaN;
                else
                    val2 = instance2.value(i);
                double d = distance.distance(i, val1, val2);
                // if (d < 0 || d > 1) System.out.println("ERROR!!!");
                dist += d * d;
            }
        }
        return Math.sqrt(dist);
    }

    /**
     * Detect the attributes to use for learning.
     */
    protected void computeUsedAttributes() {
        usedAttribute = new int[attrCount];
        for (int i = 0; i < attrCount; i++) {
            if (i != classIndex && i != timeIndex && isUsedAttribute(i)) {
                Attribute att = header.attribute(i);
                if (att.isNominal()) {
                    usedAttribute[i] = NOMINAL;
                } else if (att.isNumeric())
                    usedAttribute[i] = NUMERIC;
            }
        }
    }

    public void setOperativeAttributes(List usedAttributeList) {
        this.usedAttributeList = usedAttributeList;
    }

    /**
     * Is Attribute in the used attribute list.
     * 
     * @param attribute
     *            attribute index
     * @return if this attribute should be used for learning
     */
    protected boolean isUsedAttribute(int attribute) {
        if (usedAttributeList == null)
            return true;
        else
            return (usedAttributeList.indexOf(new Integer(attribute)) >= 0);
    }

    // protected void hasDeleted(MTree<InstanceInfo> m) {
    // for (InstanceInfo inf : m) {
    // if (inf.deleted) {
    // System.out.println("ERR");
    // }
    // }
    //
    // }

}