package weka.classifiers.lazy;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import utils.indexstructure.Distance;
import utils.indexstructure.MTree;
import weka.classifiers.CaseBasedAlgorithm;
import weka.classifiers.OnLineClassifier;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NormDistance;
import weka.core.Utils;
//import xxl.core.collections.containers.MapContainer;
//import xxl.core.collections.queues.DynamicHeap;
//import xxl.core.functions.Function;
//import xxl.core.indexStructures.Sphere;
//import xxl.core.indexStructures.Tree.Query.Candidate;
//import xxl.core.io.Convertable;
//import xxl.core.io.converters.ConvertableConverter;
//import xxl.core.io.converters.Converter;

/**
 * @author beringer
 * 
 */
public class LWF2 extends OnLineClassifier implements
        CaseBasedAlgorithm {

    private static boolean DEBUG = false;

    protected class InstanceInfo{

        public Instance inst;

        public int queryID = Integer.MIN_VALUE;

        public double queryValue = 0;

        public boolean deleted = false;

        public InstanceInfo[] neighbours;

        public List depends;

        public double maxDist = Double.POSITIVE_INFINITY;

        public InstanceInfo(Instance inst) {
            this.inst = inst;
            depends = new ArrayList();
            neighbours = new InstanceInfo[kValue];
        }

        public InstanceInfo(int attrCount, Instances header) {
            this.inst = new Instance(1.0, new double[attrCount]);
            this.inst.setDataset(header);
            depends = new ArrayList();
            neighbours = new InstanceInfo[kValue];
            if (header == null)
                System.out.println("Error null!!!");
        }
    }

    

    public static int NOMINAL = 2;

    public static int NUMERIC = 1;

    
    public Distance<InstanceInfo> distanceXXL = new Distance<InstanceInfo>() {

        public double distance(InstanceInfo inf1, InstanceInfo inf2) {
            return LWF2.this.distance(inf1.inst, inf2.inst);
        }
    };

    protected Distance treeDistance;

    protected Instances header;

    protected MTree mtree;

    protected int kValue = 5;

    protected int kValue1 = 5;

    protected int kValue2 = 25;

    protected double young = 0.5;

    protected double old = 0.66;

    protected double distEpsilon = 0.05;// 0.3;

    protected double uniqueEpsilon = 0.9;

    protected double tau = 0.8;

    protected double beta = 0.04;

    protected double thresh = 0.33;

    protected int youngC;

    protected int oldC;

    protected int attrCount;

    protected int classIndex;

    protected int timeIndex = 0;

    protected int classCount;

    protected NormDistance distance;
    protected NormDistance distance2;

    protected int[] usedAttribute;

    protected List usedAttributeList;

    protected boolean normalize = true;

    protected boolean useVDM = true;

    protected int lastQueryID;

    protected int mSize;

    protected double mCount;

    protected InstanceInfo queryInstance;

    // protected QueryComparator queryComparator;

    protected List toDelete;

    public LWF2() {

    }

    public LWF2(double beta) {
        this.beta = beta;
    }

    public void buildClassifier(Instances data) throws Exception {

        header = new Instances(data, 0);
        attrCount = data.numAttributes();
        classIndex = data.classIndex();
        classCount = data.numClasses();
        youngC = (int) (young * kValue2);
        oldC = (int) (old * kValue2);
        computeUsedAttributes();
        distance = new NormDistance(data, usedAttribute, normalize, useVDM);
        distance2=(NormDistance)distance.clone();

        mtree = new MTree(distanceXXL,6,15);

        mSize = 0;
        mCount = 0;

        lastQueryID = Integer.MIN_VALUE;
        queryInstance = new InstanceInfo(null);
        Enumeration enum1 = data.enumerateInstances();
        while (enum1.hasMoreElements()) {
            Instance inst = (Instance) enum1.nextElement();
            Instance cp = (Instance)inst.copy();
            cp.setDataset(header);
            mtree.add(new InstanceInfo(cp));
            mSize++;
            mCount += inst.weight();
        }
        // queryComparator = new QueryComparator();
        // queryComparator.query = queryInstance;
        toDelete = new ArrayList();

    }

    public void updateClassifier(Instance newInst) {
        Instance inst = (Instance)newInst.copy();
        inst.setDataset(header);
        //mtree.checkRadius();
        distance2.addInstance(inst);
        updateData(inst);
        //mtree.checkRadius();
        doDelete();
        //mtree.checkRadius();
        mtree.add(new InstanceInfo(inst));
        mSize++;
        mCount += inst.weight();
        
        if (NormDistance.maxDistance(distance,distance2)>=0.1){
            //mtree.checkRadius();
            distance=(NormDistance)distance2.clone();
            mtree.updateDistance(distanceXXL);
            //mtree.checkRadius();
        }
        //mtree.checkRadius();
        // System.out.println(mCount+" -- "+checkSum());

    }

    protected void updateData(Instance inst) {
        //mtree.checkRadius();
        int instcls = (int) inst.classValue();
        int kValue = (int) Math.ceil(mSize * beta);
        // kValue = (int) Math.ceil(mCount * beta);

        queryInstance.inst = inst;
        lastQueryID++;
        queryInstance.queryID = lastQueryID;
        // queryComparator.queryID = lastQueryID;
        Iterator<InstanceInfo> nearest = mtree.getNearestNeighbours(queryInstance);
        InstanceInfo[] near = new InstanceInfo[kValue];
        double time[] = new double[kValue];
        int cls[] = new int[kValue];
        double kdist[] = new double[kValue];
        int[] clsSum =new int[classCount];
        for (int i = 0; i < kValue && nearest.hasNext(); i++) {
            InstanceInfo inf = nearest.next();
            kdist[i] = distance(inst, inf.inst);
            // System.out.print(i+". "+kdist[i]+", " );
            if (i > 0 && kdist[i] < kdist[i - 1])
                System.out.println("\n WRONG!!!!!   ");
            near[i] = inf;
            time[i] = inf.inst.value(timeIndex);
            cls[i] = (int) inf.inst.classValue();
            clsSum[cls[i]]++;
        }
        int[] sort=Utils.sort(clsSum);
        if (clsSum[instcls]>clsSum[sort[classCount-2]])
            return;
        
        boolean del = false;
        del = false;
        int[] timesort = Utils.sort(time);
        if (near[kValue - 1] != null)
            del = forget(inst, instcls, near, cls, time, timesort, kdist,
                    kValue);

    }

    protected boolean forget(Instance newInst, int cl, InstanceInfo[] kInst,
            int[] kcls, double[] time, int[] timesort, double[] kdist,
            int kValue) {
        boolean deleted = false;
        for (int i = 0; i < kValue - 1; i++) {
            double rel = lostweights(kdist[i], kdist[kValue - 1]);
            double old = kInst[i].inst.weight();
            double newWeight = old * rel;
            if (newWeight < thresh)
                newWeight = 0;
            double diff = old - newWeight;
            kInst[i].inst.setWeight(newWeight);
            mCount -= diff;
            if (newWeight < thresh) {
                deleted = true;
                toDelete.add(kInst[i]);
                kInst[i].deleted = true;
            }
        }
        return deleted;
    }

    protected double lostweights(double dist, double distk) {
        return tau + (1 - tau) * (dist * dist) / (distk * distk);
    }

    protected void doDelete() {
        if (toDelete.size() > 0) {
            if (DEBUG)
                System.out.print(toDelete.size() + " Instanzen gelscht.     ");
        } else
            return;
        Iterator it = toDelete.iterator();
        while (it.hasNext()) {
            InstanceInfo inf = (InstanceInfo) it.next();
            if (mtree.remove(inf)) {
                mSize--;
                mCount -= inf.inst.weight();
                distance2.removeInstance(inf.inst);
                //mtree.checkRadius();
            }
        }
        toDelete.clear();
    }

    public List getTrainInstances() {
        LinkedList list = new LinkedList();
        Iterator it = mtree.iterator();
        while (it.hasNext()) {
            InstanceInfo instinf = (InstanceInfo) it.next();
            if (!instinf.deleted)
                list.add(instinf.inst);
        }
        return list;
    }

    public int getTrainInstancesCount() {
        return this.mSize;
    }

    public double checkSum() {
        double sum = 0;
        Iterator it = mtree.iterator();
        while (it.hasNext()) {
            InstanceInfo instinf = (InstanceInfo) it.next();
            if (!instinf.deleted)
                sum += instinf.inst.weight();
        }
        return sum;
    }

    /**
     * Predicts the class memberships for a given instance. If an instance is
     * unclassified, the returned array elements must be all zero.
     * 
     * @param instance
     *            the instance to be classified
     * @return an array containing the estimated membership probabilities of the
     *         test instance in each class
     * @exception Exception
     *                if distribution could not be computed successfully
     */
    public double[] distributionForInstance(Instance instance) {
        queryInstance.inst = instance;
        lastQueryID++;
        queryInstance.queryID = lastQueryID;
        // queryComparator.queryID = lastQueryID;
        //mtree.checkRadius();
        Iterator<InstanceInfo> nearest = mtree.getNearestNeighbours(queryInstance);
        double[] res = new double[classCount];
        double count = 0;
        while (nearest.hasNext() && count < kValue) {
            InstanceInfo inf = nearest.next();
            if (inf.deleted) {
                toDelete.add(inf);
                continue;
            }
            res[(int) inf.inst.classValue()] += inf.inst.weight();
            count += inf.inst.weight();
        }
        for (int i = 0; i < classCount; i++) {
            res[i] /= count;
        }
        return res;
    }
    
    public double classifyIB(Instance instance, int k){
        try {
            int h=kValue;
            kValue = k;
            double res= classifyInstance(instance);
            kValue=h;
            return res;
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return -1;
    }

    public void updateNeighbours(InstanceInfo instInf) {
        for (int i = 0; i < instInf.neighbours.length; i++) {
            if (instInf.neighbours[i] != null) {
                instInf.neighbours[i].depends.remove(instInf);
                instInf.neighbours[i] = null;
            }
        }
        queryInstance.inst = instInf.inst;
        lastQueryID++;
        queryInstance.queryID = lastQueryID;
        // queryComparator.queryID = lastQueryID;
        Iterator<InstanceInfo> nearest = mtree.getNearestNeighbours(queryInstance);
        double count = 0;
        int pos = 0;
        while (nearest.hasNext() && count < kValue
                && pos < instInf.neighbours.length) {
            InstanceInfo inf = nearest.next();
            if (instInf == inf)
                continue;
            if (inf.deleted)
                continue;
            instInf.neighbours[pos] = inf;
            pos++;
            if (!inf.depends.contains(instInf))
                inf.depends.add(instInf);
            count += inf.inst.weight();
            if (count >= kValue || pos >= instInf.neighbours.length) {
                instInf.maxDist = distance(instInf.inst, inf.inst);
            }
        }
        if (count < kValue && pos < instInf.neighbours.length)
            instInf.maxDist = Double.POSITIVE_INFINITY;
    }

    public boolean getNeighbours(InstanceInfo instInf, List except,
            InstanceInfo[] result) {

        queryInstance.inst = instInf.inst;
        lastQueryID++;
        queryInstance.queryID = lastQueryID;
        // queryComparator.queryID = lastQueryID;
        Iterator<InstanceInfo> nearest = mtree.getNearestNeighbours(queryInstance);
        double count = 0;
        int pos = 0;
        boolean found = false;
        while (nearest.hasNext() && count < kValue
                && pos < instInf.neighbours.length) {
           InstanceInfo inf = (InstanceInfo) nearest.next();
            if (instInf == inf)
                continue;
            if (inf.deleted)
                continue;
            if (!except.contains(inf)) {
                result[pos] = inf;
                pos++;
                count += inf.inst.weight();
            } else
                found = true;
        }
        return found;
    }

    /**
     * Compute distance of two Instances
     * 
     * @param instance1
     *            first instance
     * @param instance2
     *            second instance
     * @return distance of instances
     */
    protected double distance(Instance instance1, Instance instance2) {
        double dist = 0;
        for (int i = 0; i < attrCount; i++) {
            if (usedAttribute[i] > 0) {
                double val1 = 0, val2 = 0;
                if (instance1.isMissing(i))
                    val1 = Double.NaN;
                else
                    val1 = instance1.value(i);
                if (instance2.isMissing(i))
                    val2 = Double.NaN;
                else
                    val2 = instance2.value(i);
                double d = distance.distance(i, val1, val2);
                // if (d < 0 || d > 1) System.out.println("ERROR!!!");
                dist += d * d;
            }
        }
        return Math.sqrt(dist);
    }

    /**
     * Detect the attributes to use for learning.
     */
    protected void computeUsedAttributes() {
        usedAttribute = new int[attrCount];
        for (int i = 0; i < attrCount; i++) {
            if (i != classIndex && i != timeIndex && isUsedAttribute(i)) {
                Attribute att = header.attribute(i);
                if (att.isNominal()) {
                    usedAttribute[i] = NOMINAL;
                } else if (att.isNumeric())
                    usedAttribute[i] = NUMERIC;
            }
        }
    }

    public void setOperativeAttributes(List usedAttributeList) {
        this.usedAttributeList = usedAttributeList;
    }

    /**
     * Is Attribute in the used attribute list.
     * 
     * @param attribute
     *            attribute index
     * @return if this attribute should be used for learning
     */
    protected boolean isUsedAttribute(int attribute) {
        if (usedAttributeList == null)
            return true;
        else
            return (usedAttributeList.indexOf(new Integer(attribute)) >= 0);
    }

}