package weka.classifiers.lazy;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import utils.indexstructure.Distance;
import utils.indexstructure.MTree;
import weka.classifiers.CaseBasedAlgorithm;
import weka.classifiers.OnLineClassifier;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NormDistance;
//import xxl.core.collections.containers.MapContainer;
//import xxl.core.collections.queues.DynamicHeap;
//import xxl.core.functions.Function;
//import xxl.core.indexStructures.Sphere;
//import xxl.core.indexStructures.Tree.Query.Candidate;
//import xxl.core.io.Convertable;
//import xxl.core.io.converters.ConvertableConverter;
//import xxl.core.io.converters.Converter;


/**
 * @author beringer
 * 
 */
public class SlidingWindowIBk extends OnLineClassifier implements CaseBasedAlgorithm{

	private static boolean DEBUG = false;

	protected class InstanceInfo {

		public Instance inst;

		public int queryID = Integer.MIN_VALUE;

		public double queryValue = 0;

		public boolean deleted = false;

		public double maxDist = Double.POSITIVE_INFINITY;

		public InstanceInfo(Instance inst) {
			this.inst = inst;
		}

		public InstanceInfo(int attrCount, Instances header) {
			this.inst = new Instance(1.0, new double[attrCount]);
			this.inst.setDataset(header);
			if (header == null)
				System.out.println("Error null!!!  (SW-IBk)");
		}

		
	}

	

	public static int NOMINAL = 2;

	public static int NUMERIC = 1;

	

	public Distance distanceXXL = new Distance() {

		public double distance(Object object1, Object object2) {
			InstanceInfo inf1 = (InstanceInfo) object1;
			InstanceInfo inf2 = (InstanceInfo) object2;
			return SlidingWindowIBk.this.distance(inf1.inst, inf2.inst);
		}
	};

	protected Distance treeDistance;

	protected Instances header;

	protected int kValue = 1;

	protected int windowSize =40;

	protected List window;

	protected int winPos;

	protected MTree mtree;

	protected int attrCount;

	protected int classIndex;

	protected int timeIndex = 0;

	protected int classCount;

	protected NormDistance distance;
    protected NormDistance distance2;

	protected int[] usedAttribute;

	protected List usedAttributeList;

	protected boolean normalize = false;

	protected boolean useVDM = false;

	protected int lastQueryID;

	protected InstanceInfo queryInstance;

	// protected QueryComparator queryComparator;
	
	public SlidingWindowIBk(int windowSize){
		this.windowSize=windowSize;
	}

	public void buildClassifier(Instances data) throws Exception {

		header = new Instances(data, 0);
		attrCount = data.numAttributes();
		classIndex = data.classIndex();
		classCount = data.numClasses();
		computeUsedAttributes();
		distance = new NormDistance(data, usedAttribute, normalize, useVDM);
        distance2=(NormDistance)distance.clone();
		window = new ArrayList(windowSize);
		winPos = 0;

		mtree = new MTree(distanceXXL,6,15);

		lastQueryID = Integer.MIN_VALUE;
		queryInstance = new InstanceInfo(null);
		Enumeration enumer = data.enumerateInstances();
		while (enumer.hasMoreElements()) {
			Instance inst = (Instance) enumer.nextElement();
			Instance cp = (Instance) (inst).copy();
			cp.setDataset(header);
			InstanceInfo instinf = new InstanceInfo(cp);
			mtree.add(instinf);
			if (window.size() > winPos) {
				mtree.remove(window.get(winPos));
				window.set(winPos, instinf);
			} else
				window.add(instinf);
			winPos++;
			if (winPos >= windowSize)
				winPos = 0;
		}
		// queryComparator = new QueryComparator();
		// queryComparator.query = queryInstance;
	}

	public void setWindowSize(int newWindowSize) {
		List win = new ArrayList(newWindowSize);
		if (window != null && window.size() > 0) {
			
			int pos = 0;
			if (window.size() > newWindowSize) {
				pos = winPos++;
				pos -= newWindowSize;
				if (pos < 0)
					pos += window.size();
			} else {
				pos = winPos++;
				if (pos >= window.size())
					pos = 0;
			}
			while (pos != winPos) {
				win.add(window.get(pos));
				pos++;
				if (pos >= window.size())
					pos = 0;
			}
			window.clear();
		}
		window=win;
		windowSize=newWindowSize;
		winPos=window.size();
		if (winPos>=windowSize) winPos=0;
	}

	public void updateClassifier(Instance newInst) {
		Instance inst = (Instance) (newInst).copy();
		inst.setDataset(header);
		InstanceInfo newInstInf = new InstanceInfo(inst);
		InstanceInfo old = null;
        if (inst.value(0)==157) 
            System.out.print("");
        distance2.addInstance(inst);
		if (window.size() > winPos)
			old = (InstanceInfo) window.get(winPos);
		if (old != null) {
            //verifyMTree();
			if (!mtree.remove(old)) {
                System.out.print("");
                mtree.remove(old);
            }
            old.deleted = true;
            //verifyMTree();
            distance2.removeInstance(old.inst);
		}
		if (window.size() > winPos)
			window.set(winPos, newInstInf);
		else
			window.add(newInstInf);
		winPos++;
		if (winPos >= windowSize)
			winPos = 0;
		mtree.add(newInstInf);
        //verifyMTree();
        if (NormDistance.maxDistance(distance,distance2)>=0.1){
            distance=(NormDistance)distance2.clone();
            mtree.updateDistance(distanceXXL);
        }
	}

	public List getTrainInstances() {
		LinkedList list = new LinkedList();
		Iterator it = mtree.iterator();
		while (it.hasNext()) {
			InstanceInfo instinf = (InstanceInfo) it.next();
			if (!instinf.deleted)
				list.add(instinf.inst);
		}
		return list;
	}

	public int getTrainInstancesCount() {
		return window.size();
	}

	/**
	 * Predicts the class memberships for a given instance. If an instance is
	 * unclassified, the returned array elements must be all zero.
	 * 
	 * @param instance
	 *            the instance to be classified
	 * @return an array containing the estimated membership probabilities of the
	 *         test instance in each class
	 * @exception Exception
	 *                if distribution could not be computed successfully
	 */

	public double[] distributionForInstance(Instance instance) {
		queryInstance.inst = instance;
		lastQueryID++;
		queryInstance.queryID = lastQueryID;
		// queryComparator.queryID = lastQueryID;
		Iterator<InstanceInfo> nearest = mtree.getNearestNeighbours(queryInstance);
		double[] res = new double[classCount];
		double count = 0;
        
        
		while (nearest.hasNext() && count < kValue) {
			InstanceInfo inf = nearest.next();
			if (inf.deleted) {
				System.out.println("DELETE ERROR (SW-IBk)");
				continue;
			}
			res[(int) inf.inst.classValue()] += inf.inst.weight();
			count += inf.inst.weight();
		}
		for (int i = 0; i < classCount; i++) {
			res[i] /= count;
		}
		return res;
	}
    
    public double classifyIB(Instance instance, int k){
        try {
            int h=kValue;
            kValue = k;
            double res= classifyInstance(instance);
            kValue=h;
            return res;
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return -1;
    }

	/**
	 * Compute distance of two Instances
	 * 
	 * @param instance1
	 *            first instance
	 * @param instance2
	 *            second instance
	 * @return distance of instances
	 */
	protected double distance(Instance instance1, Instance instance2) {
		double dist = 0;
		for (int i = 0; i < attrCount; i++) {
			if (usedAttribute[i] > 0) {
				double val1 = 0, val2 = 0;
				if (instance1.isMissing(i))
					val1 = Double.NaN;
				else
					val1 = instance1.value(i);
				if (instance2.isMissing(i))
					val2 = Double.NaN;
				else
					val2 = instance2.value(i);
				double d = distance.distance(i, val1, val2);
				// if (d < 0 || d > 1) System.out.println("ERROR!!!");
				dist += d * d;
			}
		}
		return Math.sqrt(dist);
	}

	/**
	 * Detect the attributes to use for learning.
	 */
	protected void computeUsedAttributes() {
		usedAttribute = new int[attrCount];
		for (int i = 0; i < attrCount; i++) {
			if (i != classIndex && i != timeIndex && isUsedAttribute(i)) {
				Attribute att = header.attribute(i);
				if (att.isNominal()) {
					usedAttribute[i] = NOMINAL;
				} else if (att.isNumeric())
					usedAttribute[i] = NUMERIC;
			}
		}
	}

	public void setOperativeAttributes(List usedAttributeList) {
		this.usedAttributeList = usedAttributeList;
	}

	/**
	 * Is Attribute in the used attribute list.
	 * 
	 * @param attribute
	 *            attribute index
	 * @return if this attribute should be used for learning
	 */
	protected boolean isUsedAttribute(int attribute) {
		if (usedAttributeList == null)
			return true;
		else
			return (usedAttributeList.indexOf(new Integer(attribute)) >= 0);
	}
    
//    protected boolean verifyMTree() {
//        Iterator<InstanceInfo> it = mtree.iterator();
//        int s = mtree.size();
//        int c = 0;
//        List<InstanceInfo> li = new ArrayList();
//        while (it.hasNext()) {
//            InstanceInfo inf = it.next();
//            if (inf == null) {
//                System.out.println("Verify:BigNullProb");
//                return false;
//            }
//            if (inf.deleted) {
//                System.out.println("Verify:BigDeleteProb");
//                return false;
//            }
//            c++;
//            Iterator<InstanceInfo> it2 = li.iterator();
//            while (it2.hasNext()) {
//                InstanceInfo inf2 = it2.next();
//                if (inf2 == inf) {
//                    System.out.println("Verify:BigDoubleProb");
//                    return false;
//                }
//            }
//            li.add(inf);
//        }
//        if (s != c)
//            System.out.println("Verify:BigSizeProb");
//        return true;
//
//    }


}