package weka.classifiers.lazy;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import utils.indexstructure.Distance;
import weka.classifiers.CaseBasedAlgorithm;
import weka.classifiers.OnLineClassifier;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NormDistance;


/**
 * @author beringer
 * 
 */
public class TWF extends OnLineClassifier implements CaseBasedAlgorithm{

	private static boolean DEBUG = false;

	protected class InstanceInfo{

		public Instance inst;

		public int queryID = Integer.MIN_VALUE;

		public double queryValue = 0;

		public boolean deleted = false;

		public double maxDist = Double.POSITIVE_INFINITY;

		public InstanceInfo(Instance inst) {
			this.inst = inst;
		}

		public InstanceInfo(int attrCount, Instances header) {
			this.inst = new Instance(1.0, new double[attrCount]);
			this.inst.setDataset(header);
			if (header == null)
				System.out.println("Error null!!!");
		}

		public void read(DataInput dataInput) throws IOException {
			queryID = dataInput.readInt();
			queryValue = dataInput.readDouble();
			deleted = dataInput.readBoolean();
			inst.setWeight(dataInput.readDouble());
			int num = inst.numAttributes();
			for (int i = 0; i < num; i++) {
				inst.setValue(i, dataInput.readDouble());
			}
			maxDist = Double.POSITIVE_INFINITY;
			System.out.println("InstanceInfo read Problem!!!");
		}

		public void write(DataOutput dataOutput) throws IOException {
			dataOutput.writeInt(queryID);
			dataOutput.writeDouble(queryValue);
			dataOutput.writeBoolean(deleted);
			dataOutput.writeDouble(inst.weight());
			int num = inst.numAttributes();
			for (int i = 0; i < num; i++) {
				dataOutput.writeDouble(inst.value(i));
			}
			System.out.println("InstanceInfo write Problem!!!");
		}
	}

	

	public static int NOMINAL = 2;

	public static int NUMERIC = 1;

	

	

	public Distance distanceXXL = new Distance() {

		public double distance(Object object1, Object object2) {
			InstanceInfo inf1 = (InstanceInfo) object1;
			InstanceInfo inf2 = (InstanceInfo) object2;
			return TWF.this.distance(inf1.inst, inf2.inst);
		}
	};

	protected Distance treeDistance;

	protected Instances header;

	protected int kValue = 5;

	protected double beta=0.996;
    
    protected double theta=0.2;
    
    protected double oneWeight=1.0;
    
	protected LinkedList<InstanceInfo> window;


	protected int attrCount;

	protected int classIndex;

	protected int timeIndex = 0;

	protected int classCount;

	protected NormDistance distance;

	protected int[] usedAttribute;

	protected List usedAttributeList;

	protected boolean normalize = true;

	protected boolean useVDM = true;

	protected int lastQueryID;

	protected InstanceInfo queryInstance;

	// protected QueryComparator queryComparator;
	
	public TWF(double beta){
		this.beta=beta;
	}

	public void buildClassifier(Instances data) throws Exception {

		header = new Instances(data, 0);
		attrCount = data.numAttributes();
		classIndex = data.classIndex();
		classCount = data.numClasses();
		computeUsedAttributes();
		distance = new NormDistance(data, usedAttribute, normalize, useVDM);
		window = new LinkedList();



		lastQueryID = Integer.MIN_VALUE;
		queryInstance = new InstanceInfo(null);
		Enumeration enumer = data.enumerateInstances();
		while (enumer.hasMoreElements()) {
			Instance inst = (Instance) enumer.nextElement();
			Instance cp = (Instance) (inst).copy();
			cp.setDataset(header);
			InstanceInfo instinf = new InstanceInfo(cp);
			window.add(instinf);
			
		}
		// queryComparator = new QueryComparator();
		// queryComparator.query = queryInstance;
	}

	
	public void updateClassifier(Instance newInst) {
		Instance inst = (Instance) (newInst).copy();
		inst.setDataset(header);
		InstanceInfo newInstInf = new InstanceInfo(inst);
        oneWeight=oneWeight/beta;
        inst.setWeight(oneWeight);
        double w=oneWeight*theta;
        distance.addInstance(inst);
		
		window.add(newInstInf);
        while(window.getFirst().inst.weight()<w){
            InstanceInfo r=window.removeFirst();
            distance.removeInstance(r.inst);
        }
	}

	public List getTrainInstances() {
		LinkedList list = new LinkedList();
		Iterator it = window.iterator();
		while (it.hasNext()) {
			InstanceInfo instinf = (InstanceInfo) it.next();
			if (!instinf.deleted)
				list.add(instinf.inst);
		}
		return list;
	}

	public int getTrainInstancesCount() {
		return window.size();
	}
    
    /**
     * Insert Instance in the sorted list for search of nearest instances.
     * 
     * @param list
     * @param inf
     * @param maxSize
     * @return
     */
    protected double insert(List list, InstanceInfo inf, int maxSize) {
        double val = inf.queryValue;
        double max = Double.MAX_VALUE;
        if (list.size() >= maxSize) {
            max = ((InstanceInfo) list.get(maxSize - 1)).queryValue;
        }
        if (max < val)
            return max;
        int p = 0;
        while (p < list.size() && ((InstanceInfo) list.get(p)).queryValue < val) {
            p++;
        }
        list.add(p, inf);
        if (max < Double.MAX_VALUE) {
            max = ((InstanceInfo) list.get(maxSize - 1)).queryValue;
            while (((InstanceInfo) list.get(list.size() - 1)).queryValue > max) {
                list.remove(list.size() - 1);
            }
        }
        return max;
    }

    protected List findNearest(Instance inst2, int count) {
        List nearest=new ArrayList();
        double minDist = Double.MAX_VALUE;
        Iterator it = window.iterator();// mtree.query();
        double maxValue = Double.MAX_VALUE;

        while (it.hasNext()) {
            InstanceInfo instinf = (InstanceInfo) it.next();
            Instance inst = instinf.inst;

            double dist = distance(inst2, inst);
            if (dist <= maxValue) {
                instinf.queryValue = dist;
                maxValue = insert(nearest, instinf, count);
            }
        }
        return nearest;
    }
    
	/**
	 * Predicts the class memberships for a given instance. If an instance is
	 * unclassified, the returned array elements must be all zero.
	 * 
	 * @param instance
	 *            the instance to be classified
	 * @return an array containing the estimated membership probabilities of the
	 *         test instance in each class
	 * @exception Exception
	 *                if distribution could not be computed successfully
	 */

	public double[] distributionForInstance(Instance instance) {
		queryInstance.inst = instance;
		lastQueryID++;
		queryInstance.queryID = lastQueryID;
		// queryComparator.queryID = lastQueryID;
		Iterator<InstanceInfo> nearest = findNearest(queryInstance.inst,kValue*3).iterator();
		double[] res = new double[classCount];
		double count = 0;
		while (nearest.hasNext() && count < kValue) {
			InstanceInfo inf = nearest.next();
			if (inf.deleted) {
				System.out.println("DELETE ERROR");
				continue;
			}
			res[(int) inf.inst.classValue()] += inf.inst.weight();
			count += inf.inst.weight();
		}
		for (int i = 0; i < classCount; i++) {
			res[i] /= count;
		}
		return res;
	}

    public double classifyIB(Instance instance, int k){
        try {
            int h=kValue;
            kValue = k;
            double res= classifyInstance(instance);
            kValue=h;
            return res;
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return -1;
    }
    
	/**
	 * Compute distance of two Instances
	 * 
	 * @param instance1
	 *            first instance
	 * @param instance2
	 *            second instance
	 * @return distance of instances
	 */
	protected double distance(Instance instance1, Instance instance2) {
		double dist = 0;
		for (int i = 0; i < attrCount; i++) {
			if (usedAttribute[i] > 0) {
				double val1 = 0, val2 = 0;
				if (instance1.isMissing(i))
					val1 = Double.NaN;
				else
					val1 = instance1.value(i);
				if (instance2.isMissing(i))
					val2 = Double.NaN;
				else
					val2 = instance2.value(i);
				double d = distance.distance(i, val1, val2);
				// if (d < 0 || d > 1) System.out.println("ERROR!!!");
				dist += d * d;
			}
		}
		return Math.sqrt(dist);
	}

	/**
	 * Detect the attributes to use for learning.
	 */
	protected void computeUsedAttributes() {
		usedAttribute = new int[attrCount];
		for (int i = 0; i < attrCount; i++) {
			if (i != classIndex && i != timeIndex && isUsedAttribute(i)) {
				Attribute att = header.attribute(i);
				if (att.isNominal()) {
					usedAttribute[i] = NOMINAL;
				} else if (att.isNumeric())
					usedAttribute[i] = NUMERIC;
			}
		}
	}

	public void setOperativeAttributes(List usedAttributeList) {
		this.usedAttributeList = usedAttributeList;
	}

	/**
	 * Is Attribute in the used attribute list.
	 * 
	 * @param attribute
	 *            attribute index
	 * @return if this attribute should be used for learning
	 */
	protected boolean isUsedAttribute(int attribute) {
		if (usedAttributeList == null)
			return true;
		else
			return (usedAttributeList.indexOf(new Integer(attribute)) >= 0);
	}

}