package weka.classifiers.lazy;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import utils.indexstructure.Distance;
import utils.indexstructure.MTree;
import weka.classifiers.CaseBasedAlgorithm;
import weka.classifiers.Classifier;
import weka.classifiers.UpdateableClassifier;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NormDistance;
//import xxl.core.collections.containers.MapContainer;
//import xxl.core.collections.queues.DynamicHeap;
//import xxl.core.functions.Function;
//import xxl.core.indexStructures.Sphere;
//import xxl.core.indexStructures.Tree.Query.Candidate;
//import xxl.core.io.Convertable;
//import xxl.core.io.converters.ConvertableConverter;
//import xxl.core.io.converters.Converter;


/**
 * @author beringer
 * 
 */
public class TreeIBk extends Classifier implements CaseBasedAlgorithm, UpdateableClassifier{

	private static boolean DEBUG = false;

	protected class InstanceInfo implements Serializable{

		public Instance inst;

		public double queryValue = 0;

		public boolean deleted = false;

		public double maxDist = Double.POSITIVE_INFINITY;

		public InstanceInfo(Instance inst) {
			this.inst = inst;
		}

		public InstanceInfo(int attrCount, Instances header) {
			this.inst = new Instance(1.0, new double[attrCount]);
			this.inst.setDataset(header);
			if (header == null)
				System.out.println("Error null!!!");
		}

		
	}

	

	public static int NOMINAL = 2;

	public static int NUMERIC = 1;

	

	public Distance distanceMTree = new Distance() {

		public double distance(Object object1, Object object2) {
			InstanceInfo inf1 = (InstanceInfo) object1;
			InstanceInfo inf2 = (InstanceInfo) object2;
			return TreeIBk.this.distance(inf1.inst, inf2.inst);
		}
	};

	protected Distance treeDistance;

	protected Instances header;

	protected int kValue = 5;

	protected MTree mtree;

	protected int attrCount;

	protected int classIndex;

	protected int classCount;

	protected NormDistance distance;
    protected NormDistance distance2;

	protected int[] usedAttribute;

	protected List usedAttributeList;

	protected boolean normalize = true;

	protected boolean useVDM = false;

	protected InstanceInfo queryInstance;

	// protected QueryComparator queryComparator;
	
    public TreeIBk(){
        this.kValue=kValue;
    }
    
	public TreeIBk(int kValue){
        this.kValue=kValue;
	}

	public void buildClassifier(Instances data) throws Exception {

		header = new Instances(data, 0);
		attrCount = data.numAttributes();
		classIndex = data.classIndex();
		classCount = data.numClasses();
		computeUsedAttributes();
		distance = new NormDistance(data, usedAttribute, normalize, useVDM);
        distance2=(NormDistance)distance.clone();

		mtree = new MTree(distanceMTree,6,15);

		queryInstance = new InstanceInfo(null);
		Enumeration enumer = data.enumerateInstances();
		while (enumer.hasMoreElements()) {
			Instance inst = (Instance) enumer.nextElement();
			Instance cp = (Instance) (inst).copy();
			cp.setDataset(header);
			InstanceInfo instinf = new InstanceInfo(cp);
			mtree.add(instinf);
		}
	}

	

	public void updateClassifier(Instance newInst) {
		Instance inst = (Instance) (newInst).copy();
		inst.setDataset(header);
		InstanceInfo newInstInf = new InstanceInfo(inst);
        distance2.addInstance(inst);
		mtree.add(newInstInf);
        if (NormDistance.maxDistance(distance,distance2)>=0.1){
            distance=(NormDistance)distance2.clone();
            mtree.updateDistance(distanceMTree);
        }
	}

	public List getTrainInstances() {
		LinkedList list = new LinkedList();
		Iterator it = mtree.iterator();
		while (it.hasNext()) {
			InstanceInfo instinf = (InstanceInfo) it.next();
			if (!instinf.deleted)
				list.add(instinf.inst);
		}
		return list;
	}

	public int getTrainInstancesCount() {
		return mtree.size();
	}

	/**
	 * Predicts the class memberships for a given instance. If an instance is
	 * unclassified, the returned array elements must be all zero.
	 * 
	 * @param instance
	 *            the instance to be classified
	 * @return an array containing the estimated membership probabilities of the
	 *         test instance in each class
	 * @exception Exception
	 *                if distribution could not be computed successfully
	 */

	public double[] distributionForInstance(Instance instance) {
		queryInstance.inst = instance;

		Iterator<InstanceInfo> nearest = mtree.getNearestNeighbours(queryInstance);
		double[] res = new double[classCount];
		double count = 0;
        int nr=0;
        double init=1.0/Math.max(1.0,mtree.size());
        for(int i=0;i<classCount;i++){
            res[i]=init;
        }
        count=classCount*init;
		while (nearest.hasNext() && nr < kValue) {
			InstanceInfo inf = nearest.next();
            double alpha=distance(instance, inf.inst);
            double comp=getCompetence(inf,alpha);
			res[(int) inf.inst.classValue()] += comp;
			count += comp;
            nr++;
		}
		for (int i = 0; i < classCount; i++) {
			res[i] /= count;
		}
		return res;
	}
    
    public double getCompetence(InstanceInfo instinf, double alpha){

        Iterator<InstanceInfo> nearest = mtree.getNearestNeighbours(instinf);
        int count=0;
        double prob=0;
        double d=0;
        while (d<=alpha){
            InstanceInfo inf = nearest.next();
            if (inf==null){
                
                break;
            }
            if (inf==instinf) continue;
            d=distance(instinf.inst, inf.inst);
            if (d<= alpha){
                count++;
                if ((int)instinf.inst.classValue()==(int)inf.inst.classValue()){
                    prob++;
                }
            }
        }
        //System.out.println("count: "+count);
        if (count==0) return 1;
        else return (prob/count);
    }
    
    public double classifyIB(Instance instance, int k){
        try {
            int h=kValue;
            kValue = k;
            double res= classifyInstance(instance);
            kValue=h;
            return res;
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return -1;
    }

	/**
	 * Compute distance of two Instances
	 * 
	 * @param instance1
	 *            first instance
	 * @param instance2
	 *            second instance
	 * @return distance of instances
	 */
	protected double distance(Instance instance1, Instance instance2) {
		double dist = 0;
		for (int i = 0; i < attrCount; i++) {
			if (usedAttribute[i] > 0) {
				double val1 = 0, val2 = 0;
				if (instance1.isMissing(i))
					val1 = Double.NaN;
				else
					val1 = instance1.value(i);
				if (instance2.isMissing(i))
					val2 = Double.NaN;
				else
					val2 = instance2.value(i);
				double d = distance.distance(i, val1, val2);
				// if (d < 0 || d > 1) System.out.println("ERROR!!!");
				dist += d * d;
			}
		}
		return Math.sqrt(dist);
	}

	/**
	 * Detect the attributes to use for learning.
	 */
	protected void computeUsedAttributes() {
		usedAttribute = new int[attrCount];
		for (int i = 0; i < attrCount; i++) {
			if (i != classIndex && isUsedAttribute(i)) {
				Attribute att = header.attribute(i);
				if (att.isNominal()) {
					usedAttribute[i] = NOMINAL;
				} else if (att.isNumeric())
					usedAttribute[i] = NUMERIC;
			}
		}
	}

	public void setOperativeAttributes(List usedAttributeList) {
		this.usedAttributeList = usedAttributeList;
	}

	/**
	 * Is Attribute in the used attribute list.
	 * 
	 * @param attribute
	 *            attribute index
	 * @return if this attribute should be used for learning
	 */
	protected boolean isUsedAttribute(int attribute) {
		if (usedAttributeList == null)
			return true;
		else
			return (usedAttributeList.indexOf(new Integer(attribute)) >= 0);
	}

}