/*
 * Created on 03.05.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package weka.classifiers.meta;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import weka.classifiers.Classifier;
import weka.classifiers.rules.ZeroR;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.Option;
import weka.core.OptionHandler;
import weka.core.UnsupportedClassTypeException;
import weka.core.Utils;
import weka.filters.Filter;

/**
 * @author George
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class InversClassifier extends Classifier {

    /** The classifiers. */
    private List classifiers;
    
    protected List[] sorted;

    /** The filters used to transform the class. */
    private Filter[][] classFilters;

    /** The class name of the base classifier. */
    private Classifier classifier = new ZeroR();

    /** Internal copy of the class attribute for output purposes */
    private Attribute classAttribute;

    /** A transformed dataset header used by the 1-against-1 method */
    private Instances dataset;
    
    protected double[] clsDistrib;

    protected int numClasses;

    /** Random number seed */
    protected int m_Seed = 1;

    protected boolean weight = false;
    
    public InversClassifier(){
    }
    
    /**
     * Set the base classifier.
     * 
     * @param newClassifier
     *            the Classifier to use.
     */
    public void setClassifier(Classifier newClassifier) {

        classifier = newClassifier;
    }
    
    public Classifier getClassifier(){
        return classifier;
    }

    /*
     * (non-Javadoc)
     * 
     * @see eyke.Ranking#buildClassifier(weka.core.Instances)
     */
    public void buildClassifier(Instances insts) throws Exception {
        dataset=insts;
        // Check for non-nominal classes
        if (!insts.classAttribute().isNominal()) {
            throw new UnsupportedClassTypeException(
                    "MultiClassClassifier: class should " + "be nominal!");
        }
        if (classifier == null) {
            throw new Exception("No base classifier has been set!");
        }
        // m_ZeroR = new ZeroR();
        // m_ZeroR.buildClassifier(insts);

        // construct a two-class header version of the dataset
        

        numClasses = insts.numClasses();
        
        sorted=new List[numClasses];
        for (int i=0;i<numClasses;i++){
            sorted[i]=new ArrayList();
        }
        
        clsDistrib=new double[numClasses];
        int n=insts.numInstances();
        for (int i=0;i<n;i++){
            Instance in=insts.instance(i);
            clsDistrib[(int)in.classValue()]+=1.0/n;
            sorted[(int)in.classValue()].add(in);
        }
        
        classifiers=new ArrayList();
        Classifier c=Classifier.makeCopies(classifier, 1)[0];
        c.buildClassifier(insts);
        classifiers.add(new Object[]{getAll(),c});
        

    }
    
    public boolean[] getAll(){
        boolean[] all=new boolean[numClasses];
        
        return all;
    }
    
    public Classifier getClassifier(boolean[] clss){
        Iterator it=classifiers.iterator();
        while(it.hasNext()){
            Object[] set=(Object[])it.next();
            if (equals(clss,(boolean[])set[0])){
                return (Classifier)set[1];
            }
        }
        Classifier c=createClassifier(clss);
        classifiers.add(new Object[]{clone(clss),c});
        return c;
    }
    
    public boolean[] clone(boolean[] arr){
        boolean[] arr2=new boolean[arr.length];
        for (int i=0;i< arr.length;i++){
            arr2[i]=arr[i];
        }
        return arr2;
    }
    
    public Classifier createClassifier(boolean[] locked){
        Instances set=new Instances(dataset,0);
        for (int i=0;i<numClasses;i++){
            if (!locked[i]) {
                Iterator it=sorted[i].iterator();
                while(it.hasNext()){
                    set.add((Instance)it.next());
                }
            }
        }
        Classifier c;
        try {
            c = Classifier.makeCopies(classifier, 1)[0];
            try{
                c.buildClassifier(set);
            } catch (Exception e) {
                c=new ZeroR();
                c.buildClassifier(set);
            }
            return c;
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        return null;
    }
    
    public boolean equals(boolean[] b1, boolean[] b2){
        for (int i=0;i<numClasses;i++){
            if (b1[i]!=b2[i]) return false;
        }
        return true;
    }
    
    public double classifyInstance(Instance instance) throws Exception {
        boolean[]locked=new boolean[numClasses];
        do{
            double[] res=distributionForInstance(instance,locked);
            
            int minIndex=-1;
            double min=Double.MAX_VALUE;
            for (int i=0;i<numClasses;i++){
                if (!locked[i]&&res[i]<min){
                    minIndex=i;
                    min=res[i];
                }
            }
            locked[minIndex]=true;
        }while(countFalse(locked)>1);
        for(int i=0;i<numClasses;i++){
            if (!locked[i]) return i;
        }
        return -1;
    }
    
    protected int countFalse(boolean[] locked){
        int c=0;
        for(int i=0;i<numClasses;i++){
            if (!locked[i]) c++;
        }
        return c;
    }

    protected double[] distributionForInstance(Instance inst, boolean[] locked)
            throws Exception {
        
        Classifier cl=getClassifier(locked);
        if (cl==null){
            cl=getClassifier(locked);
        }
        return cl.distributionForInstance(inst);
    }

    
    
    protected int maxIndex(double[] dist,boolean[] locked) {

        double max = -1;
        int maxIndex = -1;

        for (int i = 0; i < numClasses; i++) {
            if(locked[i]) continue;
            if (dist[i] > max) {
                maxIndex = i;
                max = dist[i];
            }
        }
        return maxIndex;
    }
    
    /**
     * String describing default classifier.
     */
    protected String defaultClassifierString() {
      return "weka.classifiers.rules.ZeroR";
    }

    /**
     * Returns an enumeration describing the available options.
     *
     * @return an enumeration of all the available options.
     */
    public Enumeration listOptions() {

      Vector newVector = new Vector(3);

      Enumeration enum1 = super.listOptions();
      while (enum1.hasMoreElements()) {
        newVector.addElement(enum1.nextElement());
      }

      newVector.addElement(new Option(
          "\tFull name of base classifier.\n"
          + "\t(default: " + defaultClassifierString() +")",
          "W", 1, "-W"));

      newVector.addElement(new Option(
         "",
         "", 0, "\nOptions specific to classifier "
         + classifier.getClass().getName() + ":"));
      enum1 = ((OptionHandler)classifier).listOptions();
      while (enum1.hasMoreElements()) {
        newVector.addElement(enum1.nextElement());
      }

      return newVector.elements();
    }

    /**
     * Parses a given list of options. Valid options are:<p>
     *
     * -W classname <br>
     * Specify the full class name of the base learner.<p>
     *
     * Options after -- are passed to the designated classifier.<p>
     *
     * @param options the list of options as an array of strings
     * @exception Exception if an option is not supported
     */
    public void setOptions(String[] options) throws Exception {

      super.setOptions(options);

      String classifierName = Utils.getOption('W', options);

      if (classifierName.length() > 0) { 
        setClassifier(Classifier.forName(classifierName,
                       Utils.partitionOptions(options)));
      } else {
        setClassifier((Classifier)Class.forName(defaultClassifierString()).newInstance());
      }
    }

    /**
     * Gets the current settings of the Classifier.
     *
     * @return an array of strings suitable for passing to setOptions
     */
    public String [] getOptions() {

      String [] classifierOptions = ((OptionHandler)classifier).getOptions();
      int extraOptionsLength = classifierOptions.length;
      if (extraOptionsLength > 0) {
        extraOptionsLength++; // for the double hyphen
      }

      String [] superOptions = super.getOptions();
      String [] options = new String [superOptions.length + 
                   extraOptionsLength + 2];

      int current = 0;
      options[current++] = "-W";
      options[current++] = getClassifier().getClass().getName();

      System.arraycopy(superOptions, 0, options, current, 
             superOptions.length);
      current += superOptions.length;

      if (classifierOptions.length > 0) {
        options[current++] = "--";
        System.arraycopy(classifierOptions, 0, options, current, 
               classifierOptions.length);
      }

      return options;
    }
    
    /**
     * Returns the tip text for this property
     * @return tip text for this property suitable for
     * displaying in the explorer/experimenter gui
     */
    public String classifierTipText() {
      return "The base classifier to be used.";
    }
    
}
