package weka.classifiers.meta;

import java.util.Iterator;
import java.util.List;

import weka.classifiers.Classifier;
import weka.classifiers.OnLineClassifier;
import weka.classifiers.UpdateableClassifier;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NormDistance;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;
import cern.jet.random.Normal;
import edu.cornell.lassp.houle.RngPack.RandomJava;

public class MetaAdapting2 extends OnLineClassifier {

    public static int NOMINAL = NormDistance.NOMINAL;

    public static int NUMERIC = NormDistance.NUMERIC;

    protected int classIndex;

    protected int classCount, attrCount;

    protected int timeIndex = 0;

    protected int dataStart = 0;

    protected int[] usedAttribute;

    protected List usedAttributeList;

    protected Instances header;

    protected Classifier classifier;

    protected Classifier current;

    protected UpdateableClassifier currentU;
    
    protected Remove attFilter;
    
    protected Instances newData;
    
    protected int correct;
    protected boolean[] last;
    protected int first;
    protected int all;
    protected double p,s,pmin,smin;
    protected double error,warning;
        
    protected int m1=100,m2=50;
    
    public MetaAdapting2(){
    }
    
    public MetaAdapting2(Classifier newClassifier) {
        setClassifier(newClassifier);
    }
    public double classifyInstance(Instance inst){
        try {
            attFilter.input(inst);
            attFilter.batchFinished();
            Instance filt=attFilter.output();
            filt.setDataset(newData);
            return current.classifyInstance(filt);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return -1;
    }
    
    public double[] distributionForInstance(Instance inst){
        try {
            attFilter.input(inst);
            attFilter.batchFinished();
            Instance filt=attFilter.output();
            return current.distributionForInstance(filt);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
    

    public void setOperativeAttributes(List usedAttributeList) {
        this.usedAttributeList = usedAttributeList;
    }

    public void updateClassifier(Instance inst) {
        try {
            attFilter.input(inst);
            attFilter.batchFinished();
            Instance filt=attFilter.output();
            if (last[m2-1]) first--;
            if (last[m1-1]) correct--;
            System.arraycopy(last,0,last,1,m1-1);
            int cls=(int)current.classifyInstance(filt);
            if (cls==(int)filt.classValue()){
                correct++;
                first++;
                last[0]=true;
            }
            else{
                last[0]=false;
            }
            if (all<m1) all++;
            int state=getState();
            if (state==2){
                exchange();
            }
            if (state==1){
                newData.add(filt);
            }else{
                newData.delete();
            }
            currentU.updateClassifier(filt);
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        

    }
    
    public int getState(){
        p=1-(correct/(double)all);
        s=Math.sqrt(p*(1-p)/all);
        if (all<m2)return 0;
        if (p<pmin){
            pmin=p;
            smin=s;
        }
        p=1-(first/(double)m2);
        s=Math.sqrt(p*(1-p)/m2);
        if(p+s>pmin+error*smin) 
            return 2;
        else if(p+s>pmin+warning*smin) return 1;
        else return 0;
    }
    
    public void exchange(){
        System.out.println("EXCHANGE");
        resetClassifier(new Instances(newData));
        newData.delete();
        all=0;
        correct=0;
        first=0;
        pmin=Double.MAX_VALUE;
        smin=Double.MAX_VALUE;
    }

    public void buildClassifier(Instances data) throws Exception {
        header = new Instances(data, 0);
        attrCount = data.numAttributes();
        classIndex = data.classIndex();
        classCount = data.numClasses();
        correct=0;
        all=0;
        pmin=Double.MAX_VALUE;
        smin=Double.MAX_VALUE;
        error=6;
        warning=3;
        computeUsedAttributes();
        attFilter=new Remove();
        attFilter.setInvertSelection(true);
        attFilter.setAttributeIndicesArray(usedAttribute);
        attFilter.setInputFormat(data);
        Instances filtered=Filter.useFilter(data,attFilter);
        newData=new Instances(attFilter.getOutputFormat(),0);
        last=new boolean[m1];
        resetClassifier(filtered);
        
    }

    protected void resetClassifier(Instances train) {
        try {
            current = Classifier.makeCopies(classifier, 1)[0];

            current.buildClassifier(train);
            dataStart = train.numInstances();
            currentU = (UpdateableClassifier) current;
            first=0;
            correct=0;
            all=0;
            last=new boolean[m1];
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * Detect the attributes to use for learning.
     */
    protected void computeUsedAttributes() {
        if (usedAttributeList.indexOf(new Integer(classIndex))<0){
            usedAttributeList.add(new Integer(classIndex));
        }
        usedAttribute = new int[usedAttributeList.size()];
        int i=0;
        for (Iterator it=usedAttributeList.iterator();it.hasNext();){
            usedAttribute[i]=((Integer)it.next()).intValue();
            i++;
        }
    }



    public void setClassifier(Classifier newClassifier) {

        classifier = newClassifier;
    }

}
