package weka.classifiers.rules;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import weka.classifiers.Classifier;
import weka.classifiers.rules.rise.SimpleConjunctiveRule;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NoSupportForMissingValuesException;
import weka.core.Option;
import weka.core.OptionHandler;
import weka.core.UnsupportedAttributeTypeException;
import weka.core.UnsupportedClassTypeException;
import weka.core.Utils;

/**
 * @author beringer
 *
 */
public class ICN2 extends Classifier implements OptionHandler {

	public static double CONFIDENCE_0_9 = 2.706;

	public static double CONFIDENCE_0_95 = 3.841;

	public static double CONFIDENCE_0_99 = 6.635;

	

	protected double convidence = CONFIDENCE_0_95;

	protected long seed = 1;

	protected boolean ordered = false;

	protected int beamCount = 2;

	protected List unusedAttribute;

	protected boolean[] usedAttribute;

	protected Random random;

	protected int classAttr;

	protected FastVector ruleset;

	protected FastVector classruleset;

	protected SimpleConjunctiveRule defaultRule;

	protected double[] orderedClasses;

	protected int[] classOrder;

	protected double dataCount;

	protected Instances allData;

	protected FastVector data;

	protected int attrCount;

	protected int classCount;

	protected SimpleConjunctiveRule[] beamArray;

	public boolean isUsedAttribute(int attribute) {
		if (unusedAttribute == null)
			return true;
		else
			return (unusedAttribute.indexOf(new Integer(attribute)) >= 0);
	}

	/**
	 * Generates a classifier. Must initialize all fields of the classifier
	 * that are not being set via options (ie. multiple calls of buildClassifier
	 * must always lead to the same result). Must not change the dataset
	 * in any way.
	 *
	 * @param dataDFT set of instances serving as training data 
	 * @exception Exception if the classifier has not been 
	 * generated successfully
	 */
	public void buildClassifier(Instances instances) throws Exception {
		if (instances.numInstances() == 0)
				throw new Exception(" No instances with a class value!");

		if (instances.checkForStringAttributes())
				throw new UnsupportedAttributeTypeException(
						" Cannot handle string attributes!");

		if (!instances.classAttribute().isNominal())
				throw new UnsupportedClassTypeException(
						" Only nominal class, please.");

		Enumeration enumAtt = instances.enumerateAttributes();
		while (enumAtt.hasMoreElements()) {
			Attribute attr = (Attribute) enumAtt.nextElement();
			Enumeration enum1 = instances.enumerateInstances();
			while (enum1.hasMoreElements()) {
				if (((Instance) enum1.nextElement()).isMissing(attr)) { throw new NoSupportForMissingValuesException(
						"CN2: no missing values, please."); }
			}
		}


		allData = instances;

		defaultRule = getRule(allData);
		defaultRule.computeDistribution(allData);
		defaultRule.computeClass();

		orderedClasses = defaultRule.getDistribution();

		classOrder = Utils.sort(orderedClasses);

		dataCount = allData.numInstances();

		if (allData == null)
				throw new Exception(" Unable to randomize the class orders.");

		allData.deleteWithMissingClass();
		if (allData.numInstances() == 0)
				throw new Exception(" No instances with a class value!");

		ruleset = new FastVector();
		classruleset = new FastVector();

		classCount = allData.numClasses();
		attrCount = allData.numAttributes();
		classAttr = allData.classIndex();
		beamArray = new SimpleConjunctiveRule[beamCount];

		computeUsedAttributes(allData);

		//CN2 erzeugt Regeln von der seltensten Klasse bis zur hufigsten (Reihenfolge!!!)
		for (int cl = 0; cl < allData.numClasses() - 1; cl++) {
			if (!ordered || cl == 0) data = instanceVector(allData);
			addRulesetForOneClass(classOrder[cl]);
			ruleset.appendElements(classruleset);
		}
		if (!ordered) data = instanceVector(allData);
		if (!ordered) {
			addRulesetForOneClass(classOrder[allData.numClasses() - 1]);
			ruleset.appendElements(classruleset);
		}
	}

	protected FastVector instanceVector(Instances data) {
		int size = data.numInstances();
		FastVector result = new FastVector(size);

		for (int i = size - 1; i >= 0; i--) {
			result.addElement(data.instance(i));
		}
		return result;
	}

	protected void computeUsedAttributes(Instances inst) {
		usedAttribute = new boolean[inst.numAttributes()];

		for (int i = 0; i < inst.numAttributes(); i++) {
			if ((i != classAttr) && inst.attribute(i).isNominal()
					&& isUsedAttribute(i)) usedAttribute[i] = true;
		}
	}

	protected void addRulesetForOneClass(int classIndex) {
		SimpleConjunctiveRule bestRule = null;
		classruleset.removeAllElements();
		do {
			//System.out.println(data.size());
			bestRule = findBestRule(classIndex);
			if (bestRule != null) {
				classruleset.addElement(bestRule);
				//Alle untersttzten datenstze lschen
				/*
				 int size = data.size();
				 int pos = 0;
				 while (pos < size) {
				 Instance inst = (Instance)data.elementAt(pos);
				 if (bestRule.covers(inst)) {
				 data.removeElementAt(pos);
				 size--;
				 } else
				 pos++;
				 }*/
			}
		} while (bestRule != null && data.size() > 0);
	}

	protected SimpleConjunctiveRule getRule(Instances data) {
		return new SimpleConjunctiveRule(data, true);
	}

	protected SimpleConjunctiveRule getRule(SimpleConjunctiveRule rule) {
		return new SimpleConjunctiveRule(rule);
	}

	protected SimpleConjunctiveRule getEmptyRule(int classIndex) {
		SimpleConjunctiveRule empty = getRule(allData);
		empty.setClass(classIndex);
		if (!ordered) {
			for (int i = 0; i < classCount; i++)
				empty.addClass(i, orderedClasses[i]);
			empty.addCovered(data);
		} else {
			empty.computeDistribution(data);
			//if (data.numInstances()==0) empty.addClass(classIndex);
		}
		return empty;
	}

	/**
	 * @param dataDFT
	 * @return
	 */
	protected SimpleConjunctiveRule findBestRule(int classIndex) {
		SimpleConjunctiveRule empty = getEmptyRule(classIndex);
		FastVector star = new FastVector();
		star.addElement(empty);
		SimpleConjunctiveRule best = null;
		int predCount = 0;
		double bestCriteria = -Double.MAX_VALUE;
		while (star.size() > 0) {
			for (int i = 0; i < beamCount; i++)
				beamArray[i] = null;
			Enumeration enum1 = star.elements();
			while (enum1.hasMoreElements()) {
				SimpleConjunctiveRule rule = (SimpleConjunctiveRule) enum1.nextElement();
				if (rule.getPredicateCount() == predCount) {
					Iterator it = specialize(rule);
					while (it.hasNext()) {
						SimpleConjunctiveRule newRule = (SimpleConjunctiveRule) it.next();
						newRule.computeDistribution(rule.getCovered());
						//newRule.computeDistribution(data);
						double crit = computeCriteria(newRule);
						if (newRule.getCoverage() > 0 && canSpecialize(newRule)) {
							if (newRule.frequentClass() == classIndex
									&& crit > bestCriteria
									&& canBest(newRule, classIndex)) {
								best = newRule;
								bestCriteria = computeCriteria(newRule);
							}
							if (!significance(newRule, classIndex)) {
								int pos = 0;
								while (pos < beamCount
										&& beamArray[pos] != null
										&& computeCriteria(beamArray[pos]) > crit)
									pos++;
								if (pos < beamCount) {
									if (beamArray[pos] != null)
											System.arraycopy(beamArray, pos,
													beamArray, pos + 1,
													beamCount - pos - 1);
									beamArray[pos] = newRule;
								}
							}
						}
					}
				}
			}
			star.removeAllElements();
			int pos = 0;
			while (pos < beamCount && beamArray[pos] != null) {
				star.addElement(beamArray[pos]);
				pos++;
			}
			predCount++;
		}
		return best;
	}

	protected boolean canBest(SimpleConjunctiveRule rule, int classIndex) {
		if (orderedClasses[classIndex] / dataCount < rule.getDistribution()[classIndex]
				/ rule.getDistributionSum()) return true;
		for (int i = 0; i < attrCount; i++) {
			if (usedAttribute[i]) {
				if (rule.getNominalValue(i) == -1) return false;
			}
		}
		return true;
	}

	protected boolean canSpecialize(SimpleConjunctiveRule rule) {
		Enumeration enum1 = classruleset.elements();
		while (enum1.hasMoreElements()) {
			SimpleConjunctiveRule rule2 = (SimpleConjunctiveRule) enum1.nextElement();
			if (rule2.contains(rule)) return false;
		}
		return true;
	}

	protected double computeCriteria(SimpleConjunctiveRule rule) {
		return (rule.getDistribution()[rule.getClassValue()] + 1)
				/ (rule.getDistributionSum() + classCount);
	}

	protected boolean significance(SimpleConjunctiveRule rule, int classIndex) {
		double[] distribution = rule.getDistribution();
		double sum = rule.getDistributionSum();
		int cl = classIndex;
		double p = distribution[cl];
		if (p <= 0) return false;
		if (sum - p <= 0) return false;
		if (p < sum / 2) return false;
		if (orderedClasses[cl] / dataCount >= p / sum) return false;

		double val = 2 * (p * log2(p * dataCount / (orderedClasses[cl] * sum)) + (sum - p)
				* log2((sum - p) * dataCount
						/ ((dataCount - orderedClasses[cl]) * sum)));
		return val >= convidence;
		//return true;
	}

	protected double log2(double d) {
		double v = Math.log(d);
		return v; /*/ log2*/
	}

	protected Iterator specialize(final SimpleConjunctiveRule rule) {
		return new Iterator() {

			SimpleConjunctiveRule next = null;

			int attr = 0;

			int val = 0;

			int valCount = allData.numDistinctValues(attr);

			public void remove() {
			}

			public boolean hasNext() {
				if (next == null)
					return computeNext();
				else
					return true;
			}

			public Object next() {
				if (next == null) computeNext();
				Object val = next;
				next = null;
				return val;
			}

			private boolean computeNext() {
				if (attr >= attrCount) return false;
				next = getRule(rule);
				next.setNominalValue(attr, val);

				val++;
				if (val >= valCount) {
					attr++;
					while (attr < attrCount
							&& ((!usedAttribute[attr]) || rule.getNominalValue(attr) >= 0))
						attr++;
					val = 0;
					if (attr < attrCount)
							valCount = allData.numDistinctValues(attr);
				}
				return true;
			}

		};
	}

	/**
	 * Predicts the class memberships for a given instance. If
	 * an instance is unclassified, the returned array elements
	 * must be all zero. If the class is numeric, the array
	 * must consist of only one element, which contains the
	 * predicted value. Note that a classifier MUST implement
	 * either this or classifyInstance().
	 *
	 * @param instance the instance to be classified
	 * @return an array containing the estimated membership 
	 * probabilities of the test instance in each class 
	 * or the numeric prediction
	 * @exception Exception if distribution could not be 
	 * computed successfully
	 */
	public double[] distributionForInstance(Instance instance) throws Exception {
		double[] distribution = new double[classCount];
		double sum = 0;
		try {
			boolean found = false;
			int lastClass = -1;
			for (int i = 0; i < ruleset.size(); i++) {
				SimpleConjunctiveRule rule = (SimpleConjunctiveRule) ruleset.elementAt(i);
				//if (found && rule.getClassIndex()==lastClass) continue;
				if (rule.covers(instance)) {
					if (ordered) return rule.getRelativeDistribution();
					found = true;
					//lastClass=rule.getClassIndex();
					addDistribution(distribution, rule.getRelativeDistribution());
					sum += 1;
				}
			}
			if (!found) {
				return defaultRule.getRelativeDistribution();
			} else {
				for (int i = 0; i < classCount; i++) {
					distribution[i] /= sum;
				}
				return distribution;
			}
		} catch (Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		return defaultRule.getRelativeDistribution();
	}

	public void addDistribution(double[] dist, double[] newDist) {
		for (int i = 0; i < classCount; i++) {
			dist[i] += newDist[i];
		}
	}

	/**
	 * Returns an enumeration of all the available options..
	 *
	 * @return an enumeration of all available options.
	 */
	public Enumeration listOptions() {
		Vector newVector = new Vector(3);
		newVector.addElement(new Option("\tSet the number of rules\n"
				+ "\tused for beam search\n" + "\t(default 2)", "B", 1,
				"-B <number of rules>"));

		newVector.addElement(new Option("\tSet whether class ordered\n"
				+ "\trules are used (Default: false)", "O", 0, "-O"));


		newVector.addElement(new Option("\tThe seed of randomization\n"
				+ "\t(Default: 1)", "S", 1, "-S <seed>"));

		return newVector.elements();
	}

	/**
	 * Sets the OptionHandler's options using the given list. All options
	 * will be set (or reset) during this call (i.e. incremental setting
	 * of options is not possible).
	 *
	 * @param options the list of options as an array of strings
	 * @exception Exception if an option is not supported
	 */
	public void setOptions(String[] options) throws Exception {
		String numBeamString = Utils.getOption('B', options);
		if (numBeamString.length() != 0)
			beamCount = Integer.parseInt(numBeamString);
		else
			beamCount = 2;

		String seedString = Utils.getOption('S', options);
		if (seedString.length() != 0)
			seed = Integer.parseInt(seedString);
		else
			seed = 1;

		ordered = Utils.getFlag('O', options);
	}

	/**
	 * Gets the current option settings for the OptionHandler.
	 *
	 * @return the list of current option settings as an array of strings
	 */
	public String[] getOptions() {
		String[] options = new String[11];
		int current = 0;
		options[current++] = "-B";
		options[current++] = "" + beamCount;
		options[current++] = "-S";
		options[current++] = "" + seed;


		if (!ordered) options[current++] = "-O";

		while (current < options.length)
			options[current++] = "";

		return options;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		Enumeration enum1 = ruleset.elements();
		while (enum1.hasMoreElements()) {
			SimpleConjunctiveRule rule = (SimpleConjunctiveRule) enum1.nextElement();
			sb.append(rule.toString());
			sb.append("\n");
		}
		sb.append(defaultRule.toString());
		sb.append("\n");
		return sb.toString();
	}

}