/*
 * Created on 07.06.2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package weka.classifiers.rules;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import weka.classifiers.rules.rise.SimpleConjunctiveRule;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NoSupportForMissingValuesException;
import weka.core.Option;
import weka.core.UnsupportedAttributeTypeException;
import weka.core.UnsupportedClassTypeException;
import weka.core.Utils;

/**
 * @author beringer
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class RI3 extends CN2{

	public double[] changes;

	public boolean isUsedAttribute(int attribute) {
		if (usedAttributeList == null)
			return true;
		else
			return (usedAttributeList.indexOf(new Integer(attribute)) >= 0);
	}

	/**
	 * Generates a classifier. Must initialize all fields of the classifier
	 * that are not being set via options (ie. multiple calls of buildClassifier
	 * must always lead to the same result). Must not change the dataset
	 * in any way.
	 *
	 * @param dataDFT set of instances serving as training data 
	 * @exception Exception if the classifier has not been 
	 * generated successfully
	 */
	public void buildClassifier(Instances instances) throws Exception {
		if (instances.numInstances() == 0)
				throw new Exception(" No instances with a class value!");

		if (instances.checkForStringAttributes())
				throw new UnsupportedAttributeTypeException(
						" Cannot handle string attributes!");

		if (!instances.classAttribute().isNominal())
				throw new UnsupportedClassTypeException(
						" Only nominal class, please.");

		Enumeration enumAtt = instances.enumerateAttributes();
		while (enumAtt.hasMoreElements()) {
			Attribute attr = (Attribute) enumAtt.nextElement();
			if (!attr.isNominal()) { throw new UnsupportedAttributeTypeException(
					"Id3: only nominal attributes, please."); }
			Enumeration enum1 = instances.enumerateInstances();
			while (enum1.hasMoreElements()) {
				if (((Instance) enum1.nextElement()).isMissing(attr)) { throw new NoSupportForMissingValuesException(
						"Id3: no missing values, please."); }
			}
		}

		allData = instances;

		defaultRule = getRule(allData);
		defaultRule.computeDistribution(allData);
		defaultRule.computeClass();

		orderedClasses = defaultRule.getDistribution();

		classOrder = Utils.sort(orderedClasses);

		dataCount = allData.numInstances();

		if (allData == null)
				throw new Exception(" Unable to randomize the class orders.");

		allData.deleteWithMissingClass();
		if (allData.numInstances() == 0)
				throw new Exception(" No instances with a class value!");

		ruleset = new FastVector();

		classCount = allData.numClasses();
		attrCount = allData.numAttributes();
		classAttr = allData.classIndex();

		computeUsedAttributes(allData);

		data = instanceVector(allData);

		changes = new double[attrCount];

		addRulesetForOneClass();

	}

	
	protected void computeUsedAttributes(Instances inst) {
		usedAttribute = new boolean[inst.numAttributes()];

		for (int i = 0; i < inst.numAttributes(); i++) {
			if ((i != classAttr) && inst.attribute(i).isNominal()
					&& isUsedAttribute(i)) usedAttribute[i] = true;
		}
	}

	protected void addRulesetForOneClass() {
		SimpleConjunctiveRule bestRule = null;
		do {
			//System.out.println(data.numInstances());
			bestRule = findBestRule();
			if (bestRule != null) {
				ruleset.addElement(bestRule);
				//Alle untersttzten datenstze lschen
				int size = data.size();
				int pos = 0;
				while (pos < size) {
					Instance inst = (Instance)data.elementAt(pos);
					if (bestRule.covers(inst)) {
						data.removeElementAt(pos);
						size--;
					} else
						pos++;
				}
			}
		} while (bestRule != null && data.size() > 0);
	}

	protected SimpleConjunctiveRule getEmptyRule() {
		SimpleConjunctiveRule empty = getRule(allData);
		empty.computeDistribution(data);
		empty.computeClass();
		return empty;
	}

	/**
	 * @param dataDFT
	 * @return
	 */
	protected SimpleConjunctiveRule findBestRule() {
		SimpleConjunctiveRule empty = getEmptyRule();
		FastVector star = new FastVector();
		SimpleConjunctiveRule[] beamArray = new SimpleConjunctiveRule[beamCount];
		star.addElement(empty);
		SimpleConjunctiveRule best = null;
		int predCount = 0;
		double bestCriteria = -Double.MAX_VALUE;
		while (star.size() > 0) {
			for (int i = 0; i < beamCount; i++)
				beamArray[i] = null;
			Enumeration enum1 = star.elements();
			while (enum1.hasMoreElements()) {
				SimpleConjunctiveRule rule = (SimpleConjunctiveRule) enum1.nextElement();
				if (rule.getPredicateCount() == predCount) {
					Iterator it = specialize(rule);
					while (it.hasNext()) {
						SimpleConjunctiveRule newRule = (SimpleConjunctiveRule) it.next();
						newRule.computeDistribution(rule.getCovered());
						//newRule.computeDistribution(data);

						newRule.computeClass();
						double crit = computeCriteria(newRule);
						if (newRule.getCoverage() > 0) {
							if (crit > bestCriteria) {
								best = newRule;
								bestCriteria = computeCriteria(newRule);
							}
							if ((crit > bestCriteria)
									|| beamArray[beamCount - 1] == null
									|| computeCriteria(beamArray[beamCount - 1]) < crit) {
								if (newRule.getConsequent() != rule
										.getConsequent()) {
									changes[rule.getPredicateCount()]++;
								}
							}
							if ((beamArray[beamCount - 1] == null || computeCriteria(beamArray[beamCount - 1]) < crit)
									&& !significance(newRule,newRule.getClassValue())) {
								int pos = 0;
								while (pos < beamCount
										&& beamArray[pos] != null
										&& computeCriteria(beamArray[pos]) > crit)
									pos++;
								if (pos < beamCount) {
									if (beamArray[pos] != null)
											System.arraycopy(beamArray, pos,
													beamArray, pos + 1,
													beamCount - pos - 1);
									beamArray[pos] = newRule;
								}
							}
						}
					}
				}
			}
			star.removeAllElements();
			int pos = 0;
			while (pos < beamCount && beamArray[pos] != null) {
				star.addElement(beamArray[pos]);
				pos++;
			}
			predCount++;
		}
		return best;
	}

	
	/**
	 * Predicts the class memberships for a given instance. If
	 * an instance is unclassified, the returned array elements
	 * must be all zero. If the class is numeric, the array
	 * must consist of only one element, which contains the
	 * predicted value. Note that a classifier MUST implement
	 * either this or classifyInstance().
	 *
	 * @param instance the instance to be classified
	 * @return an array containing the estimated membership 
	 * probabilities of the test instance in each class 
	 * or the numeric prediction
	 * @exception Exception if distribution could not be 
	 * computed successfully
	 */
	public double[] distributionForInstance(Instance instance) throws Exception {
		double[] distribution = new double[classCount];
		double sum = 0;
		try {
			int lastClass = -1;
			for (int i = 0; i < ruleset.size(); i++) {
				SimpleConjunctiveRule rule = (SimpleConjunctiveRule) ruleset.elementAt(i);
				//if (found && rule.getClassIndex()==lastClass) continue;
				if (rule.covers(instance)) { return rule
						.getRelativeDistribution(); }
			}
			return defaultRule.getRelativeDistribution();
		} catch (Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		return defaultRule.getRelativeDistribution();
	}

	/**
	 * Returns an enumeration of all the available options..
	 *
	 * @return an enumeration of all available options.
	 */
	public Enumeration listOptions() {
		Vector newVector = new Vector(3);
		newVector.addElement(new Option("\tSet the number of rules\n"
				+ "\tused for beam search\n" + "\t(default 2)", "B", 1,
				"-B <number of rules>"));

		newVector.addElement(new Option("\tSet whether class ordered\n"
				+ "\trules are used (Default: false)", "O", 0, "-O"));

		newVector.addElement(new Option("\tThe seed of randomization\n"
				+ "\t(Default: 1)", "S", 1, "-S <seed>"));

		return newVector.elements();
	}

	/**
	 * Sets the OptionHandler's options using the given list. All options
	 * will be set (or reset) during this call (i.e. incremental setting
	 * of options is not possible).
	 *
	 * @param options the list of options as an array of strings
	 * @exception Exception if an option is not supported
	 */
	public void setOptions(String[] options) throws Exception {
		String numBeamString = Utils.getOption('B', options);
		if (numBeamString.length() != 0)
			beamCount = Integer.parseInt(numBeamString);
		else
			beamCount = 2;

		
	}

	/**
	 * Gets the current option settings for the OptionHandler.
	 *
	 * @return the list of current option settings as an array of strings
	 */
	public String[] getOptions() {
		String[] options = new String[11];
		int current = 0;
		options[current++] = "-B";
		options[current++] = "" + beamCount;
		

		while (current < options.length)
			options[current++] = "";

		return options;
	}

	public static String changeStats(double[] changes) {
		String text = "";
		int c = 0;
		for (int i = 0; i < changes.length; i++) {
			text += "with " + i + " Predicates: " + changes[i] + "\n";
			c += changes[i];
		}
		text += "insgesamt " + c + " Klassenwechsel";
		return text;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		Enumeration enum1 = ruleset.elements();
		while (enum1.hasMoreElements()) {
			SimpleConjunctiveRule rule = (SimpleConjunctiveRule) enum1.nextElement();
			sb.append(rule.toString());
			sb.append("\n");
		}
		sb.append(defaultRule.toString());
		sb.append("\n");
		return sb.toString();
	}

}