//DONE - DI constructor - complete the initialization of a description item from an instance
//DONE - hypoth - getgeneralization sorted candidates
//DONE - move learn and forget functions as members of the hypothesis class   
//DONE - complete includeDescriptorItem - check for subsumption before inserting item

//NOTE::DI - subsumed item does not implement hierarchical values - only an exact match or "ANY" value is considered a subsumption
//NOTE::DI - coversInstance - no generalization performed - equality comparison only - no restriction on an attribute covers instances with any value for this attribute

//accommodate hierarchical attribute values
//hypoth - generalize function - go through all the attributes of sorted DIs and relax/generalize them as needed
//LOW: hypoth - more flexible forget function - put the functions to forget positive and negative instances together

//FIXED BUGS
//FIXED BY ROUNDING ATTRIBUTE VALUES TO TWO DECIMAL PLACES UPON GENERATION:
//essentially identical hypotheses get added to the hypothesis store, because the bucket boundaries don't match
//because of the discretizing process - algorithm cannot be used with real-valued inputs that do not map
//to identical buckets - 0.299999 != 0.30444

//FIXED BY CHANGING HYPOTHESIS COMPARISON: why does a new hypothesis get saved every 
//time a test instance is processed? a problem in the clone() method?

//FIXED - removing redundant items from ADES before adding a generalized DI:ADES contains redundant items

package weka.classifiers.rules.flora;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import weka.classifiers.CaseBasedAlgorithm;
import weka.classifiers.OnLineClassifier;
import weka.core.Instance;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;
//Q: should I aggressively drop instances only after the window has been filled
//Q: should the maximum window size be used in the call to getInstancesToBeForgottenCount

//make sure window is full before dropping instances
//make the window size modification more explicit
//guarantee that after concept drift occurs the window grows to sufficient size to learn the new concept

//post-flora3 implementation
//learning not reported as stable by FLORA2 problem above causes a very basic hypothesis to be used
//constant drift reported by the WAH prevents saving more accurate hypotheses

//flora3 theoretical problem - do not implement changes to the policy proposed in the paper
//validate paper results first
//select the hypothesis in terms of complexity of the description with a more concise ADES set - literal count
//this is not always the case - after the sine is flipped, the ADES is 8 instead of 2 description items
//so after the flip, the better hypothesis will not be selected because it has 8 ADES description items
//compared to 2 for the old inaccurate one. Consequently the new one will not be selected because the 
//algorithm selects a hypothesis only if its ADES literal count is smaller

//NEXT:
//when adding a DI, generalize it as much as possible so as not to be subsumed by PDES or NDES or ADES

//NEXT: investigate why the performance of flora4 is so bad - and why no dis get added to PDES and NDES
//investigate how an item in ADES gets generalized despite the tight constraints

public class FloraClassifier extends OnLineClassifier implements CaseBasedAlgorithm{
	static final long serialVersionUID = 100;
	protected Hypothesis currentHypothesis = new Hypothesis();
	protected final int INITIAL_WINDOW_SIZE = 10;
	protected final boolean OUTPUT_DEBUG = true; 
	protected boolean isWindowFull = false;
	protected Window instanceWindow = new Window(INITIAL_WINDOW_SIZE);
	protected int numberOfTerms = 5;
	//index of the flora algorithm to use - can disable window adjustment and hypothesis storage
	private static int FLORA_ALGORITHM_MODE = 4;
	public static int getFLORAMODE()
	{
		return FLORA_ALGORITHM_MODE;
	}
	private static double zValue = 1.28;//corresponds to a two-sided 80% confidence interval 
	//keeps the index of the instance currently processed
	private int instanceIndex = 0;
	//stores the last <numberOfTerms> instances used when training
	//used to adjust the window size during training
	protected ArrayList classificationResults;
	//a randomly generated set of instances which is fed in before each classifier update
	//it's used to compute the predictive accuracy metric which is used in the Window Adjustment Heuristic
	protected Instances predictiveAccuracyInstanceSet = null;
	protected Discretize discretizer;
	protected Remove removeFilter;
    protected Discretize discrete;
	protected ArrayList hypothesisStore = new ArrayList();
    
	public FloraClassifier()
	{
	}
    
	public static double getConfidenceInterval(double successes, double trials)
	{
		double p = successes/trials;
		double nominator = p + (Math.pow(zValue,2.0)/(2*trials)) + 
				((zValue / Math.sqrt(trials))*Math.sqrt((p*(1.0-p)) + (Math.pow(zValue,2.0)/(4*trials))));
		return round(nominator / (1.0 + ((Math.pow(zValue,2.0))/trials)),2);
	}
    /**
     * Round a double value to a specified number of decimal 
     * places.
     *
     * @param val the value to be rounded.
     * @param places the number of decimal places to round to.
     * @return val rounded to places decimal places.
     */
    public static double round(double val, int places) 
    {
		long factor = (long)Math.pow(10,places);
	
		// Shift the decimal the correct number of places
		// to the right.
		val = val * factor;
	
		// Round to the nearest integer.
		long tmp = Math.round(val);
	
		// Shift the decimal the correct number of places
		// back to the left.
		return (double)tmp / factor;
    }
    
   

	/**
	 * Returns a prediction of the class for an instance
	 */
	public double classifyInstance(Instance instance) 
	{
		//remove first attribute and discretize
		removeFilter.input(instance);
		instance=removeFilter.output();
		//commented out in order to use the Flora3testGenerator
		//discretizer.input(instance);
		//instance=discretizer.output();
		
		//classifying instance
		double classValue = currentHypothesis.classifyInstance(instance);

		return classValue; 
	}
	/**
	 * Builds the classifier's target concept space representation
	 */
	public void buildClassifier(Instances data) throws Exception
	{
		//using a remove filter to remove the first (time) attribute
		removeFilter=new Remove();
		removeFilter.setInputFormat(data);
		removeFilter.setAttributeIndicesArray(new int[]{0});
		removeFilter.setInputFormat(data);
		removeFilter.batchFinished();
		data=Filter.useFilter(data,removeFilter);
        
        discrete=new Discretize(5);
        discrete.setInputFormat(data);
        
        Instances output=new Instances(discrete.outputFormat);
        for(int i=0;i<data.numInstances();i++){
            Instance in=discrete.convert(data.instance(i));
            in.setDataset(output);
            output.add(in);
        }
		data=output;
		//commented in order to use Flora3TestGenerator
		//discretizing double values into bins
		//discretizer=new Discretize();
		//discretizer.setBins(10);
		//discretizer.setInputFormat(data);
		//data=Filter.useFilter(data,discretizer);
		
		//outputting tuples in the training dataset
//		for(int i=0;i<data.numInstances();i++){
//			Instance in=data.instance(i);
//			for(int a=0;a<data.numAttributes();a++){
//				System.out.print(in.stringValue(a)+", ");
//			}
//			System.out.println();
//		}
		
		
		classificationResults = new ArrayList(numberOfTerms);
		//statusFileWriter.write("training\n");
		//goes through all the instances in the training set, learning the hypothesis
		Enumeration instanceEnum = data.enumerateInstances();
		while(instanceEnum.hasMoreElements())
		{
			//1.get accuracy of the hypothesis before we have learned from this instance
			Instance instance = ((Instance)instanceEnum.nextElement());

			//2. add instance to window
			instanceWindow.addInstance(instance);
			
			//compute the predictive accuracy
			//double predictiveAccuracy = getPredictiveAccuracyInTraining(instance);
            double predictiveAccuracy = getPredictiveAccuracyInTest(data);
			
			//3. learn from this instance 
			currentHypothesis.LearnFromInstance(instance, instanceWindow);
			//4. adjust the size of the window
			//drop instances only if more instances than the initial window size are stored
			//determine number of instances to be dropped from window
			AdjustWindow(predictiveAccuracy, instance);
			
			++instanceIndex;
		}
		//statusFileWriter.write("end of training\n");
		instanceIndex = 0;
	}
	/**
	 * Implements FLORA3 context changes
	 *
	 */
	public void ChangeContext(Instance instance) throws IOException
	{
		//check the status of the concept - stable or drifting, and store/load a hypothesis
		if(instanceWindow.getIsStable())//is stable
		{
			//1. store current hypothesis if a hypothesis with the same ADES is not already stored
			boolean isAlreadyStored = false;
			for(int hypothesisIndex = 0; hypothesisIndex < hypothesisStore.size(); ++hypothesisIndex)
			{
				Hypothesis hypothesis = ((Hypothesis)hypothesisStore.get(hypothesisIndex));
				if(hypothesis.IsIdenticalTo(currentHypothesis))
				{
					isAlreadyStored = true;
					break;
				}
			}
			//store hypothesis if not already stored
			if(!isAlreadyStored)
			{
				hypothesisStore.add(currentHypothesis.clone());
				if(OUTPUT_DEBUG)
				{
					//statusFileWriter.write("Saved hypothesis to store.\n");
					//statusFileWriter.write(getRulesString());
				}
			}
		}
		//concept drift is suspected
		else if(instanceWindow.getIsDrifting() && hypothesisStore.size() > 0)//is drifting
		{
			//1.get candidate hypotheses - only hypotheses consistent with the current example
			//get the best candidate with respect to the ratio of positive and negative instances
			//it matches from the current window
			int b;
			if(instanceIndex == 39)
				b=10;
			
			ArrayList candidateHypotheses = new ArrayList();
			for(int hypothesisIndex=0; hypothesisIndex < hypothesisStore.size(); ++hypothesisIndex)
			{
				Hypothesis hypothesis = (Hypothesis)((Hypothesis)hypothesisStore.get(hypothesisIndex)).clone();
				//hypothesis correctly covers current instance, therefore is a valid candidate
				if(!hypothesis.IsIdenticalTo(currentHypothesis) && hypothesis.correctlyCoversInstance(instance))
					candidateHypotheses.add(hypothesis);
			}
			//if none of the candidate hypotheses correctly covers the current instance continue
			if(candidateHypotheses.size() == 0)
				return;
			
			//1.1 process instances in the window with all candidate hypotheses
			//computing the ratio of positive and negative instances they cover
			double maxRatio = -1;
			Hypothesis bestCandidate = null;
			for(int hypothesisIndex = 0; hypothesisIndex < candidateHypotheses.size(); ++hypothesisIndex)
			{
				Hypothesis hypothesis = ((Hypothesis)candidateHypotheses.get(hypothesisIndex));
				double ratioCorrectlyCoveredWindowInstaces = 0;
				//evaluating ratio of instances covered in the current window
				for(int instanceIndex=0; instanceIndex < instanceWindow.getInstanceCount(); ++instanceIndex)
				{
					Instance windowInstance = ((Instance)instanceWindow.getInstances().get(instanceIndex));
					if(hypothesis.correctlyCoversInstance(windowInstance))
						ratioCorrectlyCoveredWindowInstaces += 1;
				}
				ratioCorrectlyCoveredWindowInstaces /= instanceWindow.getInstances().size();
				//if hypothesis covers more window instances than the stored one, store it
				if( ratioCorrectlyCoveredWindowInstaces > maxRatio)
				{
					bestCandidate = hypothesis;
					maxRatio = ratioCorrectlyCoveredWindowInstaces;
				}
			}
			//2.update the best candidate hypothesis
			//set all counters in all DES to 0 and reprocess the window
			bestCandidate.resetInstancesCoveredCounters();
			//reprocessing window instances
			for(int instanceIndex=0; instanceIndex < instanceWindow.getInstanceCount(); ++instanceIndex)
			{
				Instance windowInstance = ((Instance)instanceWindow.getInstances().get(instanceIndex));
				bestCandidate.LearnFromInstance(windowInstance, instanceWindow);
			}
			
			//remove description items which do not cover any instances in the window
			bestCandidate.removeZeroCoverageDescriptionItems();
			
			//3.compare the updated best candidate to the current concept description
			//select the hypothesis in terms of complexity of the description - 
			//with a more concise ADES set - literal count
			double bestCandidateLiteralCount = bestCandidate.getADESLiteralCount();
			double currentLiteralCount = currentHypothesis.getADESLiteralCount();
			if(bestCandidateLiteralCount < currentLiteralCount)
			{
				currentHypothesis = ((Hypothesis)bestCandidate.clone());
//				if(OUTPUT_DEBUG)
//					statusFileWriter.write("Loaded hypothesis from store.\n");
			}
			else
			{
//				if(OUTPUT_DEBUG)
//					statusFileWriter.write("Didn't load a hypothesis from store because ADES count is bigger.Old HYPO ADI#=" + 
//						String.valueOf(currentLiteralCount) + " new HYPO ADI#=" + String.valueOf(bestCandidateLiteralCount) + " \n");
			}
		}
	}
	/**
	 * Sets the randomly generated predictive accuracy test set
	 * @param instances Instances to be used to compute the predictive accuracy of the current hypothesis
	 */
	public void setPredictiveAccuracyInstances(Instances instances)
	{
		predictiveAccuracyInstanceSet = instances;
	}
	/**
	 * Returns the description items of the description sets in the current hypothesis in string format
	 * @return A string containing all the description items in all description sets of the current hypothesis
	 */
	public String getRulesString()
	{
		String result = "";
		//outputting ades items
		result += "\n===================================\n";
		result += "\nWindow contains:" + String.valueOf(instanceWindow.getInstanceCount()) + " instances";
		result += "\nPositive instances: " + String.valueOf(instanceWindow.getCurrentPositiveInstanceCount()) + " instances";
		result += " Negative instances: " + String.valueOf(instanceWindow.getCurrentNegativeInstanceCount()) + " instances";
		result += "\n===================================\n\n";
		result += "ADES: " + String.valueOf(currentHypothesis.getADES().getDesciptionItems().size())  + " Description Items\n";
		for(int index=0; index < currentHypothesis.getADES().getDesciptionItems().size(); ++index)
		{
			result += "ADES Item No:" + String.valueOf(index) + "\n";;
			result += (((DescriptionItem)currentHypothesis.getADES().getDesciptionItems().get(index)).printItem());
		}
		result += "\n===================================\n\n";
		
		//outputting ndes items
		result += "NDES: " + String.valueOf(currentHypothesis.getNDES().getDesciptionItems().size())  + " Description Items\n";
		for(int index=0; index < currentHypothesis.getNDES().getDesciptionItems().size(); ++index)
		{
			result += "NDES Item No:" + String.valueOf(index) + "\n";
			result += ((DescriptionItem)currentHypothesis.getNDES().getDesciptionItems().get(index)).printItem();
		}
		result += "\n===================================\n\n";
		
		//outputting pdes items
		result += "PDES: " + String.valueOf(currentHypothesis.getPDES().getDesciptionItems().size())  + " Description Items\n";
		
		for(int index=0; index < currentHypothesis.getPDES().getDesciptionItems().size(); ++index)
		{
			result += "PDES Item No:" + String.valueOf(index) + "\n";;
			result += (((DescriptionItem)currentHypothesis.getPDES().getDesciptionItems().get(index)).printItem());
		}
		result += "\n===================================\n";
		
		return result;
	}
	/**
	 * Returns the predictive accuracy of the hypothesis on a specific number of last items 
	 * @param instance
	 * @return Predictive accuracy of the hypothesis on a specific number of last items
	 */
	public double getPredictiveAccuracyInTraining(Instance instance)
	{
		int isCorrect = 0;
		double classValue = currentHypothesis.classifyInstance(instance);
		if( (classValue > 0 && instance.classValue() > 0) || (classValue <= 0 && instance.classValue() <= 0))
			isCorrect = 1;
		
		classificationResults.add(new Integer(isCorrect));
		
		//compute predictive accuracy only over a specific number of old instances
		if(classificationResults.size() == instanceWindow.getInstanceCount())
			classificationResults.remove(0);
		
		int totalCorrectInSet = 0;
		for(int i=0; i < classificationResults.size(); ++i)
			totalCorrectInSet += Integer.parseInt(classificationResults.get(i).toString());

		return ((double)totalCorrectInSet) / (double)classificationResults.size();
	}
	/**
	 * Returns the predictive accuracy of the hypothesis on randomly generated data
	 * @param instances
	 * @return Predictive accuracy of the hypothesis on the set of instances
	 */
	public double getPredictiveAccuracyInTest(Instances instances)
	{
		int correctCount = 0;
		int totalCount = 0;
		Enumeration instanceEnum = instances.enumerateInstances();
		while(instanceEnum.hasMoreElements())
		{
			++totalCount;
			Instance instance = ((Instance)instanceEnum.nextElement());
			
//			removeFilter.input(instance);
//			instance=removeFilter.output();
			//commented out in order to use the Flora3testGenerator
			//discretizer.input(instance);
			//instance=discretizer.output();
			
			double classification = currentHypothesis.classifyInstance(instance);
			//add a 1 in case the instance was correctly classified, 0 otherwise
			if((classification > 0 && instance.classValue() > 0) ||
					(classification <= 0 && instance.classValue() <= 0))
				++correctCount;
			
		}
		
		//returns the correct classification ratio for the last instances
		return (double)(correctCount)/(double)(totalCount);
	}

	public void setOperativeAttributes(List usedAttributeList) {
		// TODO Auto-generated method stub
		
	}
	/**
	 * Update classfier, dropping instances from window as determined by the FLORA2 heuristic
	 */
	public void updateClassifier(Instance instance) 
	{
		// TODO Auto-generated method stub
		removeFilter.input(instance);
        
		instance=removeFilter.output();
        
        instance=discrete.convert(instance);
        
		//commented out in order to use the Flora3testGenerator
		//discretizer.input(instance);
		//instance=discretizer.output();
        
        
//      if(predictiveAccuracyInstanceSet == null)
//      System.out.println("updateClassifier: predictiveAccuracyInstanceSet is null");
//  //determine number of instances to be dropped from window
//  double predictiveAccuracy = getPredictiveAccuracyInTest(predictiveAccuracyInstanceSet);
    double predictiveAccuracy = getPredictiveAccuracyInTraining(instance);
        
        

		//adding instance to window
		instanceWindow.addInstance(instance);
		//learing from instance
		currentHypothesis.LearnFromInstance(instance, instanceWindow);


		
		AdjustWindow(predictiveAccuracy, instance);
		
		++instanceIndex;
	}
	/**
	 * Implements the Window Adjustment Heuristic
	 * @param predictiveAccuracy
	 */
	public void AdjustWindow(double predictiveAccuracy, Instance instance)
	{
		
		int toBeForgottenCount = 1;
		//make sure the window fills up at the start of the algorithm
		if(instanceWindow.getInstanceCount() == INITIAL_WINDOW_SIZE)
			isWindowFull = true;
		
		if(!isWindowFull)
			toBeForgottenCount = 0;
		if(isWindowFull && FLORA_ALGORITHM_MODE > 1)
		{
			toBeForgottenCount = instanceWindow.getInstancesToBeForgottenCount(
				currentHypothesis.getADES().getInstancesCoveredCount(true),
				currentHypothesis.getADESLiteralCount(),
				instanceWindow.getInstanceCount(), predictiveAccuracy);
		}

		try
		{
//			statusFileWriter.write(
//					String.valueOf(instanceIndex) + ", " 
//			    + String.valueOf(instance.classValue() > 0 ? "+" : "-") + ", "
//				+ String.valueOf(toBeForgottenCount) + ", " 
//				+ String.valueOf(instanceWindow.getInstanceCount()) + ", " 
//				+ String.valueOf(round(predictiveAccuracy,2)) 
//				+ ", drift:" + instanceWindow.getIsDrifting() 
//				+ ", stable:" + instanceWindow.getIsStable()
//				+ ", ADIC: " + currentHypothesis.getADES().getDesciptionItems().size()
//				+ ", PDIC: " + currentHypothesis.getPDES().getDesciptionItems().size()
//				+ ", NDIC: " + currentHypothesis.getNDES().getDesciptionItems().size()
//				+ "\n");
			
			if(instanceWindow.getInstanceCount() >= toBeForgottenCount)//instanceWindow.getSize())
			{
				//dropping instances from window
				for(int i=0; i < toBeForgottenCount; ++i)
				{
					Instance droppedInstance = instanceWindow.getDroppedInstance();
					currentHypothesis.ForgetInstance(droppedInstance, instanceWindow);
				}
			}
			//implements flora3 - hypothesis storage/retrieval
			if(FLORA_ALGORITHM_MODE > 2)
				ChangeContext(instance);
		}
		catch(Exception ex)
		{
			ex.printStackTrace(System.out);
		}
	}

	public List getTrainInstances() {
		// TODO Auto-generated method stub
		return instanceWindow.getInstances();
	}

	public int getTrainInstancesCount() {
		return instanceWindow.getInstanceCount();
	}

	public double classifyIB(Instance instance, int k) {
		// TODO Auto-generated method stub
		return 0;
	}
}