package weka.classifiers.rules.flora;
import java.util.*;
import weka.core.Instance;
public class Window {
	private ArrayList m_instanceWindow = new ArrayList();
	private int m_size;
	private boolean m_isDrifting;
	private boolean m_isStable;
	protected double oldPredictiveAccuracy = 0;
	private int m_cumulativePositiveInstanceCount = 0;
	private int m_cumulativeNegativeInstanceCount = 0;
	private int m_currentPositiveInstanceCount = 0;
	private int m_currentNegativeInstanceCount = 0;
	private int m_instancesProcessedCount = 0;
	public int getCurrentPositiveInstanceCount()
	{
		return m_currentPositiveInstanceCount;
	}
	public int getCurrentNegativeInstanceCount()
	{
		return m_currentNegativeInstanceCount;
	}
	public double getPositiveGammaU()
	{
		return FloraClassifier.getConfidenceInterval(m_cumulativePositiveInstanceCount, m_instancesProcessedCount);
	}
	public double getPositiveGammaL()
	{
		return ((double)m_cumulativePositiveInstanceCount / (double)m_instancesProcessedCount) - 
				(
				FloraClassifier.getConfidenceInterval(m_cumulativePositiveInstanceCount, m_instancesProcessedCount) - 
				((double)m_cumulativePositiveInstanceCount / (double)m_instancesProcessedCount)
				);
	}
	
	public double getNegativeGammaU()
	{
		return FloraClassifier.getConfidenceInterval(m_cumulativeNegativeInstanceCount, m_instancesProcessedCount);
	}
	public double getNegativeGammaL()
	{
		return ((double)m_cumulativeNegativeInstanceCount / (double)m_instancesProcessedCount) - 
				(
				FloraClassifier.getConfidenceInterval(m_cumulativeNegativeInstanceCount, m_instancesProcessedCount) - 
				((double)m_cumulativeNegativeInstanceCount / (double)m_instancesProcessedCount)
				);
	}
	
	public boolean getIsStable()
	{
		return m_isStable;
	}
	public boolean getIsDrifting()
	{
		return m_isDrifting;
	}
	private int m_instanceCount;
	/**
	 * Returns the static size of the window
	 * @return
	 */
	public int getInitialSize()
	{
		return m_size;
	}
	/**
	 * Returns the instances in the window
	 * @return
	 */
	public ArrayList getInstances()
	{
		return m_instanceWindow;
	}
	/**
	 * Returns the number of instances in the window
	 * @return
	 */
	public int getInstanceCount()
	{
		return m_instanceCount;
	}
	public Window(int initialSize)
	{
		m_size = initialSize;
		m_instanceCount = 0;
		m_instanceWindow = new ArrayList();
	}
	/**
	 * Removes and returns the instance to be dropped from the window
	 * @return
	 */
	public Instance getDroppedInstance()
	{
		--m_instanceCount;
		Instance droppedInstance = ((Instance)m_instanceWindow.remove(0));
		//decrement the appropriate count of +/- instances
		if(droppedInstance.classValue() > 0)
			--m_currentPositiveInstanceCount;
		else
			--m_currentNegativeInstanceCount;
		return droppedInstance;
	}
	/**
	 * Adds an instance to the window
	 * @param instance
	 */
	public void addInstance(Instance instance)
	{
		//increment the total number of instances
		++m_instancesProcessedCount;
		//increment the appropriate count of +/- instances
		if(instance.classValue() > 0)
		{
			++m_cumulativePositiveInstanceCount;
			++m_currentPositiveInstanceCount;
		}
		else
		{
			++m_cumulativeNegativeInstanceCount;
			++m_currentNegativeInstanceCount;
		}
		//increment current instance count
		++m_instanceCount;
		m_instanceWindow.add(instance);
	}
	/**
	 * Returns the total number of literals over all description items in ADES
	 * @return
	 */
	public int getInstancesToBeForgottenCount(double positiveInstancesCoveredCount, double ADESLiteralCount, 
			int windowSize, double predictedAccuracy)
	{
		double LC = 1.2;
		double HC = 4.0;
		double minAcceptablePredictiveAccuracy = 0.7;
		//predictedAccuracy - ratio of positively classified instances of an independently generated dataset labelled according to the concept
		//default: grow window by 1
		int instancesToDrop = 0;
		//drift is suspected - reduce window size by 20%
		m_isDrifting = false;
		m_isStable = false;
		if( positiveInstancesCoveredCount / ADESLiteralCount < LC || (predictedAccuracy < minAcceptablePredictiveAccuracy && oldPredictiveAccuracy > predictedAccuracy))
		{
			instancesToDrop = (int)Math.round(0.2 * windowSize);
			m_isDrifting = true;
		}
		//extremely stable - reduce window size by 1
		else if( positiveInstancesCoveredCount / ADESLiteralCount > 2 * HC && predictedAccuracy > minAcceptablePredictiveAccuracy)
		{
			instancesToDrop = 2;
			m_isStable = true;
		}
		//stable enough - keep window size fixed
		else if( positiveInstancesCoveredCount / ADESLiteralCount > HC && predictedAccuracy > minAcceptablePredictiveAccuracy)
		{
			instancesToDrop = 1;
			m_isStable = true;
		}
		
		//update the predicted accuracy metric
		oldPredictiveAccuracy = predictedAccuracy;
		return instancesToDrop;
	}
}
