package weka.classifiers.trees;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import weka.classifiers.Classifier;
import weka.core.Instance;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.supervised.attribute.Discretize;

public class Copy_2_of_LAT extends Classifier implements Serializable {

    protected static class Split implements Serializable {
        public int attribut;

        public int value;

        public Instances data;

        public double[] instDistrib;

        public double[] sortDistrib;

        public int[] usedAtts;

    }

    private static final long serialVersionUID = 695872586476417759L;

    protected Instances data;

    protected int attrCount;

    protected int classCount;

    protected int classIndex;

    protected Split split;

    protected int[] attrValCount;
    
    protected Discretize disc;
    
    public void buildClassifier(Instances builddata) throws Exception {
        disc = new Discretize();
        disc.setInputFormat(builddata);
        this.data = Filter.useFilter(builddata, disc);
        this.data = new Instances(data);
        classIndex = data.classIndex();
        classCount = data.attribute(data.classIndex()).numValues();
        attrCount = data.numAttributes();
        attrValCount = new int[attrCount];
        for (int a = 0; a < attrCount; a++) {
            attrValCount[a] = data.attribute(a).numValues();
        }
        split = new Split();
        split.data = data;
        split.attribut = -1;
        split.usedAtts = new int[attrCount];
        split.instDistrib = getClassDistribution(data);
        split.sortDistrib = split.instDistrib.clone();
        Arrays.sort(split.sortDistrib);
    }

    public double[] distributionForInstance(Instance inst) {
        
        disc.input(inst);
        inst=disc.output();
//        System.out.println("\n");
        return buildNode(split, inst);
    }

    protected double[] buildNode(Split split, Instance inst) {
        Split[] splits = getSplits(split, inst);
        if (splits == null || splits.length == 0) {
            double[] dist = split.instDistrib;
            return dist;
        }
        int sc = splits.length;
        double[] dist = new double[classCount];
        //System.out.println("");
        for (int i = 0; i < sc; i++) {
//            System.out.println("  attribut=" + splits[i].attribut + "  value="
//                  + splits[i].value);
            double[] subDist = buildNode(splits[i], inst);
            for (int c = 0; c < classCount; c++) {
                dist[c] = dist[c] + (subDist[c] / sc);
            }
        }
        return dist;
    }

    protected double[] getClassDistribution(Instances data) {
        double[] dist = new double[classCount];
        for (int i = 0; i < data.numInstances(); i++) {
            dist[(int) data.instance(i).classValue()]++;
        }
        for (int i = 0; i < classCount; i++) {
            dist[i] = dist[i] / data.numInstances();
        }
        return dist;
    }

    protected double[] getWeighedClassDistribution(Instances data,
            double[] preDist) {
        double[] dist = new double[classCount];
        double sum = 0;
        for (int i = 0; i < data.numInstances(); i++) {
            int cl = (int) data.instance(i).classValue();
            dist[cl] += 1.0 / preDist[cl];
            sum += 1.0 / preDist[cl];
        }
        for (int i = 0; i < classCount; i++) {
            dist[i] = dist[i] / sum;
        }
        return dist;
    }

    protected Split[] getSplits(Split lastSplit, Instance inst) {
        List<Split> list = new ArrayList();
        Instances data = lastSplit.data;
        double[] weightdist = lastSplit.instDistrib;
        for (int i = 0; i < attrCount; i++) {
            if (lastSplit.usedAtts[i] >= 1 || i == classIndex
                    || inst.isMissing(i))
                continue;

            int n = attrValCount[i];
            int[][] distrib = new int[n + 1][classCount];
            int[] sum = new int[n + 1];
            int in = data.numInstances();
            int iValue =-1;
            if (inst.isMissing(i)) iValue=n;
            else iValue=(int) inst.value(i);

            for (int nr = 0; nr < in; nr++) {
                Instance instance = data.instance(nr);
                if (instance.isMissing(i)) {
                    distrib[n][(int) instance.classValue()]++;
                    sum[n]++;
                } else {
                    int val = (int) instance.value(i);
                    distrib[val][(int) instance.classValue()]++;
                    sum[val]++;
                }
            }
            for (int v = 0; v < n; v++) {
                if (v != iValue || sum[v] == 0 || sum[v] == data.numInstances())
                    continue;
                Split sp = new Split();

                sp.attribut = i;
                sp.value = v;
                double gain = 0;
                boolean insert = true;

                double[] dv2 = new double[classCount];
                double sum2 = 0;
                for (int a = 0; a <= n; a++) {
                    if (a == v) {
                        for (int c = 0; c < classCount; c++) {
                            if (weightdist[c] > 0) {
                                dv2[c] += distrib[a][c] * 1 / weightdist[c];
                                sum2 += distrib[a][c] * 1 / weightdist[c];
                            }
                        }
                    }
                }
                for (int c = 0; c < classCount; c++) {
                    dv2[c] /= sum2;
                }
                Arrays.sort(dv2);
                sp.sortDistrib = dv2;

                Iterator<Split> it = list.iterator();
                while (insert && it.hasNext()) {
                    Split s = it.next();
                    if (pref(s.sortDistrib, sp.sortDistrib)) {
                        insert = false;
                        break;
                    }
                    if (pref(sp.sortDistrib, s.sortDistrib)) {
                        it.remove();
                    }
                }
                if (insert) {
                    sp.usedAtts = lastSplit.usedAtts.clone();
                    sp.usedAtts[i]++;
                    list.add(sp);
                }
            }
        }
        Iterator<Split> it = list.iterator();
        while (it.hasNext()) {
            Split newSplit = it.next();
            if (newSplit != null) {
                newSplit.data = new Instances(data, 0);
                int in = data.numInstances();
                for (int nr = 0; nr < in; nr++) {
                    Instance instance = data.instance(nr);
                    if (!instance.isMissing(newSplit.attribut)
                            && (int) instance.value(newSplit.attribut) == newSplit.value) {
                        newSplit.data.add(instance);
                    }
                    newSplit.instDistrib = getClassDistribution(newSplit.data);
                }
            }
        }
        Split[] splits = new Split[list.size()];
        splits = list.toArray(splits);
        return splits;
    }

    public void weight(double[] dist1, double[] normdist) {
        double sum = 0;
        for (int i = 0; i < classCount; i++) {
            double val = dist1[i] / normdist[i];
            sum += val;
            dist1[i] = val;
        }
        for (int i = 0; i < classCount; i++) {
            dist1[i] /= sum;
        }
    }

    public boolean pref(double[] dist1, double[] dist2) {
        double sum1 = 0;
        double sum2 = 0;
        for (int c = classCount-1; c >=0; c--) {
            sum1 += dist1[c];
            sum2 += dist2[c];
            if (sum2 > sum1)
                return false;
        }
        return true;
    }

}
