package weka.classifiers.trees;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import weka.classifiers.Classifier;
import weka.core.Instance;
import weka.core.Instances;

public class LAT extends Classifier implements Serializable {

    protected static class Split implements Serializable {
        private static final long serialVersionUID = 4552136387032540913L;

        public int attribut;

        public int value;

        public Instances data;

        public double[] instDistrib;

        public double[] sortDistrib;

        public int[] usedAtts;

    }

    private static final long serialVersionUID = 695872586476417759L;

    protected Instances data;

    protected int attrCount;

    protected int classCount;

    protected int classIndex;

    protected Split split;

    protected int[] attrValCount;

    protected int treeCount = 5;

    public void buildClassifier(Instances data) throws Exception {
        this.data = new Instances(data);
        classIndex = data.classIndex();
        classCount = data.attribute(data.classIndex()).numValues();
        attrCount = data.numAttributes();
        attrValCount = new int[attrCount];
        for (int a = 0; a < attrCount; a++) {
            attrValCount[a] = data.attribute(a).numValues();
        }
        split = new Split();
        split.data = data;
        split.attribut = -1;
        split.usedAtts = new int[attrCount];
        split.instDistrib = getClassDistribution(data);
        split.sortDistrib = split.instDistrib.clone();
        Arrays.sort(split.sortDistrib);
    }

    public double[] distributionForInstance(Instance inst) {
        //System.out.println();
        double[] dist = new double[classCount];
        for (int i = 0; i < treeCount; i++) {
            double[] subdist = buildNode(split, inst,0);
            for (int c = 0; c < classCount; c++) {
                dist[c] += subdist[c] / treeCount;
            }
            //System.out.print(i);
        }
        return dist;
    }

    protected double[] buildNode(Split split, Instance inst, int dep) {
        Split[] splits = getSplits(split, inst);
        if (splits == null || splits.length == 0) {
            System.out.println(dep);
            double[] dist = split.instDistrib;
            return dist;
        }
        int sc = splits.length;
        //System.out.print(sc+" ");
        int sel = (int) (Math.random() * sc);
        
        splits[sel].data = new Instances(split.data, 0);
        int in = split.data.numInstances();
        for (int nr = 0; nr < in; nr++) {
            Instance instance = split.data.instance(nr);
            if (instance.isMissing(splits[sel].attribut)
                    || (int) instance.value(splits[sel].attribut) != splits[sel].value) {
                splits[sel].data.add(instance);
            }
        }
        splits[sel].instDistrib = getClassDistribution(splits[sel].data);
        
        // System.out.println(" attribut=" + splits[sel].attribut + " value="
        // + splits[sel].value);
        return buildNode(splits[sel], inst,dep+1);
    }

    protected double[] getClassDistribution(Instances data) {
        double[] dist = new double[classCount];
        for (int i = 0; i < data.numInstances(); i++) {
            dist[(int) data.instance(i).classValue()]++;
        }
        for (int i = 0; i < classCount; i++) {
            dist[i] = dist[i] / data.numInstances();
        }
        return dist;
    }

    protected Split[] getSplits(Split lastSplit, Instance inst) {
        List<Split> list = new ArrayList();
        Instances data = lastSplit.data;
        double[] weightdist = lastSplit.instDistrib;

        int[][][] distrib = new int[attrCount][][];
        int[][] sum = new int[attrCount][];
        for (int i = 0; i < attrCount; i++) {
            if (lastSplit.usedAtts[i] >= attrValCount[i] - 1 || i == classIndex
                    || inst.isMissing(i))
                continue;
            distrib[i] = new int[attrValCount[i] + 1][classCount];
            sum[i] = new int[attrValCount[i] + 1];
        }
        int in = data.numInstances();

        for (int nr = 0; nr < in; nr++) {
            Instance instance = data.instance(nr);
            for (int i = 0; i < attrCount; i++) {
                if (lastSplit.usedAtts[i] >= attrValCount[i] - 1
                        || i == classIndex || inst.isMissing(i))
                    continue;
                if (instance.isMissing(i)) {
                    distrib[i][attrValCount[i]][(int) instance.classValue()]++;
                    sum[i][attrValCount[i]]++;
                } else {
                    int val = (int) instance.value(i);
                    distrib[i][val][(int) instance.classValue()]++;
                    sum[i][val]++;
                }
            }
        }

        for (int i = 0; i < attrCount; i++) {
            if (lastSplit.usedAtts[i] >= attrValCount[i] - 1 || i == classIndex
                    || inst.isMissing(i))
                continue;

            int n = attrValCount[i];
            int iValue = (int) inst.value(i);
            for (int v = 0; v < n; v++) {
                if (v == iValue || sum[i][v] == 0
                        || sum[i][v] == data.numInstances())
                    continue;
                Split sp = new Split();

                sp.attribut = i;
                sp.value = v;
                boolean insert = true;

                double[] dv2 = weightdist.clone();
                double sum2 = 0;
                int a = v;
                for (int c = 0; c < classCount; c++) {
                    if (weightdist[c] > 0) {
                        dv2[c] = (dv2[c] * in - distrib[i][a][c]) * 1
                                / weightdist[c];
                        sum2 += dv2[c];
                    }
                }
                for (int c = 0; c < classCount; c++) {
                    dv2[c] /= sum2;
                }
                Arrays.sort(dv2);
                sp.sortDistrib = dv2;

                Iterator<Split> it = list.iterator();
                while (insert && it.hasNext()) {
                    Split s = it.next();
                    if (pref(s.sortDistrib, sp.sortDistrib)) {
                        insert = false;
                        break;
                    }
                    if (pref(sp.sortDistrib, s.sortDistrib)) {
                        it.remove();
                    }
                }
                if (insert) {
                    sp.usedAtts = lastSplit.usedAtts.clone();
                    sp.usedAtts[i]++;
                    list.add(sp);
                }
            }
        }
//        Iterator<Split> it = list.iterator();
//        while (it.hasNext()) {
//            Split newSplit = it.next();
//            if (newSplit != null) {
//                newSplit.data = new Instances(data, 0);
//                int instCount = data.numInstances();
//                for (int nr = 0; nr < instCount; nr++) {
//                    Instance instance = data.instance(nr);
//                    if (instance.isMissing(newSplit.attribut)
//                            || (int) instance.value(newSplit.attribut) != newSplit.value) {
//                        newSplit.data.add(instance);
//                    }
//                }
//                newSplit.instDistrib = getClassDistribution(newSplit.data);
//            }
//        }
        Split[] splits = new Split[list.size()];
        splits = list.toArray(splits);
        return splits;
    }

    public void weight(double[] dist1, double[] normdist) {
        double sum = 0;
        for (int i = 0; i < classCount; i++) {
            double val = dist1[i] / normdist[i];
            sum += val;
            dist1[i] = val;
        }
        for (int i = 0; i < classCount; i++) {
            dist1[i] /= sum;
        }
    }

    public boolean pref(double[] dist1, double[] dist2) {
        double sum1 = 0;
        double sum2 = 0;
        for (int c = 0; c < classCount; c++) {
            sum1 += dist1[c];
            sum2 += dist2[c];
            if (sum2 > sum1)
                return false;
        }
        return true;
    }

}
