package weka.classifiers.trees;

import weka.classifiers.Classifier;
import weka.core.Instance;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;

public class SampleForest extends Classifier {

    /**
     * 
     */
    private static final long serialVersionUID = -7432525881218566151L;

    protected J48[] trainedClassifier;

    protected Remove[] removeFilters;

    protected double sub=0.5;

    protected int subspace;

    protected int classifierCount=100;

    protected Instances data;

    protected int classIndex;

    protected int classCount;

    protected int attrCount;
    
    
    public void SampleForest(){
        
    }

    public void buildClassifier(Instances data) throws Exception {
        this.data = new Instances(data);
        classIndex = data.classIndex();
        classCount = data.attribute(data.classIndex()).numValues();
        attrCount = data.numAttributes();

        if (sub < 1)
            subspace = (int) ((attrCount - 1) * sub);
        else
            subspace = Math.min(attrCount - 1, (int) (sub));

        trainedClassifier = new J48[classifierCount];
        removeFilters = new Remove[classifierCount];
        for (int i = 0; i < classifierCount; i++) {
            int[] sel = getSelected();
            removeFilters[i] = new Remove();
            removeFilters[i].setInvertSelection(true);
            removeFilters[i].setAttributeIndicesArray(sel);
            removeFilters[i].setInputFormat(data);
            Instances subData = Filter.useFilter(data, removeFilters[i]);
            trainedClassifier[i] = new J48();
            trainedClassifier[i].buildClassifier(subData);
        }

    }

    public double[] distributionForInstance(Instance inst) {
        try {
            double[] dist = new double[classCount];
            for (int i = 0; i < classifierCount; i++) {
                removeFilters[i].input(inst);
                Instance instc = removeFilters[i].output();
                double[] subdist;
                subdist = trainedClassifier[i].distributionForInstance(instc);

                for (int c = 0; c < classCount; c++) {
                    dist[c] += subdist[c] / classifierCount;
                }
            }

            return dist;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    protected int[] getSelected() {
        int a = 0;
        for (int c = 0; c < attrCount; c++) {
            if (c != classIndex)
                a++;
        }
        int count = Math.min(a, subspace);
        boolean[] select = new boolean[attrCount];
        for (int i = 0; i < count; i++) {
            int sel = -1;
            do {
                sel = (int) (Math.random() * attrCount);
            } while (sel == classIndex || select[sel]);
            select[sel] = true;
        }
        int[] result = new int[count + 1];
        int p = 0;
        for (int c = 0; c < attrCount; c++) {
            if (select[c]) {
                result[p] = c;
                p++;
            }
        }
        result[count] = classIndex;
        return result;
    }

    

    public Classifier getClassifier() {
        return null;//classifier;
    }

    public void setSubspace(double subspace) {
        this.sub = subspace;
    }

    public double getSubspace() {
        return sub;
    }

    public void setClassifierCount(int c) {
        this.classifierCount = c;
    }

    public int getClassifierCount() {
        return classifierCount;
    }

}
