package weka.datagenerators;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Properties;
import java.util.StringTokenizer;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;

public class FileData extends AdaptiveGenerator {

    public BufferedReader read;

    public int attCount;

    public int clsCount;

    public Instances header;

    public FileData(Reader read, int attCount, int clsCount) {
        this.read = new BufferedReader(read);
        this.attCount = attCount;
        this.clsCount = clsCount;
        reset();
    }

    public void update() {

    }

    @Override
    protected Instance generate(int time) {
        // TODO Auto-generated method stub
        return generateExample(time);
    }

    public int getLabel(Instance point) {
        return (int) point.classValue();
    }

    public int getNoiseLabel(Instance point) {
        return (int) point.classValue();
    }

    public void reset() {
        // try {
        // read.reset();

        FastVector inf = new FastVector(2 + attCount);
        Properties prop = new Properties();
        prop.setProperty("range", "[0,inf]");

        inf.add(new Attribute("Time", new ProtectedProperties(prop)));

        for (int i = 0; i < attCount; i++) {
            prop.setProperty("range", "[0,1]");
            inf.add(new Attribute("X" + i + "-val", new ProtectedProperties(
                    prop)));
        }
        FastVector clVal = new FastVector(clsCount);
        for (int i = 0; i < clsCount; i++) {
            clVal.add("class " + i);
        }
        inf.add(new Attribute("class", clVal));

        header = new Instances("Data", inf, 0);
        header.setClassIndex(attCount + 1);

        // } catch (IOException e) {
        // e.printStackTrace();
        // }
    }

    @Override
    public Instances getInitData(int count) {

        Instances insts = new Instances(header, 0);
        for (int i = 0; i < count; i++) {
            insts.add(generateExample(i));
        }
        return insts;
    }

    @Override
    public Instance generateExample(int time) {

        try {
            double[] val = new double[attCount + 2];
            val[0] = time;
            String line = read.readLine();
            StringTokenizer toks = new StringTokenizer(line, " ,;\t");
            for (int i = 0; i < attCount; i++) {
                val[i + 1] = Double.parseDouble(toks.nextToken());
            }
            val[attCount + 1] = Integer.parseInt(toks.nextToken());
            if (Math.random()<noise){
                int up=((int)Math.random()*(clsCount-1))+1;
                val[attCount + 1] =(val[attCount + 1]+up)%clsCount;
            }
            Instance inst = new Instance(1.0, val);
            inst.setDataset(header);
            return inst;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (NullPointerException e) {
            e.printStackTrace();
        }

        return null;
    }

    public Instances defineDataFormat() {
        return header;
    }

    @Override
    public Instance generateExample() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Instances generateExamples() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String generateFinished() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean getSingleModeFlag() {
        // TODO Auto-generated method stub
        return false;
    }

}
