package weka.datagenerators;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;
import cern.jet.random.Normal;
import edu.cornell.lassp.houle.RngPack.RandomJava;
import edu.cornell.lassp.houle.RngPack.Ranmar;

//nach Widmer Kubat "Adapting to drift in continous domains"
public class GenGAUSS extends AdaptiveGenerator {

    Random ran = new Random(17363421);
    Random ranData = new Random(355346237);
    Random ranModell = new Random(293546957);

    // Random ran2=new Random(317);

    public int dimension = 2;

    public boolean init = false;

    public long count = 0;

    protected int model = 0;

    protected int speed = 2000;

    protected Normal normal0;

    protected Normal normal1;
    
    protected Instances header;

    /*
     * public double dx=-0.1; public double dy=-0.075; public double dr=0.01;
     */

    public GenGAUSS() {
        reset();
    }
    public void reset() {
        reset(new Random());
    }
    public void reset(Random rnd) {
        ran = new Random(rnd.nextLong());
        ranData = new Random(rnd.nextLong());
        ranModell = new Random(rnd.nextLong());
        normal0 = new Normal(0, 4, new Ranmar(rnd.nextInt()));
        normal1 = new Normal(0, 1, new Ranmar(rnd.nextInt()));
        init = true;
        header= new Instances("GAUSS", getAttsInfo(), 0);
        header.setClassIndex(3);
    }

    public boolean isInitialised() {
        return init;
    }

    public void update() {
        if (count % speed == 0) {
            model = 1 - model;
        }
    }

    public double dist(double[] point, double[] middle) {
        double max = 0;
        for (int i = 0; i < dimension; i++) {
            // max=Math.max(max,Math.abs(point[i+1]-middle[i]));
            double val = point[i + 1] - middle[i];
            max += val * val;
        }
        return Math.sqrt(max);
    }

    public int getNumClasses() {
        return 2;
    }

    protected Instance generate(int time) {
        double[] point = new double[dimension + 3];
        point[0] = time;
        if (ranData.nextDouble() > 0.5) {
            point[1] = normal0.nextDouble() + 2;
            point[2] = normal0.nextDouble();
            point[dimension + 1] = 0;
            point[dimension + 2] = 0;
        } else {
            point[1] = normal1.nextDouble();
            point[2] = normal1.nextDouble();
            point[dimension + 1] = 1;
            point[dimension + 2] = 1;
        }
        Instance res = new Instance(1.0, point);
        res.setDataset(header);
        res.setClassValue(getNoiseLabel(res));
        if (time!=0) count++;
        return res;
    }

    public int getLabel(Instance inst) {
        double[] point=inst.toDoubleArray();
        if (point.length > dimension + 2) {
            if (model == 0)
                return (int) point[dimension + 2];
            else
                return 1 - (int) point[dimension + 2];
        } else {
            int cls=0;
            if (point[1]*point[1]+point[2]*point[2]<1.7*1.7) cls=1;
            if (model == 1) cls=1-cls;
            return cls;
        }
    }

    public int getNoiseLabel(Instance point) {
        int cls = getLabel(point);
        if (ran.nextDouble() < noise) {
            cls = 1 - cls;
        }
        return cls;

    }

    public Instances getInitData(int count) {
        Instances insts = new Instances("ContData", getAttsInfo(), count);
        for (int i = 0; i < count; i++) {
            insts.add(generate(0));
        }
        insts.setClassIndex(3);
        return insts;
    }

    public Instance generateExample(int time) {
        update();
        Instance next=generate(time);
        next.setDataset(header);
        return next;
    }

    protected FastVector getAttsInfo() {
        FastVector inf = new FastVector(4);
        Properties prop=new Properties();
        prop.setProperty("range","[0,inf]");
        
        inf.add(new Attribute("Time",new ProtectedProperties(prop)));
        
        prop.setProperty("range","["+getMinimum(0)+","+getMaximum(0)+"]");
        inf.add(new Attribute("X-val",new ProtectedProperties(prop)));
        prop.setProperty("range","["+getMinimum(1)+","+getMaximum(1)+"]");
        inf.add(new Attribute("Y-val",new ProtectedProperties(prop)));
        FastVector clVal = new FastVector(2);
        clVal.add("out");
        clVal.add("in");
        inf.add(new Attribute("class", clVal));
        return inf;
    }

    public List producedata() {
        List data = new ArrayList(5000);
        for (int i = 0; i < 200; i++) {
            data.add(generate(0));
        }
        for (int i = 1; i < 2000; i++) {
            update();
            data.add(generate(i));
        }
        return data;
    }

    public void output(List data, PrintStream out) {
        Iterator it = data.iterator();
        while (it.hasNext()) {
            int[] dat = (int[]) it.next();
            for (int i = 0; i < dat.length; i++) {
                out.print(dat[i]);
                if (i + 1 < dat.length)
                    out.print(", ");
            }
            out.println();
        }
    }

    public double getMinimum(int dimension) {
        return -8;
    }

    public double getMaximum(int dimension) {
        return 10;
    }
    
    public Instances defineDataFormat(){
        return header;
    }

    public Instance generateExample(){
        return generateExample(0);
    }

    public Instances generateExamples(){
        return header;
    }

    public String generateFinished(){
        return "";
    }

    public boolean getSingleModeFlag(){
        return true;
    }

}
