package weka.datagenerators;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import utils.random.RngStream;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;

public class GeneratorEllipse extends AdaptiveGenerator {

	Random ran = new RngStream(new long[] { 654522, 769431, 876549, 287657,
			254561, 123457, 134567 });

	Random ranData = new RngStream(new long[] { 654322, 765431, 876541, 987651,
			234567, 123457, 134567 });

	// Random ran=new Random(317);

	Random ranModell = new RngStream(new long[] { 185423, 634523, 957641,
			265427, 234567, 123459, 134567 });

	// Random ran2=new Random(317);

	public int dimension = 2;

	public double[] middle;

	public boolean init = false;

	/*
	 * public double mx=50; public double my=50;
	 */

	public double r = 35;

	public double interval = 0;

	public double[] difMiddle;

	public double difRad;

	public double noise = 0.05;

	/*
	 * public double dx=-0.1; public double dy=-0.075; public double dr=0.01;
	 */

	public GeneratorEllipse() {
		reset();
	}

	public void reset() {
		middle = new double[dimension];
		difMiddle = new double[dimension];
		difRad = 0.001;
		r = 20;

		for (int i = 0; i < dimension; i++) {
			middle[i] = 50;
			difMiddle[i] = (ran.nextDouble() * 0.04) - 0.02;
		}
		init = true;
	}

	public boolean isInitialised() {
		return init;
	}
    
    

	public void update() {
		r += difRad;
		difRad += (ranModell.nextDouble() - 0.5) / 30.0;
		if (Math.abs(difRad) >= 0.03)
			difRad /= 2.0;
		if (10 >= r)
			difRad = Math.abs(difRad);
		if (r >= 40)
			difRad = -Math.abs(difRad);

		for (int i = 0; i < dimension; i++) {
			middle[i] += difMiddle[i];
			difMiddle[i] += (ranModell.nextDouble() - 0.5) / 800.0;
			if (Math.abs(difMiddle[i]) >= 0.015)
				difMiddle[i] /= 2.0;
			if (0 >= middle[i])
				difMiddle[i] = Math.abs(difMiddle[i]);
			if (middle[i] >= 100)
				difMiddle[i] = -Math.abs(difMiddle[i]);
		}
	}

	public double dist(double[] point) {
		double max = 0;
		for (int i = 0; i < dimension; i++) {
			// max=Math.max(max,Math.abs(point[i+1]-middle[i]));
			double val = point[i + 1] - middle[i];
			max += val * val;
		}
		return Math.sqrt(max);
	}

	public int getNumClasses() {
		return 2;
	}

	protected Instance generate(int time) {
		double[] point = new double[dimension + 2];
		point[0] = time;
		double d = ranData.nextDouble() *ranData.nextDouble()*80+10;
		do {
			for (int i = 0; i < dimension; i++) {
				do {
					double rn = ranData.nextDouble();
					point[i + 1] = rn * 100;
					if (ranData.nextDouble() >= 0.5)
						point[i + 1] = -point[i + 1];
					point[i + 1] += middle[i];
				} while (point[i + 1] < 0 || point[i + 1] >= 100);

			}
		} while (dist(point) > d);
		point[dimension + 1] = getNoiseLabel(point);
		return point;
	}

	public int getLabel(Instance point) {
		double dist = dist(point) - r;
		return dist <= 0 ? 1 : 0;
	}

	public int getNoiseLabel(Instance point) {
		int cls = getLabel(point);
		if (ran.nextDouble() < noise) {
			cls += 1;
			cls = cls % 2;
		}
		return cls;
		/*
		 * double dist=dist(x,y)-r; if (Math.abs(dist) <interval){ if
		 * (ran.nextDouble()>Math.abs(dist)/interval) dist*=-1; }
		 * 
		 * return dist <=0?1:0;
		 */

		// return ran.nextInt()&1;
	}

	public Instances getInitData(int count) {
		Instances insts = new Instances("ContData", getAttsInfo(), count);
		for (int i = 0; i < count; i++) {
			insts.add(generate(0));
		}
		insts.setClassIndex(3);
		return insts;
	}

	public Instance generateExample(int time) {
		update();
		return generate(time);
	}

	protected FastVector getAttsInfo() {
		FastVector inf = new FastVector(4);
		inf.add(new Attribute("Time"));
		inf.add(new Attribute("X-val"));
		inf.add(new Attribute("Y-val"));
		FastVector clVal = new FastVector(2);
		clVal.add("out");
		clVal.add("in");
		inf.add(new Attribute("class", clVal));
		return inf;
	}

	public List producedata() {
		List data = new ArrayList(5000);
		for (int i = 0; i < 200; i++) {
			data.add(generate(0));
		}
		for (int i = 1; i < 2000; i++) {
			update();
			data.add(generate(i));
		}
		return data;
	}

	public void output(List data, PrintStream out) {
		Iterator it = data.iterator();
		while (it.hasNext()) {
			int[] dat = (int[]) it.next();
			for (int i = 0; i < dat.length; i++) {
				out.print(dat[i]);
				if (i + 1 < dat.length)
					out.print(", ");
			}
			out.println();
		}
	}

	public static void main(String[] args) {
		AdaptiveGenerator gen = new GeneratorEllipse();
		Instances insts = gen.getInitData(250);
		OutputStream os = null;
		try {
			os = new FileOutputStream("R:/Datasets/weka/test/boxingK2b.arff");
			os.write(insts.toString().getBytes());
			os.flush();
			os.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public double getMinimum(int dimension) {
		return 0;
	}

	public double getMaximum(int dimension) {
		return 100;
	}

    @Override
    public Instances defineDataFormat() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Instance generateExample() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Instances generateExamples() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String generateFinished() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean getSingleModeFlag() {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public String generateStart() throws Exception {
        // TODO Auto-generated method stub
        return null;
    }

}
