package weka.datagenerators;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.Writer;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import utils.random.RngStream;
import weka.classifiers.lazy.FuzzyIB;
import weka.classifiers.lazy.FuzzyIBLukas;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ArffLoader;

public class GeneratorFuzzyIB {

	Random ran = new RngStream(RngStream.getRandomSeed());

	public double[] generate() {
		double x = ran.nextDouble();
		double y = (x * 2) - 1;
		y *= y;
		return new double[] { x, 1.0-y};
	}

	public Instances getData(int count) {
		Instances insts = new Instances("FuzzyIB-Data", getAttsInfo(), count);
		for (int i = 0; i < count; i++) {
			insts.add(new Instance(1.0, generate()));
		}
		insts.setClassIndex(1);
		return insts;
	}

	public Instance getNextInstance() {
		return new Instance(1.0, generate());
	}

	public FastVector getAttsInfo() {
		FastVector inf = new FastVector(4);
		inf.add(new Attribute("X-val"));
		inf.add(new Attribute("Y-val"));
		return inf;
	}

	public void output(List data, PrintStream out) {
		Iterator it = data.iterator();
		while (it.hasNext()) {
			int[] dat = (int[]) it.next();
			for (int i = 0; i < dat.length; i++) {
				out.print(dat[i]);
				if (i + 1 < dat.length) out.print(", ");
			}
			out.println();
		}
	}

	public static void produce(String file) {
		GeneratorFuzzyIB gen = new GeneratorFuzzyIB();
		Instances insts = gen.getData(250);
		OutputStream os = null;
		try {
			os = new FileOutputStream(file);
			os.write(insts.toString().getBytes());
			os.flush();
			os.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void control(String file) {
		try {
			ArffLoader loader = new ArffLoader();
			loader.setFile(new File(file));
			Instances inst = loader.getDataSet();
			inst.setClassIndex(1);
			FuzzyIB fuzzyib = new FuzzyIB();
			fuzzyib.buildClassifier(inst);
			
			
			
			int zones=500;
			
			double[][][] fuzzyval = new double[zones+1][zones+1][];
			for (int i = 0; i <= zones; i++) {
				for (int j = 0; j <= zones; j++) {
					fuzzyval[i][j] = fuzzyib.getFuzzyValue(new Instance(1.0,
							new double[] { (double)i / zones, (double)j / zones}));
				}
			}
			Writer writed = new FileWriter(
					"X:/Research/Eyke/FuzzyIB/fuzzydelta.csv");
			Writer writep = new FileWriter(
					"X:/Research/Eyke/FuzzyIB/fuzzypi.csv");
			for (int i = 0; i <= zones; i++) {
				for (int j = 0; j <= zones; j++) {
					writed.write("" + fuzzyval[i][j][0]);
					writep.write("" + fuzzyval[i][j][1]);
					if (j < zones) {
						writed.write("  ");
						writep.write("  ");
					}
				}
				if (i < zones) {
					writed.write("\n");
					writep.write("\n");
				}
			}
			writed.flush();
			writep.flush();
			writed.close();
			writep.close();
			Instance in=new Instance(1.0,
					new double[] { 0.8, 0.95});
			double[] res=fuzzyib.getFuzzyValue(in);
			int i=0;
			//TODO
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		//String file = "R:/Datasets/weka/test/FuzzyIB1.arff";
		String file = "R:/Datasets/weka/test/FuzzyIBTest.arff";
		//produce(file);
		control(file);
	}

}