package weka.datagenerators;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;

public class GeneratorHyperplane extends AdaptiveGenerator {
    Random ran = new Random(17363421);

    Random ranData = new Random(355346237);

    // Random ran=new Random(317);

    Random ranModell = new Random(293546957);

    // Random ran2=new Random(317);

    public int dimension = 2;

    public int classes = 5;

    public double[] weights;

    public double limit = .2;

    double[] fak;

    public double interval = 0;

    public long count = 0;

    public int conceptdrift = 1;

    public double speed = 4.0;

    protected double v1 = 0.25;

    protected double v2 = 0.75;
    
    protected Instances header;

    public boolean init = false;

    public GeneratorHyperplane() {
        reset();
    }

    public GeneratorHyperplane(int dim, int classes, double speed) {
        dimension = dim;
        this.classes = classes;
        this.speed = speed;
        v1 = 0.45;
        v2 = 1.0 - v1;
        reset();
    }

    public int getNumClasses() {
        return classes;
    }

    public void reset() {
        reset(new Random());
    }
    
    public void reset(Random rnd) {
        ran = new Random(rnd.nextLong());
        ranData = new Random(rnd.nextLong());
        ranModell = new Random(rnd.nextLong());
        weights = new double[dimension];
        fak = new double[dimension];
        double sum = 0;
        for (int i = 0; i < dimension; i++) {
            weights[i] = ran.nextDouble();//(i+1)/(double)(dimension+1);
            sum = sum + weights[i];
            fak[i] = ran.nextInt(2) * 2 - 1;
        }
        limit = 0.5 * sum;

        count = 0;
        header = new Instances("Means", getAttsInfo(), 0);
        header.setClassIndex(dimension + 1);
        init = true;
    }

    public boolean isInitialised() {
        return init;
    }

    public void update() {
        if (count % 2000 == 0) {
            for (int i = 0; i < dimension; i++) {
                fak[i] = ran.nextInt(2) * 2 - 1;
            }
        }
        double sum = 0;
        for (int i = 0; i < dimension; i++) {
            weights[i] += speed * 0.0001 * dimension * fak[i];
            if (weights[i]>1) fak[i]=-1;
            if (weights[i]<-1) fak[i]=1;
            // System.out.println(i+": "+weights[i]);
            sum = sum + weights[i];
        }
        limit = sum * 0.5;

    }

    protected double value(double[] point) {
        double max = 0;
        for (int i = 0; i < dimension; i++) {
            max += point[i + 1] * weights[i];
        }
        return max;
    }

    protected int classvalue(double value) {
        return Math.min(Math.max(((int) (value / limit)), 0), classes - 1);
        // for (int i=0;i<classes-1;i++){
        // if (value<limits[i]) return i;
        // }
        // return classes-1;
    }

    protected Instance generate(int time) {
        double[] point = new double[dimension + 2];
        point[0] = time;
        for (int i = 0; i < dimension; i++) {
            point[i + 1] = ranData.nextDouble();
        }
        Instance res = new Instance(1.0, point);
        res.setDataset(header);
        res.setClassValue(getNoiseLabel(res));
        if (time > 0)
            count++;
        return res;
    }

    public int getLabel(Instance point) {
        return classvalue(value(point.toDoubleArray()));

    }

    public int getNoiseLabel(Instance point) {
        int cls = getLabel(point);
        if (ran.nextDouble() < noise) {
            cls += ran.nextInt(classes - 1)+1;
            cls = cls % classes;
        }
        return cls;

        // return ranData.nextInt()&1;
    }

    public Instances getInitData(int count) {
        Instances insts = new Instances("ContData", getAttsInfo(), count);
        for (int i = 0; i < count; i++) {
            insts.add(generate(0));
        }
        insts.setClassIndex(dimension + 1);
        return insts;
    }

    protected FastVector getAttsInfo() {
        FastVector inf = new FastVector(dimension + 2);
        Properties prop=new Properties();
        prop.setProperty("range","[0,inf]");
        
        inf.add(new Attribute("Time",new ProtectedProperties(prop)));
        for (int i=0;i<dimension;i++){
            prop.setProperty("range","["+getMinimum(i)+","+getMaximum(i)+"]");
            inf.add(new Attribute("X"+(i+1)+"-val",new ProtectedProperties(prop)));
        }
        FastVector clVal = new FastVector(2);
        for (int i = 0; i < classes; i++) {
            clVal.add("Class " + (i + 1));
        }
        inf.add(new Attribute("class", clVal));
        return inf;
    }

    public Instance generateExample(int time) {
        // if (count%conceptdrift==0)
        update();
        return generate(time);
    }

    public void output(List data, PrintStream out) {
        Iterator it = data.iterator();
        while (it.hasNext()) {
            int[] dat = (int[]) it.next();
            for (int i = 0; i < dat.length; i++) {
                out.print(dat[i]);
                if (i + 1 < dat.length)
                    out.print(", ");
            }
            out.println();
        }
    }

    public double getMinimum(int dimension) {
        return 0;
    }

    public double getMaximum(int dimension) {
        return 1;
    }

    public static void main(String[] args) {
        GeneratorHyperplane gen = new GeneratorHyperplane();
        Instances insts = gen.getInitData(100);
        OutputStream os = null;
        try {
            os = new FileOutputStream("R:/Datasets/weka/test/t.arff");
            os.write(insts.toString().getBytes());
            os.flush();
            os.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    @Override
    public Instances defineDataFormat() {
        // TODO Auto-generated method stub
        return header;
    }

    @Override
    public Instance generateExample() {
        // TODO Auto-generated method stub
        return generateExample(0);
    }

    @Override
    public Instances generateExamples() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String generateFinished() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean getSingleModeFlag() {
        // TODO Auto-generated method stub
        return false;
    }

}
