/*
 * Created on 06.05.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package weka.datagenerators;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;

/**
 * @author George
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class GeneratorMeans extends AdaptiveGenerator {

    protected int classCount = 4;

    protected int dimension = 4;

    protected double[][] means;

    protected double[][] diff;

    protected boolean init = false;

    public double speed = 10;

    protected Random ranData;

    protected Random ran;

    protected Instances header;

    public GeneratorMeans(int cls, int dim, double speed) {
        classCount = cls;
        dimension = dim;
        this.speed = speed;
        reset();
    }

    public boolean isInitialised() {
        return init;
    }

    public void reset() {
        reset(new Random());
    }
    
    public void reset(Random rnd) {
        ranData = new Random(rnd.nextLong());
        ran = new Random(rnd.nextLong());
        means = new double[classCount][dimension];
        diff = new double[classCount][dimension];
        for (int i = 0; i < classCount; i++) {
            for (int j = 0; j < dimension; j++) {
                double v = (2 * i + 3 * j + 5) / 6.7;
                means[i][j] = v - (int) v;// ranData.nextDouble();
                v = (3 * i + 2 * j + 4) / 6.7;
                diff[i][j] = (ranData.nextDouble()-0.5)*0.001*speed;
                //((v - (int) v) - 0.5) * 0.001 * speed;// 
            }
        }
        header = new Instances("Means", getAttsInfo(), 0);
        header.setClassIndex(dimension + 1);
        init = true;
    }

    public Instances getInitData(int count) {
        Instances insts = new Instances("ContData", getAttsInfo(), count);
        for (int i = 0; i < count; i++) {
            insts.add(generate(0));
        }
        insts.setClassIndex(dimension + 1);
        return insts;
    }

    public void update() {
        for (int i = 0; i < classCount; i++) {
            for (int j = 0; j < dimension; j++) {
                means[i][j] += diff[i][j];
                if (means[i][j] > 1)
                    diff[i][j] = -Math.abs(diff[i][j]);
                if (means[i][j] < 0)
                    diff[i][j] = Math.abs(diff[i][j]);
            }
        }
    }

    protected Instance generate(int time) {
        double[] point = new double[dimension + 2];
        point[0] = time;
        for (int i = 0; i < dimension; i++) {
            point[i + 1] = ranData.nextDouble();
        }
        Instance res = new Instance(1.0, point);
        res.setDataset(header);
        res.setClassValue(getNoiseLabel(res));
        return res;
    }

    /*
     * (non-Javadoc)
     * 
     * @see utils.data.Generator#getLabel(double[])
     */
    public int getLabel(Instance point) {
        double minD = Double.MAX_VALUE;
        int cls = -1;
        for (int i = 0; i < classCount; i++) {
            double d = dist(point.toDoubleArray(), means[i]);
            if (d < minD) {
                minD = d;
                cls = i;
            }
        }
        return cls;
    }

    public int getNoiseLabel(Instance point) {
        int cls = getLabel(point);
        if (ran.nextDouble() < noise) {
            cls += ran.nextInt(classCount - 1);
            cls = cls % classCount;
        }
        return cls;
    }

    public double dist(double[] p, double[] m) {
        double dist = 0;
        for (int i = 0; i < dimension; i++) {
            double d = Math.abs(p[i + 1] - m[i]);
            // dist=Math.max(d,dist);
            dist = d + dist;
        }
        return dist;
        // for (int i=0;i<dimension;i++){
        // double d=p[i+1]-m[i];
        // dist+=d*d;
        // }
        // return Math.sqrt(dist);
    }

    public Instance generateExample(int time) {
        if (time > 0)
            update();
        Instance next = generate(time);
        next.setDataset(header);
        return next;
    }

    protected FastVector getAttsInfo() {
        FastVector inf = new FastVector(dimension + 2);
        Properties prop = new Properties();
        prop.setProperty("range", "[0,inf]");

        inf.add(new Attribute("Time", new ProtectedProperties(prop)));
        for (int i = 0; i < dimension; i++) {
            prop.setProperty("range", "[" + getMinimum(i) + "," + getMaximum(i)
                    + "]");
            inf.add(new Attribute("X" + (i + 1) + "-val",
                    new ProtectedProperties(prop)));
        }
        FastVector clVal = new FastVector(2);
        for (int i = 0; i < classCount; i++) {
            clVal.add("Class " + (i + 1));
        }
        inf.add(new Attribute("class", clVal));
        return inf;
    }

    public int getNumClasses() {
        return classCount;
    }

    public List producedata() {
        List data = new ArrayList(5000);
        for (int i = 0; i < 200; i++) {
            data.add(generate(0));
        }
        for (int i = 1; i < 2000; i++) {
            update();
            data.add(generate(i));
        }
        return data;
    }

    public double getMinimum(int dimension) {
        return 0;
    }

    public double getMaximum(int dimension) {
        return 1;
    }

    public void output(List data, PrintStream out) {
        Iterator it = data.iterator();
        while (it.hasNext()) {
            int[] dat = (int[]) it.next();
            for (int i = 0; i < dat.length; i++) {
                out.print(dat[i]);
                if (i + 1 < dat.length)
                    out.print(", ");
            }
            out.println();
        }
    }

    public Instances defineDataFormat(){
        return header;
    }

    public Instance generateExample(){
        return generateExample(0);
    }

    public Instances generateExamples(){
        return header;
    }

    public String generateFinished(){
        return "";
    }

    public boolean getSingleModeFlag(){
        return true;
    }

    

}
