
package weka.datagenerators;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;


public class GeneratorRandom extends AdaptiveGenerator {
	Random ran=new Random(117);
	
	Random ranData=new Random(3757431);
	//Random ran=new Random(317);
	
	public int dimension=2;
	
	public boolean init=false;
	
	public int speed=2000;
	
	public double p=0.5;
	
	public double diff;
    
    public Instances header;
    
    public int lastTime=0;
	
	
	public GeneratorRandom(){
		reset();
	}
	
	public GeneratorRandom(int dimension){
		this.dimension=dimension;
		reset();
	}
	public int getNumClasses(){
		return 2;
	}
	
    public void reset() {
        reset(new Random());
    }
    
    public void reset(Random rnd) {
        ran=new Random(rnd.nextLong());
        ranData=new Random(rnd.nextLong());
		p=0.5;
		diff=1.0/speed;
		init=true;
        header=new Instances("ContData",getAttsInfo(),0);
        header.setClassIndex(dimension+1);
	}
	
	public boolean isInitialised(){
		return init;
	}
	
	public void update(){
		p+=diff;
		if (p>1.0 || p<0){
			diff=-diff;
			p+=diff;
		}
//        if (lastTime%10==0){
//            System.out.println(p);
//            lastTime++;
//        }
	}
	
	
	
	protected Instance generate(int time){
		double[] point=new double[dimension+2];
		point[0]=time;
		for (int i=0;i<dimension;i++){
			point[i+1]=ranData.nextDouble();
		}
        Instance inst= new Instance(1.0,point);
        inst.setDataset(header);
        inst.setClassValue(getNoiseLabel(inst));
        lastTime=time;
        return inst;
	}
	
	
	
	public int getLabel(Instance point){
		return ranData.nextDouble()>=p?1:0;
	}
	
	public int getNoiseLabel(Instance point){
		int cls=getLabel(point);
		return cls;
	}
		
		
		
	
	public Instances getInitData(int count){
		Instances insts=new Instances("ContData",getAttsInfo(),count);
		for (int i=0;i<count;i++){
			insts.add(generate(0));
		}
		insts.setClassIndex(dimension+1);
		return insts;
	}
	
    double sum=0;
    double c=0;
    
	public Instance generateExample(int time){
		update();
//        sum+=Math.max(p,1-p);
//        c++;
//        if (c%100==0) System.out.println("best rate: "+(sum/c));
        return generate(time);
	}
	
	
	
	protected FastVector getAttsInfo(){
		FastVector inf=new FastVector(dimension+2);
        Properties prop = new Properties();
        prop.setProperty("range", "[0,inf]");

        inf.add(new Attribute("Time", new ProtectedProperties(prop)));
        for (int i = 0; i < dimension; i++) {
            prop.setProperty("range", "[" + getMinimum(i) + "," + getMaximum(i)
                    + "]");
            inf.add(new Attribute("X" + (i + 1) + "-val",
                    new ProtectedProperties(prop)));
        }
		FastVector clVal=new FastVector(2);
		clVal.add("out");
		clVal.add("in");
		inf.add(new Attribute("class",clVal));
		return inf;
	}
	
	public List producedata(){
		List data=new ArrayList(5000);
		for (int i=0;i<200;i++){
			data.add(generate(0));
		}
		for (int i=1;i<2000;i++){
			update();
			data.add(generate(i));
		}
		return data;
	}
	
	public void output(List data, PrintStream out){
		Iterator it=data.iterator();
		while (it.hasNext()){
			int[] dat=(int[])it.next();
			for (int i=0;i<dat.length;i++){
				out.print(dat[i]);
				if (i+1<dat.length) out.print(", ");
			}
			out.println();
		}
	}
	
	public double getMinimum(int dimension) {
		return 0;
	}

	public double getMaximum(int dimension) {
		return 1;
	}

    @Override
    public Instances defineDataFormat() {
        return header;
    }

    @Override
    public Instance generateExample() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Instances generateExamples() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String generateFinished() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean getSingleModeFlag() {
        // TODO Auto-generated method stub
        return false;
    }
}
