package weka.datagenerators;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;

public class GeneratorRecht extends AdaptiveGenerator {
	Random ran = new Random(117);

	Random ranData = new Random(59);

	// Random ran=new Random(317);

	Random ranModell = new Random(139);

	// Random ran2=new Random(317);

	public int dimension = 2;

	public double[] middle;

	/*
	 * public double mx=50; public double my=50;
	 */
	public double[] radius;

	public double[] difMiddle;

	public double[] difRad;

	public boolean init = false;

	/*
	 * public double dx=-0.1; public double dy=-0.075; public double dr=0.01;
	 */

	public double limit1 = 0.35;

	public double limit2 = 0.35;

	public double speed = 0;

	public GeneratorRecht() {
		reset();
	}

	public GeneratorRecht(int dimension) {
		this.dimension = dimension;
		reset();
	}

	public int getNumClasses() {
		return 2;
	}

	public void reset() {
		middle = new double[dimension];
		radius = new double[dimension];
		difMiddle = new double[dimension];
		difRad = new double[dimension];
		//limit1 = Math.pow(0.9, 1.0 / dimension)/2;
		//limit2 = Math.pow(0.3, 1.0 / dimension)/2;

		for (int i = 0; i < dimension; i++) {
			middle[i] = limit2;//(((1-limit2*2) * i) / dimension) + limit2;
			radius[i] = ((limit1-limit2)/(dimension-1) * i ) + limit2;
			difMiddle[i] = 0.00001 * speed ;
			difRad[i] = (limit1-limit2) * speed /1000.0;
		}
		
		init = true;
	}

	public boolean isInitialised() {
		return init;
	}

	public void update() {
		for (int i = 0; i < dimension; i++) {
			middle[i] += difMiddle[i];
			radius[i] += difRad[i];
			// difMiddle[i]+=(ranModell.nextDouble()-0.5)/(0.4/speed);
			// difRad[i]+=(ranModell.nextDouble()-0.5)/(0.6/speed);
			// if (Math.abs(difMiddle[i])>=0.15*speed) difMiddle[i]/=2.0;
			// if (Math.abs(difRad[i])>=0.1*speed) difRad[i]/=2.0;
			if (limit2 >= middle[i])
				difMiddle[i] = Math.abs(difMiddle[i]);
			if (middle[i] >= 1.0-limit2)
				difMiddle[i] = -Math.abs(difMiddle[i]);
			if (limit2 >= radius[i]) {
				difRad[i] = Math.abs(difRad[i]);
				radius[i] = limit2 + difRad[i];
			}
			if (radius[i] >= limit1) {
				difRad[i] = -Math.abs(difRad[i]);
				radius[i] = limit1 + difRad[i];
			}

		}
	}

	public double dist(double[] point) {
		double max = 0;
		for (int i = 0; i < dimension; i++) {
			max = Math.max(max, Math.abs(point[i + 1] - middle[i]));
		}
		return max;
	}

	public double distModell(double[] point) {
		double max = 0;
		for (int i = 0; i < dimension; i++) {
			double d = Math.abs(point[i + 1] - middle[i]);
			d -= radius[i];
			if (d < 0)
				d = 0;
			max = Math.max(max, d);
		}
		return max;
	}

	protected Instance generate(int time) {
		double[] point = new double[dimension + 2];
		point[0] = time;
		for (int i = 0; i < dimension; i++) {
			point[i + 1] = ranData.nextDouble();
		}
		point[dimension + 1] = getNoiseLabel(point);
		return point;
	}

	int s0 = 0;

	int s1 = 0;

	public int getLabel(Instance point) {
		double dist = distModell(point);
		int cl = dist <= 0 ? 1 : 0;
		if (cl == 0)
			s0++;
		else
			s1++;
		return cl;

		// int val=ranData.nextDouble()>=0.8?1:0;
		// if (val==0)s0++;
		// else s1++;
		// //System.out.println("output "+ s0 +" .. "+ s1);
		// return val;
	}

	public int getNoiseLabel(Instance point) {
		int cls = getLabel(point);
		if (ran.nextDouble() < noise) {
			cls += 1;
			cls = cls % 2;
		}
		return cls;

		/*
		 * double dist=distModell(point); if (Math.abs(dist) <interval){ if
		 * (ran.nextDouble()>Math.abs(dist)/interval) dist*=-1; }
		 * 
		 * return dist <=0?1:0;
		 */

		// return ranData.nextInt()&1;
	}

	public Instances getInitData(int count) {
		Instances insts = new Instances("ContData", getAttsInfo(), count);
		for (int i = 0; i < count; i++) {
			insts.add(new Instance(1.0, generate(0)));
		}
		insts.setClassIndex(dimension + 1);
		return insts;
	}

	public Instance generateExample(int time) {
		update();
		return new Instance(1.0, generate(time));
	}

	protected FastVector getAttsInfo() {
		FastVector inf = new FastVector(dimension + 2);
		inf.add(new Attribute("Time"));
		for (int i = 0; i < dimension; i++) {
			inf.add(new Attribute("X" + (i + 1) + "-val"));
		}
		FastVector clVal = new FastVector(2);
		clVal.add("out");
		clVal.add("in");
		inf.add(new Attribute("class", clVal));
		return inf;
	}

	public List producedata() {
		List data = new ArrayList(5000);
		for (int i = 0; i < 200; i++) {
			data.add(generate(0));
		}
		for (int i = 1; i < 2000; i++) {
			update();
			data.add(generate(i));
		}
		return data;
	}

	public void output(List data, PrintStream out) {
		Iterator it = data.iterator();
		while (it.hasNext()) {
			int[] dat = (int[]) it.next();
			for (int i = 0; i < dat.length; i++) {
				out.print(dat[i]);
				if (i + 1 < dat.length)
					out.print(", ");
			}
			out.println();
		}
	}

	public double getMinimum(int dimension) {
		return 0;
	}

	public double getMaximum(int dimension) {
		return 1;
	}

	public static void main(String[] args) {
		GeneratorRecht gen = new GeneratorRecht();
		Instances insts = gen.getInitData(100);
		OutputStream os = null;
		try {
			os = new FileOutputStream("R:/Datasets/weka/test/boxingR2.arff");
			os.write(insts.toString().getBytes());
			os.flush();
			os.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

}
