package weka.datagenerators;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;

public class GeneratorRectDist extends AdaptiveGenerator {

	Random ran = new Random(17363421);
    Random ranData = new Random(355346237);
    Random ranModell = new Random(293546957);
    protected Instances header;

	public int dimension = 2;

	public boolean init = false;

	public long count = 0;

	public int mode = 0;



	/*
	 * public double dx=-0.1; public double dy=-0.075; public double dr=0.01;
	 */

	public GeneratorRectDist() {
		reset();
	}
    
    public void reset() {
        reset(new Random());
    }
    
    public void reset(Random rnd) {
        ran = new Random(rnd.nextLong());
        ranData = new Random(rnd.nextLong());
        ranModell = new Random(rnd.nextLong());
        header= new Instances("Means",getAttsInfo(),0);
        header.setClassIndex(dimension+1);
		init = true;
        mode=1;
	}

	public boolean isInitialised() {
		return init;
	}

	public void update() {
		if (count % 2000== 0)
			mode = ++mode % 4;
	}

	public double dist(double[] point, double[] middle) {
		double max = 0;
		for (int i = 0; i < dimension; i++) {
			// max=Math.max(max,Math.abs(point[i+1]-middle[i]));
			double val = point[i + 1] - middle[i];
			max += val * val;
		}
		return Math.sqrt(max);
	}

	public int getNumClasses() {
		return 2;
	}

	protected Instance generate(int time) {
		if (time==0){
			double[] point = new double[dimension + 2];
			point[0] = time;
			point[1] = ranData.nextDouble() * 100;
			point[2] = ranData.nextDouble() * 100;
            Instance inst= new Instance(1.0,point);
            inst.setDataset(header);
			inst.setClassValue(getNoiseLabel(inst));
			return inst;
		}
		count++;
		double[] point = new double[dimension + 2];
		point[0] = time;
		point[1] = ranData.nextDouble() * 50;
		point[2] = ranData.nextDouble() * 50;
		if (mode==1|| mode==2) point[1]+=50;
		if (mode>1) point[2]+=50;
        Instance inst= new Instance(1.0,point);
        inst.setDataset(header);
        inst.setClassValue(getNoiseLabel(inst));
        return inst;
	}

	public int getLabel(Instance inst) {
        double[] point=inst.toDoubleArray();
		int cl=0;
		if (point[1]>=50) cl=1-cl;
		if (point[2]>=50) cl=1-cl;
		return cl;
	}

	public int getNoiseLabel(Instance point) {
		int cls = getLabel(point);
		if (ran.nextDouble() < noise) {
			cls += 1;
			cls = cls % 2;
		}
		return cls;
		
	}

	public Instances getInitData(int count) {
		Instances insts = new Instances("ContData", getAttsInfo(), count);
		for (int i = 0; i < count; i++) {
			insts.add(generate(0));
		}
		insts.setClassIndex(3);
		return insts;
	}

	public Instance generateExample(int time) {
		update();
        return generate(time);
	}

	protected FastVector getAttsInfo() {
		FastVector inf = new FastVector(4);
        Properties prop=new Properties();
        prop.setProperty("range","[0,inf]");
        
        inf.add(new Attribute("Time",new ProtectedProperties(prop)));
        
        prop.setProperty("range","["+getMinimum(0)+","+getMaximum(0)+"]");
		inf.add(new Attribute("X-val",new ProtectedProperties(prop)));
        prop.setProperty("range","["+getMinimum(1)+","+getMaximum(1)+"]");
		inf.add(new Attribute("Y-val",new ProtectedProperties(prop)));
		FastVector clVal = new FastVector(2);
		clVal.add("out");
		clVal.add("in");
		inf.add(new Attribute("class", clVal));
		return inf;
	}

	public List producedata() {
		List data = new ArrayList(5000);
		for (int i = 0; i < 200; i++) {
			data.add(generate(0));
		}
		for (int i = 1; i < 2000; i++) {
			update();
			data.add(generate(i));
		}
		return data;
	}

	public void output(List data, PrintStream out) {
		Iterator it = data.iterator();
		while (it.hasNext()) {
			int[] dat = (int[]) it.next();
			for (int i = 0; i < dat.length; i++) {
				out.print(dat[i]);
				if (i + 1 < dat.length)
					out.print(", ");
			}
			out.println();
		}
	}


	public double getMinimum(int dimension) {
		return 0;
	}

	public double getMaximum(int dimension) {
		return 100;
	}

    @Override
    public Instances defineDataFormat() {
        return header;
    }

    @Override
    public Instances generateExamples() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String generateFinished() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean getSingleModeFlag() {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public Instance generateExample() {
        
        return generateExample(0);
    }

}
