package weka.datagenerators;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import utils.random.RngStream;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.ProtectedProperties;

public class GeneratorRectSwitch extends AdaptiveGenerator {

	Random ran = new RngStream(new long[] { 654522, 769431, 876549, 287657,
			254561, 123457, 134567 });

	Random ranData = new RngStream(new long[] { 654322, 765431, 876541, 987651,
			234567, 123457, 134567 });

	// Random ran=new Random(317);

	Random ranModell = new RngStream(new long[] { 185423, 634523, 957641,
			265427, 234567, 123459, 134567 });

	// Random ran2=new Random(317);

	public int dimension = 2;

	public boolean init = false;

	public long count = 0;

	public int mode = 0;
    
    public Instances header;

	/*
	 * public double dx=-0.1; public double dy=-0.075; public double dr=0.01;
	 */

	public GeneratorRectSwitch() {
		reset();
	}

	public void reset() {
        header =new Instances("ContData", getAttsInfo(), 0);
        header.setClassIndex(3);
		init = true;
	}

	public boolean isInitialised() {
		return init;
	}

	public void update() {
		if (count % 500== 0){
			mode = ++mode % 4;
			count=0;
		}
	}

	public double dist(double[] point, double[] middle) {
		double max = 0;
		for (int i = 0; i < dimension; i++) {
			// max=Math.max(max,Math.abs(point[i+1]-middle[i]));
			double val = point[i + 1] - middle[i];
			max += val * val;
		}
		return Math.sqrt(max);
	}

	public int getNumClasses() {
		return 2;
	}

	protected Instance generate(int time) {
		if (time>0) count++;
		double[] point = new double[dimension + 2];
		point[0] = time;
		point[1] = ranData.nextDouble() * 100;
		point[2] = ranData.nextDouble() * 100;
		Instance inst=new Instance(1.0, point);
		point[dimension + 1] = getNoiseLabel(inst);
        return new Instance(1.0, point) ;
	}

	public int getLabel(Instance point) {
		int cl=0;
		if (mode==0 && point.value(1)<50 && point.value(2)<50) cl=1;
		if (mode==1 && point.value(1)<50 && point.value(2)>=50) cl=1;
		if (mode==2 && point.value(1)>=50 && point.value(2)>=50) cl=1;
		if (mode==3 && point.value(1)>=50 && point.value(2)<50) cl=1;
		return cl;
	}

	public int getNoiseLabel(Instance point) {
		int cls = getLabel(point);
		if (ran.nextDouble() < noise) {
			cls += 1;
			cls = cls % 2;
		}
		return cls;
	}

	public Instances getInitData(int count) {
		Instances insts = new Instances("ContData", getAttsInfo(), count);
		for (int i = 0; i < count; i++) {
			insts.add(generate(0));
		}
		insts.setClassIndex(3);
		return insts;
	}

	public Instance generateExample(int time) {
		update();
		return generate(time);
	}

	protected FastVector getAttsInfo() {
		FastVector inf = new FastVector(4);
        Properties prop=new Properties();
        prop.setProperty("range","[0,inf]");
        
        inf.add(new Attribute("Time",new ProtectedProperties(prop)));
        prop.setProperty("range","["+getMinimum(0)+","+getMaximum(0)+"]");
        inf.add(new Attribute("X-val",new ProtectedProperties(prop)));
        prop.setProperty("range","["+getMinimum(1)+","+getMaximum(1)+"]");
        inf.add(new Attribute("Y-val",new ProtectedProperties(prop)));
		FastVector clVal = new FastVector(2);
		clVal.add("out");
		clVal.add("in");
		inf.add(new Attribute("class", clVal));
		return inf;
	}

	public List producedata() {
		List data = new ArrayList(5000);
		for (int i = 0; i < 200; i++) {
			data.add(generate(0));
		}
		for (int i = 1; i < 2000; i++) {
			update();
			data.add(generate(i));
		}
		return data;
	}

	public void output(List data, PrintStream out) {
		Iterator it = data.iterator();
		while (it.hasNext()) {
			int[] dat = (int[]) it.next();
			for (int i = 0; i < dat.length; i++) {
				out.print(dat[i]);
				if (i + 1 < dat.length)
					out.print(", ");
			}
			out.println();
		}
	}

	public double getMinimum(int dimension) {
		return 0;
	}

	public double getMaximum(int dimension) {
		return 100;
	}

    @Override
    public Instances defineDataFormat() {
        // TODO Auto-generated method stub
        return header;
    }

    @Override
    public Instance generateExample() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Instances generateExamples() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String generateFinished() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean getSingleModeFlag() {
        // TODO Auto-generated method stub
        return false;
    }

}
