package weka.datagenerators;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import utils.random.RngStream;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;

public class GeneratorSin extends AdaptiveGenerator {

    Random ran = new RngStream(new long[] { 654522, 769431, 876549, 287657,
            254561, 123457, 134567 });

    Random ranData = new RngStream(new long[] { 654322, 765431, 876541, 987651,
            234567, 123457, 134567 });

    // Random ran=new Random(317);

    Random ranModell = new RngStream(new long[] { 185423, 634523, 957641,
            265427, 234567, 123459, 134567 });

    // Random ran2=new Random(317);

    public int dimension = 2;

    public boolean init = false;

    public long count = 0;
    
    protected double model=0;
    
    protected double delta=0.0001;
    
    protected double speed=0;
    
    protected Instances header;

    /*
     * public double dx=-0.1; public double dy=-0.075; public double dr=0.01;
     */

    public GeneratorSin() {
        reset();
    }

    public void reset() {
        init = true;
        delta=0.0002*speed;
        header=getInitData(0);
    }

    public boolean isInitialised() {
        return init;
    }

    public void update() {
        model+=delta;
    }

    public double dist(double[] point, double[] middle) {
        double max = 0;
        for (int i = 0; i < dimension; i++) {
            // max=Math.max(max,Math.abs(point[i+1]-middle[i]));
            double val = point[i + 1] - middle[i];
            max += val * val;
        }
        return Math.sqrt(max);
    }

    public int getNumClasses() {
        return 2;
    }

    protected Instance generate(int time) {
        double[] point = new double[dimension + 2];
        point[0] = time;
        point[1] = ranData.nextDouble() ;
        point[2] = ranData.nextDouble() ;
        Instance res = new Instance(1.0, point);
        res.setDataset(header);
        res.setClassValue(getNoiseLabel(res));
        return res;
    }

    public int getLabel(Instance inst) {
        int cl = 0;
        double[] point=inst.toDoubleArray();
        double val=(Math.sin(model+point[1]*Math.PI*4)*0.6*0.5)+0.5;
        if (point[2] >= val)
            cl = 1 - cl;
        return cl;
    }

    public int getNoiseLabel(Instance point) {
        int cls = getLabel(point);
        if (ran.nextDouble() < noise) {
            cls += 1;
            cls = cls % 2;
        }
        return cls;

    }

    public Instances getInitData(int count) {
        Instances insts = new Instances("ContData", getAttsInfo(), count);
        for (int i = 0; i < count; i++) {
            insts.add(generate(0));
        }
        insts.setClassIndex(3);
        return insts;
    }

    public Instance generateExample(int time) {
        if (time>0) update();
        return generate(time);
    }

    protected FastVector getAttsInfo() {
        FastVector inf = new FastVector(4);
        inf.add(new Attribute("Time"));
        inf.add(new Attribute("X-val"));
        inf.add(new Attribute("Y-val"));
        FastVector clVal = new FastVector(2);
        clVal.add("out");
        clVal.add("in");
        inf.add(new Attribute("class", clVal));
        return inf;
    }

    public List producedata() {
        List data = new ArrayList(5000);
        for (int i = 0; i < 200; i++) {
            data.add(generate(0));
        }
        for (int i = 1; i < 2000; i++) {
            update();
            data.add(generate(i));
        }
        return data;
    }

    public void output(List data, PrintStream out) {
        Iterator it = data.iterator();
        while (it.hasNext()) {
            int[] dat = (int[]) it.next();
            for (int i = 0; i < dat.length; i++) {
                out.print(dat[i]);
                if (i + 1 < dat.length)
                    out.print(", ");
            }
            out.println();
        }
    }

   

    public double getMinimum(int dimension) {
        return 0;
    }

    public double getMaximum(int dimension) {
        return 1;
    }
    
    public Instances defineDataFormat(){
        return header;
    }

    public Instance generateExample(){
        return generateExample(0);
    }

    public Instances generateExamples(){
        return header;
    }

    public String generateFinished(){
        return "";
    }

    boolean getSingleModeFlag(){
        return true;
    }

}
