/*
 * Created on 15.06.2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package weka.multiValue;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.SparseInstance;
import weka.core.UnassignedClassException;

public class MVInstances extends Instances {

	public static MVInstances convertInstances(Instances data) {
		FastVector attributes = new FastVector(data.numAttributes()
				+ data.numClasses());

		FastVector attribValues = new FastVector(2);
		attribValues.addElement("false");
		attribValues.addElement("true");

		int classIndex = data.classIndex();
		for (int i = 0; i < data.numAttributes(); i++) {
			attributes.addElement(data.attribute(i).copy());
		}
		for (int i = 0; i < data.numClasses(); i++) {
			attributes.addElement(new Attribute(data.attribute(classIndex)
					.value(i), attribValues));
		}

		MVInstances newData = new MVInstances(data.relationName(), attributes,
				0);
		
		newData.setClassIndex(data.classIndex());

		for (int i = 0; i < data.numClasses(); i++) {
			newData.addClassIndex(i + data.numAttributes());
		}

		Enumeration enum1 = data.enumerateInstances();
		while (enum1.hasMoreElements()) {
			Instance instance = (Instance) enum1.nextElement();
			double[] vals = new double[newData.numAttributes()];
			// Copy and convert the values
			for (int i = 0; i < data.numAttributes(); i++) {
				vals[i] = instance.value(i);
			}
			int cl = (int) instance.classValue();
			for (int i = 0; i < data.numClasses(); i++) {
				if (i == cl) {
					vals[i + data.numAttributes()] = 1;
				} else
					vals[i + data.numAttributes()] = 0;
			}
			Instance newVal = null;
			if (instance instanceof SparseInstance) {
				newVal = new SparseInstance(instance.weight(), vals);
			} else {
				newVal = new Instance(instance.weight(), vals);
			}
			newVal.setDataset(newData);
			newData.add(newVal);
		}

		return newData;
	}

	protected int[] classIndices;

	/**
	 * Constructor copying all instances and references to
	 * the header information from the given set of instances.
	 *
	 * @param instances the set to be copied
	 */
	public MVInstances(Instances dataset) {
		super(dataset);
		if (dataset instanceof MVInstances) {
			MVInstances mvData = (MVInstances) dataset;
			if (mvData.classIndices != null) {
				classIndices = new int[mvData.classIndices.length];
				System.arraycopy(mvData.classIndices, 0, classIndices, 0,
						classIndices.length);
			}
		}
	}

	/**
	 * Constructor creating an empty set of instances. Copies references
	 * to the header information from the given set of instances. Sets
	 * the capacity of the set of instances to 0 if its negative.
	 *
	 * @param instances the instances from which the header 
	 * information is to be taken
	 * @param capacity the capacity of the new dataset 
	 */
	public MVInstances(Instances dataset, int capacity) {
		super(dataset, capacity);
		if (dataset instanceof MVInstances) {
			MVInstances mvData = (MVInstances) dataset;
			if (mvData.classIndices != null) {
				classIndices = new int[mvData.classIndices.length];
				System.arraycopy(mvData.classIndices, 0, classIndices, 0,
						classIndices.length);
			}
		}
	}

	/**
	 * Creates a new set of instances by copying a 
	 * subset of another set.
	 *
	 * @param source the set of instances from which a subset 
	 * is to be created
	 * @param first the index of the first instance to be copied
	 * @param toCopy the number of instances to be copied
	 * @exception IllegalArgumentException if first and toCopy are out of range
	 */
	public MVInstances(Instances source, int first, int toCopy) {
		super(source, first, toCopy);
		if (source instanceof MVInstances) {
			MVInstances mvData = (MVInstances) source;
			if (mvData.classIndices != null) {
				classIndices = new int[mvData.classIndices.length];
				System.arraycopy(mvData.classIndices, 0, classIndices, 0,
						classIndices.length);
			}
		}
	}

	/**
	 * Creates an empty set of instances. Uses the given
	 * attribute information. Sets the capacity of the set of 
	 * instances to 0 if its negative. Given attribute information
	 * must not be changed after this constructor has been used.
	 *
	 * @param name the name of the relation
	 * @param attInfo the attribute information
	 * @param capacity the capacity of the set
	 */
	public MVInstances(String name, FastVector attInfo, int capacity) {
		super(name, attInfo, capacity);
	}

	/**
	 * Create a copy of the structure, but "cleanse" string types (i.e.
	 * doesn't contain references to the strings seen in the past).
	 *
	 * @return a copy of the instance structure.
	 */
	public Instances stringFreeStructure() {

		FastVector atts = (FastVector) m_Attributes.copy();
		for (int i = 0; i < atts.size(); i++) {
			Attribute att = (Attribute) atts.elementAt(i);
			if (att.type() == Attribute.STRING) {
				atts.setElementAt(new Attribute(att.name(), (FastVector) null),
						i);
			}
		}
		MVInstances result = new MVInstances(relationName(), atts, 0);
		result.setClassIndex(m_ClassIndex);
		if (classIndices != null) {
			result.classIndices = new int[classIndices.length];
			System.arraycopy(classIndices, 0, result.classIndices, 0,
					classIndices.length);
		}
		return result;
	}

	/** 
	 * Sets the class attribute.
	 *
	 * @param att attribute to be the class
	 */
	public void setClasses(Attribute att) {
		addClassIndex(att.index());
	}

	public void addClassIndex(int classIndex) {
		if (classIndex >= numAttributes()) { throw new IllegalArgumentException(
				"Invalid class index: " + classIndex); }
		if (classIndices != null) {
			for (int i = 0; i < classIndices.length; i++) {
				if (classIndices[i] == classIndex) return;
			}
			int[] newCI = new int[classIndices.length + 1];
			System.arraycopy(classIndices, 0, newCI, 0, classIndices.length);
			newCI[classIndices.length] = classIndex;
			classIndices = newCI;
		} else {
			classIndices = new int[] { classIndex};
		}
	}

	public boolean isClassAttribute(int attrIndex) {
		if (classIndices != null) {
			for (int i = 0; i < classIndices.length; i++) {
				if (classIndices[i] == attrIndex) { return true; }
			}
		}
		return false;
	}

	private void removeClassIndex(int classIndex) {
		if (classIndices != null) {
			for (int i = 0; i < classIndices.length; i++) {
				if (classIndices[i] == classIndex) {
					int[] newCI = new int[classIndices.length - 1];
					if (i > 0) System.arraycopy(classIndices, 0, newCI, 0, i);
					if (i < classIndices.length - 1)
							System.arraycopy(classIndices, i + 1, newCI, i,
									classIndices.length - i - 1);
					classIndices = newCI;
					return;
				}
			}
		}
	}

	public int numClassIndices() {
		if (classIndices == null)
			return 0;
		else
			return classIndices.length;
	}

	/**
	 * Returns the class attribute.
	 *
	 * @return the class attribute
	 * @exception UnassignedClassException if the class is not set
	 */
	public Attribute classAttribute(int indexNr) {

		if (classIndices == null || classIndices.length <= indexNr) { throw new UnassignedClassException(
				"Class index is not set!"); }
		return attribute(classIndices[indexNr]);
	}

	/**
	 * Returns the class attribute's index.
	 *
	 * @return the class index as an integer
	 */
	public int classIndex(int indexNr) {
		if (classIndices == null || classIndices.length <= indexNr) { throw new UnassignedClassException(
				"Class index is not set!"); }
		return classIndices[indexNr];
	}

	public void setClassIndices(int[] classIndices) {
		this.classIndices = classIndices;
	}

	public int[] classIndices() {
		return classIndices;
	}

	public int numLabeledClasses(Instance instance) {
		if (classIndices == null)
			return 0;
		else {
			int c = 0;
			for (int i = 0; i < classIndices.length; i++) {
				if ((int) instance.value(classIndices[i]) == 1) c++;
			}
			return c;
		}
	}

	public int[] getLabeledClasses(Instance instance) {
		if (classIndices == null)
			return new int[0];
		else {
			int[] cls = new int[classIndices.length];
			int pos=0;
			for (int i = 0; i < classIndices.length; i++) {
				if ((int) instance.value(classIndices[i]) == 1){
						cls[pos]=i;
						pos++;
				}
			}
			int[] result = new int[pos];
			System.arraycopy(cls,0,result,0,pos);
			return result;
		}
	}

	public double[] getClassValues(Instance instance) {
		if (classIndices == null)
			return new double[0];
		else {
			double[] result = new double[classIndices.length];
			for (int i = 0; i < classIndices.length; i++) {
				result[i] = instance.value(classIndices[i]);
			}
			return result;
		}
	}

	/**
	 * Copies instances from one set to the end of another 
	 * one.
	 *
	 * @param source the source of the instances
	 * @param from the position of the first instance to be copied
	 * @param dest the destination for the instances
	 * @param num the number of instances to be copied
	 */
	protected void copyInstances(int from, Instances dest, int num) {

		for (int i = 0; i < num; i++) {
			dest.add(instance(from + i));
		}
	}

	/**
	 * Creates the test set for one fold of a cross-validation on 
	 * the dataset.
	 *
	 * @param numFolds the number of folds in the cross-validation. Must
	 * be greater than 1.
	 * @param numFold 0 for the first fold, 1 for the second, ...
	 * @return the test set as a set of weighted instances
	 * @exception IllegalArgumentException if the number of folds is less than 2
	 * or greater than the number of instances.
	 */
	public Instances testCV(int numFolds, int numFold) {

		int numInstForFold, first, offset;
		MVInstances test;

		if (numFolds < 2) { throw new IllegalArgumentException(
				"Number of folds must be at least 2!"); }
		if (numFolds > numInstances()) { throw new IllegalArgumentException(
				"Can't have more folds than instances!"); }
		numInstForFold = numInstances() / numFolds;
		if (numFold < numInstances() % numFolds) {
			numInstForFold++;
			offset = numFold;
		} else
			offset = numInstances() % numFolds;
		test = new MVInstances(this, numInstForFold);
		first = numFold * (numInstances() / numFolds) + offset;
		copyInstances(first, test, numInstForFold);
		return test;
	}

	/**
	 * Creates the training set for one fold of a cross-validation 
	 * on the dataset. The data is subsequently randomized based
	 * on the given random number generator.
	 *
	 * @param numFolds the number of folds in the cross-validation. Must
	 * be greater than 1.
	 * @param numFold 0 for the first fold, 1 for the second, ...
	 * @return the training set 
	 * @exception IllegalArgumentException if the number of folds is less than 2
	 * or greater than the number of instances.
	 */
	public Instances trainCV(int numFolds, int numFold) {

		int numInstForFold, first, offset;
		MVInstances train;

		if (numFolds < 2) { throw new IllegalArgumentException(
				"Number of folds must be at least 2!"); }
		if (numFolds > numInstances()) { throw new IllegalArgumentException(
				"Can't have more folds than instances!"); }
		numInstForFold = numInstances() / numFolds;
		if (numFold < numInstances() % numFolds) {
			numInstForFold++;
			offset = numFold;
		} else
			offset = numInstances() % numFolds;
		train = new MVInstances(this, numInstances() - numInstForFold);
		first = numFold * (numInstances() / numFolds) + offset;
		copyInstances(0, train, first);
		copyInstances(first + numInstForFold, train, numInstances() - first
				- numInstForFold);

		return train;
	}

}