/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui;

import java.awt.Font;
import java.beans.Introspector;
import java.io.File;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.FontUIResource;

import xxl.applications.gui.windows.MainFrame;

/**
 * @author maxim
 * @version 1.0
 */
public final class Main {

	private int uiFontSize = 11;
	private Class laf = null;

	public static void main(String[] args) {
		try {
			new Main(args);
		}
		catch (Throwable th) {
			th.printStackTrace();
			//System.err.println(th.getMessage());
			System.exit(1);
		}
	}

	private Main(String[] args) throws Exception {
		processArgs(args);
		Introspector.setBeanInfoSearchPath(
			new String[] {
				"xxl.applications.gui.graph.beaninfo",
				"xxl.applications.gui.code.beaninfo" });
		if (laf != null) {
			UIManager.setLookAndFeel(laf.getName());
		}
		String cwd = System.getProperty("xxl.user.home");
		//TODO do not check the validity of userdir twice...
		if (cwd == null) {
			cwd =
				System.getProperty("user.home")
					+ System.getProperty("file.separator")
					+ ".xxlviewer";
			setUserDirectory(cwd);
		}
		init();
	}

	private void setUserDirectory(String path) {
		File userdir = new File(path);
		String msg = "Could not create directory structure";
		if (!userdir.exists() && !userdir.isDirectory() && !userdir.mkdir()) {
			throw new RuntimeException(msg);
		}
		if (!userdir.canRead() && !userdir.canWrite())
			throw new RuntimeException("Could not read from and write to specified directory");
		File src = new File(path + File.separator + "src");
		if (!src.exists() && !src.mkdir()) {
			throw new RuntimeException(msg);
		}
		File dest = new File(path + File.separator + "classes");
		if (!dest.exists() && !dest.mkdir())
			throw new RuntimeException(msg);
		File cat = new File(path + File.separator + "categories");
		if (!cat.exists() && !cat.mkdir()) {
			throw new RuntimeException(msg);
		}
		System.getProperties().put("xxl.user.home", userdir.getAbsolutePath());
	}

	private void init() {
		if (!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {

				public void run() {
					init();
				}
			});
			return;
		}
		if (uiFontSize > 0) {
			Font dialogPlain =
				new javax.swing.plaf.FontUIResource("Dialog", java.awt.Font.PLAIN, uiFontSize);
			Font dialogBold =
				new javax.swing.plaf.FontUIResource("Dialog", java.awt.Font.BOLD, uiFontSize);
			Font serifPlain =
				new javax.swing.plaf.FontUIResource("Serif", java.awt.Font.PLAIN, uiFontSize);
			Font sansSerifPlain = new FontUIResource("SansSerif", java.awt.Font.PLAIN, uiFontSize);
			Font monospacedPlain =
				new FontUIResource("Monospaced", java.awt.Font.PLAIN, uiFontSize);
			Font editorFont = new FontUIResource("Monospaced", java.awt.Font.PLAIN, uiFontSize);
			UIManager.getDefaults().put("EditorFont", editorFont);
			UIManager.getDefaults().put("TextArea.font", monospacedPlain);
			UIManager.getDefaults().put("TextField.font", dialogPlain);
			UIManager.getDefaults().put("Button.font", dialogPlain);
			UIManager.getDefaults().put("ComboBox.font", dialogPlain);
			UIManager.getDefaults().put("RadioButton.font", dialogPlain);
			UIManager.getDefaults().put("CheckBox.font", dialogPlain);
			UIManager.getDefaults().put("Label.font", dialogPlain);
			UIManager.getDefaults().put("List.font", dialogPlain);
			UIManager.getDefaults().put("MenuBar.font", dialogPlain);
			UIManager.getDefaults().put("MenuItem.font", dialogPlain);
			UIManager.getDefaults().put("MenuItem.acceleratorFont", dialogPlain);
			UIManager.getDefaults().put("RadioButtonMenuItem.font", dialogPlain);
			UIManager.getDefaults().put("CheckBoxMenuItem.font", dialogPlain);
			UIManager.getDefaults().put("Menu.font", dialogPlain);
			UIManager.getDefaults().put("PopupMenu.font", dialogPlain);
			UIManager.getDefaults().put("OptionPane.font", dialogPlain);
			UIManager.getDefaults().put("Panel.font", dialogPlain);
			UIManager.getDefaults().put("ScrollPane.font", dialogPlain);
			UIManager.getDefaults().put("Viewport.font", dialogPlain);
			UIManager.getDefaults().put("TabbedPane.font", dialogPlain);
			UIManager.getDefaults().put("Table.font", dialogPlain);
			UIManager.getDefaults().put("EditorPane.font", monospacedPlain);
			UIManager.getDefaults().put("TableHeader.font", dialogPlain);
			UIManager.getDefaults().put("PasswordField.font", monospacedPlain);
			UIManager.getDefaults().put("TitledBorder.font", dialogPlain);
			UIManager.getDefaults().put("ToolBar.font", dialogPlain);
			UIManager.getDefaults().put("ToolTip.font", sansSerifPlain);
			UIManager.getDefaults().put("InternalFrame.titleFont", dialogBold);
			UIManager.getDefaults().put("Dialog.titleFont", dialogBold);
			UIManager.getDefaults().put("windowTitleFont", dialogBold);
			UIManager.getDefaults().put("controlFont", dialogPlain);
		}
		//setDefaultLookAndFeel();
		MainFrame ds = MainFrame.getInstance();
		ds.pack();
		ds.setVisible(true);
	}

	private static void setDefaultLookAndFeel() {
		try {
			//UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			//UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
			UIManager.setLookAndFeel("com.incors.plaf.kunststoff.KunststoffLookAndFeel");
		}
		catch (Exception e) {
		}
	}

	private void processArgs(String[] args) {
		for (int i = 0; i < args.length; i++) {
			String arg = args[i];
			if (arg.equals("--help") || arg.equals("-h")) {
				printUsage();
				System.exit(0);
			}
			else if (arg.equals("--fontsize")) {
				try {
					uiFontSize = Integer.parseInt(args[i + 1]);
					i++;
				}
				catch (NumberFormatException nfe) {
					String msg = "Could not parse font size";
					throw new RuntimeException(msg);
				}
				catch (ArrayIndexOutOfBoundsException obe) {
					String msg =
						"You must specify the font size when using the --fontsize argument";
					throw new RuntimeException(msg);
				}
			}
			else if (arg.equals("--userdir")) {
				try {
					setUserDirectory(args[i + 1]);
					i++;
				}
				catch (ArrayIndexOutOfBoundsException obe) {
					String msg = "You must specify the directory when using the --userdir argument";
					throw new RuntimeException(msg);
				}
			}
			else if (arg.equals("--laf")) {
				String lafClass = args[i + 1];
				i++;
				try {
					laf = Class.forName(lafClass);
				}
				catch (ClassNotFoundException cnfe) {
					String msg = "Could not find specified LaF class";
					throw new RuntimeException(msg);
				}
				catch (ArrayIndexOutOfBoundsException obe) {
					String msg = "You must specify the class name when using the --laf argument";
					throw new RuntimeException(msg);
				}
			}
		}
	}

	private void printUsage() {
		StringBuffer str_buf = new StringBuffer();
		str_buf.append("Usage: MainApp {options} arguments");
		str_buf.append("\n\n");
		str_buf.append("Options:");
		str_buf.append('\n');
		str_buf.append("  --help, -h \t\t prints this message");
		str_buf.append('\n');
		str_buf.append("  --fontsize <size> \t base font size of the UI");
		str_buf.append('\n');
		str_buf.append(
			"  --userdir <path> \t directory to store user settings and generated files");
		str_buf.append('\n');
		str_buf.append("  --laf <classname> \t use given LaF class instead of the default");
		str_buf.append('\n');
		System.out.println(str_buf.toString());
	}
}
