/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.actions;

import java.util.HashMap;

import javax.swing.Action;
import javax.swing.KeyStroke;

/**
 * Basisklasse zur Verwaltung von Action-Objekten. Zur Identifikation von Actions wird
 * ihr actionCommand verwendet (String). Eine Anwendung implementiert lediglich die Methode
 * addActions(). 
 *
 */

public abstract class ActionManager {

	private HashMap action2NameMap;
	private HashMap action2KeystrokeMap;
	private String name;

	protected static ActionManager sharedInstance;

	protected ActionManager() {
		this("");
	}
	
	protected ActionManager(String name) {
		action2NameMap = new HashMap();
		addActions();
//		setName(name);
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public static ActionManager sharedInstance() {
		return sharedInstance;
	}

	protected abstract void addActions();
	
	protected abstract void configureAction(Action a);
		
	
	protected void addAction(String actionCommand, Action action) {
		action2NameMap.put(actionCommand, action);
		configureAction(action);
	}
	
	protected void addKeyStroke(Action action, KeyStroke keyStroke) {
		action2KeystrokeMap.put(action, keyStroke);
	}

	public Action getAction(String name) {
		return (Action) action2NameMap.get(name);
	}

	public CallableAction getCallableAction(String actionCommand) {
		Action a = getAction(actionCommand);
		if (a instanceof CallableAction) {
			return (CallableAction) a;
		}
		return null;
	}
	
	public ToggleAction getToggleAction(String actionCommand) {
		Action a = getAction(actionCommand);		
		if(a instanceof ToggleAction) {
			return (ToggleAction) a;
		}
		return null;
	}

	public void setActionEnabled(String name, boolean enabled) {
		Action action = getAction(name);
		if (action != null) {
			action.setEnabled(enabled);
		}
	}
}
