/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.actions;

import javax.swing.Action;
import javax.swing.SwingUtilities;

/**
 * @author maxim
 * @version 1.0
 */
public final class MainActionManager extends ActionManager {

	protected void addActions() {
		if (!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					addActions();
				}
			});
			return;
		}
		//file
		addAction(OpenAction.ACTION_COMMAND_VALUE, new OpenAction());
		addAction(NewAction.ACTION_COMMAND_VALUE, new NewAction());
		addAction(SaveAction.ACTION_COMMAND_VALUE, new SaveAction());
		addAction(ExitAction.ACTION_COMMAND_VALUE, new ExitAction());
		//edit
		addAction(DeleteAction.ACTION_COMMAND_VALUE, new DeleteAction());
		// graph
		addAction(ConnectingToolAction.ACTION_COMMAND_VALUE, new ConnectingToolAction());
		addAction(SelectionToolAction.ACTION_COMMAND_VALUE, new SelectionToolAction());
		addAction(MovingToolAction.ACTION_COMMAND_VALUE, new MovingToolAction());
		addAction(ZoomInAction.ACTION_COMMAND_VALUE, new ZoomInAction());
		addAction(ZoomOutAction.ACTION_COMMAND_VALUE, new ZoomOutAction());
		addAction(FoldSelectionAction.ACTION_COMMAND_VALUE, new FoldSelectionAction());
		addAction(UnfoldSelectionAction.ACTION_COMMAND_VALUE, new UnfoldSelectionAction());
		//code
		addAction(GenerateCodeAction.ACTION_COMMAND_VALUE, new GenerateCodeAction());
		addAction(AddNewAction.ACTION_COMMAND_VALUE, new AddNewAction());
		addAction(ExecuteAction.ACTION_COMMAND_VALUE, new ExecuteAction());
		//windows
		addAction(TileVerticalAction.ACTION_COMMAND_VALUE, new TileVerticalAction());
		addAction(TileHorizontalAction.ACTION_COMMAND_VALUE, new TileHorizontalAction());
		//help
		addAction(AboutAction.ACTION_COMMAND_VALUE, new AboutAction());
	}

	public static ActionManager sharedInstance() {
		if (sharedInstance == null) {
			sharedInstance = new MainActionManager();
			sharedInstance.setName("main");
		}
		return sharedInstance;
	}

	protected void configureAction(Action a) {
		a.setEnabled(false);
	}

}
