/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;

import xxl.applications.gui.actions.ActionManager;
import xxl.applications.gui.actions.ToggleAction;

/**
 * @author maxim
 * @version 1.0
 */
public abstract class AbstractMenuBar extends JMenuBar {

	protected ActionManager manager;

	protected AbstractMenuBar(ActionManager manager) {
		setManager(manager);
		configureMenu();
	}

	protected ActionManager getManager() {
		return manager;
	}

	protected void setManager(ActionManager manager) {
		this.manager = manager;
	}

	protected abstract void configureMenu();

	protected static void configureMenuItem(JMenuItem item, Action action) {
		item.setName((String) action.getValue(Action.NAME));
		KeyStroke stroke = (KeyStroke) action.getValue(Action.ACCELERATOR_KEY);
		if (stroke != null)
			item.setAccelerator(stroke);
	}

	protected static JMenu createMenu(String name, char mnemonic) {
		JMenu menu = new JMenu(name);
		menu.setMnemonic(mnemonic);
		return menu;
	}

	protected static JRadioButtonMenuItem addRadioButton(JMenuItem item, ToggleAction action) {
		JRadioButtonMenuItem button = new JRadioButtonMenuItem(action);
		button.setSelected(action.isToggled());
		button.addItemListener(action);
		action.addPropertyChangeListener(new AbstractToolBar.PropertyChangeAdapter(button));
		configureMenuItem(button, action);
		item.add(button);
		return button;
	}

	protected void addMenuItem(JMenu menu, Action action) {
		menu.add(action);
	}
}
