/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.JPopupMenu;

/**
 * Basisklasse fuer Werkzeuge. Stellt Funktionen zum Zeichen von Tools auf einer
 * {@link xxl.applications.gui.graph.viewer2d.GraphPane}, implementiert
 * {@link java.awt.event.MouseListener} und {@link java.awt.event.MouseMotionListener}
 * und stellt ein Kontextmenue zur Verfuegung.
 * @author maxim
 */
public abstract class AbstractTool implements MouseListener, MouseMotionListener {

	public static final String CURSOR_PROPERTY = "cursor";
	public static final String PANE_PROPERTY = "pane";

	private transient PropertyChangeSupport changes;
	private DefaultGraphPanel pane;

	public AbstractTool() {
	}

	public AbstractTool(DefaultGraphPanel pane) {
		this();
		setPane(pane);
	}

	protected abstract JPopupMenu getContexMenu();

	public void setPane(DefaultGraphPanel pane) {
		DefaultGraphPanel oldPane = this.pane;
		if (oldPane != pane) {
			this.pane = pane;
			firePropertyChange(PANE_PROPERTY, oldPane, pane);
		}
	}

	public abstract void draw(Graphics g);

	public DefaultGraphPanel getPane() {
		return this.pane;
	}

	protected PropertyChangeSupport getPropertySupport() {
		if (changes == null)
			changes = new PropertyChangeSupport(this);
		return changes;
	}

	protected void firePropertyChange(PropertyChangeEvent evt) {
		getPropertySupport().firePropertyChange(evt);
	}
	protected void firePropertyChange(String name, Object oldValue, Object newValue) {
		getPropertySupport().firePropertyChange(name, oldValue, newValue);
	}
	public void addPropertyChangeListener(PropertyChangeListener l) {
		getPropertySupport().addPropertyChangeListener(l);
	}

	public void removePropertyChangeListener(PropertyChangeListener l) {
		getPropertySupport().removePropertyChangeListener(l);
	}

	private void checkPopupTrigger(MouseEvent e) {
		JPopupMenu menu = getContexMenu();
		if (e.isPopupTrigger()) {
			if (menu != null) {
				Point location = e.getPoint();
				//if (pane != null) {
				//TODO transform works not 100%
					location = pane.transformFromGraphToView(location);
				//}
				menu.show(e.getComponent(), location.x, location.y);
				if (menu instanceof AbstractPopupMenu) {
					((AbstractPopupMenu) menu).setLastScreenLocation(location.x, location.y);
				}
				return;
			}
		}
	}

	public void mouseClicked(MouseEvent e) {
		checkPopupTrigger(e);
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
		checkPopupTrigger(e);
	}

	public void mouseReleased(MouseEvent e) {
		checkPopupTrigger(e);
	}

	public void mouseDragged(MouseEvent e) {
	}

	public void mouseMoved(MouseEvent e) {
	}

}
