/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import java.awt.Dimension;
import java.awt.Insets;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import xxl.applications.gui.actions.ActionManager;
import xxl.applications.gui.actions.ToggleAction;
import xxl.applications.gui.util.Utilities;

public abstract class AbstractToolBar extends JToolBar {

	private ActionManager manager;
	private Dimension buttonSize;
	private Insets buttonInsets;

	protected AbstractToolBar(ActionManager manager) {
		setManager(manager);
		buttonSize = new Dimension(Utilities.BUTTCON_PREF_SIZE);
		buttonInsets = new Insets(0, 0, 0, 0);
		addButtons();
	}

	protected void setManager(ActionManager manager) {
		this.manager = manager;
	}

	protected ActionManager getManager() {
		return manager;
	}

	protected abstract void addButtons();

	protected JButton addButton(Action action) {
		JButton button = add(action);
		button.setToolTipText((String) action.getValue(Action.SHORT_DESCRIPTION));
		return button;
	}

	protected JToggleButton addToggleButton(ToggleAction action) {
		JToggleButton button = new JToggleButton(action);
		button.setSelected(action.isToggled());
		button.addItemListener(action);
		action.addPropertyChangeListener(new PropertyChangeAdapter(button));
		button.setText(null);
		configureToggleButton(button);
		add(button);
		return button;
	}

	protected void configureToggleButton(JToggleButton button) {
	}

	protected JButton createActionComponent(Action a) {
		Icon icon = a != null ? (Icon) a.getValue(Action.SMALL_ICON) : null;
		boolean enabled = a != null ? a.isEnabled() : true;
		String tooltip = a != null ? (String) a.getValue(Action.SHORT_DESCRIPTION) : null;
		//Overwrite to use ToolBarButtons (but i dont really use them now...)
		JButton b = new JButton(icon) {

			//JButton b = new ToolbarButton(icon) {
			protected PropertyChangeListener createActionPropertyChangeListener(Action a) {
				PropertyChangeListener pcl = createActionChangeListener(this);
				if (pcl == null) {
					pcl = super.createActionPropertyChangeListener(a);
				}
				return pcl;
			}
		};
		if (icon != null) {
			b.putClientProperty("hideActionText", Boolean.TRUE);
		}
		b.setHorizontalTextPosition(JButton.CENTER);
		b.setVerticalTextPosition(JButton.BOTTOM);
		b.setEnabled(enabled);
		b.setToolTipText(tooltip);
		return b;
	}

	protected static class PropertyChangeAdapter implements PropertyChangeListener {

		private AbstractButton button;

		public PropertyChangeAdapter(AbstractButton button) {
			this.button = button;
		}

		public void propertyChange(PropertyChangeEvent evt) {
			if (evt.getPropertyName().equals(ToggleAction.TOGGLE_PROPERTY)) {
				Boolean b = (Boolean) evt.getNewValue();
				button.setSelected(b.booleanValue());
			}
		}
	}
}
