/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import java.awt.BorderLayout;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import xxl.applications.gui.awt.tree.BootPathTreeObject;
import xxl.applications.gui.awt.tree.CategoryTreeObject;
import xxl.applications.gui.awt.tree.ClassPackageTree;
import xxl.applications.gui.awt.tree.ClassPathTreeObject;
import xxl.applications.gui.awt.tree.JarTreeObject;
import xxl.applications.gui.awt.tree.PackageTreeObject;
import xxl.applications.gui.awt.tree.PathTreeObject;
import xxl.applications.gui.awt.tree.TreeObject;
import xxl.applications.gui.serializer.CategorySerializer;

/**
 * Contains a tree component displaying a structure of classpath and boot path
 * @author maxim
 * 
 */
public class ClassTreePanel extends JPanel {

	private JTree packageTree;
	private DefaultTreeModel model;

	public ClassTreePanel() {
		super();
		setLayout(new BorderLayout());
		initTree();

	}

	private void initTree() {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		model = new DefaultTreeModel(root);
		CategoryTreeObject[] categories = CategorySerializer.readAll();
		for (int i = 0; i < categories.length; i++) {
			categories[i].setModel(model);
			root.add(categories[i]);
		}
		packageTree = new ClassPackageTree(model);
		BootPathTreeObject pto = new BootPathTreeObject();
		ClassPathTreeObject cto = new ClassPathTreeObject();
		root.add(cto);
		root.add(pto);
		pto.setModel(model);

		cto.setModel(model);
		model.nodeStructureChanged(root);

		packageTree.addTreeExpansionListener(new TreeExpansionAdapter());
		packageTree.collapsePath(new TreePath(pto));
		JScrollPane pane = new JScrollPane(packageTree);
		add(pane, BorderLayout.CENTER);

	}

	private class TreeExpansionAdapter implements TreeExpansionListener {

		public void treeExpanded(TreeExpansionEvent event) {
			TreePath path = event.getPath();
			TreeObject to = (TreeObject) path.getLastPathComponent();
			if (to == null)
				return;
			switch (to.getType()) {
				case TreeObject.PATH_ENTRY_OBJECT :
					 ((PathTreeObject) to).addJarEntries();
					break;
				case TreeObject.JAR_FILE_OBJECT :
					 ((JarTreeObject) to).addPackageEntries();
					break;
				case TreeObject.PACKAGE_OBJECT :
					 ((PackageTreeObject) to).addClassEntries();
			}
		}

		public void treeCollapsed(TreeExpansionEvent event) {
			TreePath path = event.getPath();
			TreeObject to = (TreeObject) path.getLastPathComponent();
			if (to == null)
				return;
			int children = to.getChildCount();
			if (children == 0)
				treeExpanded(event);
			else
				to.invalidate();
		}
	}

	public static void main(String[] args) throws Exception {
		//UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		JFrame frame = new JFrame();
		frame.setSize(640, 480);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(new ClassTreePanel());
		frame.setVisible(true);

	}
}
