/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import java.awt.Graphics;

import javax.swing.JPopupMenu;

import xxl.applications.gui.graph.ClassGraph;
import xxl.applications.gui.graph.Graph;
import xxl.applications.gui.graph.Vertex;
import xxl.applications.gui.graph.event.SelectionEvent;
import xxl.applications.gui.graph.event.SelectionListener;
import xxl.applications.gui.graph.viewer2d.GraphDrawer;
import xxl.applications.gui.graph.viewer2d.GraphPane;
import xxl.applications.gui.graph.viewer2d.Grid;
import xxl.applications.gui.property.PropertyPane;

/**
 * @author maxim
 */
public final class DefaultGraphPanel extends GraphPane implements SelectionListener {

	protected transient AbstractTool tool;
	private JPopupMenu popupMenu;
	private transient Vertex [] selectedVertexes;

	public DefaultGraphPanel() {
		this(new Graph());
	}

	public DefaultGraphPanel(Graph graph) {
		super(new GraphDrawer(graph));
		setAdapt(false);
		setGrid(new Grid());
	}

	public void setTool(AbstractTool tool) {
		removeTool();
		this.tool = tool;
		tool.setPane(this);
		addMouseListener(tool);
		addMouseMotionListener(tool);
		System.out.println("DefaultGraphPanel.setTool() " + this.hashCode());
	}

	public AbstractTool getTool() {
		return tool;
	}

	public void removeTool() {
		if (tool != null) {
			removeMouseListener(tool);
			removeMouseMotionListener(tool);
			tool.setPane(null);
			tool = null;
			System.out.println("DefaultGraphPanel.removeTool() " + this.hashCode());
		}
	}

	public ClassGraph getClassGraph() {
		return (ClassGraph) getDrawer().getGraph();
	}

	public void paint(Graphics g) {
		super.paint(g);
		if (tool != null) {
			tool.draw(g);
		}
	}

	//	TODO i dont like this..
	public Vertex[] getSelection() {
		return selectedVertexes;
	}
	

	public void handleSelection(SelectionEvent e) {
		if (e.getPane() == this) {
			//Rectangle cachedSelection = e.getSelectionBounds();
			if (selectedVertexes != null) {
				PropertyPane.getInstance().setBean(null);
				for (int i = 0; i < selectedVertexes.length; i++) {
					selectedVertexes[i].setSelected(false);
				}
			}
			//selectedVertexes = drawer.getEclosedVertexes(cachedSelection);
			selectedVertexes = e.getSelection();
			if (selectedVertexes.length == 1) {
				PropertyPane.getInstance().setBean(selectedVertexes[0]);
			}
			for (int i = 0; i < selectedVertexes.length; i++) {
				selectedVertexes[i].setSelected(true);
			}
		}
	}

}
