/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import javax.swing.JMenu;

import xxl.applications.gui.actions.AboutAction;
import xxl.applications.gui.actions.ActionManager;
import xxl.applications.gui.actions.ConnectingToolAction;
import xxl.applications.gui.actions.DeleteAction;
import xxl.applications.gui.actions.ExecuteAction;
import xxl.applications.gui.actions.ExitAction;
import xxl.applications.gui.actions.FoldSelectionAction;
import xxl.applications.gui.actions.GenerateCodeAction;
import xxl.applications.gui.actions.MainActionManager;
import xxl.applications.gui.actions.MovingToolAction;
import xxl.applications.gui.actions.NewAction;
import xxl.applications.gui.actions.OpenAction;
import xxl.applications.gui.actions.SaveAction;
import xxl.applications.gui.actions.SelectionToolAction;
import xxl.applications.gui.actions.TileHorizontalAction;
import xxl.applications.gui.actions.TileVerticalAction;
import xxl.applications.gui.actions.UnfoldSelectionAction;
import xxl.applications.gui.actions.ZoomInAction;
import xxl.applications.gui.actions.ZoomOutAction;

/**
 * @author maxim
 * @version 1.0
 */
public class MainMenu extends AbstractMenuBar {

	public MainMenu() {
		super(MainActionManager.sharedInstance());
	}

	protected void configureMenu() {
		add(createFileMenu("File", 'F'));
		add(createEditMenu("Edit", 'E'));
		add(createGraphMenu("Graph", 'G'));
		add(createCodeMenu("Code", 'o'));
		add(createWindowMenu("Window", 'W'));
		add(createHelpMenu("Help", 'H'));
	}

	private JMenu createFileMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);
		addMenuItem(menu, getManager().getAction(NewAction.ACTION_COMMAND_VALUE));
		addMenuItem(menu, getManager().getAction(OpenAction.ACTION_COMMAND_VALUE));
		menu.addSeparator();
		addMenuItem(menu, getManager().getAction(SaveAction.ACTION_COMMAND_VALUE));
		menu.addSeparator();
		addMenuItem(menu, getManager().getAction(ExitAction.ACTION_COMMAND_VALUE));
		return menu;
	}

	private JMenu createEditMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);
		addMenuItem(menu, getManager().getAction(FoldSelectionAction.ACTION_COMMAND_VALUE));
		addMenuItem(menu, getManager().getAction(UnfoldSelectionAction.ACTION_COMMAND_VALUE));
		addMenuItem(menu, getManager().getAction(DeleteAction.ACTION_COMMAND_VALUE));
		return menu;
	}

	private JMenu createWindowMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);
		addMenuItem(menu, getManager().getAction(TileHorizontalAction.ACTION_COMMAND_VALUE));
		addMenuItem(menu, getManager().getAction(TileVerticalAction.ACTION_COMMAND_VALUE));
		//menu.addSeparator();
		return menu;
	}

	private JMenu createCodeMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);
		addMenuItem(menu, getManager().getAction(GenerateCodeAction.ACTION_COMMAND_VALUE));
		addMenuItem(menu, getManager().getAction(ExecuteAction.ACTION_COMMAND_VALUE));
		return menu;
	}

	private JMenu createGraphMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);
		ActionManager manager = getManager();
		addMenuItem(menu, manager.getAction(ZoomInAction.ACTION_COMMAND_VALUE));
		addMenuItem(menu, manager.getAction(ZoomOutAction.ACTION_COMMAND_VALUE));
		menu.addSeparator();
		addRadioButton(menu, manager.getToggleAction(ConnectingToolAction.ACTION_COMMAND_VALUE));
		addRadioButton(menu, manager.getToggleAction(SelectionToolAction.ACTION_COMMAND_VALUE));
		addRadioButton(menu, manager.getToggleAction(MovingToolAction.ACTION_COMMAND_VALUE));
		//	menu.add(createViewMenu("View", 'i'));
		//	menu.add(createToolMenu("Tools", 'T'));
		return menu;
	}

	private JMenu createToolMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);
		ActionManager manager = getManager();
		addRadioButton(menu, manager.getToggleAction(ConnectingToolAction.ACTION_COMMAND_VALUE));
		addRadioButton(menu, manager.getToggleAction(SelectionToolAction.ACTION_COMMAND_VALUE));
		addRadioButton(menu, manager.getToggleAction(MovingToolAction.ACTION_COMMAND_VALUE));
		return menu;
	}

	private JMenu createViewMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);
		ActionManager manager = getManager();
		addMenuItem(menu, manager.getAction(ZoomInAction.ACTION_COMMAND_VALUE));
		addMenuItem(menu, manager.getAction(ZoomOutAction.ACTION_COMMAND_VALUE));
		return menu;
	}
	
	private JMenu createHelpMenu(String menuName, char mnemonic) {
		JMenu menu = createMenu(menuName, mnemonic);		
		addMenuItem(menu, manager.getAction(AboutAction.ACTION_COMMAND_VALUE));
		return menu;
	}

}
