/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.JPopupMenu;

import xxl.applications.gui.graph.Vertex;
import xxl.applications.gui.graph.viewer2d.GraphDrawer;

/**
 * Moving graph tool 
 * @author maxim
 * 
 */
public final class MovingTool extends AbstractTool {

	private static Cursor HAND_CURSOR = new Cursor(Cursor.HAND_CURSOR);
	private static Cursor DEFAULT_CURSOR = new Cursor(Cursor.DEFAULT_CURSOR);
	private static MovingTool sharedInstance;

	public static MovingTool getInstance() {
		if (sharedInstance == null)
			sharedInstance = new MovingTool();
		return sharedInstance;
	}

	private transient Point lastClicked = new Point();
	private transient Point origin = new Point();
	private transient Vertex[] selection;
	private transient Rectangle cachedBounds;

	private MovingTool() {
		super();
	}

	public void mousePressed(MouseEvent e) {		
		super.mousePressed(e);
		if (getPane() == null) {
			return;
		}
		GraphDrawer drawer = getPane().getDrawer();
		//determine selection - try to get any from pane
		selection = getPane().getSelection();
		//if it's empty 		
		if (selection == null || selection.length == 0) {
			//look for vertex under cursor			
			Vertex v = drawer.vertexAtPoint(e.getPoint());
			if (v != null)
				selection = new Vertex[] { v };

		}
		// selection still empty - return
		if (selection == null || selection.length == 0)
			return;
		cachedBounds = new Rectangle();
		//now cache bounds for selection to draw it during drugging
		for (int i = 0; i < selection.length; i++) {
			Rectangle bounds = drawer.getVertexBounds(selection[i]);
			if (i == 0)
				cachedBounds.setBounds(bounds.x, bounds.y, bounds.width, bounds.height);
			else
				cachedBounds.add(bounds);
		}
		//and finally remeber the point of the click
		if (cachedBounds.contains(e.getPoint())) {
			lastClicked.setLocation(e.getPoint());
			origin.setLocation(cachedBounds.getLocation());
			getPane().setCursor(HAND_CURSOR);
		}
		else
			cachedBounds = null;
	}

	public void mouseReleased(MouseEvent e) {
		super.mouseReleased(e);
		if (cachedBounds != null && getPane() != null) {
			for (int i = 0; i < selection.length; i++) {
				int dx = cachedBounds.x - origin.x;
				int dy = cachedBounds.y - origin.y;
				//sonst klappt propertyEvent nicht....
				Point p = new Point(selection[i].getLocation());
				p.translate(dx, dy);
				selection[i].setLocation(p);
			}
			cachedBounds = null;
			getPane().setCursor(DEFAULT_CURSOR);
		}
	}

	public void draw(Graphics g) {
		if (cachedBounds != null) {
			Point upperLeft = getPane().transformFromGraphToView(new Point(cachedBounds.x, cachedBounds.y));
			Point lowerRight =
				getPane().transformFromGraphToView(
					new Point(cachedBounds.x + cachedBounds.width, cachedBounds.y + cachedBounds.height));
			g.drawRect(upperLeft.x, upperLeft.y, lowerRight.x - upperLeft.x, lowerRight.y - upperLeft.y);
		}
	}

	public void mouseDragged(MouseEvent e) {
		if (cachedBounds != null && getPane() != null) {
			Point p = e.getPoint();
			int dx = p.x - lastClicked.x;
			int dy = p.y - lastClicked.y;
			cachedBounds.setLocation(origin.x + dx, origin.y + dy);
			getPane().repaint();
		}
	}

	protected JPopupMenu getContexMenu() {
		return null;
	}
}
