/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt;

import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseEvent;

import javax.swing.JPopupMenu;
import javax.swing.event.EventListenerList;

import xxl.applications.gui.graph.Vertex;
import xxl.applications.gui.graph.event.SelectionEvent;
import xxl.applications.gui.graph.event.SelectionListener;

/**
 * @author maxim
 */

public final class SelectionTool extends AbstractTool {

	private static SelectionTool sharedInstance;
	private static Stroke SELECTOR_STROKE = new BasicStroke(1, BasicStroke.CAP_BUTT,
			BasicStroke.JOIN_MITER, 8, new float[]{10, 5, 5, 5}, 0);

	public static SelectionTool getInstance() {
		if (sharedInstance == null)
			sharedInstance = new SelectionTool();
		return sharedInstance;
	}
	/** Point of initial click */
	private transient Point start;
	/** Bounds of the selected area */
	private transient Rectangle bounds;
	private transient EventListenerList eventListeners = new EventListenerList();

	private SelectionTool() {
		super();
	}

	public void setPane(DefaultGraphPanel pane) {
		removeSelectionListener(getPane());
		if (pane != null)
			addSelectionListener(pane);
		super.setPane(pane);

	}

	public void addSelectionListener(SelectionListener listener) {
		eventListeners.add(SelectionListener.class, listener);
	}

	public void removeSelectionListener(SelectionListener listener) {
		eventListeners.remove(SelectionListener.class, listener);
	}

	protected void fireSelectionEvent(SelectionEvent se) {
		Object[] listeners = eventListeners.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == SelectionListener.class)
				((SelectionListener) listeners[i + 1]).handleSelection(se);
		}
	}

	public void mousePressed(MouseEvent e) {
		super.mousePressed(e);
		if (e.getButton() == MouseEvent.BUTTON1) {
			start = e.getPoint();
			bounds = new Rectangle();
		}
	}

	public void mouseDragged(MouseEvent e) {
		if (start != null && getPane() != null) {
			Point end = e.getPoint();
			int x = Math.min(start.x, end.x);
			int y = Math.min(start.y, end.y);
			int width = Math.abs(end.x - start.x);
			int height = Math.abs(end.y - start.y);
			bounds.setBounds(x, y, width, height);
			getPane().repaint();
		}
	}

	public void mouseReleased(MouseEvent e) {
		super.mouseReleased(e);
		if (bounds != null && getPane() != null) {
			fireSelectionEvent(new SelectionEvent(getPane(), getPane().getDrawer()
					.getEclosedVertexes(bounds)));
			start = null;
			bounds = null;
			getPane().repaint();
		}
	}

	public void mouseClicked(MouseEvent e) {
		super.mouseClicked(e);
		Vertex v = getPane().getDrawer().vertexAtPoint(e.getPoint());
		if (v != null) {
			fireSelectionEvent(new SelectionEvent(getPane(), new Vertex[]{v}));
		}

	}

	public void draw(Graphics g) {
		if (bounds != null) {
			Point upperLeft = getPane().transformFromGraphToView(new Point(bounds.x, bounds.y));
			Point lowerRight = getPane().transformFromGraphToView(
					new Point(bounds.x + bounds.width, bounds.y + bounds.height));
			Graphics2D g2d = (Graphics2D) g;
			Stroke old = g2d.getStroke();
			g2d.setStroke(SELECTOR_STROKE);
			g2d.drawRect(upperLeft.x, upperLeft.y, lowerRight.x - upperLeft.x, lowerRight.y
					- upperLeft.y);
			g2d.setStroke(old);
		}
	}

	protected JPopupMenu getContexMenu() {
		return null;
	}

}
