/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt.tree;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * This class provides functionality around the class path.
 * @author K. Buerger, maxim 
 * @version 1.0
 */
public class ClassPackageInfo {

	/** Adds a new jar file to the tree.
	 * @param filIn the jar file
	 * @param tstIn the treeset
	 */
	private static void addClassesOfJarFile(JarFile filIn, TreeSet tstIn) {
		Enumeration enuTmp;
		JarEntry jarTmp;
		File filTmp = null;
		try {
			enuTmp = filIn.entries();
			while (enuTmp.hasMoreElements()) {
				jarTmp = (JarEntry) enuTmp.nextElement();
				try {
					if (jarTmp.getName().toLowerCase().endsWith(".class"))
						addClassFile(jarTmp.getName(), tstIn);
				}
				catch (Exception e) {
				}
			}
		}
		catch (Exception e) {
			// XXXTODO ersetzen
			// Jar File nicht gefunden
			System.out.println("Error Jar" + e);
		}
	}

	/** Adds a filname to the tree set and converts it to a standard form
	 * @param strFile the file
	 * @param tstIn the tree set
	 */
	public static void addClassFile(String strFile, TreeSet tstIn) {
		String strTmp;

		strTmp = strFile.replace('\\', '/');

		if (strTmp.length() > 0 && strTmp.charAt(0) == '/')
			strTmp = strTmp.substring(1);

		if (strTmp.length() > 0) {
			// filter all objects which ends with $[0-9]+.class
			if (!Pattern.matches(".*\\$\\d+\\.class", strTmp))
				tstIn.add(strTmp);
		}
	}

	public static TreeSet getAllJars(final String path) {
		TreeSet jarFiles = new TreeSet();
		StringTokenizer str_tok = new StringTokenizer(path, getPathSeparator());
		while (str_tok.hasMoreElements()) {
			String jarEntry = str_tok.nextToken();
			if (jarEntry.endsWith(".jar") || jarEntry.endsWith(".zip"))
				jarFiles.add(jarEntry);
		}
		return jarFiles;
	}

	/** Returns the boot class path.
	 * @return the boot class path
	 */
	public static String getBootClassPath() {
		return System.getProperty("sun.boot.class.path");
	}

	public static TreeSet getBootPathJars() {
		return getAllJars(getBootClassPath());
	}

	/**
	 * Gets classes contained in the given package
	 * @param path location of the package, maybe a jar/zip file or a directory
	 * @param packageName name of the package inside the path
	 * @return 
	 * @throws IllegalArgumentException
	 */
	public static TreeSet getClassesOfPackage(String path, String packageName)
		throws IllegalArgumentException {
		TreeSet classes = new TreeSet();
		FilenameFilter filter = new FilenameFilter() {
			public boolean accept(File dir, String name) {
				return name.endsWith(".class") && !Pattern.matches(".*\\$\\d+\\.class", name);
			}
		};
		if (path.endsWith(".zip") || path.endsWith(".jar")) {
			String pckg = packageName.replace('.', '/');
			Enumeration entries = null;
			try {
				ZipFile file = new ZipFile(new File(path), ZipFile.OPEN_READ);
				entries = file.entries();
			}
			catch (IOException ioe) {
				throw new IllegalArgumentException(path + " is not a valid path entry");
			}
			while (entries.hasMoreElements()) {
				String entry = ((ZipEntry) entries.nextElement()).getName();
				//if package name begins with desired...
				if (entry.startsWith(pckg) && entry.endsWith(".class")) {
					String clazz = entry.substring(0, entry.lastIndexOf('/'));

					//exclude subpackages
					if (clazz.equals(pckg))
						addClassFile(entry, classes);
				}
			}
		}
		else {
			String pckg = packageName.replace('.', File.separatorChar);
			File dir = new File(path + File.separator + pckg);
			if (!dir.isDirectory())
				throw new IllegalArgumentException(dir + " is not a directory");
			File[] entries = dir.listFiles(filter);
			for (int i = 0; i < entries.length; i++) {
				String clazz = entries[i].getName();
				classes.add(packageName + "." + clazz.substring(0, clazz.length() - 6));
			}
		}
		return classes;
	}

	/** Returns the complete class path.
	 * @return the complete class path
	 */
	public static String getCompleteClassPath() {
		return getBootClassPath() + getPathSeparator() + getUserClassPath();
	}

	/** Returns the file separator.
	 * @return the file separator
	 */
	public static String getFileSeparator() {
		return System.getProperty("file.separator");
	}

	//TODO fertig stellen...
	public static TreeSet getPackages(String path) {
		TreeSet packages = new TreeSet();
		if (path.endsWith(".zip") || path.endsWith(".jar")) {
			Enumeration entries = null;
			try {
				ZipFile file = new ZipFile(new File(path), ZipFile.OPEN_READ);
				entries = file.entries();
			}
			catch (IOException e) {
				throw new IllegalArgumentException(e.getMessage());
			}
			while (entries.hasMoreElements()) {
				String entry = ((ZipEntry) entries.nextElement()).getName();
				if (entry.endsWith(".class")) {
					entry = entry.substring(0, entry.lastIndexOf('/'));
					entry = entry.replace('/', '.');
					packages.add(entry);
				}
			}
		}
		else {
			File dir = new File(path);
			if (!dir.isDirectory())
				throw new IllegalArgumentException(path + " is not a directory");
			File[] entries = dir.listFiles();

		}
		return null;
	}

	public static TreeSet getPackagesOfJar(String jarFilePath) {
		TreeSet packages = new TreeSet();
		JarFile file = null;
		try {
			file = new JarFile(new File(jarFilePath), false, JarFile.OPEN_READ);

		}
		catch (Exception e) {
			System.out.println("Jar erroneous: " + jarFilePath);
			//	e.printStackTrace();
			return null;
		}
		Enumeration entries = file.entries();
		while (entries.hasMoreElements()) {
			String entry = ((JarEntry) entries.nextElement()).getName();
			if (entry.endsWith(".class")) {
				entry = entry.substring(0, entry.lastIndexOf('/'));
				entry = entry.replace('/', '.');
				packages.add(entry);
			}
		}
		return packages;
	}

	/** Returns the path separator.
	 * @return the path separator
	 */
	public static String getPathSeparator() {
		return System.getProperty("path.separator");
	}

	/** Returns the user class path.
	 * @return the user class path
	 */
	public static String getUserClassPath() {
		return System.getProperty("java.class.path");
	}

	public static TreeSet getUserPathJars() {
		TreeSet set = getAllJars(getUserClassPath());
		return set;
	}

	private static void addClassesOfDirectory(String dir, TreeSet set) {
		File cwd = new File(dir);
		if (cwd.isDirectory()) {
			File[] entries = cwd.listFiles(new FileFilter() {
				public boolean accept(File pathname) {
					if (pathname.getName().endsWith(".class"))
						return true;
					return false;
				}
			});
			for (int i = 0; i < entries.length; i++)
				addClassFile(entries[i].getName(), set);
		}
	}

	private ClassPackageInfo() {
	}

	public static void main(String[] args) {
		String path = "D:\\Workspaces\\XXLViewer\\bin";
		String packagename = "xxl.applications.gui.awt";
		TreeSet set = getClassesOfPackage(path, packagename);
		Iterator iter = set.iterator();
		while (iter.hasNext()) {
			System.out.println(iter.next());
		}

	}
}
