/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt.tree;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Window;
import java.awt.dnd.Autoscroll;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import xxl.applications.gui.actions.CallableAction;
import xxl.applications.gui.actions.DefaultActionDescriptor;
import xxl.applications.gui.actions.DeleteAction;
import xxl.applications.gui.windows.MainFrame;

/**
 * Some kind of explorer view.... The whole class is one big fat hack for now.
 * @author maxim
 * 
 */
public final class ClassPackageTree extends JTree implements Autoscroll {

	public static final String CREATE_NEW = "New...";

	private DragSource dragSource;
	private TreeDragSupport dragSupport;
	private AutoscrollSupport autoScroll;
	private MouseListener mouseListener = new MouseListener();
	private Map categories2NamesCache;
	private ActionListener listener;

	public ClassPackageTree() {
		super();
		init();
		initDnD();
		listener = new NodeListener(this);
		ToolTipManager.sharedInstance().registerComponent(this);
	}

	public ClassPackageTree(TreeModel model) {
		this();
		setModel(model);
		addMouseListener(mouseListener);
	}

	public void setModel(TreeModel newModel) {
		super.setModel(newModel);
		invalidateCategoriesCache();
		if (newModel != null) {
			newModel.addTreeModelListener(new TreeModelListener() {
				public void treeNodesChanged(TreeModelEvent e) {
				}

				public void treeStructureChanged(TreeModelEvent e) {
					invalidateCategoriesCache();
				}

				public void treeNodesInserted(TreeModelEvent e) {
					if (getModel().getRoot() == e.getTreePath().getPathComponent(0))
						invalidateCategoriesCache();
				}

				public void treeNodesRemoved(TreeModelEvent e) {
					if (getModel().getRoot() == e.getTreePath().getPathComponent(0))
						invalidateCategoriesCache();
				}

			});
		}
	}

	private void init() {
		setSelectionModel(new DefaultTreeSelectionModel());
		getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		setCellRenderer(new ClassTreeCellRenderer());
		putClientProperty("JTree.lineStyle", "Angled");
		setRootVisible(false);
		setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
	}

	private void initDnD() {
		dragSupport = new TreeDragSupport(this);
		dragSource = DragSource.getDefaultDragSource();
		DragGestureRecognizer dgr = dragSupport.getDefaultGestureRecognizer();
		dgr.setSourceActions(dgr.getSourceActions() & ~InputEvent.BUTTON3_MASK);
		DropTarget dropTarget = new TreeDropSupport(this).getDropTarget();
	}

	private AutoscrollSupport getAutoscrollSupport() {
		if (autoScroll == null)
			autoScroll = new AutoscrollSupport(this, new Insets(15, 10, 15, 10));
		return autoScroll;
	}

	public void autoscroll(Point cursorLocn) {
		getAutoscrollSupport().autoscroll(cursorLocn);
	}

	public Insets getAutoscrollInsets() {
		return getAutoscrollSupport().getAutoscrollInsets();
	}

	/**
	 * Creates a popup menu from node's actions (if it has any nonempty action set)
	 * @param object node to create the menu for
	 * @return JPopupMenu created for <code>object</code> or <code>null</code> if node has no actions
	 */
	private JPopupMenu createPopup(TreeObject object) {
		JPopupMenu menu = new JPopupMenu();
		CallableAction[] nodeActions = object.getActions();
		if (nodeActions == null || nodeActions.length < 1)
			menu = null;
		else {
			for (int i = 0; i < nodeActions.length; i++) {
				CallableAction a = nodeActions[i];
				if (a != null) {
					a.setActionListener(listener);
					//TODO HACK!!!!!!
					if (a
						.getValue(Action.ACTION_COMMAND_KEY)
						.equals(ClassTreeObject.AddToAction.ACTION_COMMAND_VALUE)) {
						Set entries = categories2NamesCache.keySet();
						JMenu subMenu = new JMenu(a);
						CallableAction ca = new CategorySelectionAction(CREATE_NEW);
						ca.setActionListener(listener);
						subMenu.add(ca);
						for (Iterator iter = entries.iterator(); iter.hasNext();) {
							String name = (String) iter.next();
							ca = new CategorySelectionAction(name);
							ca.setActionListener(listener);
							subMenu.add(new JMenuItem(ca));
						}
						menu.add(subMenu);
					}
					else {
						JMenuItem item = new JMenuItem(a);
						item.setToolTipText(null);
						menu.add(item);
					}
				}
				else {
					if (i > 0 && i < nodeActions.length - 1)
						menu.addSeparator();
				}
			}
		}
		return menu;
	}

	private void invalidateCategoriesCache() {
		if (categories2NamesCache == null)
			categories2NamesCache = new HashMap();
		else
			categories2NamesCache.clear();
		TreeNode node = (TreeNode) getModel().getRoot();
		for (int i = 0; i < node.getChildCount(); i++) {
			TreeNode child = node.getChildAt(i);
			if (child instanceof CategoryTreeObject)
				categories2NamesCache.put(
					((TreeObject) child).getObjectName(),
					(CategoryTreeObject) child);
		}
	}

	private class MouseListener extends MouseAdapter {
		public MouseListener() {

		}
		private void performPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				TreePath path = getPathForLocation(e.getX(), e.getY());
				if (path != null) {
					setSelectionPath(path);
					TreeObject to = (TreeObject) path.getLastPathComponent();
					JPopupMenu menu = createPopup(to);
					if (menu != null) {
						menu.show(e.getComponent(), e.getX(), e.getY());
					}
				}
			}
		}
		public void mouseClicked(MouseEvent e) {
			performPopup((e));
		}
		public void mousePressed(MouseEvent e) {
			performPopup(e);
		}
		public void mouseReleased(MouseEvent e) {
			performPopup(e);
		}
	}

	private static class NodeListener implements ActionListener {
		private ClassPackageTree tree;
		private SingleInputDialog dlg;
		public NodeListener(ClassPackageTree tree) {
			this.tree = tree;
		}
		public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand();
			if (cmd.equals(DeleteAction.ACTION_COMMAND_VALUE)) {

			}
			else {
				boolean updateModel = false;
				CategoryTreeObject to = null;
				DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
				if (cmd.equals(CREATE_NEW)) {
					if (dlg == null)
						dlg = new SingleInputDialog("New Category", "Name:", "Add", "Cancel");
					dlg.getDialog().setVisible(true);
					Object ctg = dlg.getValue();
					if (ctg != null) {
						to = new CategoryTreeObject(ctg.toString());
						DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
						root.add(to);
						updateModel = true;
					}
				}
				else
					to = (CategoryTreeObject) tree.categories2NamesCache.get(cmd);
				if (to != null) {
					MutableTreeNode node =
						(MutableTreeNode) tree
							.getSelectionModel()
							.getSelectionPath()
							.getLastPathComponent();
					if (node != null) {
						to.add(node);
					}
					if (updateModel)
						model.nodeStructureChanged((TreeNode) model.getRoot());
				}
			}
		}
	}

	static class CategorySelectionAction extends CallableAction {
		public CategorySelectionAction(final String categoryName) {
			super(new DefaultActionDescriptor() {
				public String getActionCommand() {
					return categoryName;
				}
				public String getActionName() {
					return categoryName;
				}
			});
		}
	}

	static class SingleInputDialog {
		private String title;
		private String label;
		private String okLabel;
		private String cancelLabel;
		private Window dialog;
		private Object value;

		public SingleInputDialog(String title, String label, String okLabel, String cancelLabel) {
			this.title = title;
			this.label = label;
			this.okLabel = okLabel;
			this.cancelLabel = cancelLabel;
		}

		private void createDialog() {
			JPanel panel = new JPanel(new GridBagLayout());
			panel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
			final JTextField inputField = new JTextField();
			JLabel label = new JLabel(this.label);
			label.setLabelFor(inputField);
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.insets = new Insets(0, 0, 0, 2);
			panel.add(label, gbc);
			gbc = new GridBagConstraints();
			gbc.gridx = 1;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.weightx = 1.0;
			panel.add(inputField, gbc);

			JButton okButton = new JButton(okLabel);
			Dimension dim = new Dimension(100, 26);
			okButton.setMinimumSize(dim);
			okButton.setPreferredSize(dim);
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String text = inputField.getText();
					if (text.equals(""))
						value = null;
					else
						value = text;
					dialog.dispose();
				}
			});
			JButton cancelButton = new JButton(cancelLabel);
			cancelButton.setMinimumSize(dim);
			cancelButton.setPreferredSize(dim);
			cancelButton.setVerifyInputWhenFocusTarget(false);
			cancelButton.setDefaultCapable(false);
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					dialog.dispose();
				}
			});
			Object[] options = new Object[] { okButton, cancelButton };
			JOptionPane jop =
				new JOptionPane(
					panel,
					JOptionPane.PLAIN_MESSAGE,
					JOptionPane.NO_OPTION,
					null,
					options,
					okButton);
			dialog = jop.createDialog(MainFrame.getInstance(), title);
		}

		public Window getDialog() {
			if (dialog == null)
				createDialog();
			return dialog;
		}

		public Object getValue() {
			return value;
		}
	}
}
