/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt.tree;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.TreeSet;

import xxl.applications.gui.actions.CallableAction;
import xxl.applications.gui.actions.DefaultActionDescriptor;

/**
 * @author maxim
 * 
 */
public class ClassTreeObject extends TreeObject implements Transferable {

	public static DataFlavor CLASS_OBJECT_FLAVOR =
		new DataFlavor(ClassTreeObject.class, "ClassTreeObject");
	static DataFlavor[] flavors = { CLASS_OBJECT_FLAVOR };

	public ClassTreeObject(String classEntry) {
		super();
		setObjectEntry(classEntry.replace('/', '.'));
		setUserObject(getObjectName());
		setType(CLASS_OBJECT);
	}

	public String getIconResource() {
		return "xxl/applications/gui/awt/tree/resources/class.gif";
	}

	public String getObjectName() {
		String str =
			objectEntry.endsWith(".class")
				? objectEntry.substring(0, objectEntry.length() - 6)
				: objectEntry;
		int i = str.lastIndexOf('.');
		return i < 0 ? str : str.substring(i + 1, str.length());
	}

	public String getObjectPath() {
		String str =
			objectEntry.endsWith(".class")
				? objectEntry.substring(0, objectEntry.length() - 6)
				: objectEntry;
		int i = str.lastIndexOf('.');
		return i < 0 ? str : str.substring(0, i);
	}

	public String getFullName() {
		String name = objectEntry;
		int i = -1;
		if (name.endsWith("class"))
			i = name.lastIndexOf('.');
		return i < 0 ? name : name.substring(0, i);
	}

	public void addChildren(TreeSet children) {
	}

	public boolean isLeaf() {
		return true;
	}

	// Transferable
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return flavor.equals(CLASS_OBJECT_FLAVOR);
	}

	public Object getTransferData(DataFlavor flavor)
		throws UnsupportedFlavorException, IOException {
		if (isDataFlavorSupported(flavor))
			return this;
		throw new UnsupportedFlavorException(flavor);
	}

	public DataFlavor[] getTransferDataFlavors() {
		return flavors;
	}

	public CallableAction[] getActions() {
		return new CallableAction[] { new AddToAction()};
	}

	public static class AddToAction extends CallableAction {
		public static final String ACTION_COMMAND_VALUE = "addto";

		public AddToAction() {
			super(new DefaultActionDescriptor() {
				public String getActionCommand() {
					return ACTION_COMMAND_VALUE;
				}

				public String getActionName() {
					return "Add to ...";
				}
			});
		}
	}
}
