/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt.tree;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.Icon;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

/**
 * Should be the next version of TreeObject
 * @author maxim
 */
public abstract class Node extends DefaultMutableTreeNode {

	public static final String NAME_PROPERTY = "name";
	public static final String ICON_PROPERTY = "icon";
	public static final String LEAF_PROPERTY = "leaf";
	public static final String DISPLAY_NAME_PROPERTY = "diplayName";
	public static final String CHILDREN_PROPERTY = "children";

	private String name;
	private String displayName;
	private PropertyChangeSupport changes = new PropertyChangeSupport(this);
	protected DefaultTreeModel model;

	public Node() {
		super();
	}

	public void setDisplayName(String name) {
		String old = this.displayName;
		this.displayName = name;
		firePropertyChange(DISPLAY_NAME_PROPERTY, old, name);
	}

	public String getDisplayName() {
		return displayName;
	}

	public void setName(String name) {
		String old = this.name;
		this.name = name;
		firePropertyChange(NAME_PROPERTY, old, name);

	}

	public String getName() {
		return name;
	}

	/**
	 * @param listener
	 */
	public final synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
		changes.addPropertyChangeListener(listener);
	}

	/**
	 * @param propertyName
	 * @param listener
	 */
	public final synchronized void addPropertyChangeListener(
		String propertyName,
		PropertyChangeListener listener) {
		changes.addPropertyChangeListener(propertyName, listener);
	}

	/**
	 * @param propertyName
	 * @param oldValue
	 * @param newValue
	 */
	protected final void firePropertyChange(
		String propertyName,
		boolean oldValue,
		boolean newValue) {
		changes.firePropertyChange(propertyName, oldValue, newValue);
	}

	/**
	 * @param propertyName
	 * @param oldValue
	 * @param newValue
	 */
	protected final void firePropertyChange(
		String propertyName,
		Object oldValue,
		Object newValue) {
		changes.firePropertyChange(propertyName, oldValue, newValue);
	}

	/**
	 * @param listener
	 */
	public final synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
		changes.removePropertyChangeListener(listener);
	}

	/**
	 * @param propertyName
	 * @param listener
	 */
	public final synchronized void removePropertyChangeListener(
		String propertyName,
		PropertyChangeListener listener) {
		changes.removePropertyChangeListener(propertyName, listener);
	}

	public abstract Icon getIcon();

	public abstract boolean canDelete();

	public void performExpand() {
	}

	public void performClose() {
	}

}
