/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt.tree;

import java.awt.Image;
import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.util.ArrayList;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

import xxl.applications.gui.util.Utilities;

/**
 * @author maxim
 */
final class TreeDragSupport implements DragSourceListener, DragGestureListener {

	private DragGestureRecognizer dragGesture;
	private JTree tree;

	public TreeDragSupport(JTree tree) {
		this.tree = tree;
	}

	public void dragGestureRecognized(DragGestureEvent dge) {
		TreeObject[] nodes = obtainNodes(dge);
		if (nodes == null || nodes.length == 0) {
			return;
		}
		//TODO need transferable to handle multiple selections
		Image img = Utilities.loadImage("xxl/applications/gui/awt/tree/resources/cursorscopysingle.gif");
		dge.startDrag(
			Utilities.createCustomCursor(img, "ACTION_COPY"),
			img,
			new Point(16, 16),
			(Transferable) nodes[0],
			this);
	}

	public void dragEnter(DragSourceDragEvent dsde) {
	}

	public void dragOver(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub
	}

	public void dropActionChanged(DragSourceDragEvent dsde) {
		// TODO Auto-generated method stub
	}

	public void dragExit(DragSourceEvent dse) {
		// TODO Auto-generated method stub
	}

	public void dragDropEnd(DragSourceDropEvent dsde) {
		// TODO Auto-generated method stub
	}

	TreeObject[] obtainNodes(DragGestureEvent dge) {
		Point point = dge.getDragOrigin();

		TreePath tp = tree.getPathForLocation(point.x, point.y);
		if (tp == null)
			return null;

		TreeObject node = (TreeObject) tp.getLastPathComponent();
		if (node == null)
			return null;

		ArrayList list = new ArrayList();
		if (tree.isPathSelected(tp)) {
			TreePath[] paths = tree.getSelectionPaths();
			for (int i = 0; i < paths.length; i++) {
				TreeObject object = (TreeObject) paths[i].getLastPathComponent();
				if (object == null)
					return null;
				if (isDragSupported(object))
					list.add(object);
			}
		}
		else {
			if (isDragSupported(node))
				list.add(node);
		}
		return (TreeObject[]) list.toArray(new TreeObject[0]);
	}

	boolean isDragSupported(TreeObject t) {
		switch (t.getType()) {
			case TreeObject.CLASS_OBJECT :
				return true;
			default :
				return false;
		}
	}

	/** Safe getter for default gesture<br>
	   * (creates the gesture when called for the first time)
	   */
	DragGestureRecognizer getDefaultGestureRecognizer() {
		if (dragGesture == null) {
			DragSource ds = DragSource.getDefaultDragSource();
			dragGesture = ds.createDefaultDragGestureRecognizer(tree, DnDConstants.ACTION_COPY, this);
		}
		return dragGesture;
	}
}
