/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.awt.tree;

import java.awt.Point;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;

import javax.swing.JTree;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
 * @author maxim
 */
final class TreeDropSupport implements DropTargetListener {

	private JTree tree;
	private int pointAt = 0;
	private DropTarget dropTarget;

	public TreeDropSupport(JTree tree) {
		this.tree = tree;
	}

	public void dragEnter(DropTargetDragEvent dtde) {
		doDragOver(dtde);
	}

	public void dragOver(DropTargetDragEvent dtde) {
		doDragOver(dtde);
	}

	private void doDragOver(DropTargetDragEvent dtde) {
		TreePath tp = getTreePath(dtde);

		if (tp == null) {
			dtde.rejectDrag();

			return;
		}
		Point p = dtde.getLocation();
		TreeObject dropNode = getObjectForDrop(p);
		if (dropNode == null) {
			dtde.rejectDrag();
			return;
		}

		if (dropNode.getType() == TreeObject.CLASS_OBJECT) {
			dropNode = (TreeObject) dropNode.getParent();
		}

		if (canDrop(dropNode, dtde.getDropAction())) {			
			dtde.acceptDrag(dtde.getDropAction());
		}
		
		else {			
			dtde.rejectDrag();
		}
	}

	public void drop(DropTargetDropEvent dtde) {
		System.out.println("TreeDropSupport.drop()");
		Point p = dtde.getLocation();
		TreeObject dropNode = getObjectForDrop(p);
		TreePath tp = tree.getPathForLocation(p.x, p.y);
		if (dropNode.getType() == TreeObject.CLASS_OBJECT) {
			dropNode = (TreeObject) dropNode.getParent();
		}

		if (!canDrop(dropNode, dtde.getDropAction())) {
			dtde.rejectDrop();
			System.out.println("TreeDropSupport.drop() REJECT " + dropNode);
			return;
		}
		dtde.acceptDrop(dtde.getDropAction());
		try {

			TreeObject dragNode =
				(TreeObject) dtde.getTransferable().getTransferData(ClassTreeObject.CLASS_OBJECT_FLAVOR);
			dropNode.add(dragNode);
			dropNode.fireModelChange();
			dtde.dropComplete(true);
		}
		catch (Exception e) {
			System.out.println("TreeDropSupport.drop()" + e);
			dtde.rejectDrop();
		}
	}

	public void dropActionChanged(DropTargetDragEvent dtde) {
		// TODO Auto-generated method stub
	}

	public void dragExit(DropTargetEvent dte) {
		// TODO Auto-generated method stub

	}

	//TODO move actions?
	private boolean canDrop(TreeObject node, int dropAction) {
		if (node == null) {
			return false;
		}
		if (node.getType() == TreeObject.CATEGORY_OBJECT) {
			return true;
		}
		return false;
	}

	private int rowForNode(final TreeNode object) {
		if (object.getParent() == null)
			return 0;
		return rowForNode(object.getParent()) + 1;

	}

	private TreeObject getObjectForDrop(Point p) {
		if (p != null) {
			TreePath path = tree.getPathForLocation(p.x, p.y);
			if (path != null)
				return (TreeObject) path.getLastPathComponent();
		}
		return null;
	}

	private TreePath getTreePath(DropTargetDragEvent dtde) {
		Point point = dtde.getLocation();
		TreePath path = tree.getPathForLocation(point.x, point.y);
		if (path == null)
			return null;
		TreeObject to = (TreeObject) path.getLastPathComponent();
		return to == null ? null : path;

	}

	DropTarget getDropTarget() {
		if (dropTarget == null) {
			dropTarget = new DropTarget(tree, DnDConstants.ACTION_COPY, this);
		}
		return dropTarget;
	}

}
