/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.code;

import java.awt.Point;

import xxl.applications.gui.graph.Vertex;

/**
 * This Vertex represents an uninstantiated Class, so only static members can be accessed.
 * This item type should be used for untility classes which usually can not be intantiated and 
 * provide only static methods 
 * 
 * @author maxim
 */
public final class ClassItem extends AbstractItem {
	static final long serialVersionUID = 4657688330992096241L;
	
	public static final String DEFAULT = "java.lang.System";

	public ClassItem() {
		this(DEFAULT);
	}

	public ClassItem(String className) {
		this(new Point(), className);
	}

	public ClassItem(Point location, String className) {
		super(location, className);
		type = CLASS_ITEM;
	}

	/**
	 * Adds a new child vertex to the item. Parameter <code>child</code> will be added iff 
	 * it is of type <code>CallVertex</code> and its call element is not of type 
	 * <code>ConstructorCall</code>. The added vertex will be also inserted into the graph. 
	 */
	public void addChildVertex(Vertex child) {
		ElementCall call = ((CallVertex) child).getCall();
		if (call.getType() == ElementCall.CONSTRUCTOR_CALL) {
			return;
		}
		addChildVertex(-1, child);
	}

	/**
	 * Test whether the item is valid. In the case of ClassItem it returns <code>true</code>, if
	 * all item calls (zero or more) are valid. <code>false</code> is returned otherwise.
	 */
	public boolean isValid() {
		CallVertex[] calls = (CallVertex[]) children.toArray(new CallVertex[0]);
		for (int i = 0; i < calls.length; i++) {
			ElementCall vertexCall = calls[i].getCall();
			if (!vertexCall.isValid())
				return false;
		}
		return true;
	}

}
