/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.code;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.PrintWriter;
import java.io.Serializable;

/**
 * @author maxim
 * Basisklasse von Codeelementen
 * 
 */
public abstract class Element implements ElementProperties, Serializable {
	static final long serialVersionUID = -2119258080003902422L;

	public void generateCode(PrintWriter writer) {
		generateCode(writer, 0);
	}

	public abstract void generateCode(PrintWriter writer, int indentation);

	protected static void indent(PrintWriter writer, int indentation) {
		for (int i = 0; i < indentation; i++)
			writer.print('\t');
	}

	public static boolean generateCode(Element[] elems, PrintWriter writer) {
		for (int i = 0; i < elems.length; i++) {
			if (i > 0) {
				writer.println();
			}
			elems[i].generateCode(writer);
		}
		return elems.length > 0;
	}

	private PropertyChangeSupport changes = new PropertyChangeSupport(this);

	public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
		changes.addPropertyChangeListener(listener);
	}

	protected void firePropertyChange(PropertyChangeEvent evt) {
		changes.firePropertyChange(evt);
	}

	protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		changes.firePropertyChange(propertyName, oldValue, newValue);
	}

	public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
		changes.removePropertyChangeListener(listener);
	}

}
