/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package xxl.applications.gui.code;

import java.io.Serializable;

/** Represents one class or package import.
*
* @author Petr Hamernik
*/
public class Import extends Object implements Serializable {
    /** A package import. */
    public static final boolean PACKAGE = true;
    /** A class import. */
    public static final boolean CLASS = false;

    /** Kind of this Import element. It is true if the import means "whole package"
    * otherwise (if it is import just one class) false.
    */
    private boolean wholePackage;

    /** Identifier which is imported */
    private Identifier id;

    static final long serialVersionUID =-4111760314345461897L;

    /** Create an import.
    * @param id the name of the class or package imported
    * @param wholePackage one of {@link #PACKAGE} or {@link #CLASS}
    */
    public Import(Identifier id, boolean wholePackage) {
        this.wholePackage = wholePackage;
        this.id = id;
    }

    /** Is this a package import?
    * @return <code>true</code> if so
    */
    public boolean isPackage() {
        return wholePackage;
    }

    /** Is this a class import?
    * @return <code>true</code> if so
    */
    public boolean isClass() {
        return (!wholePackage);
    }

    /** Get the name of the import.
    * @return the identifier which is imported
    */
    public Identifier getIdentifier() {
        return id;
    }

    /** Get this import as a string.
    * @return e.g. <code>import com.mycom.Class</code> or <code>import com.mycom.*</code>
    */
    public String toString() {
        StringBuffer buf = new StringBuffer("import "); // NOI18N
        buf.append(id.getFullName());
        if (wholePackage)
            buf.append(".*"); // NOI18N
        buf.append(';');
        return buf.toString();
    }

    /** @return the hash code for this import */
    public int hashCode() {
        return id.getFullName().hashCode();
    }

    /** @return true if the specified object is also Import of the same class or package.
    */
    public boolean equals(Object o) {
        if (o instanceof Import) {
            Import imp = (Import) o;
            return (wholePackage == imp.wholePackage) && (id.equals(imp.id));
        }
        return false;
    }
}
