/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.code;

import java.io.PrintWriter;
import java.lang.reflect.Method;

/**
 * @author maxim
 *
 */
public final class MethodCall extends ConstructorCall {	
	static final long serialVersionUID = 7614829496978414263L;

	/** name of the called method */
	private String methodName;
	/** name of the object this method is called on */
	private AbstractItem callee;
	/** is the return value of the call used by another calls? */
	private int useReturn;

	public MethodCall(CallVertex caller) {
		super(caller);
		setType(METHOD_CALL);
	}

	/** creates new method call from method and name of the object the method is called on */
	public static MethodCall create(CallVertex caller, Method method, AbstractItem callee) {
		MethodCall call = new MethodCall(caller);
		//type of returned object
		Class[] params = method.getParameterTypes();
		Type retType = Type.createFromClass(method.getReturnType());		
		call.setCallType(retType);
		Type[] types = new Type[params.length];
		for (int i = 0; i < types.length; i++)
			types[i] = Type.createFromClass(params[i]);
		call.setParamTypes(types);
		call.setMethodName(method.getName());
		call.callee = callee;
		return call;
	}

	/** set the usage of the return value of the call */
	void setUseReturn(boolean use) {
		if (use)
			useReturn++;
		else
			useReturn -= useReturn > 0 ? 1 : 0;
		System.out.println("MethodCall.setUseReturn()" + useReturn);
	}

	/** is the return value of the call used by another calls? */
	boolean isReturnUsed() {
		return useReturn != 0;
	}

	/** observer for the method name */
	public String getMethodName() {
		return methodName;
	}

	public void setMethodName(String methodName) {
		String old = this.methodName;
		this.methodName = methodName;
		//firePropertyChange(METHOD_NAME_PROPERTY, old, methodName);
	}

	public void setCallee(AbstractItem callee) {
		this.callee = callee;
	}

	public AbstractItem getCalle() {
		return callee;
	}

	/** generates java code representing this call */
	public void generateCode(PrintWriter writer, int indentation) {
		//		if (!isValid())
		//			throw new IllegalStateException("MethodCall is not valid");
		indent(writer, indentation);
		//is the return value in use?
		if (isReturnUsed() && !callType.compareTo(Type.VOID, false)) {
			writer.print(getTypeIdentifier(callType));
			writer.print(' ');
			writer.print(getIdentifier().getName());
			writer.print(" = ");
		}
		writer.print(callee.getName());
		writer.print('.');
		writer.print(getMethodName());
		writer.print('(');
		for (int i = 0; i < getParamValues().length; i++) {
			writer.print(i > 0 ? ", " : "");
			writer.print(getParamValues()[i] == null ? "?" : getParamValues()[i].getIdentifier().getName());
		}
		writer.print(");");
	}
}
