/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.code;

import java.io.PrintWriter;

/**
 * @author maxim
 */
public final class PrimitiveCall extends ElementCall {
	
	static final long serialVersionUID = 2746444308145712770L;

	private String value;
	public static Type STRING_TYPE = Type.createFromClass(String.class);

	public PrimitiveCall(CallVertex caller) {
		super(caller);
		setType(PRIMITIVE_CALL);
	}

	/** Creates a new instance of PrimitiveCall */
	public static PrimitiveCall create(CallVertex caller, Type type, String value, String name) {
		PrimitiveCall call = new PrimitiveCall(caller);
		call.setCallType(type);
		call.setValue(value);
		call.setIdentifier(name);
		return call;
	}

	public static boolean isPrimitive(Type type) {
		if (type.compareTo(STRING_TYPE, false))
			return true;
		if (type.isPrimitive() && !type.compareTo(Type.VOID, false))
			return true;
		System.out.println("PrimitiveCall.is(not)Primitive() " + type.getFullString());
		return false;

	}

	public void setCallType(Type type) {
		//boolean n = type.compareTo(STRING_TYPE, false);
		//System.out.println("PrimitiveCall.setCallType() " + type + " is " +
		// (n ? " " : "not ") + STRING_TYPE);
		if (!isPrimitive(type))
			throw new IllegalArgumentException("not a valid callable primitive type: " + type);
		super.setCallType(type);
		if (value != null)
			setValue(value);
	}

	public Type getCallType() {
		return callType == null ? STRING_TYPE : callType;
	}

	public void setValue(Object o) {
		String value = o == null ? "" : o.toString();
		if (!isValueOfType(getCallType(), value)) {
			throw new IllegalArgumentException(
				"value " + value + " is not assignable to type " + callType);
		}
		this.value = value;
	}

	public Object getValue() {
		return value;
	}

	//TODO!
	public boolean isValid() {
		return true;
	}

	public static boolean isValueOfType(Type type, String value) {
		try {
			if (type.compareTo(Type.DOUBLE, false))
				Double.parseDouble(value);
			else if (type.compareTo(Type.FLOAT, false))
				Float.parseFloat(value);
			else if (type.compareTo(Type.LONG, false))
				Long.parseLong(value);
			else if (type.compareTo(Type.INT, false))
				Integer.parseInt(value);
			else if (type.compareTo(Type.SHORT, false))
				Short.parseShort(value);
			else if (type.compareTo(Type.CHAR, false))
				return value.length() == 1;
			else if (type.compareTo(Type.BYTE, false))
				Byte.parseByte(value);
			else if (type.compareTo(Type.BOOLEAN, false))
				return value.equals("true") || value.equals("false");
		}
		catch (NumberFormatException nfe) {
			return false;
		}
		//String Type
		return true;
	}

	public void generateCode(PrintWriter writer, int indentation) {
		indent(writer, indentation);
		writer.print(getTypeIdentifier(callType));
		writer.print(' ');
		writer.print(getIdentifier().getName());
		writer.print(" = ");
		if (callType.compareTo(STRING_TYPE, false)) {
			writer.print('\"');
			writer.print(value);
			writer.print('\"');
		}
		else if (callType.compareTo(Type.CHAR, false)) {
			writer.print('\'');
			writer.print(value);
			writer.print('\'');
		}
		else
			writer.print(value);
		writer.print(';');
	}
}
