/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.code;

import java.awt.Point;

/**
 * @author maxim
 */
public class PrimitiveItem extends AbstractItem {
	static final long serialVersionUID = 935599482519819557L;
	
	public PrimitiveItem() {
		this("java.lang.String");
	}

	public PrimitiveItem(String className) {
		this(new Point(), className);
	}

	public PrimitiveItem(Point location, String className) {
		super(location, className);
		type = PRIMITIVE_ITEM;
	}

	public void setInitCall(PrimitiveCall call) {
		CallVertex vertex = call.getCallVertex();
		PrimitiveCall old = getInitCall();
		if (old == call)
			return;
		if (old != null) {
			old.setCallType(call.getCallType());
			old.setValue(call.getValue());
		}
		else
			addChildVertex(0, vertex);
	}

	public PrimitiveCall getInitCall() {
		if (children.size() < 1)
			return null;
		CallVertex callVertex = (CallVertex) children.get(0);
		ElementCall call = callVertex.getCall();
		return (PrimitiveCall) (call instanceof PrimitiveCall ? call : null);
	}

	public boolean isValid() {
		CallVertex[] calls = (CallVertex[]) children.toArray(new CallVertex[children.size()]);
		if (calls.length < 1) {
			return false;
		}
		else if (calls.length == 1) {
			ElementCall call = calls[0].getCall();
			return (call.getType() == ElementCall.PRIMITIVE_CALL);
		}
		else {
			for (int i = 1; i < calls.length; i++) {
				ElementCall call = calls[i].getCall();
				if (!call.isValid()) {
					return false;
				}
			}
		}
		return (getInitCall() != null && getInitCall().isValid());
	}
}
