/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph;

import java.io.PrintWriter;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.code.CallVertex;
import xxl.applications.gui.code.ClassItem;
import xxl.applications.gui.code.Identifier;
import xxl.applications.gui.code.Import;
import xxl.applications.gui.code.Type;

/**
 * 
 * @author maxim
 */
public final class ClassGraph extends Graph {

	public static final char DELIMERER = '_';

	static final long serialVersionUID = -3881986563550558918L;

	protected NameManager namemanager;

	public ClassGraph() {
		super();	
	}

	//TODO entkomplizieren...
	public void addVertex(Vertex vertex) {
		Type itemType = null;
		if (vertex instanceof AbstractItem) {
			itemType = ((AbstractItem) vertex).getEnclosedType();
		}
		else if (vertex instanceof CallVertex) {
			itemType = ((CallVertex) vertex).getCall().getCallType();
		}
		if (itemType != null) {
			if (vertex instanceof ClassItem) {
				vertex.setName(itemType.getClassName().getName());
			}
			else {
				String name = vertex.getName();
				if (name == null) { // || getManager().isManagerId(name,
					// itemType)) {
					vertex.setName(getManager().getNameFor(itemType));
				}
			}
		}
		super.addVertex(vertex);
	}

	public void generateCode(String className, PrintWriter writer) {
		StringWriter swr = new StringWriter();
		PrintWriter pwr = new PrintWriter(swr);
		pwr.print("public class " + className + " {\n\tpublic static void main(String[] args){\n");
		List list = new ArrayList(dfs().entrySet());
		Collections.sort(list, VertexComparator.getInstance());
		Set imports = new HashSet();
		for (int i = 0; i < list.size(); i++) {
			Map.Entry entry = (Map.Entry) list.get(i);
			Vertex v = (Vertex) entry.getKey();
			if (v instanceof CallVertex) {
				CallVertex cv = (CallVertex) v;
				cv.getImport(imports);
				cv.generateCode(pwr, 2);
				pwr.println();
			}
			//TODO imports ueber AbstractItem abfragen
			if (v instanceof ClassItem) {
				Type t = ((ClassItem) v).getEnclosedType();
				Identifier id = t.getTypeIdentifier();
				if (id == null || id.getQualifier().equals("java.lang"))
					continue;
				else
					imports.add(new Import(id, false));
			}
		}
		pwr.print("\t}\n}");
		Iterator iter = imports.iterator();
		while (iter.hasNext())
			writer.println(iter.next());
		writer.println();
		writer.print(swr.getBuffer());
	}

	protected NameManager getManager() {
		if (namemanager == null)
			namemanager = new NameManager();
		return namemanager;
	}

	public String getNameFor(Type clazz) {
		return getManager().getNameFor(clazz);
	}

	public void printGraphCode() {
		StringWriter stw = new StringWriter();
		PrintWriter pwr = new PrintWriter(stw);
		generateCode("NameClass", pwr);
		System.out.println(stw.getBuffer());
	}
	

	static class NameManager implements Serializable {

		//TODO this method also residents in TypeTableCellRenderer, make it to
		// utility method?
		static String getTypeIdentifier(Type type) {
			if (type.isArray())
				return getTypeIdentifier(type.getElementType());
			else if (type.isPrimitive())
				return type.toString();
			return type.getTypeIdentifier().getName();
		}
		private Map class2CountMap;
		//TODO set the default length as system property?
		private int idLength = 14;

		public NameManager() {
			class2CountMap = new HashMap();
		}

		public int getIdLength() {
			return idLength;
		}

		/**
		 * Created an unique identifier for the given type.
		 * 
		 * @param clazz
		 * @return
		 */
		public String getNameFor(Type clazz) {
			Integer counter = (Integer) class2CountMap.get(clazz);
			if (counter == null) {
				counter = new Integer(0);
				class2CountMap.put(clazz, counter);
			}
			StringBuffer str_buf = new StringBuffer(getTypeIdentifier(clazz));
			str_buf.setCharAt(0, Character.toLowerCase(str_buf.charAt(0)));
			int num_length = getIdLength() - counter.toString().length();
			//what should i do when even the number part of id is longer?
			if (num_length < 0)
				num_length = 1;
			class2CountMap.put(clazz, new Integer(counter.intValue() + 1));
			if (num_length < str_buf.length()) {
				return new StringBuffer(str_buf.substring(0, num_length - 1)).append(DELIMERER)
						.append(counter).toString();
			}
			return str_buf.append(counter).toString();
		}

		public boolean isManagerId(String toCheck, Type clazz) {
			String identifier = getNameFor(clazz);
			int index = identifier.indexOf(DELIMERER);
			if (index > 0)
				identifier = identifier.substring(0, index);
			else {
				Integer i = (Integer) class2CountMap.get(clazz);
				int diff = idLength - i.toString().length();
				if (diff < identifier.length())
					identifier = identifier.substring(0, diff);
			}
			if (toCheck.startsWith(identifier))
				return true;
			return false;
		}

		public void setIdLength(int newLength) {
			if (newLength > 0)
				this.idLength = newLength;
		}
	}

	private static class VertexComparator implements Comparator {
		private static Comparator instance;
		public static Comparator getInstance() {
			if (instance == null)
				instance = new VertexComparator();
			return instance;
		}

		private VertexComparator() {

		}

		public int compare(Object entry1, Object entry2) {
			if (entry1 instanceof Map.Entry && entry2 instanceof Map.Entry) {
				Map.Entry o1 = (Map.Entry) entry1;
				Map.Entry o2 = (Map.Entry) entry2;
				int i1 = ((Integer) o1.getValue()).intValue();
				int i2 = ((Integer) o2.getValue()).intValue();
				return i2 - i1;
			}
			else
				throw new IllegalArgumentException("Items are not comparable");
		}
	}
}
