/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph;

import java.awt.Point;
import java.util.List;
import java.util.Vector;

/**
 * Vertex which can hold another ones
 * @author maxim
 */
public class ComplexVertex extends Vertex {

	protected List children;

	public ComplexVertex(ComplexVertex vertex) {
		throw new UnsupportedOperationException("Copy constructor not suppoted yet");
	}

	public ComplexVertex() {
		this((String) null);
	}

	public ComplexVertex(String name) {
		this(name, new Point());
	}

	public ComplexVertex(String name, Point location) {
		super(name, location);
		init();
	}

	private void init() {
		children = new Vector();
	}

	//TODO shouldn't we add the child to the graph?
	public void addChildVertex(Vertex child) {
		addChildVertex(-1, child);
	}

	public Vertex[] getChildren() {
		return (Vertex[]) children.toArray(new Vertex[0]);
	}

	//TODO was tut isValid hier?
	public boolean isValid() {
		return true;
	}

	protected Vertex addChildVertex(int position, Vertex child) {
		if (child == null)
			throw new IllegalArgumentException("Child is null");
		if(child == this) throw new IllegalArgumentException("Adding vertex to itself");
		child.setParent(this);
		Vertex ret = null;
		//add vertex to list of position is not valid
		if (position < 0 || position >= children.size()) {
			children.add(child);
		}
		else {
			ret = (Vertex) children.set(position, child);
		}
		return ret;
	}

	public void removeAllChildren() {
		Vertex[] children = getChildren();
		for (int i = 0; i < children.length; i++)
			removeChildVertex(children[i]);
	}

	public void removeChildVertex(Vertex child) {
		if (child == null)
			throw new IllegalArgumentException("Child is null");
		children.remove(child);
		child.setParent(null);
	}

}
