/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;

/**
 * Basic class of all graph elements, providing support for property changes and
 * defining properties of elements. All Graph, Vertex and Edge Classes should
 * extend this class.
 * 
 * @author maxim
 */
public abstract class GraphElement extends Object implements Serializable {
	static final long serialVersionUID = 3736405417217662450L;
	
	public static final String GRAPH_PROPERTY = "graph".intern();
	public static final String NAME_PROPERTY = "name".intern();
	public static final String PARENT_PROPERTY = "parent".intern();
	public static final String LOCATION_PROPERTY = "location".intern();
	public static final String SELECTED_PROPERTY = "selected".intern();
	public static final String VISIBLE_PROPERTY = "visible".intern();
	public static final String VALIDITY_PROPERTY = "valid".intern();

	private transient PropertyChangeSupport propertySupport;

	protected PropertyChangeSupport getPropertySupport() {
		if (propertySupport == null)
			propertySupport = new PropertyChangeSupport(this);
		return propertySupport;
	}

	protected void firePropertyChange(String property, Object oldValue, Object newValue) {
		getPropertySupport().firePropertyChange(property, oldValue, newValue);
	}

	protected void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
		getPropertySupport().firePropertyChange(propertyName, oldValue, newValue);
	}

	public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
		getPropertySupport().addPropertyChangeListener(listener);
	}

	public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
		getPropertySupport().removePropertyChangeListener(listener);
	}

}
