/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph.event;

import java.awt.Rectangle;
import java.util.EventObject;

import xxl.applications.gui.graph.Vertex;
import xxl.applications.gui.graph.viewer2d.GraphPane;

/**
 * @author maxim
 */
public final class SelectionEvent extends EventObject {
	private static Rectangle EMPTY_SELECTION = new Rectangle();

	public static final int MULTIPLY_SELECTION = 0;
	public static final int SINGLE_SELECTION = 1;
	private Rectangle selectionBounds;
	private Vertex[] selection;

	private int type;

	public SelectionEvent(GraphPane source, Vertex[] selection) {
		super(source);
		this.selection = selection;
	}

	public SelectionEvent(GraphPane source, Rectangle bounds) {
		super(source);
		setSelectionBounds(bounds);
	}

	public GraphPane getPane() {
		return (GraphPane) source;
	}

	public Rectangle getSelectionBounds() {
		return selectionBounds;
	}

	public int getType() {
		return type;
	}

	public Vertex[] getSelection() {
		if (selection == null)
			return new Vertex[0];
		return selection;
	}

	void setPane(GraphPane pane) {
		if (pane == null)
			throw new IllegalArgumentException("Selection Source is null");
		source = pane;
	}

	void setSelectionBounds(Rectangle selection) {
		if (selection == null)
			selectionBounds = EMPTY_SELECTION;
		else
			selectionBounds = selection;
	}

	void setType(int type) {
		if (type != SINGLE_SELECTION && type != MULTIPLY_SELECTION)
			this.type = SINGLE_SELECTION;
		else
			this.type = type;

	}

	public String toString() {
		StringBuffer str_buf = new StringBuffer();
		str_buf.append(getClass().getName());
		return str_buf.toString();
	}
}
