/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph.viewer2d;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Rectangle2D;

import xxl.applications.gui.graph.ComplexVertex;
import xxl.applications.gui.graph.Vertex;

/**
 * @author maxim
 */
public class ComplexVertexDrawer extends VertexDrawer {

	private static final String EXPAND = "+";
	private static final String VALID = "v";
	private static final String NOTVALID = "x";

	public ComplexVertexDrawer() {
		super();
	}

	protected void drawBackground(Graphics g, Vertex state, Point point, Color color) {
		g.setColor(color);
		g.fillRect(point.x - getWidth(), point.y - getHeight(), 2 * getWidth(), 2 * getHeight());

	}

	public void drawVertex(Graphics g, Vertex vertex1, Point point, Color color) {
		if (!vertex1.isVisible())
			return;
		ComplexVertex vertex = (ComplexVertex) vertex1;
		Color old = g.getColor();
		drawBackground(g, vertex, point, color);
		g.setColor(vertex.isSelected() ? VertexDrawer.VERTEX_SELECTED_COLOR : Color.BLACK);
		FontMetrics fm = g.getFontMetrics();
		Rectangle2D bounds = getBoundsAtPoint(point);
		int dx = (int) fm.getStringBounds(EXPAND, g).getWidth();
		int dy = (int) fm.getAscent();
		g.drawString(EXPAND, (int) (bounds.getX() + bounds.getWidth() - 2 * dx), (int) bounds.getY() + dy);
		Color oldCol = g.getColor();
		//TODO FARBE und in die oberklasse packen ?
		//g.setColor(vertex.isValid()? Color.GREEN : Color.RED);
		//g.drawString(vertex.isValid()? VALID : NOTVALID, (int) bounds.getX() + dx, (int) bounds.getY() + dy);
		//g.setColor(oldCol);
		String name = vertex.getName();
		dx = ((int) fm.getStringBounds(name, g).getWidth()) >> 1;
		dy = ((int) fm.getAscent()) >> 1;
		g.drawString(name, point.x - dx, point.y + dy);
		g.drawRect(point.x - getWidth(), point.y - getHeight(), 2 * getWidth(), 2 * getHeight());
		g.setColor(old);
	}

}
