/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph.viewer2d;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;

/** Grid is for the main panel and provides functionality like snap.
 * @author K. Buerger
 * @version 1.0
 */
public final class Grid {

	public static final int DEFAULT_GRID_SIZE = 10;	
	/** The distance in x-direction. */
	private int gridX;
	/** The distance in y-direction. */
	private int gridY;
	/** The color of the grid. */
	private Color color;

	/** Creates a new instance of Grid */
	public Grid() {
		this(DEFAULT_GRID_SIZE);
	}

	public Grid(int cell) {
		color = Color.LIGHT_GRAY;
		setGrid(Math.abs(cell), Math.abs(cell));
	}

	/**
	 * Sets the grid distances
	 * @param x the distance in x-direction
	 * @param y the distance in y-direction
	 */
	public void setGrid(int x, int y) {
		gridX = x;
		gridY = y;
	}

	/**
	 * Draws this grid inside a rectangle.
	 * @param gphIn the graphics context
	 * @param recIn the bounding rectangle
	 */
	public void draw(Graphics g, Rectangle rect) {
		Color colSave;
		colSave = g.getColor();
		g.setColor(color);
		int maxX = rect.x + rect.width;
		int maxY = rect.y + rect.height;

		for (int x = rect.x; x < maxX; x += gridX)
			g.drawLine(x, rect.y, x, maxY);
		for (int y = rect.y; y < maxY; y += gridY)
			g.drawLine(rect.x, y, maxX, y);

		g.setColor(colSave);
	}

	/**
	 * Places a point on a grid point.
	 * @param potIn the point to place
	 */
	public void snap(Point potIn) {

		int x, y = 0;

		if (gridX == 0)
			x = potIn.x;
		else
			x = (potIn.x / gridX) * gridX;
		if (gridY == 0)
			y = potIn.y;
		else
			y = (potIn.y / gridY) * gridY;

		potIn.setLocation(x, y);
	}

	/**
	 * Places a point on a grid point.
	 * @param recIn the rectangle to place
	 */
	public void snap(Rectangle2D recIn) {
		double x;
		double w;
		double y;
		double h;

		if (gridX == 0) {
			x = recIn.getX();
			w = recIn.getWidth();
		}
		else {
			x = Math.round(recIn.getX() / gridX) * gridX;
			w = Math.ceil(recIn.getWidth() / (2 * gridX)) * (2 * gridX);
		}

		if (gridY == 0) {
			y = recIn.getY();
			h = recIn.getHeight();
		}
		else {
			y = Math.round(recIn.getY() / gridY) * gridY;
			h = Math.ceil(recIn.getHeight() / (2 * gridY)) * (2 * gridY);
		}

		recIn.setFrame(x, y, w, h);
	}	

}
