/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph.viewer2d;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

/**
 * @author maxim
 * 
 */
public class SimpleArrow {

	private static final int ARROW_LENGTH = 15;
	private static final double ARROW_ANGLE = Math.PI / 10;

	protected Point start;
	protected Point end;
	protected Line2D line;
	protected String label;
	private AffineTransform textTransform;
	private Rectangle2D bounds;
	private boolean isValid;

	public SimpleArrow(Point start, Point end) {
		line = new Line2D.Float();
		bounds = new Rectangle();
		setStart(start);
		setEnd(end);
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getLabel() {
		return label;
	}

	public void setStart(Point start) {
		this.start = start;
		isValid = false;
	}

	public Point getStart() {
		return start;
	}

	public void setEnd(Point end) {
		this.end = end;
		isValid = false;
	}

	public Point getEnd() {
		return end;
	}

	public boolean isOnArrow(Point point, int distance) {
		if (point == null)
			return false;
		if (!isValid)
			refresh();
		return line.intersects(point.x - distance, point.y - distance, distance << 1, distance << 1);
	}

	public void draw(Graphics2D g) {
		if (!isValid)
			refresh();
		g.draw(line);
		g.fill(getArrowHead());
		g.draw(getArrowHead());
		drawLabel(g);
	}

	public void drawSelected(final Graphics2D g) {
		if (!isValid)
			refresh();
		Graphics2D gr = (Graphics2D) g.create();
		gr.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		gr.setStroke(new BasicStroke(2.0f));
		gr.setColor(Color.BLUE);
		draw(gr);
		gr.dispose();
	}

	private Polygon getArrowHead() {
		double angle = Math.atan2(start.x - end.x, start.y - end.y);
		Polygon arrow = new Polygon();
		arrow.addPoint(end.x, end.y);
		angle += ARROW_ANGLE;
		int endX = ((int) (Math.sin(angle) * ARROW_LENGTH)) + end.x;
		int endY = ((int) (Math.cos(angle) * ARROW_LENGTH)) + end.y;
		arrow.addPoint(endX, endY);
		angle -= 2 * ARROW_ANGLE;
		endX = ((int) (Math.sin(angle) * ARROW_LENGTH)) + end.x;
		endY = ((int) (Math.cos(angle) * ARROW_LENGTH)) + end.y;
		arrow.addPoint(endX, endY);
		return arrow;
	}

	protected void drawLabel(Graphics2D gr) {

	}

	protected void refresh() {
		line.setLine(getStart().x, getStart().y, getEnd().x, getEnd().y);		
		textTransform = new AffineTransform();
		textTransform.rotate(Math.atan2(line.getX2() - line.getX1(), line.getY2() - line.getY1()));
		if (getEnd().x < getStart().x)
			textTransform.rotate(Math.PI);
		isValid = true;
	}

	public Rectangle2D getBounds() {
		if (!isValid)
			refresh();
		Rectangle2D b = line.getBounds2D();		
		b.add(getArrowHead().getBounds2D());
		return b;
	}

}
