/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.graph.viewer2d;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;

import xxl.applications.gui.graph.Vertex;

public class VertexDrawer {

	public static final Color VERTEX_BKG_COLOR = new Color(255, 255, 240);
	public static final Color VERTEX_SELECTED_COLOR = new Color(190, 190, 230);
	public static final BasicStroke VERTEX_SELECTED_STROKE =
		new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 8, new float[] { 10, 10 }, 0);
	public static final int VERTEX_WIDTH = 40;
	public static final int VERTEX_HEIGHT = 20;
	public static final int ARC_RADIUS = 10;

	private int width;
	private int height;
	private Rectangle2D bounds;

	public VertexDrawer() {
		this(VERTEX_WIDTH, VERTEX_HEIGHT);
	}

	public VertexDrawer(int width, int height) {
		this.width = width;
		this.height = height;
	}

	protected void drawBackground(Graphics g, Vertex state, Point point, Color color) {
		g.setColor(color);
		g.fillRoundRect(
			point.x - getWidth(),
			point.y - getHeight(),
			2 * getWidth(),
			2 * getHeight(),
			ARC_RADIUS,
			ARC_RADIUS);
	}

	protected void drawSelected(Graphics g, Vertex vertex, Point point) {
		Graphics2D g2d = (Graphics2D) g;
		String name = vertex.getName();
		Color old_c = g.getColor();
		Stroke old_s = g2d.getStroke();
		//g2d.setStroke(VERTEX_SELECTED_STROKE);

		FontMetrics fm = g.getFontMetrics();
		int dx = ((int) fm.getStringBounds(name, g).getWidth()) >> 1;
		int dy = ((int) fm.getAscent()) >> 1;
		g2d.setColor(VERTEX_SELECTED_COLOR);
		g.drawString(name, point.x - dx, point.y + dy);
		g.drawRoundRect(
			point.x - getWidth(),
			point.y - getHeight(),
			2 * getWidth(),
			2 * getHeight(),
			ARC_RADIUS,
			ARC_RADIUS);
		g2d.setColor(old_c);
		//g2d.setStroke(old_s);
	}

	public Rectangle2D getBoundsAtPoint(Point point) {
		return new Rectangle2D
			.Float(point.x - getWidth(), point.y - getHeight(), 2 * getWidth(), 2 * getHeight())
			.getBounds2D();
	}

	public void drawVertex(Graphics g, Vertex vertex) {
		drawVertex(g, vertex, vertex.getLocation());
	}

	public void drawVertex(Graphics g, Vertex vertex, Point point) {
		drawVertex(g, vertex, point, VERTEX_BKG_COLOR);
	}

	public void drawVertex(Graphics g, Vertex vertex, Point point, Color color) {
		//skip invisible vertexes
		if (!vertex.isVisible())
			return;
		if (vertex.isSelected()) {
			drawSelected(g, vertex, point);
			return;
		}

		Color old = g.getColor();
		drawBackground(g, vertex, point, color);
		g.setColor(Color.BLACK);
		FontMetrics fm = g.getFontMetrics();
		String name = vertex.getName();
		int dx = ((int) fm.getStringBounds(name, g).getWidth()) >> 1;
		int dy = ((int) fm.getAscent()) >> 1;
		g.drawString(name, point.x - dx, point.y + dy);
		g.drawRoundRect(
			point.x - getWidth(),
			point.y - getHeight(),
			2 * getWidth(),
			2 * getHeight(),
			ARC_RADIUS,
			ARC_RADIUS);
		g.setColor(old);
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}

}
