/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.property.editors;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.beans.PropertyEditor;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.JLabel;
import javax.swing.JTextField;

import xxl.applications.gui.code.ConstructorCall;
import xxl.applications.gui.code.ElementCall;
import xxl.applications.gui.code.PrimitiveCall;
import xxl.applications.gui.code.Type;

/**
 *
 * @author  maxim
 */
final class ConstructorCallCustomEditor
	extends javax.swing.JPanel
	implements CustomEditorSupport {

	private Map fields2OrderMap;
	private ConstructorCall call2Edit;
	private PropertyEditor editor;

	/** Creates new form ConstructorCallCustomEditor */
	public ConstructorCallCustomEditor(final ConstructorCallEditor editor) {
		this.editor = editor;
		initComponents();
		call2Edit = (ConstructorCall) ConstructorCallCustomEditor.this.editor.getValue();
		createMask(call2Edit);		
	}

	private void createMask(ConstructorCall call) {
		int params = call.getParamCount();
		if (params <= 0)
			return;
		fields2OrderMap = new Hashtable();
		GridBagConstraints gbc = null;
		Insets rightIns = new Insets(0, 0, 2, 0);
		int done = 0;
		for (int i = 0; i < params; i++) {
			ElementCall parameter = call.getParamValue(i);
			Type t = call.getParamType(i);
			System.out.println("ConstructorCallCustomEditor.createMask() " + t);
			//if(parameter instanceof PrimitiveCall) {
			if (PrimitiveCall.isPrimitive(t)) {
				JLabel label = new JLabel(t.toString());
				gbc = new GridBagConstraints();
				gbc.gridx = 0;
				gbc.gridy = done++;
				gbc.fill = GridBagConstraints.HORIZONTAL;
				gbc.weightx = 1.0;
				gbc.insets = rightIns;
				innerPanel.add(label, gbc);
				JTextField field = new JTextField();
				//what happens when string is an object?
				PrimitiveCall primitive = (PrimitiveCall) call.getParamValue(i);
				field.setText(primitive == null ? "?" : primitive.getValue().toString());
				gbc = new GridBagConstraints();
				gbc.gridx = 0;
				gbc.gridy = done++;
				gbc.fill = GridBagConstraints.HORIZONTAL;
				gbc.weightx = 1.0;
				label.setLabelFor(field);
				innerPanel.add(field, gbc);
				fields2OrderMap.put(new Integer(i), field);

			}
		}
	}

	public Object getPropertyValue() throws IllegalStateException {
		try {
			Set entries = fields2OrderMap.entrySet();
			for (Iterator iter = entries.iterator(); iter.hasNext();) {
				Map.Entry entry = (Map.Entry) iter.next();
				int param = ((Integer) entry.getKey()).intValue();
				Object value = ((JTextField) entry.getValue()).getText();
				PrimitiveCall call = (PrimitiveCall) call2Edit.getParamValue(param);
				call.setValue(value);
			}
			call2Edit.setBody(codeEditorPane.getText());
			return call2Edit;
		}
		catch (Exception e) {
			e.printStackTrace();
			throw new IllegalStateException(e.getMessage());
		}
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	private void initComponents() { //GEN-BEGIN:initComponents
		java.awt.GridBagConstraints gridBagConstraints;

		innerPanel = new javax.swing.JPanel();
		codePanel = new javax.swing.JPanel();
		jScrollPane1 = new javax.swing.JScrollPane();
		codeEditorPane = new javax.swing.JEditorPane();

		setLayout(new java.awt.GridBagLayout());

		setAutoscrolls(true);
		innerPanel.setLayout(new java.awt.GridBagLayout());

		innerPanel.setBorder(
			new javax.swing.border.CompoundBorder(
				new javax.swing.border.TitledBorder("Parameters"),
				new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 1, 1, 1))));
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 2, 0);
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 0.7;
		add(innerPanel, gridBagConstraints);

		codePanel.setLayout(new java.awt.BorderLayout());

		codePanel.setBorder(
			new javax.swing.border.CompoundBorder(
				new javax.swing.border.TitledBorder("Code"),
				new javax.swing.border.EmptyBorder(new java.awt.Insets(2, 2, 2, 2))));
		jScrollPane1.setViewportView(codeEditorPane);

		codePanel.add(jScrollPane1, java.awt.BorderLayout.CENTER);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 0.5;
		add(codePanel, gridBagConstraints);

	} //GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JEditorPane codeEditorPane;
	private javax.swing.JPanel codePanel;
	private javax.swing.JPanel innerPanel;
	private javax.swing.JScrollPane jScrollPane1;
	// End of variables declaration//GEN-END:variables

}
