/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.property.editors;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

/**
 * @author maxim
 */
public class ConstructorCallEditor extends SwingEditorSupport {

	private JTextField textValue;
	private JButton editorButton;
	private EditorDisplayer dlg;
	private Component customEditor;

	public ConstructorCallEditor() {
		createComponents();
		initListeners();
	}

	private void initListeners() {
		editorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (dlg == null)
					dlg = new EditorDisplayer("Edit constructor", true, ConstructorCallEditor.this);
				dlg.getDialog().setVisible(true);
			}
		});
	}

	public void setValue(Object value) {
		super.setValue(value);
		editorValueChanged(value);
	}

	void editorValueChanged(Object value) {
		if (dlg == null)
			dlg = new EditorDisplayer("Edit constructor", true, ConstructorCallEditor.this);
		dlg.setComponent();

	}

	public void setAsText(String text) throws IllegalArgumentException {
	}

	private void createComponents() {
		textValue = new JTextField();
		textValue.setPreferredSize(MEDIUM_DIMENSION);
		textValue.setMinimumSize(MEDIUM_DIMENSION);
		textValue.setEditable(false);
		setAlignment(textValue);
		editorButton = new JButton("...");
		editorButton.setContentAreaFilled(false);
		editorButton.setMargin(BUTTON_MARGIN);
		setAlignment(editorButton);
		panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(textValue);
		panel.add(Box.createRigidArea(new Dimension(5, 0)));
		panel.add(editorButton);
		panel.setAlignmentX(Component.LEFT_ALIGNMENT);
	}

	static class EditorDisplayer {

		private static final String OK_CAPTION = "OK";
		private static final String CANCEL_CAPTION = "Cancel";
		private static final String CLOSE_CAPTION = "Close";
		private static final Dimension BTN_DIMENSION = new Dimension(100, 26);

		private Object oldValue;
		private ConstructorCallEditor editor;
		private Component component;
		private Window dialog;
		private String title;
		private boolean isModal;

		private Object[] options;
		private Object defaultOption;

		/** Creates a new instance of EditorDisplayer */
		public EditorDisplayer(
			final String title,
			final boolean isModal,
			final ConstructorCallEditor editor) {
			this.editor = editor;
			this.title = title;
			this.isModal = isModal;
			setComponent();

		}

		void setComponent() {
			System.out.println("EditorDisplayer.setComponent()");
			this.component = new ConstructorCallCustomEditor(editor);
			createDialog();
			//createListeners();
		}

		private void createDialog() {
			if (component == null)
				throw new NullPointerException("Could not create Dialog for null component");

			if (editor == null) {
				JButton closeButton = new JButton(CLOSE_CAPTION);
				closeButton.setMinimumSize(BTN_DIMENSION);
				closeButton.setPreferredSize(BTN_DIMENSION);
				options = new Object[] { closeButton };
				defaultOption = closeButton;
			}
			else {
				JButton okButton = new JButton(OK_CAPTION);
				okButton.setPreferredSize(BTN_DIMENSION);
				okButton.setMinimumSize(BTN_DIMENSION);
				JButton cancelButton = new JButton(CANCEL_CAPTION);
				cancelButton.setPreferredSize(BTN_DIMENSION);
				cancelButton.setMinimumSize(BTN_DIMENSION);
				cancelButton.setVerifyInputWhenFocusTarget(false);
				cancelButton.setDefaultCapable(false);
				options = new Object[] { okButton, cancelButton };
				defaultOption = okButton;
			}

			JOptionPane jop =
				new JOptionPane(
					component,
					JOptionPane.PLAIN_MESSAGE,
					JOptionPane.NO_OPTION,
					null,
					options,
					defaultOption);
			dialog =
				jop.createDialog(SwingUtilities.getAncestorOfClass(Frame.class, component), title);

		}

		private void cancelValue() {

		}

		public Window getDialog() {
			return dialog;
		}

		private void createListeners() {
			oldValue = editor.getValue();
			dialog.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent we) {
					if (editor != null || !(component instanceof Window)) {
						cancelValue();
					}
					dialog.dispose();
				}
				public void windowClosed(WindowEvent we) {
					if (component instanceof CustomEditorSupport) {
						try {
							Object newValue = ((CustomEditorSupport) component).getPropertyValue();
							editor.setValue(newValue);
						}
						catch (Exception e) {
							e.printStackTrace();
						}
					}
					dialog.removeWindowListener(this);
				}
			});
		}
	}
}
