/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.property.editors;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListModel;

import xxl.applications.gui.windows.MainFrame;

//FIXME ist als editor fr listen gedacht, momentan jedoch nur fr listen von.
// interfaces
/**
 * @author maxim
 */
public class InterfaceListPropertyEditor extends SwingEditorSupport {
	
	private JList items;
	private JTextField field;
	private JButton expandBtn;
	
	public InterfaceListPropertyEditor() {
		createComponents();
		addComponentListeners();
	}
	
	private void addComponentListeners() {
		expandBtn.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				Frame owner = MainFrame.getInstance();
				JDialog d = new ListViewer(owner, "Interfaces");
				d.setLocationRelativeTo(owner);
				d.setVisible(true);
			}
		});
	}
	
	private void createComponents() {
		items = new JList();
		field = new JTextField();
		field.setPreferredSize(MEDIUM_DIMENSION);
		field.setMinimumSize(MEDIUM_DIMENSION);		
		setAlignment(field);
		
		expandBtn = new JButton("...");		
		expandBtn.setToolTipText("press to show the list view");
		expandBtn.setMargin(BUTTON_MARGIN);		
		expandBtn.setContentAreaFilled(false);	
		setAlignment(expandBtn);
		
		panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(field);
		panel.add(Box.createRigidArea(new Dimension(5, 10)));		
		panel.add(expandBtn);
		panel.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	
	public void setAsText(String s) {
		StringTokenizer str_tok = new StringTokenizer(s, " ,");
		ArrayList list = new ArrayList();
		while (str_tok.hasMoreTokens()) {
			list.add(str_tok.nextToken());
		}
		setValue(list.toArray());
	}
	
	public String getAsText() {
		ListModel model = items.getModel();
		String text = "";
		if (model != null) {
			StringBuffer str_buf = new StringBuffer();
			for (int i = 0; i < model.getSize(); i++) {
				if (i > 0)
					str_buf.append(", ");
				str_buf.append(model.getElementAt(i));
			}
			text = str_buf.toString();
		}
		return text;
	}
	
	public void setValue(Object value) {
		super.setValue(value);
		editorValueChanged(value);
	}
	
	void editorValueChanged(Object value) {
		Object[] values;
		if (value.getClass().isArray()) {
			values = (Object[]) value;
		}
		else
			values = new Object[]{value};
			DefaultListModel model = new DefaultListModel();
			StringBuffer str_buf = new StringBuffer();
			for (int i = 0; i < values.length; i++) {
				if (i > 0)
					str_buf.append(", ");
				model.addElement(values[i]);
				str_buf.append(values[i]);
			}
			items.setModel(model);
			field.setText(str_buf.toString());
	}
	
	private class ListViewer extends JDialog {
		
		public ListViewer(Frame owner, String title) {
			super(owner, title);
			initComponents();
		}
		
		private void initComponents() {
			JButton btn = new JButton("Close");
			Dimension dim = new Dimension(100, 26);
			btn.setMinimumSize(dim);
			btn.setMaximumSize(dim);
			btn.setPreferredSize(dim);
			btn.addActionListener(new ActionListener() {
				
				public void actionPerformed(ActionEvent e) {
					ListViewer.this.setVisible(false);
					ListViewer.this.dispose();
				}
			});
			JPanel panel = new JPanel(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.fill = GridBagConstraints.BOTH;
			gbc.weightx = 1.0;
			gbc.weighty = 1.0;
			JScrollPane pane = new JScrollPane(items);
			pane.setBorder(BorderFactory.createCompoundBorder(BorderFactory
			.createTitledBorder("Implemented Interfaces"), BorderFactory.createEmptyBorder(5, 5, 5,
			5)));
			panel.add(pane, gbc);
			gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 1;
			gbc.insets = new Insets(5, 0, 0, 0);
			panel.add(btn, gbc);
			panel.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
			getRootPane().setDefaultButton(btn);
			getContentPane().add(panel);
			pack();
		}
	}
}
