/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.serializer;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import xxl.applications.gui.awt.tree.CategoryTreeObject;
import xxl.applications.gui.awt.tree.ClassTreeObject;

import com.thoughtworks.xstream.alias.ClassMapper;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

/**
 * @author maxim
 */
public final class CategoryConverter implements Converter {
	public static final String ENTRY_PROPERTY = "entry";
	public static final String CATEGORY_PROPERTY = "category";

	private ClassMapper classMapper;

	public CategoryConverter(ClassMapper classMapper) {
		this.classMapper = classMapper;
	}

	public boolean canConvert(Class clazz) {
		return clazz.equals(CategoryTreeObject.class);
	}

	public void marshal(
		Object source,
		HierarchicalStreamWriter writer,
		MarshallingContext context) {
		CategoryTreeObject cto = (CategoryTreeObject) source;
		Enumeration children = cto.children();		
		writer.addAttribute("name", cto.getObjectName());
		while (children.hasMoreElements()) {
			ClassTreeObject clazz = (ClassTreeObject) children.nextElement();
			writeEntry(clazz.getFullName(), context, writer);
		}		
	}

	public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
		List children = new ArrayList();
		String name = null;
		Class type;
		while (reader.hasMoreChildren()) {		
			name = reader.getAttribute("name");
			reader.moveDown();			
			children.add(readEntry(reader, context));
			reader.moveUp();
		}
		CategoryTreeObject cto = new CategoryTreeObject(name);
		for (int i = 0; i < children.size(); i++) {
			cto.add(new ClassTreeObject(children.get(i).toString()));
		}
		return cto;

	}

	private void writeEntry(
		Object entry,
		MarshallingContext ctx,
		HierarchicalStreamWriter writer) {
		if (entry == null || entry.equals(""))
			return;
		writer.startNode(ENTRY_PROPERTY);
		ctx.convertAnother(entry);
		writer.endNode();
	}

	private Object readEntry(HierarchicalStreamReader reader, UnmarshallingContext ctx) {		
		return ctx.convertAnother(null, String.class);
	}
}
