/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.serializer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import xxl.applications.gui.awt.tree.CategoryTreeObject;

import com.thoughtworks.xstream.XStream;

/**
 * @author maxim
 */
public class CategorySerializer {
	private static CategorySerializer instance;

	private static CategorySerializer getInstance() {
		if (instance == null) {
			instance = new CategorySerializer();
		}
		return instance;
	}
	private XStream stream;
	private CategorySerializer() {
		stream = new XStream();
		stream.registerConverter(new CategoryConverter(stream.getClassMapper()));
		stream.alias("category", CategoryTreeObject.class);
	}

	private static String createPath(String name) {
		StringBuffer str_buf = new StringBuffer(System.getProperty("xxl.user.home"));
		str_buf.append(File.separatorChar);
		str_buf.append("categories");
		str_buf.append(File.separatorChar);
		str_buf.append(name);
		return str_buf.toString();
	}
	public static void write(CategoryTreeObject category) {
		File f = new File(createPath(category.getObjectName()) + ".xml");
		try {
			FileWriter fw = new FileWriter(f);
			BufferedWriter writer = new BufferedWriter(fw);
			getInstance().stream.toXML(category, writer);
			writer.flush();
			writer.close();
			fw.close();
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	public static CategoryTreeObject read(String name) {
		File f = new File(createPath(name));
		CategoryTreeObject to = null;
		try {
			FileReader fr = new FileReader(f);
			BufferedReader reader = new BufferedReader(fr);
			to = (CategoryTreeObject) getInstance().stream.fromXML(reader);
			reader.close();
			fr.close();
		}
		catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return to;
	}

	public static CategoryTreeObject[] readAll() {
		List list = new ArrayList();
		File cwd = new File(System.getProperty("xxl.user.home") + File.separator + "categories");
		File[] entries = cwd.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isFile() && pathname.getName().endsWith(".xml");
			}
		});
		for (int i = 0; i < entries.length; i++) {
			list.add(read(entries[i].getName()));
		}
		return (CategoryTreeObject[]) list.toArray(new CategoryTreeObject[list.size()]);
	}
}
