/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.serializer;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import xxl.applications.gui.code.CallVertex;
import xxl.applications.gui.code.ClassItem;
import xxl.applications.gui.code.ConstructorCall;
import xxl.applications.gui.code.ObjectItem;
import xxl.applications.gui.code.PrimitiveCall;
import xxl.applications.gui.code.PrimitiveItem;
import xxl.applications.gui.code.Type;
import xxl.applications.gui.graph.ClassGraph;
import xxl.applications.gui.graph.Edge;
import xxl.applications.gui.graph.Graph;
import xxl.applications.gui.graph.Vertex;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.core.ReferenceByIdMarshallingStrategy;

/*
 * Created on 08.04.2004
 */
/**
 * @author Tim Dörnemann
 */
public class XMLSerializeConverter extends Converter {
	
	public static final XStream XMLSERIALIZER = new XStream(new ReflectionProvider());
	public static final XMLSerializeConverter DEFAULT_INSTANCE = new XMLSerializeConverter();
	
	XMLSerializeConverter() {
	  XMLSERIALIZER.setMarshallingStrategy(new ReferenceByIdMarshallingStrategy());
	      
	  XMLSERIALIZER.alias("Vertex",Vertex.class);
	  XMLSERIALIZER.alias("Graph",Graph.class);
	  XMLSERIALIZER.alias("Edge",Edge.class);
	  XMLSERIALIZER.alias("ClassGraph", ClassGraph.class);
	  
	  XMLSERIALIZER.alias("CallVertex", CallVertex.class);
	  XMLSERIALIZER.alias("ObjectItem", ObjectItem.class);
	  XMLSERIALIZER.alias("ClassItem", ClassItem.class);
	  XMLSERIALIZER.alias("PrimitiveItem", PrimitiveItem.class);
	  XMLSERIALIZER.alias("PrimitiveCall", PrimitiveCall.class);
	  
	  XMLSERIALIZER.alias("ConstructorCall", ConstructorCall.class);
	  XMLSERIALIZER.alias("Type", Type.class);
	}
	
	public void write(DataOutput dataOutput, Object object) throws IOException {
		String xmlData = XMLSERIALIZER.toXML(object);
		
		if(dataOutput instanceof XMLDataOutput)
			((XMLDataOutput)dataOutput).writeNextElement(xmlData);		
	}
	
	public Object read(DataInput dataInput, Object object) throws IOException{
		if(dataInput instanceof XMLDataInput) {
			String xmlData = ((XMLDataInput)dataInput).readNextElement();
			
			object = XMLSERIALIZER.fromXML(xmlData);
		}	
		return object;
	}
}
