/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.serializer;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import xxl.applications.gui.graph.Edge;
import xxl.applications.gui.graph.Graph;
import xxl.applications.gui.graph.Vertex;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.core.ReferenceByIdMarshallingStrategy;

/**
 * @author maxim, doernemt
 */
public final class XMLSerializer extends Converter {

	private static Converter sharedInstance;

	public static Converter getInstance() {
		if (sharedInstance == null) {
			sharedInstance = new XMLSerializer();
		}
		return sharedInstance;
	}
	private XStream stream;

	private XMLSerializer() {
		stream = new XStream();
		stream.setMarshallingStrategy(new ReferenceByIdMarshallingStrategy());
		stream.alias("vertex", Vertex.class);
		stream.alias("edge", Edge.class);
		stream.alias("graph", Graph.class);
		stream.registerConverter(new GraphConverter(stream.getClassMapper()));
	}

	public Object read(DataInput dataInput, Object object) throws IOException {
		if (dataInput instanceof XMLDataInput) {
			String xmlData = ((XMLDataInput) dataInput).readNextElement();
			object = stream.fromXML(xmlData);
		}
		return object;
	}

	public void write(DataOutput dataOutput, Object object) throws IOException {
		String xmlData = stream.toXML(object);
		if (dataOutput instanceof XMLDataOutput)
			((XMLDataOutput) dataOutput).writeNextElement(xmlData);
	}

	public static void main(String[] args) throws Exception {
		Graph graph = new Graph();
		for (int i = 0; i < 10; i++) {
			graph.addVertex(new Vertex("Vertex " + (i + 1)));
		}
		DataOutputStream out = new XMLDataOutput(new FileOutputStream("test.xml"));
		XMLSerializer.getInstance().write(out, graph);
		out.flush();
		out.close();
		DataInputStream in = new XMLDataInput(new FileInputStream("test.xml"));
		graph = (Graph) XMLSerializer.getInstance().read(in);
		in.close();
		System.out.println(graph);
	}
}
