/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;

import com.sun.tools.javac.Main;

/**
 * @author maxim
 */
public class CompilerClassLoader extends ClassLoader {

	private static CompilerClassLoader classLoader;

	protected CompilerClassLoader() {
		super();
	}

	protected CompilerClassLoader(ClassLoader parent) {
		super(parent);
	}

	public static ClassLoader getClassLoader() {
		if (classLoader == null) {
			classLoader = new CompilerClassLoader();
		}
		return classLoader;
	}

	private byte[] getClassBytes(File file) throws IOException {
		if (file.isDirectory())
			throw new IOException("Class File is a directory");
		FileInputStream fis = new FileInputStream(file);
		long length = file.length();
		byte[] classData = new byte[(int) length];
		if (fis.read(classData) != length)
			throw new IOException("Could not read all bytes from file");
		fis.close();
		return classData;
	}

	private boolean compile(String javaFileName) throws IOException {
		ArrayList args = new ArrayList();
		//String tmp = System.getProperty("xxl.user.home") + File.separator + "src";
		//	if (tmp != null) {
		//		args.add("-sourcepath");
		//		args.add(tmp);
		//	}
		String tmp = ClassPath.createClassPath().getClassPath();
		if (tmp != null) {
			args.add("-classpath");
			args.add(tmp);
		}
		tmp = System.getProperty("xxl.user.home") + File.separator + "classes";
		if (tmp != null) {
			args.add("-d");
			args.add(tmp);
		}
		args.add(
			System.getProperty("xxl.user.home") + File.separator + "src" + File.separator + javaFileName);
		//TODO redirect OUTPUT				
		return Main.compile((String[]) args.toArray(new String[args.size()])) == 0;
	}

	public Class loadClass(String className, boolean resolve) throws ClassNotFoundException {
		//check if the class was already loaded
		Class clazz = findLoadedClass(className);
		String tmpName = className.replace('.', File.separatorChar);
		StringBuffer str_buf = new StringBuffer(System.getProperty("xxl.user.home"));
		str_buf.append(File.separator);
		str_buf.append("src");
		str_buf.append(File.separator);
		str_buf.append(tmpName);
		str_buf.append(".java");
		File javaFile = new File(str_buf.toString());
		str_buf.delete(0, str_buf.length());
		str_buf.append(System.getProperty("xxl.user.home"));
		str_buf.append(File.separator);
		str_buf.append("classes");
		str_buf.append(File.separatorChar);
		str_buf.append(tmpName);
		str_buf.append(".class");

		File classFile = new File(str_buf.toString());

		//if yes and the class file needs to be recompiled, try to do it
		if (javaFile.exists()
			&& !classFile.exists()
			|| javaFile.lastModified() > classFile.lastModified()) {
			try {
				if (!compile(javaFile.getName()) || !classFile.exists())
					throw new ClassNotFoundException(
						"Could not load class " + javaFile + " Compilation failed");
			}
			catch (IOException ie) {
				throw new ClassNotFoundException(ie.getMessage());
			}
		}
		//load class from file
		try {
			byte[] classData = getClassBytes(classFile);
			clazz = defineClass(className, classData, 0, classData.length);
		}
		// loading fails if we try load a system class from our directory, so it is normal		
		catch (IOException ie) {
		}
		// if class is still null, it is a system class, try to load it
		if (clazz == null) {
			clazz = findSystemClass(className);
		}
		if (resolve && clazz != null) {
			resolveClass(clazz);
		}
		//give up...
		if (clazz == null)
			throw new ClassNotFoundException(className);
		return clazz;
	}

	/**
	 * Tests the ClassLoader. Note that the system property "xxl.user.home" must be
	 * set to a directory <tt>A</tt>, since the loader tries to fetch classes from
	 * the <tt>A + File.separator + classes</tt> directory.	  
	 * @throws Exception if something goes wrong, resulting in printing the exception's stack
	 */
	public static void main(String[] args) throws Exception {
		//to run main system property xxl.user.home must be set!
		CompilerClassLoader ccl = new CompilerClassLoader();
		Class clazz = Class.forName("HelloWorld", true, ccl);
		Method main = clazz.getMethod("main", new Class[] { String[].class });
		main.invoke(null, new Object[] { args });
	}
}
