/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

/**
 * @author maxim
 */
public final class ClassDialog extends JDialog {

	private final static double TEXTFIELDWEIGHT = 0.90;
	private final static double LABELWEIGHT = 0.10;

	private JTextField classNameField;
	private String className;
	private JButton okButton;

	private ActionListener buttonListener = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			className = null;
			if (e.getSource() == okButton)
				className = new String(classNameField.getText());
			setVisible(false);
			dispose();
		}
	};

	private ClassDialog(Frame parent, boolean isModal) {
		super(parent, isModal);
		initComponents();
		JRootPane root = SwingUtilities.getRootPane(okButton);
		if (root != null)
			root.setDefaultButton(okButton);
		className = "<default>";
		classNameField.setText(className);
		classNameField.selectAll();
	}

	private JLabel makeDialogLabel(String labelText) {
		JLabel label = new JLabel();
		label.setText(labelText);
		label.setHorizontalTextPosition(SwingConstants.LEADING);
		return label;
	}

	private void initComponents() {
		
		if (!SwingUtilities.isEventDispatchThread()) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					initComponents();
				}
			});
			return;
		}

		GridBagConstraints gridBagConstraints;
		classNameField = new JTextField();
		setTitle("New class name");
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				doClose();
			}
		});

		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new java.awt.GridBagLayout());
		mainPanel.setBorder(new EmptyBorder(11, 11, 11, 11));

		GridBagConstraints gbc = new java.awt.GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.insets = new Insets(0, 0, 0, 4);
		mainPanel.add(makeDialogLabel("Class Name:"), gbc);

		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.weightx = 1.0;
		mainPanel.add(classNameField, gbc);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridBagLayout());

		Dimension dim = new Dimension(100, 26);
		JButton button;
		okButton = new JButton();
		okButton.setText("OK");
		okButton.setPreferredSize(dim);
		okButton.setMinimumSize(dim);
		okButton.setMaximumSize(dim);
		okButton.setMnemonic('K');
		okButton.addActionListener(buttonListener);
		gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.insets = new Insets(0, 0, 0, 2);
		buttonPanel.add(okButton, gbc);

		button = new JButton();
		button.setText("Cancel");
		button.setPreferredSize(dim);
		button.setMinimumSize(dim);
		button.setMaximumSize(dim);
		button.setMnemonic('a');
		button.addActionListener(buttonListener);
		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 0;
		buttonPanel.add(button, gbc);

		gbc = new GridBagConstraints();
		gbc.gridx = 1;
		gbc.gridy = 2;
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1.0;
		gbc.anchor = GridBagConstraints.SOUTHEAST;
		gbc.insets = new Insets(12, 0, 0, 0);
		mainPanel.add(buttonPanel, gbc);
		getContentPane().add(mainPanel);
		pack();

	}
	private void doClose() {
		setVisible(false);
		dispose();
	}
	public static String getName(Frame parent) {
		ClassDialog dlg = new ClassDialog(parent, true);
		dlg.className = null;
		dlg.pack();
		dlg.setLocationRelativeTo(parent);
		dlg.setVisible(true);
		return dlg.className;
	}
}
