/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows;

import java.awt.Component;
import java.awt.Container;
import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import xxl.applications.gui.actions.AddNewAction;
import xxl.applications.gui.actions.GenerateCodeAction;
import xxl.applications.gui.awt.AbstractPopupMenu;
import xxl.applications.gui.awt.tree.ClassTreeObject;
import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.code.CallVertex;
import xxl.applications.gui.code.ClassItem;
import xxl.applications.gui.code.ConstructorCall;
import xxl.applications.gui.code.ObjectItem;
import xxl.applications.gui.code.PrimitiveCall;
import xxl.applications.gui.code.PrimitiveItem;
import xxl.applications.gui.code.Type;
import xxl.applications.gui.graph.ClassGraph;
import xxl.applications.gui.graph.Graph;
import xxl.applications.gui.graph.Vertex;
import xxl.applications.gui.graph.event.ConnectionEvent;
import xxl.applications.gui.graph.viewer2d.ClassGraphDrawer;
import xxl.applications.gui.windows.wizards.AddWizard;
import xxl.applications.gui.windows.wizards.ConstructorWizard;
import xxl.applications.gui.wizard.JWizardPane;

/**
 * @author maxim
 */
class ClassGraphFrame extends GraphFrame {

	//corresponding code window
	private transient CodeFrame codeFrame;
	private transient String className;

	protected ClassGraphFrame(String title, ClassGraph graph) {
		super(title, graph);
		graphPane.setDrawer(new ClassGraphDrawer());
		DropTarget dropTarget = new DropTarget(this, new DropTargetListener());
	}

	protected Graph createNew() {
		return new ClassGraph();
	}

	protected void registerActions() {
		super.registerActions();
		registerAction(AddNewAction.ACTION_COMMAND_VALUE, new AddNewActionImpl());
		registerAction(GenerateCodeAction.ACTION_COMMAND_VALUE, new GenerateCodeImpl());
	}

	public void handleConnection(ConnectionEvent ce) {
		callWizard(ce.getOrigin(), ce.getDestination());
	}

	private int callWizard(Vertex source, Vertex destination) {
		ConstructorWizard wizard = new ConstructorWizard();
		wizard.putProperty(ConstructorWizard.SOURCE_ITEM_PROPERTY, source);
		wizard.putProperty(ConstructorWizard.TARGET_ITEM_PROPERTY, destination);
		JDialog dlg = wizard.createDialog(null, "Constructor Selection");
		dlg.setVisible(true);
		return ((Integer) wizard.getValue()).intValue();
	}

	private class DropTargetListener extends DropTargetAdapter {

		public void drop(DropTargetDropEvent dtde) {
			Transferable tr = dtde.getTransferable();
			if (!tr.isDataFlavorSupported(ClassTreeObject.CLASS_OBJECT_FLAVOR)) {
				dtde.rejectDrop();
			}
			try {
				ClassTreeObject cto =
					(ClassTreeObject) tr.getTransferData(ClassTreeObject.CLASS_OBJECT_FLAVOR);
				Point point = graphPane.transformFromViewToGraph(dtde.getLocation());
				ClassGraph g = graphPane.getClassGraph();
				/*
				AddWizard wizard = new AddWizard(cto.getFullName());
				wizard.putProperty(AddWizard.ITEM_CLASS_PROPERTY, cto.getFullName());
				wizard.putProperty(
					AddWizard.ITEM_TYPE_PROPERTY,
					new Integer(AbstractItem.OBJECT_ITEM));
				
				JDialog dialog = wizard.createDialog(MainFrame.getInstance(), "Add new Item");
				dialog.show();
				int option = ((Integer) wizard.getValue()).intValue();
				if (option == JWizardPane.FINISH_OPTION) {
					processValues(wizard, point, g);
				}
				*/

				//TODO should one could add static utility classes using dnd?
				// reject primitives, interfaces and and abstract classes
				Class clazz = Class.forName(cto.getFullName());
				if (clazz == null
					|| clazz.isPrimitive()
					|| clazz.isInterface()
					|| Modifier.isAbstract(clazz.getModifiers())) {
					dtde.rejectDrop();
					return;
				}
				Constructor[] cons = clazz.getConstructors();
				//reject classes with no constructors
				if (cons == null || cons.length == 0) {
					dtde.rejectDrop();
					return;
				}
				AbstractItem item = new ObjectItem(point, cto.getFullName());
				g.addVertex(item);

			}
			catch (Exception e) {
				e.printStackTrace();
				dtde.rejectDrop();
			}
		}

		public void dragOver(DropTargetDragEvent dtde) {
			dtde.acceptDrag(DnDConstants.ACTION_COPY);
		}
	}

	private void processValues(AddWizard addWizard, Point location, ClassGraph graph) {
		Integer item = (Integer) addWizard.getProperty(AddWizard.ITEM_TYPE_PROPERTY);
		AbstractItem newItem = null;
		String className = (String) addWizard.getProperty(AddWizard.ITEM_CLASS_PROPERTY);
		String name = (String) addWizard.getProperty(AddWizard.ITEM_NAME_PROPERTY);
		String value = (String) addWizard.getProperty(AddWizard.ITEM_INITVALUE_PROPERTY);
		String code = (String) addWizard.getProperty(AddWizard.ITEM_CODE_PROPERTY);
		//go through item types
		switch (item.intValue()) {
			//class_item is the simpliest case, just add the vertex
			case AbstractItem.CLASS_ITEM :
				newItem = new ClassItem(location, className);
				graph.addVertex(newItem);
				break;
			case AbstractItem.OBJECT_ITEM :
				//create object from classname, since we've allready
				// parsed the classname
				// no exception should be thrown
				newItem = new ObjectItem(location, className);
				if (name == null || name.equals(""))
					name = graph.getNameFor(newItem.getEnclosedType());
				newItem.setName(name);
				//try to add a constructor
				ConstructorCall cCall = new ConstructorCall(new CallVertex(graph));
				cCall.setCallType(newItem.getEnclosedType());
				cCall.setIdentifier(name);
				//try to determine if the class has a default
				// constructor
				try {
					Class clazz = newItem.getEnclosedType().toClass();
					int modif = clazz.getModifiers();
					//it is the case if the type is interface or the
					// default constructor is specified
					boolean hasDefault =
						Modifier.isInterface(modif) || (clazz.getConstructor(new Class[0]) != null);
					if (hasDefault)
						cCall.setParamTypes(new Type[0]);
					if (!Modifier.isFinal(modif)) {
						cCall.setBody(code);
					}
				}
				// else one has to set the constructor manually using
				// the wizard
				catch (ClassNotFoundException nfe) {
				}
				catch (NoSuchMethodException nsme) {
				}
				//finally add the created vertex to the graph
				graph.addVertex(newItem);
				((ObjectItem) newItem).setConstructor(cCall);
				break;
			case AbstractItem.PRIMITIVE_ITEM :
				newItem = new PrimitiveItem(location, className);
				graph.addVertex(newItem);
				if (name == null || name.equals(""))
					name = graph.getNameFor(newItem.getEnclosedType());
				newItem.setName(name);
				((PrimitiveItem) newItem).setInitCall(
					PrimitiveCall.create(
						new CallVertex(graph),
						Type.parse(className),
						value,
						newItem.getName()));
				break;
		}
	}

	private final class AddNewActionImpl implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			Container c =
				SwingUtilities.getAncestorOfClass(JPopupMenu.class, (Component) e.getSource());
			Point location = null;
			if (c != null && c instanceof AbstractPopupMenu) {
				location = ((AbstractPopupMenu) c).getLastScreenLocation();
			}
			else
				location = new Point();
			AddWizard addWizard = new AddWizard();
			ClassGraph graph = (ClassGraph) graphPane.getDrawer().getGraph();
			JDialog dialog = addWizard.createDialog(MainFrame.getInstance(), "Add new Item");
			dialog.setVisible(true);
			int option = ((Integer) addWizard.getValue()).intValue();
			if (option == JWizardPane.FINISH_OPTION) {
				processValues(addWizard, location, graph);
			}
		}
	}

	private final class GenerateCodeImpl implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			ClassGraph graph = (ClassGraph) graphPane.getDrawer().getGraph();
			StringWriter printer = new StringWriter();
			if (className == null) {
				className =
					ClassDialog.getName(
						(JFrame) SwingUtilities.getAncestorOfClass(
							JFrame.class,
							ClassGraphFrame.this));
				if (className.equals("<default>") || className == null)
					className = "AutoGen" + System.currentTimeMillis();
			}
			graph.generateCode(className, new PrintWriter(printer));			
			if (codeFrame == null) {
				codeFrame = new CodeFrame(className);
				MainFrame.getInstance().desktopPane.add(codeFrame);
				codeFrame.setSize(MainFrame.getInstance().desktopPane.getSize());
			}
			codeFrame.setCode(printer.getBuffer().toString());
			codeFrame.setVisible(true);
			codeFrame.toFront();
			printer.flush();
		}
	}
}
