/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.swing.JInternalFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;

import xxl.applications.gui.actions.ExecuteAction;
import xxl.applications.gui.actions.SaveAction;
import xxl.applications.gui.awt.StatusBar;
import xxl.applications.gui.util.CompilerClassLoader;

/**
 * @author maxim
 */
final class CodeFrame extends InternalFrame {

	private JTextArea codeEditorPane;
	private JTextArea outputArea;
	private JSplitPane splitPane;
	private transient boolean fileWasSaved;

	public CodeFrame(String title) {
		super(title);
		codeEditorPane = new JTextArea();
		//outputArea = new JTextArea();
		JScrollPane codeScroller = new JScrollPane(codeEditorPane);
		getContentPane().add(codeScroller);
		//JScrollPane outputScroller = new JScrollPane(outputArea);
		//splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		//splitPane.setTopComponent(codeScroller);
		//getContentPane().add(splitPane);	
		setDefaultCloseOperation(JInternalFrame.HIDE_ON_CLOSE);	
	}

	public void setCode(String code) {
		codeEditorPane.setText(code);
	}

	public String getCode() {
		return codeEditorPane.getText();
	}

	protected void registerActions() {
		registerAction(SaveAction.ACTION_COMMAND_VALUE, new SaveActionImpl());
		registerAction(ExecuteAction.ACTION_COMMAND_VALUE, new ExecuteActionImpl());
	}	

	private final class SaveActionImpl implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			File userdir =
				new File(
					System.getProperty("xxl.user.home")
						+ File.separator
						+ "src"
						+ File.separator
						+ CodeFrame.this.getTitle()
						+ ".java");
			try {
				PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(userdir)));
				out.write(getCode());
				out.close();
				fileWasSaved = true;
			}
			catch (IOException ioe) {

			}
		}
	}

	private final class ExecuteActionImpl implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (fileWasSaved) {
				try {
					Class clazz =
						Class.forName(
							CodeFrame.this.getTitle(),
							true,
							CompilerClassLoader.getClassLoader());
					if (clazz != null) {
						Method main = clazz.getMethod("main", new Class[] { String[].class });
						main.invoke(null, new Object[] { new String[0] });
						StatusBar.sharedInstance().notify("Class " + clazz.getName() + " loaded");
					}
				}
				catch (ClassNotFoundException cnfe) {
					StatusBar.sharedInstance().notify(cnfe.getMessage());
					//cnfe.printStackTrace();
				}
				catch (NoSuchMethodException nsme) {
					nsme.printStackTrace();
				}
				catch(IllegalAccessException iae) {
					iae.printStackTrace();					
				}
				catch(InvocationTargetException ite) {
					ite.printStackTrace();					
				}
			}
			else System.out.println("ExecuteActionImpl.actionPerformed() - save first!" );
		}
	}
}
