/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows;

import java.awt.event.ActionListener;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.JInternalFrame;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import xxl.applications.gui.actions.ActionManager;
import xxl.applications.gui.actions.CallableAction;

/**
 * @author maxim
 */
abstract class InternalFrame extends JInternalFrame {

	protected Map action2CommandMap;

	public InternalFrame() {
		this(null);
	}

	public InternalFrame(String title) {
		this(title, true);
	}

	public InternalFrame(String title, boolean resizable) {
		this(title, resizable, true);
	}

	public InternalFrame(String title, boolean resizable, boolean closable) {
		this(title, resizable, closable, true);
	}

	public InternalFrame(String title, boolean resizable, boolean closable, boolean maximizable) {
		this(title, resizable, closable, maximizable, true);
	}

	public InternalFrame(
		String title,
		boolean resizable,
		boolean closable,
		boolean maximizable,
		boolean iconifiable) {
		super(title, resizable, closable, maximizable, iconifiable);
		addInternalFrameListener(new InternalFrameHandler());
		registerActions();
	}

	public void doActivate() {
		enableActions(true);		
	}

	public void doDeactivate() {
		enableActions(false);		
	}

	public void doClose() {
		unregisterActions();		
	}

	protected final void registerAction(String actionCommand, ActionListener listener) {
		if (action2CommandMap == null)
			action2CommandMap = new Hashtable();
		CallableAction action = ActionManager.sharedInstance().getCallableAction(actionCommand);
		if (action != null) {
			action.setEnabled(true);
			action.setActionListener(listener);
			action2CommandMap.put(actionCommand, listener);
		}		
	}

	private void enableActions(boolean enable) {
		if (action2CommandMap != null) {
			Set commands = action2CommandMap.keySet();
			Iterator iter = commands.iterator();
			while (iter.hasNext()) {
				String ac = (String) iter.next();
				CallableAction action = ActionManager.sharedInstance().getCallableAction(ac);
				if (action != null) {
					if (!enable)
						action.removeActionListener();
					else
						action.setActionListener((ActionListener) action2CommandMap.get(ac));
					action.setEnabled(enable);
				}
			}

		}
	}

	protected abstract void registerActions();

	protected final void unregisterActions() {
		if (action2CommandMap != null) {
			String[] commands = (String[]) action2CommandMap.keySet().toArray(new String[0]);
			for (int i = 0; i < commands.length; i++) {
				unregisterAction(commands[i]);
			}
		}
	}

	protected final void unregisterAction(String actionCommand) {
		CallableAction action = ActionManager.sharedInstance().getCallableAction(actionCommand);
		if (action != null) {
			action.setEnabled(false);
			action.removeActionListener();
			if (action2CommandMap != null)
				action2CommandMap.remove(actionCommand);
		}
	}

	private final class InternalFrameHandler extends InternalFrameAdapter {

		public void internalFrameActivated(InternalFrameEvent e) {
			doActivate();
		}

		public void internalFrameClosed(InternalFrameEvent e) {
			doClose();
		}

		public void internalFrameDeactivated(InternalFrameEvent e) {
			doDeactivate();
		}

		public void internalFrameOpened(InternalFrameEvent e) {
			doActivate();
		}

	}
}
