/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows.wizards;

import java.awt.CardLayout;
import java.awt.Component;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.wizard.JWizardPane;
import xxl.applications.gui.wizard.StepDescriptor;
import xxl.applications.gui.wizard.WizardStep;

/**
 *
 * @author  maxim
 */
public class AddStep2 extends StepDescriptor implements WizardStep {
	private StepPanel stepPane;

	/** Creates a new instance of AddStep2 */
	public AddStep2() {
		//stepPane = new AddStep2Panel(this);
		stepPane = new StepPanel();
	}

	public StepDescriptor getStepDescription() {
		return this;
	}

	public Component getComponent() {
		return stepPane;
	}

	public String getContentItem() {
		return "Setting up item";
	}

	public String getStepTitle() {
		return "Set item parameters:";
	}

	public void hidingStep(JWizardPane wizard) {
		Integer i = (Integer) wizard.getValue();
		if (i != null) {
			int option = i.intValue();
			if (option != JWizardPane.PREVIOUS_OPTION && option != JWizardPane.CANCEL_OPTION) {
				Integer item = (Integer) wizard.getProperty(AddWizard.ITEM_TYPE_PROPERTY);
				JPanel panel = stepPane.getPanel(item.toString());
				switch (item.intValue()) {
					case AbstractItem.CLASS_ITEM :
						wizard.putProperty(
							AddWizard.ITEM_CLASS_PROPERTY,
							((ClassPanel) panel).getTypeString());
						break;
					case AbstractItem.OBJECT_ITEM :
						ObjectPanel op = (ObjectPanel) panel;
						wizard.putProperty(AddWizard.ITEM_CLASS_PROPERTY, op.getTypeString());
						wizard.putProperty(AddWizard.ITEM_NAME_PROPERTY, op.getNameString());
						wizard.putProperty(AddWizard.ITEM_CODE_PROPERTY, op.getCodeString());
						break;
					case AbstractItem.PRIMITIVE_ITEM :
						PrimitivePanel pp = (PrimitivePanel) panel;
						wizard.putProperty(AddWizard.ITEM_CLASS_PROPERTY, pp.getTypeString());
						wizard.putProperty(AddWizard.ITEM_NAME_PROPERTY, pp.getNameString());
						wizard.putProperty(AddWizard.ITEM_INITVALUE_PROPERTY, pp.getValueString());
						break;
				}
			}
		}
	}

	public boolean isValid() {
		boolean valid = true;
		//valid &= wizard.getProperty(AddWizard.ITEM_CLASS_PROPERTY) != null;
		//TODO etc...
		return valid;
	}

	public void showingStep(JWizardPane wizard) {
		//this.wizard = wizard;
		Integer i = (Integer) wizard.getProperty(AddWizard.ITEM_TYPE_PROPERTY);
		System.out.println("AddStep2.showingStep() " + i);
		if (i != null) {
			CardLayout layout = (CardLayout) stepPane.getLayout();
			layout.show(stepPane, i.toString());
			
			//TODO null pointer
			if (i.intValue() == AbstractItem.OBJECT_ITEM) {
				ObjectPanel panel = (ObjectPanel) stepPane.cardsMap.get(i.toString());				
				panel.setTypeString(wizard.getProperty(AddWizard.ITEM_CLASS_PROPERTY));
			}
		}

	}

	private final class StepPanel extends JPanel {
		private Map cardsMap;
		public StepPanel() {
			super();
			cardsMap = new Hashtable(3);
			setLayout(new CardLayout());

			String key = String.valueOf(AbstractItem.CLASS_ITEM);
			JPanel panel = new ClassPanel();
			add(panel, key);
			cardsMap.put(key, panel);

			panel = new ObjectPanel();
			key = String.valueOf(AbstractItem.OBJECT_ITEM);
			add(panel, key);
			cardsMap.put(key, panel);

			panel = new PrimitivePanel();
			key = String.valueOf(AbstractItem.PRIMITIVE_ITEM);
			add(panel, key);
			cardsMap.put(key, panel);

			setBorder(BorderFactory.createEmptyBorder(11, 11, 11, 11));
		}

		private JPanel getPanel(String key) {
			return (JPanel) cardsMap.get(key);
		}

	}

}
