/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows.wizards;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.code.PrimitiveCall;
import xxl.applications.gui.code.Type;
import xxl.applications.gui.wizard.JWizardPane;
import xxl.applications.gui.wizard.StepDescriptor;
import xxl.applications.gui.wizard.WizardStep;

/**
 * @author maxim
 */
class AddStep3 extends StepDescriptor implements WizardStep {

	//private AddStep3Panel stepPanel;
	private StepPanel stepPanel;
	private boolean isValid;
	private StringBuffer str_buf;

	public AddStep3() {
		str_buf = new StringBuffer();
	}

	public StepDescriptor getStepDescription() {		
		return this;
	}

	public Component getComponent() {
		if (stepPanel == null)
			//stepPanel = new AddStep3Panel(this);
			stepPanel = new StepPanel();
		return stepPanel;
	}

	public String getContentItem() {
		return "Verifing settings";

	}

	public String getStepTitle() {
		return "Verify entered settings and finish";
	}

	public void hidingStep(JWizardPane wizard) {

	}

	public boolean isValid() {
		return isValid;
	}
		
	/* validiert daten vom assistenten und erstellt eine simple meldung */
	private boolean validate(JWizardPane wizard, StringBuffer strBuf) {
		boolean valid = true;
		//loesche alte meldung
		if (strBuf.length() > 0)
			strBuf.delete(0, strBuf.length() - 1);
		//welche item wird validiert?	
		int item = ((Integer) wizard.getProperty(AddWizard.ITEM_TYPE_PROPERTY)).intValue();
		Type type = null;
		//PrimitiveItem vom Type String?
		String typeStr = (String) wizard.getProperty(AddWizard.ITEM_CLASS_PROPERTY);
		if (typeStr.equals("String")) {
			typeStr = "java.lang.String";
			wizard.putProperty(AddWizard.ITEM_CLASS_PROPERTY, typeStr);
		}
		//name neuer item
		String name = (String) wizard.getProperty(AddWizard.ITEM_NAME_PROPERTY);		
		String value = (String) wizard.getProperty(AddWizard.ITEM_INITVALUE_PROPERTY);
		String typeMsg = "Type OK", nameMsg = "Name OK", valueMsg = "Value OK";
		try {
			type = Type.parse(typeStr);
			if(type.isClass()) {
				Class.forName(typeStr);
			}

		} catch (Exception e) {
			typeMsg = "Could not parse type:";
			valid &= false;
		}
		switch (item) {
			case AbstractItem.CLASS_ITEM :
				strBuf.append(typeMsg);
				strBuf.append(' ');
				strBuf.append(typeStr);
				strBuf.append('\n');
				break;
			case AbstractItem.OBJECT_ITEM :
				if (type != null && (type.isPrimitive() || type.isArray())) {
					typeMsg = "Type is not valid - check if it is not an array or primitive:";
					valid &= false;
				}
				strBuf.append(nameMsg);
				strBuf.append(' ');
				strBuf.append(name);
				strBuf.append('\n');
				strBuf.append(typeMsg);
				strBuf.append(' ');
				strBuf.append(typeStr);
				strBuf.append('\n');
				break;
			case AbstractItem.PRIMITIVE_ITEM :
				if (type != null) {
					if (!type.isPrimitive() && !type.compareTo(PrimitiveCall.STRING_TYPE, false)) {
						typeMsg = "Type is not valid - check if it is not an array or class:";
						valid &= false;
					}
				}
				strBuf.append(nameMsg);
				strBuf.append(' ');
				strBuf.append(name);
				strBuf.append('\n');
				strBuf.append(typeMsg);
				strBuf.append(' ');
				strBuf.append(type);
				strBuf.append('\n');
				if (type != null && !PrimitiveCall.isValueOfType(type, value)) {
					valueMsg = "Value is not assignable to type";
					valid &= false;
				}
				strBuf.append(valueMsg);
				strBuf.append(' ');
				strBuf.append(value);
				strBuf.append('\n');
				break;
		}
		return valid;
	}

	public void showingStep(JWizardPane wizard) {
		isValid = validate(wizard, str_buf);
		((StepPanel) getComponent()).setText(str_buf.toString());
		fireStateChanged();
	}

	private final class StepPanel extends JPanel {
		private JTextArea textArea;

		public StepPanel() {
			super(new BorderLayout());
			textArea = new JTextArea();
			add(new JScrollPane(textArea), BorderLayout.CENTER);
			setBorder(BorderFactory.createEmptyBorder(11, 11, 11, 11));
		}

		public void setText(String text) {
			textArea.setText(text);
		}
	}
}
