/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows.wizards;

import java.awt.Component;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.code.CallVertex;
import xxl.applications.gui.code.ConstructorCall;
import xxl.applications.gui.code.ElementCall;
import xxl.applications.gui.code.MethodCall;
import xxl.applications.gui.code.ObjectItem;
import xxl.applications.gui.graph.ClassGraph;
import xxl.applications.gui.wizard.JWizardPane;
import xxl.applications.gui.wizard.StepDescriptor;
import xxl.applications.gui.wizard.WizardStep;

/**
 *
 * @author  maxim
 */
final class ConstructorFirstStep extends StepDescriptor implements WizardStep {

	private ConstructorFirstStepPanel stepPanel;

	private JWizardPane wizard;

	/** property value this step puts into wizard after completion */
	private Object selectedObject;
	private AbstractItem sourceItem;
	private AbstractItem targetItem;

	/** Creates a new instance of ConstructorFirstStep */
	public ConstructorFirstStep() {
		stepPanel = new ConstructorFirstStepPanel(this);
	}

	public StepDescriptor getStepDescription() {
		return this;
	}

	AbstractItem getSourceItem() {
		return (AbstractItem) wizard.getProperty(ConstructorWizard.SOURCE_ITEM_PROPERTY);
	}

	AbstractItem getTargetItem() {
		return (AbstractItem) wizard.getProperty(ConstructorWizard.TARGET_ITEM_PROPERTY);
	}

	void setSelectedObject(Object o) {
		selectedObject = o;
		fireStateChanged();
	}

	public Component getComponent() {
		return ConstructorFirstStep.this.stepPanel;
	}

	public String getContentItem() {
		return "Select a Method to call";
	}

	public String getStepTitle() {
		return "Call Selection Wizard";
	}

	/** put selected call into wizard */
	public void hidingStep(JWizardPane wizard) {
		int option = ((Integer) wizard.getValue()).intValue();
		if (option != JWizardPane.CANCEL_OPTION) {
			//get the caller
			AbstractItem item = getTargetItem();
			ElementCall call = null;
			//create new call, either constructor
			if (selectedObject instanceof Constructor) {
				Constructor c = (Constructor) selectedObject;				
				call =
					ConstructorCall.create(
						new CallVertex((ClassGraph) item.getGraph()),
						c,
						(ObjectItem) item);
			}
			// or method call
			else if (selectedObject instanceof Method) {
				Method m = (Method) selectedObject;
				call = MethodCall.create(new CallVertex((ClassGraph) item.getGraph()), m, item);
			}
			//put into wizard by specified key, so one can safely cancel here
			if (call != null) {
				wizard.putProperty(ConstructorWizard.SELECTED_CALL_PROPERTY, call);
			}
		}
	}

	/** is always true, since we can process any time without adding any calls */
	public boolean isValid() {
		return true;
	}

	public void showingStep(JWizardPane wizard) {
		ConstructorFirstStep.this.wizard = wizard;
	}

}
