/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows.wizards;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;

import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.code.Type;

/**
 *
 * @author  maxim
 */

final class ConstructorFirstStepPanel extends javax.swing.JPanel {

	/** WizardStep this panel belongs to */
	private ConstructorFirstStep step;
	/** ListModel containitg constructors of the target item */
	private ListModel constructorListModel;
	/** ListModel containing methods of the target item */
	private ListModel methodListModel;
	private final static DefaultListModel NULL_LIST_MODEL = new DefaultListModel();

	/** Creates new form FirstStep */
	public ConstructorFirstStepPanel(ConstructorFirstStep step) {
		this.step = step;
		initComponents();
		methodList.setCellRenderer(new ClassListCellRenderer());
		setBorder(BorderFactory.createEmptyBorder(0, 11, 11, 11));
	}

	/** Creates a model for the methodList containig methods of a given class depending on 
	 *  the type of the source item. If item is <code>ClassItem</code> only static methods are accepted.
	 *  @return <code>DefaultListModel</code> containig methods
	 */
	private DefaultListModel methods() {
		DefaultListModel model = new DefaultListModel();
		Type t = step.getTargetItem().getEnclosedType();
		if (t != null) {
			try {
				Method[] methods = t.toClass().getMethods();
				for (int i = 0; i < methods.length; i++) {
					int modif = methods[i].getModifiers();
					if (step.getTargetItem().getType() == AbstractItem.CLASS_ITEM
						&& !Modifier.isStatic(modif)) {
						continue;
					}
					model.addElement(methods[i]);
				}
			}
			catch (Exception e) {
				System.err.println("FIRSTSTEPPANEL: " + this.getClass() + " " + e);
			}
		}
		return model;
	}

	/** Creates a model for the methodList containig class' constructors. If source Item is of 
	 *  type <code>ClassItem</code> then an empty model is returned.
	 *  @return <code>DefaultListModel</code> containig constructors of the item.
	 */
	private DefaultListModel constructors() {
		DefaultListModel model = new DefaultListModel();
		if (step.getTargetItem().getType() == AbstractItem.CLASS_ITEM)
			return model;
		Type t = step.getTargetItem().getEnclosedType();
		if (t != null) {
			try {
				Constructor[] constructors = t.toClass().getConstructors();
				for (int i = 0; i < constructors.length; i++) {
					model.addElement(constructors[i]);
				}
			}
			catch (Exception e) {
				System.err.println("FIRSTSTEPPANEL: " + this.getClass() + " " + e);
			}
		}
		return model;
	}

	/** lazy initalisation of methodList */
	private ListModel getConstructorModel() {
		if (constructorListModel == null)
			constructorListModel = constructors();
		return constructorListModel;
	}

	/** lazy initalisation of methodList */
	private ListModel getMethodModel() {
		if (methodListModel == null) {
			methodListModel = methods();
		}
		return methodListModel;
	}

	/** constructs a list model containig parameter types of acc */
	private ListModel methodParamsModel(AccessibleObject acc) {
		DefaultListModel model = NULL_LIST_MODEL;
		Class[] params = null;
		if (acc instanceof Method) {
			params = ((Method) acc).getParameterTypes();
		}
		else if (acc instanceof Constructor) {
			params = ((Constructor) acc).getParameterTypes();
		}
		if (params != null) {
			model = new DefaultListModel();
			for (int i = 0; i < params.length; i++) {
				model.addElement(Type.createFromClass(params[i]).getFullString());
			}
		}

		return model;
	}

	/** method called by methodsRadioButton action listener */
	private void performMethodRButtonClick() {
		methodList.setModel(getMethodModel());
		// methodList.setSelectedIndex(0);
		returnTypeTextField.setText(null);
		paramList.setModel(NULL_LIST_MODEL);
	}

	/** method called by constructorsRadioButton action listener */
	private void performConstructorRButtonClick() {
		methodList.setModel(getConstructorModel());
		//methodList.setSelectedIndex(0);
		returnTypeTextField.setText(null);
		paramList.setModel(NULL_LIST_MODEL);
	}

	/** method called on selection changes in methodList */
	private void performListSelection(ListSelectionEvent e) {
		AccessibleObject acc = (AccessibleObject) methodList.getSelectedValue();
		if (acc != null) {
			paramList.setModel(methodParamsModel(acc));
			if (acc instanceof Method) {				
				returnTypeTextField.setText(Type.createFromClass(((Method)acc).getReturnType()).getFullString());
			}
		}
		//hold the selected method
		step.setSelectedObject(acc);

	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	private void initComponents() { //GEN-BEGIN:initComponents
		java.awt.GridBagConstraints gridBagConstraints;

		showButtonGroup = new javax.swing.ButtonGroup();
		headLabel = new javax.swing.JLabel();
		methodsLabel = new javax.swing.JLabel();
		jPanel1 = new javax.swing.JPanel();
		returnTypeTextField = new javax.swing.JTextField();
		returnLabel = new javax.swing.JLabel();
		paramTypeLabel = new javax.swing.JLabel();
		paramList = new javax.swing.JList();
		detailsLabel = new javax.swing.JLabel();
		methodsScrollPane = new javax.swing.JScrollPane();
		methodList = new javax.swing.JList();
		constructorsRadioButton = new javax.swing.JRadioButton();
		methodsRadioButton = new javax.swing.JRadioButton();

		setLayout(new java.awt.GridBagLayout());

		headLabel.setText("Select constructor or method to set");
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridwidth = 3;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 11, 0);
		add(headLabel, gridBagConstraints);

		methodsLabel.setLabelFor(methodsScrollPane);
		methodsLabel.setText("Methods:");
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 11);
		add(methodsLabel, gridBagConstraints);

		jPanel1.setLayout(new java.awt.GridBagLayout());

		returnTypeTextField.setEditable(false);
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints.weightx = 0.5;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 2, 0);
		jPanel1.add(returnTypeTextField, gridBagConstraints);

		returnLabel.setLabelFor(returnTypeTextField);
		returnLabel.setText("Return Type:");
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 2, 2);
		jPanel1.add(returnLabel, gridBagConstraints);

		paramTypeLabel.setText("Parameter Type(s):");
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 2, 2);
		jPanel1.add(paramTypeLabel, gridBagConstraints);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints.weightx = 0.5;
		gridBagConstraints.weighty = 1.0;
		jPanel1.add(paramList, gridBagConstraints);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 2;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 11, 0);
		add(jPanel1, gridBagConstraints);

		detailsLabel.setText("Method Details:");
		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 2;
		gridBagConstraints.gridy = 1;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
		add(detailsLabel, gridBagConstraints);

		methodList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		methodList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
			public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
				methodListValueChanged(evt);
			}
		});

		methodsScrollPane.setViewportView(methodList);

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 2;
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 1.0;
		gridBagConstraints.insets = new java.awt.Insets(0, 0, 11, 11);
		add(methodsScrollPane, gridBagConstraints);

		constructorsRadioButton.setText("show Constructors");
		showButtonGroup.add(constructorsRadioButton);
		constructorsRadioButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				constructorsRadioButtonActionPerformed(evt);
			}
		});

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		add(constructorsRadioButton, gridBagConstraints);

		methodsRadioButton.setText("show Methods");
		showButtonGroup.add(methodsRadioButton);
		methodsRadioButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				methodsRadioButtonActionPerformed(evt);
			}
		});

		gridBagConstraints = new java.awt.GridBagConstraints();
		gridBagConstraints.gridx = 1;
		gridBagConstraints.gridy = 3;
		gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
		add(methodsRadioButton, gridBagConstraints);

	} //GEN-END:initComponents

	private void constructorsRadioButtonActionPerformed(
		java.awt.event.ActionEvent evt) { //GEN-FIRST:event_constructorsRadioButtonActionPerformed
		performConstructorRButtonClick();
	} //GEN-LAST:event_constructorsRadioButtonActionPerformed

	private void methodListValueChanged(
		javax.swing.event.ListSelectionEvent evt) { //GEN-FIRST:event_methodListValueChanged
		performListSelection(evt);
	} //GEN-LAST:event_methodListValueChanged

	private void methodsRadioButtonActionPerformed(
		java.awt.event.ActionEvent evt) { //GEN-FIRST:event_methodsRadioButtonActionPerformed
		performMethodRButtonClick();
	} //GEN-LAST:event_methodsRadioButtonActionPerformed

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JRadioButton constructorsRadioButton;
	private javax.swing.JLabel detailsLabel;
	private javax.swing.JLabel headLabel;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JList methodList;
	private javax.swing.JLabel methodsLabel;
	private javax.swing.JRadioButton methodsRadioButton;
	private javax.swing.JScrollPane methodsScrollPane;
	private javax.swing.JList paramList;
	private javax.swing.JLabel paramTypeLabel;
	private javax.swing.JLabel returnLabel;
	private javax.swing.JTextField returnTypeTextField;
	private javax.swing.ButtonGroup showButtonGroup;
	// End of variables declaration//GEN-END:variables
}
