/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows.wizards;

import java.awt.Component;

import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.code.ConstructorCall;
import xxl.applications.gui.code.ElementCall;
import xxl.applications.gui.code.ObjectItem;
import xxl.applications.gui.wizard.JWizardPane;
import xxl.applications.gui.wizard.StepDescriptor;
import xxl.applications.gui.wizard.WizardStep;

/**
 *
 * @author  maxim
 */
final class ConstructorSecondStep extends StepDescriptor implements WizardStep {

	private ConstructorSecondStepPanel step;
	private JWizardPane wizard;
	private AbstractItem sourceItem;
	private AbstractItem targetItem;

	/** Creates a new instance of ConstructorSecondStep */
	public ConstructorSecondStep() {
		step = new ConstructorSecondStepPanel(this);
	}

	public StepDescriptor getStepDescription() {
		return this;
	}

	AbstractItem getSourceItem() {
		if (sourceItem == null)
			sourceItem = (AbstractItem) wizard.getProperty(ConstructorWizard.SOURCE_ITEM_PROPERTY);
		return sourceItem;

	}

	AbstractItem getTargetItem() {
		if (targetItem == null)
			targetItem = (AbstractItem) wizard.getProperty(ConstructorWizard.TARGET_ITEM_PROPERTY);
		return targetItem;
	}

	public Component getComponent() {
		return ConstructorSecondStep.this.step;
	}

	public String getContentItem() {
		return "Set parameters";
	}

	public String getStepTitle() {
		return "Set Parameters";
	}

	public void hidingStep(JWizardPane wizard) {
	}

	public boolean isValid() {
		return true;
	}

	public void showingStep(JWizardPane wizard) {
		ConstructorSecondStep.this.wizard = wizard;
		ElementCall call = (ElementCall) wizard.getProperty(ConstructorWizard.SELECTED_CALL_PROPERTY);
		if (call != null) {
			AbstractItem item = getTargetItem();
			if (call.getType() == ElementCall.CONSTRUCTOR_CALL) {
				if (item.getType() == AbstractItem.OBJECT_ITEM) {
					System.out.println("ConstructorSecondStep.showingStep() CON " + call);
					((ObjectItem) item).setConstructor((ConstructorCall) call);
				}
			}
			else if (call.getType() == ElementCall.METHOD_CALL) {
				System.out.println("ConstructorSecondStep.showingStep() MET " + call);				
				getTargetItem().addChildVertex(call.getCallVertex());
			}
			wizard.putProperty(ConstructorWizard.SELECTED_CALL_PROPERTY, null);
		}
		step.performInit();
	}

}
