/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.windows.wizards;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import xxl.applications.gui.code.AbstractItem;
import xxl.applications.gui.code.CallVertex;
import xxl.applications.gui.code.ElementCall;
import xxl.applications.gui.code.Type;
import xxl.applications.gui.graph.Vertex;
/**
 *
 * @author  maxim
 */
final class ValuesTableModel extends AbstractTableModel {

	/** ColumnNames, the first one should be hidden by renderer */
	private static final String[] NAMES = { "Type", "Object" };
	private ArrayList elements;

	public ValuesTableModel(AbstractItem item) {
		super();
		Vertex[] children = item.getChildren();
		elements = new ArrayList(children.length + 1);
		ArrayList row = new ArrayList(2);
		// TODO add the item as element itself?
		/*row.add("");
		row.add(item.getEnclosedType());
		row.add(item.getName());
		elements.add(row);       
		 */
		for (int i = 0; i < children.length; i++) {
			row = new ArrayList(3);
			ElementCall call = ((CallVertex) children[i]).getCall();
			row.add(call);
			row.add(call.getCallType());
			row.add(call.getIdentifier());
			elements.add(row);
		}
	}

	public int getColumnCount() {
		return NAMES.length;
	}

	public int getRowCount() {
		return elements.size();
	}
	
	public Class getColumnClass(int columnIndex) {
		return Type.class;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		ArrayList row = (ArrayList) elements.get(rowIndex);
		return row.get(columnIndex + 1);
	}

	ElementCall getCallAt(int rowIndex) {
		ArrayList row = (ArrayList) elements.get(rowIndex);
		return (ElementCall) row.get(0);

	}

	public String getColumnName(int column) {
		return NAMES[column];
	}

	//there is no need for cell editing
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

}
