/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.gui.wizard;

import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.JComponent;
import javax.swing.SwingConstants;

/**
 * @author maxim
 */
class ImagedPanel extends JComponent {

	Image image = null;
	int alignment = SwingConstants.CENTER;
	boolean visibleImage = true;

	public ImagedPanel(Image im) {
		setImage(im);
		setLayout(new BorderLayout());
		setOpaque(true);		
	}

	protected void paintComponent(Graphics graphics) {
		graphics.setColor(getBackground());
		graphics.fillRect(0, 0, getWidth(), getHeight());
		if ((image != null) && visibleImage) {
			int x = 0, y = 0;
			switch (alignment) {
				case SwingConstants.CENTER :
					x = (getWidth() - image.getWidth(null)) / 2;
					y = (getHeight() - image.getHeight(null)) / 2;
					break;
				case SwingConstants.NORTH :
					x = (getWidth() - image.getWidth(null)) / 2;
					break;
				case SwingConstants.SOUTH :
					x = (getWidth() - image.getWidth(null)) / 2;
					y = getHeight() - image.getHeight(null);
					break;
				case SwingConstants.WEST :
					y = (getHeight() - image.getHeight(null)) / 2;
					break;
				case SwingConstants.EAST :
					x = getWidth() - image.getWidth(null);
					y = (getHeight() - image.getHeight(null)) / 2;
					break;
				case SwingConstants.NORTH_WEST :
					break;
				case SwingConstants.NORTH_EAST :
					x = getWidth() - image.getWidth(null);
					break;
				case SwingConstants.SOUTH_WEST :
					y = getHeight() - image.getHeight(null);
					break;
				case SwingConstants.SOUTH_EAST :
					x = getWidth() - image.getWidth(null);
					y = getHeight() - image.getHeight(null);
					break;
			}
			graphics.drawImage(image, x, y, null);			
		}
	}

	public void setImageAlignment(int align) {
		if (alignment != align) {
			alignment = align;
			repaint();
		}
	}

	public int getImageAlignment() {
		return alignment;
	}

	public void setImage(Image image) {
		if (this.image != image) {
			this.image = image;
			if (visibleImage && isVisible()) {
				repaint();
			}
		}
	}

	public Image getImage() {
		return image;
	}

	public boolean getVisibleImage() {
		return visibleImage;
	}

	public void setVisibleImage(boolean flag) {
		if (visibleImage != flag) {
			visibleImage = flag;
			if (isVisible())
				repaint();
		}
	}
	
}

