/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package xxl.applications.gui.wizard;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.awt.List;
import java.awt.Window;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.UIDefaults;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import xxl.applications.gui.wizard.plaf.WizardPaneUI;
import xxl.applications.gui.wizard.plaf.basic.BasicWizardPaneUI;

public class JWizardPane extends JComponent {

	private static final boolean DEBUG = false;

	public final static int SELECT_BY_FONT = 0, SELECT_BY_COLOR = 1;

	private StepIterator iterator;
	private Object customizedObject;
	private Map properties;
	private ChangeListener cl;
	private PropertyChangeListener pcl;
	private JWizardPane wizard = this;

	/** Bounds property name for value. */
	public static final String VALUE_PROPERTY = "value";
	/** Bounds property name for defaultButton. */
	public static final String DEFAULT_BUTTON_PROPERTY = "defaultButton";
	/** Bound property name for */
	public static final String LEFT_OPTIONS_PROPERTY = "leftOptions";
	/** Bound property name for */
	public static final String RIGHT_OPTIONS_PROPERTY = "rightOptions";
	//** Bound property name for */
	public static final String CONTENT_PANEL_PROPERTY = "contentPanel";
	//** Bound property name for */
	public static final String STEP_PANEL_PROPERTY = "stepPanel";

	//** Bound property name for current step */
	public static final String CURRENT_STEP_PROPERTY = "currentStep";

	//      //** Bound property name for content image*/
	//      public static final String CONTENT_IMAGE_PROPERTY = "contentImage";
	//      //** Bound property name for content visibility image*/
	//      public static final String CONTENT_VISIBLE_IMAGE_PROPERTY = "contentVisibleImage";
	//      //** Bound property name for content image alignment*/
	//      public static final String CONTENT_IMAGE_ALIGNMENT_PROPERTY = "contentImageAlignment";
	//      //** Bound property name for content width*/
	//      public static final String CONTENT_WIDTH_PROPERTY = "contentWidth";
	//      //** Bound property name for content title color*/
	//      public static final String CONTENT_TITLE_COLOR_PROPERTY = "contentTitleColor";
	//      //** Bound property name for content steps font color*/
	//      public static final String CONTENT_STEPS_COLOR_PROPERTY = "contentStepsColor";
	//      //** Bound property name for content selection type*/
	//      public static final String CONTENT_SELECTION_TYPE_PROPERTY = "contentSelectionType";
	//      //** Bound property name for content selection color*/
	//      public static final String CONTENT_SELECTION_COLOR_PROPERTY = "contentSelectionColor";
	/** Bound property name for step title text*/
	public static final String STEP_TITLE_TEXT_PROPERTY = "stepTitleText";
	/** Bound property name for step title color*/
	public static final String STEP_TITLE_COLOR_PROPERTY = "stepTitleColor";

	/** */
	transient protected WizardStep currentStep;
	/** */
	transient protected Object[] leftOptions;
	/** */
	transient protected Object[] rightOptions;
	/** */
	transient protected ContentPanel contentPanel;
	/** */
	transient protected Component stepPanel = null;
	/** */
	transient protected Object value;
	/** */
	transient protected Object defaultButton;
	/** */
	transient protected int contentWidth;
	//      /** */
	//      transient protected Color contentTitleColor;
	//      /** */
	//      transient protected Color contentStepsColor;
	//      /** */
	//      transient protected int contentSelectionType;
	//      /** */
	//      transient protected Color contentSelectionColor;

	/** */
	transient protected Color stepTitleColor;
	/** */
	transient protected String stepTitleText;

	/** "Previous" button option.*/
	public static final int PREVIOUS_OPTION = 1;
	public static final int PREVIOUS_OPTION_DISABLED = 2;
	/** "Next" button option.*/
	public static final int NEXT_OPTION = 3;
	public static final int NEXT_OPTION_DISABLED = 4;
	/** "Finish" button option.*/
	public static final int FINISH_OPTION = 5;
	public static final int FINISH_OPTION_DISABLED = 6;
	/** "Last" button option.*/
	public static final int LAST_OPTION = 7;
	public static final int LAST_OPTION_DISABLED = 8;
	/** "Cancel" button option.*/
	public static final int CANCEL_OPTION = 9;
	/** Windows is closed by user*/
	public static final int CLOSED_OPTION = 10;
	/** "Help" button option */
	public static final int HELP_OPTION = 11;

	/** Create wizard for a sequence of panels, with settings
	 * defaulted to <CODE>this</CODE>.
	 *
	 * @param iterator iterator over all {@link WizardStep}s that can appear in the wizard
	 */
	public JWizardPane(StepIterator iterator) {
		this(iterator, null);
	}

	/** Create wizard for a sequence of panels, passing some settings to the panels.
	* @param iterator iterator over all {@link WizardStep}s that can appear in the wizard
	* @param setter the settings to provide to the panels (may be any data understood by them)
	*/
	public JWizardPane(StepIterator iterator, Object setter) {

		this.iterator = iterator;
		this.customizedObject = setter;

		this.setUI(BasicWizardPaneUI.createUI(this));

		setContentPanel(new ContentPanel());

		cl = new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				if (DEBUG) {
					System.out.println("JWizardPane.JWizardPane() ChangeEvent is here.");
				} // end of if ()
				updateState();
			}
		};
		pcl = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if (DEBUG) {
					System.out.println("JWizardPane.JWizardPane() PropertyChangeEvent is here: " + e.getPropertyName());
				} // end of if ()
				if (e.getPropertyName().equals(VALUE_PROPERTY)) {
					if (e.getNewValue() instanceof Integer) {
						int val = ((Integer) e.getNewValue()).intValue();
						if (DEBUG) {
							System.out.println(val);
							System.out.println("----");
						}
						switch (val) {
							case JWizardPane.PREVIOUS_OPTION :
								backAction();
								break;
							case JWizardPane.NEXT_OPTION :
								nextAction();
								break;
							case JWizardPane.LAST_OPTION :
								lastAction();
								break;
							case JWizardPane.FINISH_OPTION :
								finishAction();
								break;
							case JWizardPane.HELP_OPTION :
								break;
							case JWizardPane.CANCEL_OPTION :
								cancelAction();
								break;
						} // end of switch ()
					}
				}

				if (e.getPropertyName().equals(CURRENT_STEP_PROPERTY)) {

					WizardStep ows = (WizardStep) e.getOldValue();
					if (ows != null) {
						ows.getStepDescription().removeChangeListener(cl);
						ows.getStepDescription().hidingStep(wizard);
					}

					WizardStep nws = (WizardStep) e.getNewValue();
					nws.getStepDescription().addChangeListener(cl);
					nws.getStepDescription().showingStep(wizard);
				}
			}
		};

		this.addPropertyChangeListener(pcl);
		iterator.reset();
		updateState();
		if (DEBUG) {
			System.out.println("JWizardPane-JWizardPane()");
		} // end of if ()
	}

	/** Create a new wizard from a fixed list of panels, passing some settings to the panels.
	 * @param steps the panels to use
	 * @param settings the settings to pass to panels, or <code>null</code>
	 * @see #JWizardPane(StepIterator, Object)
	 */
	public JWizardPane(WizardStep[] steps, Object settings) {
		this(new ArrayIterator(steps), settings);
	}

	/** Create a new wizard from a fixed list of panels with settings
	* defaulted to <CODE>this</CODE>.
	*
	* @param steps the panels to use
	* @see #JWizardPane(StepIterator, Object)
	*/
	public JWizardPane(WizardStep[] steps) {
		// passing CLOSE_PREVENTER which is treated especially
		this(steps, null); //, CLOSE_PREVENTER);
	}

	//*********************************************************************************************************

	/** Action called when Back button is pressed
	 *
	 * @see #nextAction()
	 * @see #lastAction() 
	 * @see #cancelAction()
	 * @see #finishAction()
	 */
	public void backAction() {

		if (DEBUG) {
			System.out.println("BACK is pressed.");
		} // end of if ()

		setValue(new Integer(PREVIOUS_OPTION));
		iterator.previousStep();
		updateState();
	}

	/** Action called when Next button is pressed
	 *
	 * @see #backAction()
	 * @see #lastAction() 
	 * @see #cancelAction()
	 * @see #finishAction()
	 */
	public void nextAction() {

		if (DEBUG) {
			System.out.println("NEXT is pressed.");
		} // end of if ()

		setValue(new Integer(NEXT_OPTION));
		iterator.nextStep();
		updateState();
	}

	/** Action called when Lst button is pressed
	 *
	 * @see #nextAction()
	 * @see #backAction()
	 * @see #cancelAction()
	 * @see #finishAction()
	 */
	public void lastAction() {

		if (DEBUG) {
			System.out.println("LAST is pressed.");
		} // end of 

		setValue(new Integer(LAST_OPTION));
		iterator.lastStep();
		updateState();
	}

	/** Action called when Finish button is pressed
	 *
	 * @see #nextAction()
	 * @see #backAction()
	 * @see #lastAction() 
	 * @see #cancelAction()
	 */
	public void finishAction() {

		if (DEBUG) {
			System.out.println("FINISH is pressed.");
		} // end of if ()

		setValue(new Integer(FINISH_OPTION));
	}

	/** Action called when Cancel button is pressed
	 *
	 * @see #nextAction()
	 * @see #backAction()
	 * @see #lastAction() 
	 * @see #finishAction()
	 */
	public void cancelAction() {

		if (DEBUG) {
			System.out.println("CANCEL is pressed.");
		} // end of if ()

		setValue(new Integer(CANCEL_OPTION));
	}

	private void updateContent() {

		// update items
		// build actual list of steps 
		List items = new List();
		while (iterator.hasPrevious())
			iterator.previousStep();
		while (iterator.hasNext()) {
			items.add(iterator.getCurrent().getStepDescription().getContentItem());
			iterator.nextStep();
		}

		//....
		items.add(iterator.getCurrent().getStepDescription().getContentItem());

		while (iterator.hasPrevious() && (iterator.getCurrent() != this.currentStep))
			iterator.previousStep();
		String[] c = items.getItems();
		// ----

		contentPanel.setContentItems(c);
		// update selected item
		String s = iterator.getCurrent().getStepDescription().getContentItem();
		for (int i = 0; i < c.length; i++) {
			if (s.equals(c[i])) {
				contentPanel.setSelectedIndex(i);
				break;
			}
		}
	}

	private void updateButtons() {
		setLeftOptions(getUpdatedLeftOptions().toArray());
		setRightOptions(getUpdatedRightOptions().toArray());
	}

	/**
	 *
	 */
	public Vector getUpdatedLeftOptions() {

		Vector lb = new Vector();
		int b;

		// BACK button
		b = iterator.hasPrevious() ? PREVIOUS_OPTION : PREVIOUS_OPTION_DISABLED;
		lb.add(new Integer(b));

		// NEXT button
		if ((iterator.hasNext() && currentStep.getStepDescription().isValid())) {
			b = NEXT_OPTION;
			defaultButton = new Integer(NEXT_OPTION);
		} else {
			b = NEXT_OPTION_DISABLED;
			defaultButton = new Integer(NEXT_OPTION_DISABLED);
			// REMARK fix enabled next button by invalid WizardPanel, done by Gigrod
		}

		lb.add(new Integer(b));

		// LAST or FINISH button
		if (iterator.canFinish()) {
			if (iterator.hasNext()) {
				b = (currentStep.getStepDescription().isValid()) ? LAST_OPTION : LAST_OPTION_DISABLED;
			} else {
				b = (currentStep.getStepDescription().isValid()) ? FINISH_OPTION : FINISH_OPTION_DISABLED;
				if (b == FINISH_OPTION)
					defaultButton = new Integer(FINISH_OPTION);
			}
		}
		lb.add(new Integer(b));

		return lb;
	}

	/**
	 *
	 */
	public Vector getUpdatedRightOptions() {

		Vector rb = new Vector();
		int b;

		// CANCEL button
		b = CANCEL_OPTION;
		rb.add(new Integer(b));

		// HELP button
		if (currentStep.getStepDescription().getHelpAction() instanceof ActionListener) {
			b = HELP_OPTION;
			rb.add(new Integer(b));
		}

		return rb;
	}

	private synchronized void updateState() {

		setCurrentStep(iterator.getCurrent());
		setStepPanel(currentStep.getStepDescription().getComponent());
		setStepTitleText(currentStep.getStepDescription().getStepTitle());
		updateContent();
		updateButtons();
		selectDefaultButton();
	}

	/** Return object which is customized by this wizard
	 */
	public synchronized Object getCustomizedObject() {
		return customizedObject;
	}

	public synchronized void putProperty(final String name, Object value) {
		Object oldValue = null;
		if (properties == null) {			
			properties = new HashMap(7);
		}
		oldValue = properties.get(name);		
		properties.put(name, value);

	}

	public synchronized Object getProperty(String name) {		
		return properties == null ? null : properties.get(name);
	}

	public void setCurrentStep(WizardStep ws) {
		if (currentStep != ws) {
			WizardStep ows = currentStep;
			currentStep = ws;
			firePropertyChange(CURRENT_STEP_PROPERTY, ows, ws);
		}
	}

	public WizardStep getCurrentStep() {
		return currentStep;
	}

	public void setLeftOptions(Object[] options) {
		Object[] oldOptions = leftOptions;

		leftOptions = options;
		firePropertyChange(LEFT_OPTIONS_PROPERTY, oldOptions, leftOptions);
	}

	public Object[] getLeftOptions() {
		return leftOptions;
	}

	public void setRightOptions(Object[] options) {
		Object[] oldOptions = rightOptions;

		rightOptions = options;
		firePropertyChange(RIGHT_OPTIONS_PROPERTY, oldOptions, rightOptions);
	}

	public Object[] getRightOptions() {
		return rightOptions;
	}

	public void setContentPanel(JComponent content) {
		JComponent oldPanel = contentPanel;

		contentPanel = (ContentPanel) content;
		firePropertyChange(CONTENT_PANEL_PROPERTY, oldPanel, contentPanel);
	}

	public JComponent getContentPanel() {
		return contentPanel;
	}

	public void setStepPanel(Component step) {
		Component oldStep = stepPanel;

		stepPanel = step;
		firePropertyChange(STEP_PANEL_PROPERTY, oldStep, stepPanel);
	}

	public Component getStepPanel() {
		return stepPanel;
	}

	/** Setter for step title text*/
	public void setStepTitleText(String title) {
		String oldTitle = stepTitleText;
		stepTitleText = title;
		firePropertyChange(STEP_TITLE_TEXT_PROPERTY, oldTitle, stepTitleText);
	}

	/** Getter for step title text */
	public String getStepTitleText() {
		return stepTitleText;
	}

	/** Setter for step title color */
	public void setStepTitleColor(Color c) {
		Color old = stepTitleColor;
		stepTitleColor = c;
		firePropertyChange(STEP_TITLE_COLOR_PROPERTY, old, stepTitleColor);
	}

	/** Getter for step title color */
	public Color getStepTitleColor() {
		//return contentPanel.getTitleColor();
		return stepTitleColor;
	}

	/** Getter for value*/
	public Object getValue() {
		return value;
	}

	/** Setter for value*/
	public void setValue(Object newValue) {
		Object oldValue = value;
		value = newValue;
		firePropertyChange(VALUE_PROPERTY, oldValue, value);
	}

	/** Setter for contnts image*/
	public void setContentImage(Image im) {
		contentPanel.setImage(im);
	}

	/** Getter for contnts image*/
	public Image getContentImage() {
		return contentPanel.getImage();
	}

	/** Getter for content image alignment*/
	public int getContentImageAlignment() {
		return contentPanel.getImageAlignment();
	}

	/** Setter for content image alignment*/
	public void setContentImageAlignment(int al) {
		contentPanel.setImageAlignment(al);
	}

	/** Getter for content image visilility*/
	public boolean getContentVisibleImage() {
		return contentPanel.getVisibleImage();
	}

	/** Setter for content image visilility*/
	public void setContentVisibleImage(boolean vi) {
		contentPanel.setVisibleImage(vi);
	}

	public void setContentTitleColor(Color c) {
		contentPanel.setTitleColor(c);
	}

	public void setContentForeground(Color c) {
		contentPanel.setForeground(c);
	}

	public void setContentSelectionType(int val) {
		contentPanel.setSelectMetod(val);
	}

	public void setContentSelectionColor(Color c) {
		contentPanel.setSelectedColor(c);
	}

	public void setContentNumbered(boolean val) {
		contentPanel.setContentNumbered(val);
	}

	public void setDefaultButton(Object newValue) {
		Object oldValue = defaultButton;
		defaultButton = newValue;
		firePropertyChange(DEFAULT_BUTTON_PROPERTY, oldValue, defaultButton);
	}

	public Object getDefaultButton() {
		return defaultButton;
	}

	/**
	 * Requests that the initial value be selected, which will set
	 * focus to the initial value. This method
	 * should be invoked after the window containing the option pane
	 * is made visible.
	 */
	public void selectDefaultButton() {
		WizardPaneUI ui = getUI();
		if (ui != null) {
			ui.selectDefaultButton(this);
		}
	}

	/**
	 * Sets the UI object which implements the L&F for this component.
	 *
	 * @param ui  the <code>OptionPaneUI</code> L&F object
	 * @see UIDefaults#getUI
	 * @beaninfo
	 *       bound: true
	 *      hidden: true
	 * description: The UI object that implements the optionpane's LookAndFeel
	 */
	public void setUI(WizardPaneUI ui) {
		if ((WizardPaneUI) this.ui != ui) {
			super.setUI(ui);
			invalidate();
		}
	}

	/**
	 * Returns the UI object which implements the L&F for this component.
	 *
	 * @return the <code>WizardPaneUI</code> object
	 */
	public WizardPaneUI getUI() {
		return (WizardPaneUI) ui;
	}

	/**
	 * Returns the name of the UI class that implements the
	 * L&F for this component.
	 *
	 * @return the string "OptionPaneUI"
	 * @see JComponent#getUIClassID
	 * @see UIDefaults#getUI
	 */
	public String getUIClassID() {
		return "WizardPaneUI"; //uiClassID;
	}

	public JDialog createDialog(final Component parentComponent, final String title) {

		final JDialog dialog;
				
		//******
		Window window = getWindowForComponent(parentComponent);
		//= JOptionPane.getFrameForComponent(parentComponent);

		if (window instanceof Frame) {
			dialog = new JDialog((Frame) window, title, true);
		} else {
			dialog = new JDialog((Dialog) window, title, true);
		}

		Container contentPane = dialog.getContentPane();

		contentPane.setLayout(new BorderLayout());
		contentPane.add(this, BorderLayout.CENTER);
		dialog.pack();
		dialog.setLocationRelativeTo(parentComponent);
		dialog.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent we) {
				setValue(new Integer(CLOSED_OPTION));
			}

			public void windowActivated(WindowEvent we) {

				if (DEBUG) {
					System.out.println("windowActivated()");
				} // end of if ()

				selectDefaultButton();
			}
		});

		addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {

				if (dialog.isVisible()
					&& event.getSource() == JWizardPane.this
					&& (event.getPropertyName().equals(VALUE_PROPERTY))) {

					JWizardPane p = (JWizardPane) event.getSource();
					if ((p.getValue().equals(new Integer(JWizardPane.CANCEL_OPTION))
						|| (p.getValue().equals(new Integer(JWizardPane.FINISH_OPTION))))) {
						dialog.setVisible(false);
						dialog.dispose();
					}
				}
			}
		});
		return dialog;
	}

	static Window getWindowForComponent(Component parentComponent) {
		if (parentComponent == null)
			return JOptionPane.getFrameForComponent(parentComponent);
		//return getRootFrame();

		if (parentComponent instanceof Frame || parentComponent instanceof Dialog)
			return (Window) parentComponent;
		return JWizardPane.getWindowForComponent(parentComponent.getParent());
	}

	/** Special iterator that works on an array of <code>WizardStep</code>s.
	*/
	private static class ArrayIterator implements StepIterator {
		/** Array of items.
		*/
		private WizardStep[] steps;

		/** Index into the array
		*/
		private int index;

		//          /* Default constructor. It's here to allow subclasses to
		//          * be serializable easily. Panel initialization is done
		//          * through initializePanels() protected method. */
		//          public ArrayStepIterator () {
		//              steps = initializeSteps();
		//              index = 0;
		//          }

		/** Construct an iterator.
		* @param array the list of panels to use
		*/
		public ArrayIterator(WizardStep[] array) {
			steps = array;
			index = 0;
		}

		//          /** Allows subclasses to initialize their arrays of panels when
		//          * constructed using default constructor.
		//          * (for example during deserialization.
		//          * Default implementation returns empty array. */
		//          protected WizardStep[] initializeSteps () {
		//              return new WizardStep[0];
		//          }

		/* The current WizardStep.
		*/
		public WizardStep getCurrent() {
			return steps[index];
		}

		//          /* Current name of the panel */
		//          public String name () {
		//              Object[] args = {
		//                  new Integer (index + 1),
		//                  new Integer (panels.length)
		//              };
		//              MessageFormat mf = new MessageFormat (NbBundle.getBundle (WizardDescriptor.class).getString ("CTL_ArrayIteratorName"));
		//              return mf.format (args);
		//          }

		/* Is there a next panel?
		* @return true if so
		*/
		public boolean hasNext() {
			return index < steps.length - 1;
		}

		/* Is there a previous panel?
		* @return true if so
		*/
		public boolean hasPrevious() {
			return index > 0;
		}

		/* Moves to the next panel.
		* @exception NoSuchElementException if the panel does not exist
		*/
		public synchronized void nextStep() {
			if (index + 1 == steps.length)
				throw new java.util.NoSuchElementException();
			index++;
		}

		/* Moves to previous panel.
		* @exception NoSuchElementException if the panel does not exist
		*/
		public synchronized void previousStep() {
			if (index == 0)
				throw new java.util.NoSuchElementException();
			index--;
		}

		/* Ignores the listener, there are no changes in order of panels.
		*/
		public void addChangeListener(ChangeListener l) {
		}

		/* Ignored.
		*/
		public void removeChangeListener(ChangeListener l) {
		}

		/** Resets this iterator to initial state.
		* Called by subclasses when they need re-initialization of the iterator.
		*/
		public void reset() {
			index = 0;
		}

		/** Can wizard finish in current WizardStep? */
		public boolean canFinish() {
			return true;
		}

		public void lastStep() {
			index = steps.length - 1;
		}
	}
}
