/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.indexStructures;

import java.math.BigInteger;

import xxl.core.functions.Function;
import xxl.core.indexStructures.BPlusTree.KeyRange;

/** 
 * An example for {@link xxl.core.indexStructures.BPlusTree.KeyRange} with BigInteger values.
 */
public class BigIntegerKeyRange extends KeyRange {
	
	/**
	 * An factory function providing a <tt>BigIntegerKeyRange</tt> if invoked
	 * with two {@link java.math.BigInteger}.
	 */
	public static final Function<BigInteger, BigIntegerKeyRange> FACTORY_FUNCTION = new Function<BigInteger, BigIntegerKeyRange>() {
		@Override
		public BigIntegerKeyRange invoke(BigInteger min, BigInteger max) {
			return new BigIntegerKeyRange(min, max);
		}
	};

	/** Constructs a new <tt>BigIntegerKeyRange</tt>.
	 * 
	 * @param min the BigInteger at the beginning of the range
	 * @param max the BigInteger at the end of the range
	 */		
	public BigIntegerKeyRange(BigInteger min, BigInteger max) {
		super(min, max);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() {
		return new BigIntegerKeyRange((BigInteger)minBound(), (BigInteger)minBound());
	}
}
