/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.indexStructures;

import java.math.BigInteger;

import xxl.core.functions.Function;
import xxl.core.indexStructures.Separator;

/** An example for {@link xxl.core.indexStructures.Separator} with BigInteger values.
 */
class BigIntegerSeparator extends Separator {

	/**
	 * An factory function providing an <tt>BigIntegerSeparator</tt> if invoked
	 * with an {@link java.math.BigInteger}.
	 */
	public static final Function<BigInteger, BigIntegerSeparator> FACTORY_FUNCTION = new Function<BigInteger, BigIntegerSeparator>() {
		@Override
		public BigIntegerSeparator invoke(BigInteger key) {
			return new BigIntegerSeparator(key);
		}
	};
	
	/** Constructs a new <tt>BigIntegerSeparator</tt>.
	 * 
	 * @param key the BigInteger used for separating
	 */	
	public BigIntegerSeparator(BigInteger key) {
		super(key);
	}
		
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() {
		if (!isDefinite())
			return new BigIntegerSeparator(null);
		return new BigIntegerSeparator((BigInteger)sepValue);
	}
}
