/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.indexStructures;

import java.util.List;

import xxl.core.functions.Function;
import xxl.core.indexStructures.MVBTree.MVRegion;
import xxl.core.indexStructures.MVBTree.MVSeparator;

public class IntMVSeparator extends MVSeparator {
	
	public static final Function FACTORY_FUNCTION= new Function<Object,Object>() {
		public Object invoke() {throw new UnsupportedOperationException();}
		public Object invoke(Object arg1) {throw new UnsupportedOperationException();}
		public Object invoke(Object arg1,Object arg2) {throw new UnsupportedOperationException();}
		public Object invoke(List<? extends Object> arguments) {
			if(arguments.size()!=3) throw new IllegalArgumentException();
			IntVersion insertVersion=(IntVersion)arguments.get(0);
			IntVersion deleteVersion=(IntVersion)arguments.get(1);
			Integer min= (Integer)arguments.get(2);
			return new IntMVSeparator(insertVersion,deleteVersion,min);	
		}
	};
		
	public IntMVSeparator(IntVersion insertVersion, IntVersion deleteVersion, Integer min) {
		super(insertVersion, deleteVersion, min);	
	}
		
	public IntMVSeparator(IntVersion insertVersion, Integer min) {
		this(insertVersion, null, min);
	}
		
	public Object clone() {
		Integer min=new Integer(((Integer)sepValue()).intValue());
		return new IntMVSeparator((IntVersion)insertVersion(), (IntVersion)deleteVersion(), min);
	}
}

class IntMVRegion extends MVRegion {
	
	public static final Function FACTORY_FUNCTION= new Function<Object,Object>() {
		public Object invoke() {throw new UnsupportedOperationException();}
		public Object invoke(Object arg1) {throw new UnsupportedOperationException();}
		public Object invoke(Object arg1,Object arg2) {throw new UnsupportedOperationException();}
		public Object invoke(List<? extends Object> arguments) {
			if(arguments.size()!=4) throw new IllegalArgumentException();
			IntVersion beginVersion=(IntVersion)arguments.get(0);
			IntVersion endVersion=(IntVersion)arguments.get(1);
			Integer min= (Integer)arguments.get(2);
			Integer max= (Integer)arguments.get(3);
			return new IntMVRegion(beginVersion, endVersion, min, max);	
		}
	};

	public IntMVRegion(IntVersion beginVersion, IntVersion endVersion, Integer min, Integer max) {
		super(beginVersion, endVersion, min, max);	
	}
				
	public Object clone() {
		IntVersion begin=(IntVersion)beginVersion().clone();
		IntVersion end=isDead()?(IntVersion)endVersion().clone(): null;
		Integer min=new Integer(((Integer)minBound()).intValue());
		Integer max=this.isDefinite()? new Integer(((Integer)maxBound()).intValue()): null;
		return new IntMVRegion(begin, end, min, max);
	}
}

