/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.applications.indexStructures;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.List;

import xxl.core.functions.Function;
import xxl.core.indexStructures.MVBTree.MVRegion;
import xxl.core.indexStructures.MVBTree.MVSeparator;
import xxl.core.indexStructures.MVBTree.Version;
import xxl.core.io.converters.LongConverter;
import xxl.core.io.converters.MeasuredConverter;

public class MVBTreeHelper {
	public static class LongVersion implements Version {
		public static final MeasuredConverter<LongVersion> DEFAULT_CONVERTER = new MeasuredConverter<LongVersion>() {
			public LongVersion read(DataInput input, LongVersion obj) throws IOException{
				long t = LongConverter.DEFAULT_INSTANCE.readLong(input);
				return new LongVersion(t);
			}

			public void write(DataOutput output, LongVersion obj) throws IOException{
				LongConverter.DEFAULT_INSTANCE.writeLong(output, obj.time);
			}
			
			public int getMaxObjectSize() {
				return LongConverter.SIZE;
			}
		};
		
		protected long time;
				
		public LongVersion(long t) {
			time = t;
		}
		
		public int compareTo(Object object) {
			LongVersion version = (LongVersion)object;
			if(time > version.time) return 1;
			if(time < version.time) return -1;
			return 0;
		}
			
		public boolean equals(Object object) {
			if(!(object instanceof LongVersion)) return false;
			return compareTo(object) == 0;
		}
			
		public Object clone() {
			return new LongVersion(time);
		}
			
		public int hashCode() {
			return (int)time;
		}
			
		public String toString() {
			return ""+time;
		}
	}

	public static class LongMVSeparator extends MVSeparator {
		public static final Function FACTORY_FUNCTION = new Function<Object,Object>() {
			public Object invoke() { throw new UnsupportedOperationException(); }

			public Object invoke(Object arg1) { throw new UnsupportedOperationException(); }
			
			public Object invoke(Object arg1, Object arg2) { throw new UnsupportedOperationException(); }

			public Object invoke(List<? extends Object> arguments) {
				if(arguments.size() !=3 ) throw new IllegalArgumentException();
				LongVersion insertVersion = (LongVersion)arguments.get(0);
				LongVersion deleteVersion = (LongVersion)arguments.get(1);
				Integer min = (Integer)arguments.get(2);
				return new LongMVSeparator(insertVersion, deleteVersion, min);	
			}
		};
			
		public LongMVSeparator(LongVersion insertVersion, LongVersion deleteVersion, Integer min) {
			super(insertVersion, deleteVersion, min);	
		}
			
		public LongMVSeparator(LongVersion insertVersion, Integer min) {
			this(insertVersion, null, min);
		}
			
		public Object clone() {
			Integer min = new Integer(((Integer)sepValue()).intValue());
			return new LongMVSeparator((LongVersion)insertVersion(), (LongVersion)deleteVersion(), min);
		}
	}

	public static class LongMVRegion extends MVRegion {
		public static final Function FACTORY_FUNCTION = new Function<Object,Object>() {
			public Object invoke() {throw new UnsupportedOperationException();}
			public Object invoke(Object arg1) {throw new UnsupportedOperationException();}
			public Object invoke(Object arg1,Object arg2) {throw new UnsupportedOperationException();}
			public Object invoke(List<? extends Object> arguments) {
				if(arguments.size()!=4) throw new IllegalArgumentException();
				LongVersion beginVersion=(LongVersion)arguments.get(0);
				LongVersion endVersion=(LongVersion)arguments.get(1);
				Integer min= (Integer)arguments.get(2);
				Integer max= (Integer)arguments.get(3);
				return new LongMVRegion(beginVersion, endVersion, min, max);	
			}
		};

		public LongMVRegion(LongVersion beginVersion, LongVersion endVersion, Integer min, Integer max) {
			super(beginVersion, endVersion, min, max);	
		}
					
		public Object clone() {
			LongVersion begin=(LongVersion)beginVersion().clone();
			LongVersion end=isDead()?(LongVersion)endVersion().clone(): null;
			Integer min=new Integer(((Integer)minBound()).intValue());
			Integer max=this.isDefinite()? new Integer(((Integer)maxBound()).intValue()): null;
			return new LongMVRegion(begin, end, min, max);
		}
	}
}
