package xxl.connectivity.ctbot;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;

import xxl.core.pipes.elements.TemporalObject;
import xxl.core.pipes.elements.TimeInterval;
import xxl.core.pipes.processors.SourceProcessor;
import xxl.core.pipes.queryGraph.QueryExecutor;
import xxl.core.pipes.sinks.Printer;
import xxl.core.pipes.sources.AbstractSource;
import xxl.core.pipes.sources.Source;

public class CtBotConnector {
	
	protected static byte[] welcome = { 62, 87, 78, 0, 0, 0, 0, 0, 0, 0, 60 };
	protected static byte[] command =  { 62, 114, 79, 28, 0, 0, 0, 0, 0, 0, 60, 98, 111, 116, 95,  99, 111, 109, 109,  97, 110, 100, 95, 109, 97, 114, 98, 117, 114, 103, 0, -1, 0, 0, 0, -1, 0, 0, 0 };
	protected static byte[] sendChar = { 62, 114, 79, 24, 0, 0, 0, 0, 0, 0, 60, 98, 111, 116, 95, 100, 105, 115, 112, 108,  97, 121,                                      0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0 }; 
	
	public class CtBotProcessor extends SourceProcessor {

		protected ServerSocket serverSocket;
		protected Socket socket;
		protected volatile boolean connected;
		protected DataOutputStream out;
		protected DataInputStream in;
		
		public CtBotProcessor() {
			super(0);
			connected = false;
		}

		protected void connect(boolean re) {
			if (!re)
				System.out.println("Connecting to bot "+name);
			else
				System.out.println("Reconnecting to bot "+name);
			try {				
				socket = serverSocket.accept();
				out = new DataOutputStream(new BufferedOutputStream(socket.getOutputStream()));
				in = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
				out.write(welcome);
				out.flush();
//				byte[] oneByte = new byte[1];
//				do {
//					while(oneByte[0]!='>') 
//						in.read(oneByte);
//					in.read(oneByte);
//					System.out.println(oneByte[0]);
//				} while (oneByte[0]!=62);
//				byte[] nineBytes = new byte[9];
//				in.read(nineBytes);
			} catch (IOException e) {
				System.err.println("An error occured while connecting to "+name);
				terminate();
			}		
			connected = true;
			System.out.println("Connected to bot "+name);
		}
		
		@Override
		protected void onStart() {
			super.onStart();
			try {
				serverSocket = new ServerSocket(port);
			} catch (IOException e) {
				System.err.println("Cannot create server socket (port used by other application?)");
			}
			connect(false);
		}
		
		@Override
		public void process() {
			byte[] oneByte = new byte[1];
			byte[] tenBytes = new byte[10];
			try {
				// Read data
				while(oneByte[0]!='>')					
					in.read(oneByte);
				in.read(tenBytes);
				
//				for (int i=0; i<10; i++)
//					System.out.print(tenBytes[i]+" ");
//				for (int i=0; i<10; i++)
//					System.out.print(((char)tenBytes[i])+" ");				
//				System.out.println();
				
			} catch (IOException e) {
				System.out.println("Lost connection to "+name);
				connected = false;
				connect(true);
				return;
			}
			long now = System.currentTimeMillis();
			char command = (char)tenBytes[0];
			List<CtBotSource> cs = botSources.get(command);
			if (cs!=null) {
				for(CtBotSource source : cs) {
					if (source.isClosed()) 
						continue;
					Integer value  = new Integer((short) ((tenBytes[source.left?4:6] & 0xff) << 8 | (tenBytes[source.left?3:5] & 0xff )));
					TimeInterval timeInterval = new TimeInterval(now, now+1);
					TemporalObject<Integer> temporalObject = new TemporalObject<Integer>(value, timeInterval);
					source.transfer(temporalObject);
				}
			}
			checkTermination();
		}
		
		public boolean sendCommand(byte com, byte param) {
			if (!connected)
				return false;
			try {
				byte[] toSend = new byte[command.length];
				System.arraycopy(command, 0, toSend, 0, command.length);				
				toSend[31]=com;
				toSend[35]=param;
				out.write(toSend);
				out.flush();
			} catch (IOException e) {
				System.err.println("An error occured while sending bot command");
				return false;
			}
			return true;
		}
		
		public boolean sendChar(byte x, byte y, char c) {
			if (!connected)
				return false;
			try {
				byte[] toSend = new byte[sendChar.length];
				System.arraycopy(sendChar, 0, toSend, 0, sendChar.length);
				toSend[23]=x;
				toSend[27]=y;
				toSend[31]=(byte)c;
				
//				for (int i=0; i<sendChar.length; i++)
//					System.out.print(sendChar[i]+", ");
//				System.out.println();
				
				out.write(toSend);
				out.flush();
			} catch (IOException e) {
				System.err.println("An error occured while sending display character");
				return false;
			}
			return true;			
		}
		
	}
	
	public class CtBotSource extends AbstractSource {

		protected boolean left;
		
		public CtBotSource(SourceProcessor bp, boolean left) {
			super(bp);
			this.left = left;
		}
		
		@Override
		public Object next() throws NoSuchElementException {
			throw new UnsupportedOperationException();
		}
		
		@Override
		public void close() {}
		
	}
	
	protected int port;
	protected String name;
	protected CtBotProcessor botProcessor;
	protected HashMap<Character,List<CtBotSource>> botSources;
	
	public CtBotConnector(String name, int port) {
		this.name = name;
		this.port = port;
		this.botProcessor = new CtBotProcessor();
		this.botProcessor.start();
		this.botSources = new HashMap<Character,List<CtBotSource>>();
	}
	
	public String getName() {
		return name;
	}
	
	public CtBotSource getSource(char command, boolean left) {
		List<CtBotSource> cs;
		if (botSources.containsKey(command))
			cs = botSources.get(command);
		else 
			botSources.put(command, cs = new ArrayList<CtBotSource>());
		CtBotSource source = new CtBotSource(botProcessor, left);
		cs.add(source);
		return source;
	}
	
	public boolean sendCommand(byte com, byte param) {
		return botProcessor.sendCommand(com, param);
	}
	
	public boolean sendCommand(byte[] com) {
		return botProcessor.sendCommand(com[0], com[1]);
	}
	
	public boolean sendChar(byte x, byte y, char c) {
		return botProcessor.sendChar(x, y, c);
	}
	
	public static void main(String[] args) {
		CtBotConnector con = new CtBotConnector("TestBot",10001);
		Source s = con.getSource('L', true);
		Printer p = new Printer(s);
		QueryExecutor qe = new QueryExecutor();
		qe.registerQuery(p);
		qe.startAllQueries();		
	}

}
