/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/
package xxl.connectivity.ctbot;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

public class CtBotGui {
	
	static class Command {
		protected String name;
		protected byte[] raw;
		public Command(String name, byte[] raw) {
			this.name = name;
			this.raw=raw;
		}		
	}
	
	protected static List<Command> commands;
	static {
		commands = new ArrayList<Command>();
		commands.add(new Command("Stop Moving", new byte[]{ 1, 1 }));
		commands.add(new Command("Start Moving", new byte[]{ 1, 0 }));
		commands.add(new Command("Start Blinking", new byte[]{ 2, 4 }));
		commands.add(new Command("Stop Blinking", new byte[]{ 2, 0 }));
		commands.add(new Command("Sensor Display", new byte[]{ 3, 0 }));
		commands.add(new Command("Static Display", new byte[]{ 3, 1 }));
		commands.add(new Command("Remote Display", new byte[]{ 3, 2}));
	}

	protected JFrame frame;
	protected HashMap<String,CtBotConnector> connectors;
	
	public CtBotGui(CtBotConnector [] bots) {
		if (bots==null)
			throw new IllegalArgumentException();
		frame = new JFrame("c't-Bots");
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		frame.getContentPane().setLayout(new GridLayout(1,0));
		connectors = new HashMap<String,CtBotConnector>();
		addBot(null);			
		for (int i=0; i<bots.length; i++)
			addBot(bots[i]);
		frame.setVisible(true);
	}
	
	
	public static class BoundedJTextField extends JTextField {
		
	    public BoundedJTextField(int cols) {
	        super(cols);
	    }
	
	    protected Document createDefaultModel() {
		      return new BoundedDocument();
	    }
	
	    static class BoundedDocument extends PlainDocument {
	        public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
		          if (str == null) 
		        	  return;
		          if (offs<20)
		        	  super.insertString(offs, str, a);
		      }
	    }
	}
	
	public void addBot(final CtBotConnector connector) {
		if (connector!=null) 
			connectors.put(connector.getName(), connector);
		JPanel botPanel = new JPanel();
		botPanel.setLayout(new GridLayout(0,1));
		JLabel label = new JLabel(connector==null ? "All Bots" : connector.getName(), SwingConstants.CENTER);
		botPanel.add(label);		
		for (Command command : commands) {
			final JButton button = new JButton(command.name);
			final byte[] com = command.raw;
			button.addActionListener(new ActionListener() {				
				public void actionPerformed(ActionEvent e) {
					if (connector!=null)
						connector.sendCommand(com);
					else
						for (CtBotConnector c : connectors.values())
							c.sendCommand(com);
				}
			});
			botPanel.add(button);
		}	
		JButton sendText = new JButton("Send Text");
		sendText.addActionListener(
			new ActionListener() {				
				public void actionPerformed(ActionEvent e) {
					final JDialog dialog = new JDialog(frame, "Send Text");
					dialog.setSize(215, 150);
					dialog.setLayout(new GridLayout(5,1));
					final JTextField[] lines = new JTextField[4];
					final JButton sendButton = new JButton("Senden");
					for (int i=0; i<4; i++) {
						final int _i = i;
						lines[i] = new BoundedJTextField(20);	
						lines[i].setFont(new Font("Monospaced", Font.BOLD, 16));
						dialog.add(lines[i]);						
						lines[i].addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								if(_i<3) 
									lines[_i+1].requestFocusInWindow();
								else 
									sendButton.doClick();									
							}							
						});
					}										
					sendButton.addActionListener(new ActionListener() {				
						public void actionPerformed(ActionEvent e) {
							for (byte i=0; i<4; i++) {
								String line = lines[i].getText();
								for (byte j=0; j<20; j++) {
									char ch = j<line.length() ? line.charAt(j) : ' ';
									if (connector!=null)
										connector.sendChar(i, j, ch);
									else
										for (CtBotConnector con : connectors.values())
											con.sendChar(i, j, ch);
								}
								
							}
							dialog.dispose();
						}
					});
					dialog.add(sendButton,BorderLayout.SOUTH);
					dialog.setLocationRelativeTo(null);
					dialog.setVisible(true);
				}
			}
		);
		botPanel.add(sendText);
		frame.setSize(120*(connectors.size()+1), 30*(3+commands.size()));
		frame.getContentPane().add(botPanel);	
		frame.validate();
	}
	
	public static void main(String[] args) {
		try {
			javax.swing.UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} 
		catch (Exception e) {}
		CtBotConnector[] bots = new CtBotConnector[] {
			 new CtBotConnector("Marvin", 10001), 
			 new CtBotConnector("Data",   10002)
		}; 
		new CtBotGui(bots);
	}
	
}
