/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can 
subscribe to our mailing-list. Send an email to 
    
    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body. 
*/
package xxl.connectivity.jts.io;

import java.io.File;
import java.util.Iterator;

import xxl.core.cursors.mappers.Aggregator;
import xxl.core.io.converters.Converter;
import xxl.core.math.functions.AggregationFunction;
import xxl.core.spatial.geometries.Geometry2D;
import xxl.core.spatial.rectangles.Rectangle;
import xxl.core.spatial.rectangles.Rectangles;

/** This Aggregator incrementally computes the minimum bounding rectangle
 *  for an iterator of {@link Geometry2D Geometry} objects.
 *
 * @see xxl.core.cursors.mappers.Aggregator
 */
public class UniverseAggregator extends Aggregator<Geometry2D, Rectangle>{
	
	/** The aggregate function that computes the actual MBR.
	*/
	public static class UniverseFunction extends AggregationFunction<Geometry2D, Rectangle> {

		public Rectangle invoke(Rectangle aggregate, Geometry2D next){

			if(aggregate == null) aggregate = next.getMBR();
				else aggregate.union(next.getMBR());
			
			return aggregate;
		}
	}

	/** Creates a new UniverseAggregator.
	 *
	 * @param iterator input iterator containing geometries 
	 */
	public UniverseAggregator(Iterator<Geometry2D> iterator){
		super( iterator, new UniverseFunction());
	}

	/**
	 * USE CASE: compute universe for a file containing geometries
	 * and write aggregated mbr to a file "file-name.universe".
	 *
	 * @param args args[0] file-name, args[1] dimensionality of the data	 
	 */
	public static void main(String[] args){
		if(args.length == 0){
			System.out.println(	"Computes the universe of the geometries given by the specified wkb file and saves the result in the file 'inputFilename.universe'! ");
			System.out.println(	"usage: java xxl.connectivity.jts.util.UniverseAggregator file");
			return;
		}							
		
		if(args[0].toLowerCase().endsWith(".wkb")){
			Converter<xxl.connectivity.jts.Geometry2DAdapter> converter = Geometry2DConverter.DEFAULT_INSTANCE;
			Iterator<Geometry2D> iterator = Geometry2DFileIO.read(converter, new File(args[0]));
			
			Rectangle universe = new UniverseAggregator(iterator).last();
			System.out.println("Universe: "+universe);
			Rectangles.writeSingletonRectangle(new File(args[0]+".universe"),universe);
		} else
			System.out.println("Don't know how to read file: "+args[0]);			
	}
}
