/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
						Head of the Database Research Group
						Department of Mathematics and Computer Science
						University of Marburg
						Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

	http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

	xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.collections.sweepAreas;

import java.util.Iterator;

import xxl.core.pipes.elements.PNObject;
import xxl.core.predicates.Predicate;

/**
 * Specialized SweepArea for use with the positive negative stream operators.
 * Note the modified {@link #reorganize(Object, int) reorganize} and
 * {@link #expire(Object, int) expire} methods and that PNSweepArea does not support
 * remove predicates. 
 */
public class PNSweepArea<I> extends ImplementorBasedSweepArea<PNObject<I>> {
	
	public PNSweepArea(SweepAreaImplementor<PNObject<I>> impl, int ID, Predicate<? super PNObject<I>> queryPredicate, int dim) {
		super(impl, ID, false, queryPredicate, dim);
	}
	
	public PNSweepArea(SweepAreaImplementor<PNObject<I>> impl, int ID, Predicate<? super PNObject<I>>[] queryPredicates) {
		super(impl, ID, false, queryPredicates);
	}
	
	public PNSweepArea(SweepAreaImplementor<PNObject<I>> impl, int ID, int dim) {
		super(impl, ID, dim);
	}

	/**
	 * If <code>pno</code> is a negative PNObject, it is removed from the implementor.
	 * The implementor has to ensure that a value-equivalent element to the given element <code>pno</code>
	 * is deleted. 
	 */
	@Override
	public void reorganize(PNObject<I> pno, int ID){
		if (pno.isNegative() && impl.size() > 0)
			if (!impl.remove(pno)) 
				throw new IllegalStateException("Cannot remove a value-equivalent element for "+pno+" from the SweepArea.");
	}

	/**
	 * PNSweepArea does not support expire!
	 * @throws UnsupportedOperationException always!
	 * */
	@Override
	public final Iterator<PNObject<I>> expire(PNObject<I> pno, int ID) throws UnsupportedOperationException{
		throw new UnsupportedOperationException("PNJoinSA does not support the method expire().");
	}
	
	@Override
	public String toString() {
		StringBuffer buf= new StringBuffer();
		buf.append(this.getClass().toString()+": ");
		Iterator<PNObject<I>> it = this.iterator();
		if (!it.hasNext())
			 buf.append(" no content");
		for (; it.hasNext();) {
			 buf.append(it.next()+" ");			
		}
		return buf.toString();
	}
}

