/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.cursors.wrappers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import xxl.core.cursors.Cursor;
import xxl.core.cursors.Cursors;
import xxl.core.cursors.DecoratorCursor;

public class AutomaticCloseCursor extends DecoratorCursor {
	boolean closeWasCalled = false;
	public AutomaticCloseCursor(Cursor cursor) {
		super(cursor);
	}
	public void close() {
		// do not call close again if it is called below.
		if (closeWasCalled) {
			cursor.close();
			closeWasCalled = true;
		}
	}
	public boolean hasNext() {
		boolean hn = cursor.hasNext();
		if (!hn) {
			cursor.close();
			closeWasCalled = true;
		}
		return hn;
	}
	public static void main(String[] args) {
		List l = new ArrayList();
		l.add("1");
		l.add("2");
		l.add("3");
		l.add("4");
		l.add("5");
		
		Iterator it = l.iterator();
		Cursor c = new DecoratorCursor(it) { //new IteratorCursor(it));
			public void close() {
				System.out.println("Close called");
			}
		};
		
		Cursors.println(new AutomaticCloseCursor(c));
	}
}
