/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/
package xxl.core.pipes.metaData;

import xxl.core.functions.Function;
import xxl.core.math.functions.AggregationFunction;
import xxl.core.math.statistics.parametric.aggregates.LastN;

public class AggregationOverWindowMetaDataIdentifier<P,R> extends FunctionApplicationMetaDataIdentifier {

	public static class AggregationOnWindowApplicationFunction<A,B> extends Function<A[],B> {

		AggregationFunction<A, B> aggregationFunction;
		
		public AggregationOnWindowApplicationFunction(Class<? extends AggregationFunction<A, B>> aggregationFunctionClass) {
			try {
				aggregationFunction = aggregationFunctionClass.newInstance();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
		
		@Override
		public B invoke(A[] objects) {			
			B agg = null;
			for (A object : objects) 
				if (object != null && !Double.isNaN((Double) object))
					agg = aggregationFunction.invoke(agg, object);
			return agg;
		}
		
	}
	
	String aggregationFunctionName;
	
	public AggregationOverWindowMetaDataIdentifier(Class<? extends AggregationFunction<P, R>> aggregationFunctionClass, int windowSize, int minResultWindowSize, Object basisMetaDataIdentifier) {
		super(
			AggregationOnWindowApplicationFunction.class, new Object[] { aggregationFunctionClass },
			new AggregationMetaDataIdentifier(LastN.class, new Object[] { new Integer(windowSize), new Integer(minResultWindowSize) }, basisMetaDataIdentifier)
		);
		String fs = aggregationFunctionClass.toString();
		if (fs.contains("."))
			fs = fs.substring(fs.lastIndexOf('.')+1);
		if (fs.contains("$"))
			fs = fs.substring(fs.lastIndexOf('$')+1);
		aggregationFunctionName = fs;
	}

	public AggregationOverWindowMetaDataIdentifier(Class<? extends AggregationFunction<P, R>> aggregationFunctionClass, int windowSize, Object basisMetaDataIdentifier) {
		this(aggregationFunctionClass, windowSize, windowSize, basisMetaDataIdentifier);
	}
	
	@Override
	public String toString() {
		return aggregationFunctionName+"("+basisMetaDataIdentifier+")";
	}

}
