/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/
package xxl.core.pipes.metaData;

import xxl.core.functions.Function;
import xxl.core.pipes.queryGraph.Node;
import xxl.core.pipes.queryGraph.Graph.GraphMetaDataManagement;
import xxl.core.util.metaData.MetaDataHandler;
import xxl.core.util.metaData.MetaDataManagement;

public class GraphAggregationMetaDataIdentifier extends ComplexMetaDataIdentifier {
	
	protected int updatePeriod;
	
	public GraphAggregationMetaDataIdentifier (Object basisMetaDataIdentifier, int updatePeriod) {
		super(basisMetaDataIdentifier);
		this.updatePeriod = updatePeriod;
	}
	
	public Object getBasisMetaDataIdentifier() {
		return basisMetaDataIdentifier;
	}

	public Object newInstance(final MetaDataManagement mdm) {
		final GraphMetaDataManagement metaDataManagement;
		try {
			metaDataManagement = (GraphMetaDataManagement)mdm;
		}
		catch (ClassCastException e) {
			return null;
		}
		Function<Object, Double> function = new Function<Object, Double>() {
			public Double invoke() {
				synchronized(metaDataManagement) {
					double sum = 0;
					for (Node n : metaDataManagement.getNodes()) {
						try {
							if (n.getMetaData().contains(basisMetaDataIdentifier)) {
								MetaDataHandler mdh = (MetaDataHandler)(n.getMetaData().get(basisMetaDataIdentifier));
								Number num = (Number)mdh.getMetaData(); 
								if (num!=null)
									sum += num.doubleValue();
							}
						}
						catch (ClassCastException e) {
							e.printStackTrace();
						}
					}
					return sum;
				}
			}
		};
		return new PeriodicEvaluationMetaDataHandler<Double>(this, function, updatePeriod);
	}

	@Override
	public String toString() {
		return "GraphAggregation("+basisMetaDataIdentifier+")";
	}
	
	@Override
	public int hashCode() {
		return basisMetaDataIdentifier.hashCode();
	}
	
	@Override
	public boolean equals(Object o) {
		try {
			GraphAggregationMetaDataIdentifier g = (GraphAggregationMetaDataIdentifier)o;
			return basisMetaDataIdentifier.equals(g.basisMetaDataIdentifier)
				&& updatePeriod == g.updatePeriod;
		}
		catch(ClassCastException e) {
			return false;
		}
	}
	
}
