package xxl.core.pipes.operators.identities;

import static xxl.core.pipes.operators.CostModelMetaDataIdentifiers.D_ESTIMATION;

import java.util.ArrayList;

import xxl.core.collections.queues.ListQueue;
import xxl.core.collections.queues.Queue;
import xxl.core.functions.Function;
import xxl.core.pipes.elements.TemporalObject;
import xxl.core.pipes.metaData.DefaultMetaDataHandler;
import xxl.core.pipes.metaData.TriggeredEvaluationMetaDataHandler;
import xxl.core.pipes.sources.Source;

public class TemporalBufferPipe<I> extends BufferPipe<TemporalObject<I>> {
	
	class TemporalBufferPipeMetaDataManagement extends AbstractBufferPipeMetaDataManagement {
		
		
		public boolean addMetaData(Object metaDataIdentifier) {
			if (metaDataIdentifier.equals(FIRST_ELEMENT_START)) {
				metaData.add(metaDataIdentifier,new DefaultMetaDataHandler<Long>(this,metaDataIdentifier,
						new Function<Object, Long>() {
							@Override
							public Long invoke() {
								processingRLock.lock();
								try {
									if (!buffer.isEmpty()) {
										TemporalObject to = (TemporalObject) buffer.peek();
										return to.getTimeInterval().getStart();
									}
									return Long.MAX_VALUE;	
								}
								finally {
									processingRLock.unlock();
								}
							}
						}));
				return true;
			}
			if (metaDataIdentifier.equals(D_ESTIMATION)) {
				metaData.add(metaDataIdentifier, new TriggeredEvaluationMetaDataHandler<Double>(this, metaDataIdentifier, Double.class,new Function<Object,Double>() {			
						@Override
						public Double invoke() {
							if (isClosed || isDone) 
								return Double.NaN;
							return (Double) getMetaDataFragmentFromSource(0, D_ESTIMATION, Double.NaN);
						}
					})
				);
				return true;
			}
			return super.addMetaData(metaDataIdentifier);
		}
		
	}
	
	public TemporalBufferPipe(Queue<TemporalObject<I>> buffer, int batchSize, int objectSize) {
		super(buffer, batchSize, objectSize);
	}
	
	public TemporalBufferPipe(Source<? extends TemporalObject<I>>[] sources, int[] sourceIDs, Queue<TemporalObject<I>> buffer, int batchSize, int objectSize) {
		super(sources, sourceIDs, buffer, batchSize, objectSize);
	}
	
	public TemporalBufferPipe(Source<? extends TemporalObject<I>> source, int sourceID, Queue<TemporalObject<I>> buffer, int batchSize, int objectSize) {
		super(source, sourceID, buffer, batchSize, objectSize);
	}
	
	public TemporalBufferPipe(Source<? extends TemporalObject<I>> source, int batchSize, int objectSize) {
		super(source, batchSize, objectSize);
	}
	
	public TemporalBufferPipe(Source<? extends TemporalObject<I>> source, int objectSize) {
		super(source, objectSize);
	}
	
	public TemporalBufferPipe(Source<? extends TemporalObject<I>> source) {
		super(source);
	}
	
	public TemporalBufferPipe() {
		super(new ListQueue<TemporalObject<I>>(new ArrayList<TemporalObject<I>>()),150,0);
	}

	public void createMetaDataManagement() {
		if (metaDataManagement==null)
			metaDataManagement = new TemporalBufferPipeMetaDataManagement();
	}
	
}

