/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.pipes.queryGraph;

import java.util.ArrayList;

/**
 * This class represents a global query graph, which consists of several subgraphs.
 * Each subgraph is executed by an thread.
 * 
 * To avoid deadlocks an order for the subgraphs is defined.
 * (position in list)
 */

public class GlobalGraph {
    
    /**
     * list of subgraphs (determines order)
     */
    private ArrayList<Graph> subgraphs;
    
    /**
     * new instance
     * 
     * @param sgs list of subgraphs
     */
    public GlobalGraph(ArrayList<Graph> sgs) {
        subgraphs = sgs;
    }
    
    /**
     * acquire read locks for all the subgraphs in the given list
     * @param toAcquire subgraphs to be locked
     */
    public void acquireRLocks(ArrayList<Graph> toAcquire) {
        if (!subgraphs.containsAll(toAcquire))
            throw new IllegalStateException("Can't get hold of all locks");
        for (int i=0;i<subgraphs.size();i++)
            if (toAcquire.contains(subgraphs.get(i)))
                subgraphs.get(i).RLock.lock();
    }
    
    /**
     * acquire write locks for all the subgraphs in the given list
     * @param toAcquire subgraphs to be locked
     */
    public void acquireWLocks(ArrayList<Graph> toAcquire) {
        if (!subgraphs.containsAll(toAcquire))
            throw new IllegalStateException("Can't get hold of all locks");
        for (int i=0;i<subgraphs.size();i++)
            if (toAcquire.contains(subgraphs.get(i)))
                subgraphs.get(i).WLock.lock();
    }
        
    /**
     * release read locks for all the subgraphs in the given list
     * @param toAcquire subgraphs to be released
     */
    public void releaseRLocks(ArrayList<Graph> toAcquire) {
        if (!subgraphs.containsAll(toAcquire))
            throw new IllegalStateException("Can't get hold of all locks");
        for (int i=subgraphs.size()-1;i<=0;i--)
            if (toAcquire.contains(subgraphs.get(i)))
                subgraphs.get(i).RLock.unlock();
    }
    
    /**
     * release write locks for all the subgraphs in the given list
     * @param toAcquire sungraphs to be locked
     */
    public void releaseWLocks(ArrayList<Graph> toAcquire) {
        if (!subgraphs.containsAll(toAcquire))
            throw new IllegalStateException("Can't get hold of all locks");
            for (int i=subgraphs.size()-1;i<=0;i--)
                if (toAcquire.contains(subgraphs.get(i)))
                    subgraphs.get(i).WLock.unlock();
    }
   
}
