/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
						Head of the Database Research Group
						Department of Mathematics and Computer Science
						University of Marburg
						Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

	http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

	xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.pipes.scheduler;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * Pre-implementation for scheduling strategies.
 * @param <T> 
 * 
 * @since 1.1
 */
public abstract class AbstractStrategy<T extends Controllable> implements Strategy<T> {
	
    /**
     * next controllable to be chosen
     */
	protected T next;
	
    /**
     * list of controllables, which can be chosen
     */
	protected ArrayList<T> controllables = new ArrayList<T>();
		
    /**
     * new instance
     * 
     * @param nodes list of controllables controlled by this strategy
     */
	public AbstractStrategy(T[] nodes) {
		for (int i = 0; i < nodes.length; i++)
			register(nodes[i]);
	}
		
    /**
     * new instance
     */
	public AbstractStrategy() {}
	
    /**
     * @see xxl.core.pipes.scheduler.Strategy#register(xxl.core.pipes.scheduler.Controllable)
     */
	public void register(T node) {
		if (node == null)
			throw new IllegalArgumentException("No valid node for scheduling specified.");
        if (!controllables.contains(node))
            this.controllables.add(node);
	}
	
    /**
     * @see xxl.core.pipes.scheduler.Strategy#deregister(xxl.core.pipes.scheduler.Controllable)
     */
	public void deregister(T node) {
		if (node == null)
			throw new IllegalArgumentException("No valid node for scheduling specified.");
		this.controllables.remove(node);
	}
	
    /**
     * @see xxl.core.pipes.scheduler.Strategy#getControllables()
     */
	public Iterator<T> getControllables() {
		return controllables.iterator();
	}
	
    /**
     * @see xxl.core.pipes.scheduler.Strategy#getNoOfControllables()
     */
	public int getNoOfControllables() {
		return controllables.size();
	}
	
    /**
     * @see xxl.core.pipes.scheduler.Strategy#computeNext()
     */
	public abstract boolean computeNext();
	
    /**
     * @see xxl.core.pipes.scheduler.Strategy#next()
     */
	public T next() {
		return next;
	}
}
