/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/
package xxl.core.pipes.scheduler;

import java.util.Comparator;
import java.util.HashMap;

/**
 * This comparator uses a hashmap to compare two objects if type I.
 * For each key in the hashmap exists an Object of type Number.
 * If two objects of type I are compared the corresponding objects of 
 * type O are used for the comparision.
 */
public class HashComparator<I,O extends Number> implements Comparator<I> {
	
    /**
     * map: I->O
     */
	private HashMap<I,O> map;
	
    /**
     * new instance
     * 
     * @param m used hashmap
     */
	public HashComparator(HashMap<I,O> m) {
		map = m;
	}

    /**
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
	public int compare(I arg0, I arg1) {
		Number prio0 = (map.get(arg0)!=null) ? map.get(arg0) : Double.MAX_VALUE;
		Number prio1 = (map.get(arg1)!=null) ? map.get(arg1) : Double.MAX_VALUE;
		return (prio0.doubleValue() == prio1.doubleValue()) ? 0 : (prio0.doubleValue() < prio1.doubleValue()) ? -1 : 1;
	}
}