/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/
package xxl.core.pipes.scheduler;

import xxl.core.pipes.operators.identities.AbstractBufferPipe;

/**
 * naive strategy processor: it chooses every time it is called a new controllable and doesn't use
 * and updateperiod 
 */
public class NaiveStrategyProcessor<T extends AbstractBufferPipe> extends StrategyProcessor<T> {

    /**
     * new instance
     * 
     * @param str strategy used
     * @param period update period in ms
     */
	public NaiveStrategyProcessor(Strategy<T> str, long period) {
		super(str, period, 0);
	}
	
    /**
     * @see xxl.core.pipes.scheduler.StrategyProcessor#process()
     */
	@Override
	public void process() {
		if (strategy.getNoOfControllables() == 0) {
			terminate();
			return;
		}
		next = strategy.computeNext() ? strategy.next() : null;
		if (next != null) {
			if (next.isFinished()) {
				strategy.deregister(next);
				lastStrategyUpdate = 0; // cause strategy check
				return;
			}
			next.execute();
		}
	}
	
    /**
     * @see xxl.core.pipes.processors.Processor#terminate()
     */
	public void terminate() {
		super.terminate();
	}

}