/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.pipes.scheduler;

import java.util.Iterator;

// TODO: Controllable Interface in CursorSource impl.
/**
 * Top-interface modelling scheduling strategies.
 * 
 * @param <T> 
 * 
 * @since 1.1
 */
public interface Strategy<T extends Controllable> {
	
    /**
     * registers controllable to strategy
     * @param node controllable to register
     */
	public abstract void register(T node);
	
    /**
     * deregisters controllable from strategy
     * @param node controllable to deregister
     */
	public abstract void deregister(T node);
	
    /**
     * @return iterator over all registered controllables
     */
	public abstract Iterator<T> getControllables();
	
    /**
     * @return number of registered controllables
     */
	public abstract int getNoOfControllables();

    /**
     * @return ? next ready controllable available
     */
	public abstract boolean computeNext();
	
    /**
     * @return next ready controllable
     */
	public abstract T next();
	
}