/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/
package xxl.core.pipes.scheduler.strategies;

import static xxl.core.pipes.operators.identities.AbstractBufferPipe.AbstractBufferPipeMetaDataManagement.FIRST_ELEMENT_START;
import xxl.core.pipes.operators.identities.AbstractBufferPipe;
import xxl.core.pipes.operators.identities.TemporalBufferPipe;
import xxl.core.pipes.queryGraph.Graph;
import xxl.core.pipes.scheduler.AbstractStrategy;
import xxl.core.util.metaData.MetaDataHandler;

/**
 * This strategy chooses the bufferpipe with the tuple with the oldest application timestamp. This bufferpipe will 
 * transfer the tuple to the following operators. The application time of the operator will be brought forward. 
 * 
 * For more information: @see: Master Thesis of Udo Wolske: Eine Evaluation von Strategien fr die Ablaufssteuerung 
 * fr die Datenstrominfrastruktur PIPES
 */
public class BATANaiveStrategy extends AbstractStrategy<TemporalBufferPipe> {

    /**
     * new instance
     * 
     * @param g considered graph
     */
    public BATANaiveStrategy(Graph g) {
        AbstractBufferPipe[] nodes = g.getQueues();
        for (int i = 0; i < nodes.length; i++)
            register((TemporalBufferPipe) nodes[i]);
    }
    
    /**
     * @see xxl.core.pipes.scheduler.AbstractStrategy#computeNext()
     */
    @Override
    public boolean computeNext() {
        while (controllables.size() > 0) {
            next = null;
            long min = Long.MAX_VALUE;
            for (int i = 0; i < controllables.size(); i++) {
                    TemporalBufferPipe node =  controllables.get(i);
                    long v = Long.MAX_VALUE;
                    MetaDataHandler mdh = (MetaDataHandler) node.getMetaDataManagement().getMetaData().get(FIRST_ELEMENT_START);
                    // no tuples in bufferpipe -> v = Long.MAX_VALUE
                    v = (Long) mdh.getMetaData();
                    next = (v < min) ? controllables.get(i) : next;
                    min = (v < min) ? v : min;
            }
            if (next == null)
                return false;
            if (next.isFinished())
            	deregister(next);
            else if (next.isReady()) 
                return true;
        }
        return false;
    }
    
    /**
     * @see xxl.core.pipes.scheduler.AbstractStrategy#register(xxl.core.pipes.scheduler.Controllable)
     */
    @Override
    public void register(TemporalBufferPipe node) {
        super.register(node);
        node.getMetaDataManagement().include(FIRST_ELEMENT_START);
    }

    /** 
     * @see xxl.core.pipes.scheduler.AbstractStrategy#deregister(xxl.core.pipes.scheduler.Controllable)
     */
    @Override
    public void deregister(TemporalBufferPipe node) {
            super.deregister(node);
            node.getMetaDataManagement().exclude(FIRST_ELEMENT_START);
    }
}
