/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/
package xxl.core.pipes.scheduler.strategies;

import java.util.ArrayList;
import java.util.Stack;

import xxl.core.functions.Function;
import xxl.core.pipes.operators.identities.AbstractBufferPipe;
import xxl.core.pipes.queryGraph.Graph;
import xxl.core.pipes.sinks.Sink;
import xxl.core.pipes.sources.Source;

/**
 * Implementation of the MinCost-Traversal-Strategy proposed in 'Operator scheduling in a data stream manager'
 * 
 * calculates a query-tree-traversal, which induces minimal execution costs
 * This traversal is equivalent to the post-order-traversal of trees
 */
public class MinCostStrategy extends AuroraStrategy<AbstractBufferPipe> {
	
    /**
     * considered query tree
     */
    private Graph query;
    
    /**
     * function, that calculates the traversal
     */
	private Function<Graph, AbstractBufferPipe[]> getTraversals;

    /**
     * New instance
     * 
     * @param q considered query tree
     */
	public MinCostStrategy(Graph q) {
		super(q);
        query = q;
		getTraversals = new Function<Graph,AbstractBufferPipe[]>() {
            // postorder
			public AbstractBufferPipe[] invoke() {
				Sink sink = query.getSinks()[0];
				ArrayList<AbstractBufferPipe> result = new ArrayList<AbstractBufferPipe>();
				Stack<Sink> stack = new Stack<Sink>();
				stack.push(sink);
				while (!stack.isEmpty()) {
					sink = stack.pop();
					for (int j = sink.getNoOfSources() - 1; j >= 0; j--) {
						Source source = sink.getSource(j);
						if (query.containsQueue(source))
							result.add(0, (AbstractBufferPipe) source);
						if (!query.isInitialSource(source))
							stack.push((Sink) source);
					}
				}
				AbstractBufferPipe[] res = new AbstractBufferPipe[result.size()];
				for (int i = 0; i < res.length; i++) {
					res[i] = result.get(i);
				}
				return res;
			}
		};
        // calculate traversal 
		traversal = getTraversals.invoke();
	}
}
