/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.pipes.scheduler.strategies;

import java.util.Comparator;

import xxl.core.collections.queues.DynamicHeap;
import xxl.core.pipes.operators.identities.AbstractBufferPipe;
import xxl.core.pipes.queryGraph.Graph;
import xxl.core.pipes.queryGraph.OperatorPath;
import xxl.core.pipes.scheduler.AbstractStrategy;


/**
 * implementation of shortest path approach as described in master thesis of Udo Wolske
 * For more information: @see: Master Thesis of Udo Wolske: Eine Evaluation von Strategien fr die Ablaufssteuerung 
 * fr die Datenstrominfrastruktur PIPES 
 * 
 * sets static priorities for each bufferpipe.: The priority is the shortest distance from a source of the graph 
 * to this bufferpipe
 */
public class ShortestPathStrategy extends AbstractStrategy<AbstractBufferPipe> {

    /**
     * bufferpipes of the operatorpaths of the considered graph
     */
    private AbstractBufferPipe[][] operatorPaths;

    /**
     * minprioritiy = max number of bufferpipes in a graph
     */
    private int minPriority = 0;

    /**
     * new instance
     * 
     * @param graph considered graph
     *            
     */
    public ShortestPathStrategy(Graph graph) {
    	super(graph.getQueues());
        OperatorPath[] paths = graph.getOperatorPaths();
        operatorPaths = new AbstractBufferPipe[paths.length][];
        for (int i = 0; i < paths.length; i++) {
            operatorPaths[i] = paths[i].getQueues();
            minPriority = (operatorPaths[i].length - 1 > minPriority) ? operatorPaths[i].length - 1 : minPriority;
        }
    }

    /**
     * @see xxl.core.pipes.scheduler.AbstractStrategy#computeNext()
     */
    public boolean computeNext() {
        while (controllables.size() > 0) {
            int priority = 0;
            next = null;
            boolean found = false;
            while (priority <= minPriority) {
                for (int i = 0; i < operatorPaths.length; i++) {
                    if (priority < operatorPaths[i].length) {
                        if (operatorPaths[i][priority].isFinished()) {
                            deregister(operatorPaths[i][priority]);
                        }
                        if (operatorPaths[i][priority].isReady()) {
                            next = operatorPaths[i][priority];
                            found = true;
                            break;
                        }
                    }
                }
                if (found)
                    break;
                priority++;
            }
            if (next == null)
                return false;
            return true;
        }
        return false;
    }
}

class CollectionHeap<E> extends DynamicHeap<E>  {

	/**
	 * @param comparator
	 */
	public CollectionHeap(Comparator<? super E> comparator) {
		super(comparator);
	}

	/**
	 * @param o
	 * @return
	 */
	public boolean contains(Object o) {
		for (int i=0;i<array.length;i++) 
			if (array[i]==o)
				return true;
		return false;
	}

}