package xxl.core.pipes.sinks;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;

import javax.swing.JButton;
import javax.swing.JFrame;

import xxl.core.pipes.processors.Processor;
import xxl.core.pipes.queryGraph.QueryExecutor;
import xxl.core.pipes.sources.Enumerator;
import xxl.core.pipes.sources.Source;

public class PocketPCSink<I> extends AbstractSink<I> {

	protected ServerSocket serverSocket;
	protected Socket socket;
	protected OutputStream out;
	protected PrintStream stream;
		
	class PocketPCSinkProcessor extends Processor {
		
		public PocketPCSinkProcessor() {
			super(0);
			final JFrame frame = new JFrame("Pipes Mobile Sink");
			frame.getContentPane().setLayout(new BorderLayout());
			final JButton close = new JButton("Close");
			close.addActionListener(
				new ActionListener() {				
					public void actionPerformed(ActionEvent e) {
						closeAllSources();
						if (stream != null) {
							stream.println("DONE");
							stream.close();
						}
						try {
							if (socket != null)
								socket.close();
							serverSocket.close();
						}
						catch (IOException e1) {
							// sad but true
						}
						frame.dispose();
					}
				}
			);
			frame.getContentPane().add(close);
			frame.addWindowListener(
				new WindowAdapter() {
					@Override
					public void windowClosing (WindowEvent we) {
						close.doClick();
					}
				}
			);
			frame.setPreferredSize(new Dimension(200, 100));
			frame.pack();
			frame.setVisible(true);
		}

		@Override
		public void process() {
			try {
				socket = serverSocket.accept();
				out = socket.getOutputStream();
				stream = new PrintStream(out);
			} catch (IOException e) {
				stream = null;
			}
			terminate();
		}
		
	}

	public PocketPCSink(Source<I> source, int port) {
		super(source);
		try {			
			serverSocket = new ServerSocket(port);
			new PocketPCSinkProcessor().start();
		} catch (IOException e) {
			throw new IllegalArgumentException("port in use");
		}
	}
	
	@Override
	public void processObject(I o, int sourceID) throws IllegalArgumentException {
		if (stream!=null) {
			String output = o.toString();
			while (output.length() > 33) {
				stream.println(output.substring(0, 33));
				output = output.substring(33);
			}
			stream.println(output);
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Source<Integer> source = new Enumerator(1000000, 1000);
		AbstractSink<Integer> sink = new PocketPCSink<Integer>(source, 11111);
		Printer printer = new Printer(source);
		QueryExecutor qe = new QueryExecutor();
		qe.registerQuery(sink);
		qe.registerQuery(printer);
		qe.startAllQueries();
	}

}
