/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.pipes.sources;

import java.util.Iterator;
import java.util.NoSuchElementException;

import xxl.core.pipes.elements.PNObject;
import xxl.core.pipes.processors.SourceProcessor;

public class PNCursorSource<O> extends AbstractPNSource<O> {
	
	protected Iterator<? extends O> objects;
	protected Iterator<? extends Number> timeStampIncs;
	protected long lastTimeStamp;
	
	public PNCursorSource(SourceProcessor processor, Iterator<? extends O> objects, Iterator<? extends Number> timeStampIncs) {
		super(processor);
		this.objects = objects;
		this.timeStampIncs = timeStampIncs;
		lastTimeStamp = 0;
	}
	
	public PNCursorSource(long period, Iterator<? extends O> objects, Iterator<? extends Number> timeStampIncs) {
		super(period);
		this.objects = objects;
		this.timeStampIncs = timeStampIncs;
		lastTimeStamp = 0;
	}
		
	@Override
	public PNObject<O> next() throws NoSuchElementException {
		if (objects.hasNext() && timeStampIncs.hasNext()) {
			return new PNObject<O>(objects.next(),  lastTimeStamp += timeStampIncs.next().longValue(), true);
		}
		throw new NoSuchElementException();
	}
	
}
