/* XXL: The eXtensible and fleXible Library for data processing

Copyright (C) 2000-2006 Prof. Dr. Bernhard Seeger
                        Head of the Database Research Group
                        Department of Mathematics and Computer Science
                        University of Marburg
                        Germany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
USA

    http://www.xxl-library.de

bugs, requests for enhancements: request@xxl-library.de

If you want to be informed on new versions of XXL you can
subscribe to our mailing-list. Send an email to

    xxl-request@lists.uni-marburg.de

without subject and the word "subscribe" in the message body.
*/

package xxl.core.pipes.sources;

import java.util.Iterator;

import xxl.core.cursors.mappers.Mapper;
import xxl.core.cursors.sources.Repeater;
import xxl.core.functions.Function;
import xxl.core.pipes.processors.SourceProcessor;
import xxl.core.pipes.sources.SyncTimeStampSource.RecallIterator;

public class SyncPNCursorSource<O> extends PNCursorSource<O> {

	@SuppressWarnings("unchecked")
	public SyncPNCursorSource(Iterator<? extends O> randomNumbers, Iterator<? extends Number> timeStampIncs) {
		super(new SourceProcessor((Iterator<Long>)(timeStampIncs = new RecallIterator(timeStampIncs))), randomNumbers, ((RecallIterator)timeStampIncs).getRecall());
	}

	@SuppressWarnings("unchecked")
	public SyncPNCursorSource(Iterator<? extends O> randomNumbers, Iterator<? extends Number> timeStampIncs, final double speedup) {
		super(new SourceProcessor(
				new Mapper<Number,Long>(
					new Function<Number,Long>() {
						public Long invoke(Number argument) {
							return (long)(argument.doubleValue()/speedup);
						}
					}, (Iterator<Long>)(timeStampIncs = new RecallIterator(timeStampIncs))
				)				
			), randomNumbers, ((RecallIterator)timeStampIncs).getRecall());
	}

	public SyncPNCursorSource(Iterator<? extends O> randomNumbers, long period) {
		this(randomNumbers, new Repeater<Long>(period));
	}
	
}
