/*
 * Created on 14.05.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package clustering;

import xxl.core.functions.Function;

public class FuzzyKMeans extends AbstractFuzzyKMeans {

    static public FuzzyObjective OBJECTIVE = new FuzzyObjective() {
        public double invoke(FuzzyKMeansState state) {
            FuzzyKMeans fkm = (FuzzyKMeans) state.fkmc;
            double result = 0d;
            for (int i = 0; i < state.dataCount; i++) {
                double sum = 0.0;
                for (int j = 0; j < state.clusterCount; j++) {
                    double val = state.fuzzyCluster[i][j];
                    sum += val;
                    double dist = state.clusterSquaredDist[i][j];
                    if (fkm.fuzzifier == 2.0)
                        val = val * val;
                    else
                        val = Math.pow(val, fkm.fuzzifier);
                    result += val * dist;
                }
                if (fkm.noiseClustering) {
                    sum = 1.0 - sum;
                    if (fkm.fuzzifier == 2.0)
                        sum = sum * sum;
                    else
                        sum = Math.pow(sum, fkm.fuzzifier);
                    result += fkm.squaredNoiseDistance * sum;
                }
            }
            state.objective=result;
            return result;
        }

    };

    /**
     * if noise clustering is activated (one Cluster for all data which are far
     * from all means)
     */
    protected boolean noiseClustering;

    /** Distance of alle data to the noisecluster */
    protected double noiseDistance;

    /** often used help variable */
    protected double squaredNoiseDistance;

    // double mind, sumd;

    /**
     * Create FuzzyKMeans without noise clustering....
     * 
     * @param fuzzifier
     *            fuzzifier constant (use default 2)
     */
    public FuzzyKMeans(Distance distance, double fuzzifier,
            FuzzyObjective objective, FuzzyValidity validity,
            Initialization initialize, Function copyObject,
            FuzzyAverage meansFunction) {
        super(distance, fuzzifier, objective, validity, initialize, copyObject,
                meansFunction);
        noiseClustering = false;
        // System.out.println("Fuzzifier: "+fuzzifier);
    }

    public FuzzyKMeans(int dimension) {
        this(KMeansUtils.getDoubleDistance(dimension), 2.0,
                FuzzyKMeans.OBJECTIVE, FuzzyKMeans.MYVF3,
                FuzzyKMeansUtils.INITALIZE_RANDOM, KMeansUtils
                        .getDoubleCopyFunction(dimension), FuzzyAverages
                        .getDoubleMeansFunction(dimension));
    }

    /**
     * Create FuzzyKMeans with noise clustering....
     * 
     * @param fuzzifier
     *            fuzzifier constant (use default 2)
     * @param noiseDistance
     *            distance to noise cluster
     */
    public FuzzyKMeans(Distance distance, double fuzzifier,
            FuzzyObjective objective, FuzzyValidity validity,
            Initialization initialize, Function copyObject,
            FuzzyAverage meansFunction, double noiseDistance) {
        this(distance, fuzzifier, objective, validity, initialize, copyObject,
                meansFunction);
        setNoiseClustering(noiseDistance);
    }

    /**
     * Set noise clluster ing and the noise distance
     * 
     * @param noiseDistance
     *            new noise distance
     */
    public void setNoiseClustering(double noiseDistance) {
        this.noiseDistance = noiseDistance;
        this.squaredNoiseDistance = noiseDistance * noiseDistance;
        noiseClustering = true;
        if (noiseDistance <= 0) {
            noiseClustering = false;
        }
    }

    /**
     * set to no noise clustering
     */
    public void unsetNoiseClustering() {
        noiseClustering = false;
    }

    public double getNoiseDistance() {
        return noiseDistance;
    }

    /**
     * Compute degrees of memberships
     * @param clusterDist
     *            squared distances of data to each cluster center (return)
     */
    public void computeMemberships(FuzzyKMeansState state) {

        int full = 0;
        for (int k = 0; k < state.clusterCount; k++) {
            state.clusterCounts[k] = 0d;
        }
        for (int i = 0; i < state.dataCount; i++) {
            full = 0;
            double minDist = Double.MAX_VALUE;
            for (int k = 0; k < state.clusterCount; k++) {
                double d = distance.squaredDistance(state.data[i], state.kMeans[k]);
                state.clusterSquaredDist[i][k] = d;
                if (d < minDist)
                    minDist = d;
                if (d == 0)
                    full++;
            }
            for (int k = 0; k < state.clusterCount; k++) {
                if (full > 0) {
                    if (state.clusterSquaredDist[i][k] == 0)
                        state.fuzzyCluster[i][k] = 1.0 / full;
                    else
                        state.fuzzyCluster[i][k] = 0;
                    continue;
                }
                double sum = 0f;
                double d2 = state.clusterSquaredDist[i][k];
                for (int k2 = 0; k2 < state.clusterCount; k2++) {
                    // clusterFuzzyDist[k][k2]=0f;
                    if (d2==0){
                        if (state.clusterSquaredDist[i][k2]==0) sum+=1;
                        else sum+=0;
                    }
                    else if (state.clusterSquaredDist[i][k2]==0) sum=Double.POSITIVE_INFINITY;
                    else if (fuzzifier == 2.0)
                        sum += (d2 / (state.clusterSquaredDist[i][k2]));
                    else
                        sum += Math.pow((d2 / state.clusterSquaredDist[i][k2]),
                                (1.0 / (fuzzifier - 1.0)));
                }
                if (noiseClustering) {
                    double h = d2 / squaredNoiseDistance;
                    if (fuzzifier == 2.0)
                        sum += h;
                    else
                        sum += Math.pow(h, (1.0 / (fuzzifier - 1.0)));

                }
                state.fuzzyCluster[i][k] = 1 / sum;
                state.clusterCounts[k] += state.fuzzyCluster[i][k];
            }
        }
    }

}
