package clustering;

import java.io.IOException;

public class FuzzyKMeansState extends KMeansState {

    public double[][] fuzzyCluster;

    public transient FuzzyKMeansClustering fkmc;

    public double epsilon;

    public FuzzyKMeansState() {
    }

    public FuzzyKMeansState(Object[] data, int dataCount) {
        this.data = data;
        this.dataCount = dataCount;
    }

    public FuzzyKMeansState(Object[] data, int dataCount, int clusterCount) {
        this.data = data;
        this.dataCount = dataCount;
        this.clusterCount = clusterCount;
    }

    public FuzzyKMeansState(FuzzyKMeansClustering fkmc, Object[] data,
            int dataCount, int clusterCount) {
        this.data = data;
        this.dataCount = dataCount;
        this.clusterCount = clusterCount;
        this.fkmc = fkmc;
    }

    private void readObject(java.io.ObjectInputStream stream)
            throws IOException, ClassNotFoundException {
        
        this.dataCount=stream.readInt();
        this.clusterCount=stream.readInt();
        clusterCounts=new double[clusterCount];
        for (int c = 0; c < clusterCount; c++) {
            clusterCounts[c]=stream.readDouble();
        }
        
        fuzzyCluster=new double[dataCount][clusterCount];
        for (int i = 0; i < dataCount; i++) {
            for (int c = 0; c < clusterCount; c++) {
                fuzzyCluster[i][c]=stream.readDouble();
            }
        }
        int v=stream.readInt();
        if (v==0) {
            
            int l = stream.readInt();
            data=new double[dataCount][l];
            for (int i = 0; i < dataCount; i++) {
                double[] d = (double[]) data[i];
                for (int p = 0; p < l; p++) {
                    d[p]=stream.readDouble();
                }
            }
        }
        else if (v==1) {
            int l = stream.readInt();
            data=new double[dataCount][l][2];
            for (int i = 0; i < dataCount; i++) {
                double[][] d = (double[][]) data[i];
                for (int p = 0; p < l; p++) {
                    d[p][0]=stream.readDouble();
                    d[p][1]=stream.readDouble();
                }
            }
        }

    }

    private void writeObject(java.io.ObjectOutputStream stream)
            throws IOException {
        stream.writeInt(this.dataCount);
        stream.writeInt(this.clusterCount);
        for (int c = 0; c < clusterCount; c++) {
            stream.writeDouble(clusterCounts[c]);
        }
        for (int i = 0; i < dataCount; i++) {
            for (int c = 0; c < clusterCount; c++) {
                stream.writeDouble(fuzzyCluster[i][c]);
            }
        }
        if (data[0] instanceof double[]) {
            stream.writeInt(0);
            int l = ((double[])data[0]).length;
            stream.writeInt(l);
            for (int i = 0; i < dataCount; i++) {
                double[] d = (double[]) data[i];
                for (int p = 0; p < l; p++) {
                    stream.writeDouble(d[p]);
                }
            }
        }
        if (data[0] instanceof double[][]) {
            stream.writeInt(1);
            int l = ((double[][])data[0]).length;
            stream.writeInt(l);
            for (int i = 0; i < dataCount; i++) {
                double[][] d = (double[][]) data[i];
                for (int p = 0; p < l; p++) {
                    stream.writeDouble(d[p][0]);
                    stream.writeDouble(d[p][1]);
                }
            }
        }
    }

    public Object clone() {
        FuzzyKMeansState state = null;
        state = (FuzzyKMeansState) super.clone();
        state.fuzzyCluster = new double[fuzzyCluster.length][fuzzyCluster[0].length];
        for (int i = 0; i < fuzzyCluster.length; i++)
            for (int j = 0; j < fuzzyCluster[0].length; j++) {
                state.fuzzyCluster[i][j] = fuzzyCluster[i][j];
            }
        state.epsilon = epsilon;
        return state;
    }
}
