/*
 * Created on 24.04.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package pipes;

import xxl.core.pipes.operators.AbstractPipe;
import xxl.core.pipes.operators.identities.BufferPipe;
import clustering.FuzzyKMeansClustering;
import clustering.FuzzyKMeansState;
import clustering.RelativeFuzzyKMeans;

/**
 * @author beringer
 */
public class FuzzyTrivialKMeans extends
        AbstractPipe<Object[], FuzzyKMeansState> {
    static protected boolean DEBUG = false;

    static public boolean QUIET = true;

    protected double lambda = 400;

    protected FuzzyKMeansClustering fkmAlg;

    protected RelativeFuzzyKMeans rfkm;

    protected FuzzyKMeansState state;

    protected int initCount;

    // Decoupler-Object
    protected BufferPipe bufferPipe;

    // initialisiert? also ob die ersten Cluster schon berechnet wurden
    protected boolean initialized;

    // anzahl von verarbeiteten Blcken
    protected int init;

    // Beschriftung(Namen) der Streams
    protected Object[] streamsObjects;
    
    protected double[][]cluster;

    /** Creates a new instance of StreamCluster */
    public FuzzyTrivialKMeans(RelativeFuzzyKMeans rfkm,
            Object[] streamsObjects, int size, int blockCount,
            int clusterCount, double[][] cluster, Object[] kMeans) {
        super();

        this.fkmAlg = rfkm.getFuzzyKMeansClustering();
        this.rfkm = rfkm;
        this.streamsObjects = streamsObjects;
        this.cluster=cluster;

        state = new FuzzyKMeansState();
        state.fkmc = fkmAlg;
        state.dataCount = size;
        state.maxClusterCount = clusterCount;
        state.minClusterCount = clusterCount;
        state.clusterCount = clusterCount;
        state.kMeans = kMeans;
        state.clusterSquaredDist = new double[size][clusterCount];
        state.clusterCounts = new double[clusterCount];
        this.initCount = blockCount;
        
        initCluster();

        initialized = false;
        init = 0;

    }

    public FuzzyKMeansClustering getFKMC() {
        return fkmAlg;
    }

    public void setBufferPipe(BufferPipe decoupler) {
        this.bufferPipe = decoupler;
    }
    
    public void initCluster(){
        state.fuzzyCluster = new double[state.dataCount][state.clusterCount];
        for (int n = 0; n < state.dataCount; n++) {
            for (int c = 0; c < state.clusterCount; c++) {
                state.fuzzyCluster[n][c] = cluster[n][c];
            }
        }
    }

    public void processObject(Object[] normdfts, int id) {
        if (DEBUG)
            System.out.println("Block " + (init + 1));

        state.data = normdfts;
        init++;
        if (!QUIET)
            System.out.println("Step " + init);

        if (initialized) {
            initCluster();
            computeClusterCenters();
            iteration();

            transfer(state);

        } else {
            if (init == initCount + 1) {
                initCluster();
                initialized = true;
                computeClusterCenters();
                transfer(state);
                iteration();
            }
        }
    }

    public void iteration() {
        int c = 0;

        do {
            computeMemberships();
            computeClusterCenters();
            c++;
        } while ((c < 20));
    }

    protected void computeMemberships() {
        fkmAlg.computeMemberships(state);
    }

    protected void computeClusterCenters() {
        fkmAlg.computeClusterCenters(state);
    }

    public Object[] getClusterMeans() {
        return state.kMeans;
    }

    public int getClusterCount() {
        return state.clusterCount;
    }

    public int getMaxClusterCount() {
        return state.maxClusterCount;
    }

}
