

package weka.core.fuzzy;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;


/**
 * @author beringer
 *
 */
public class InterpolationFunction implements FuzzySetFunction {
	
	public static DecimalFormat format2 = new DecimalFormat("0.00",
			new DecimalFormatSymbols(Locale.US));
	
	protected double[] values;
	protected double[] fuzzyValues;
	protected int size;
	
	public InterpolationFunction(double[] sortedValues, double[] fuzzyValues){
		if (fuzzyValues.length<sortedValues.length) 
			throw new IllegalArgumentException("values and fuzzyValues must have same size");
		values=sortedValues;
		this.fuzzyValues=fuzzyValues;
		size=values.length;
	}

	/*public double getFuzzyValue(double value) {
		if (value < values[0]) return fuzzyValues[0];
		if (value >= values[size-1]) return fuzzyValues[size-1];
		int pos=1;
		while (values[pos]<=value) pos++;
		return fuzzyValues[pos-1]+(((value-values[pos-1])/(values[pos]-values[pos-1]))*(fuzzyValues[pos]-fuzzyValues[pos-1])); 
	}*/
	
	public double getFuzzyValue(double value) {
		if (value <= values[0]) return fuzzyValues[0];
		if (value > values[size-1]) return fuzzyValues[size-1];
		int pos=0;
		while (values[pos+1]<value) pos++;
		return fuzzyValues[pos]+(((value-values[pos])/(values[pos+1]-values[pos]))*(fuzzyValues[pos+1]-fuzzyValues[pos])); 
	}
	
	public double[] getValues(){
		return values;
	}
	
	public double[] getFuzzyValues(){
		return fuzzyValues;
	}
	
	public String toString(){
		StringBuffer sb=new StringBuffer();
		for (int i=0;i<size;i++){
			sb.append(format2.format(values[i])+"    ");
		}
		sb.append(System.getProperty("line.separator"));
		for (int i=0;i<size;i++){
			sb.append(format2.format(fuzzyValues[i])+"    ");
		}
		return sb.toString();
	}
	
	public static void main(String[] args){
		InterpolationFunction f=new InterpolationFunction(new double[]{2,2,3,5},new double[]{0.0,1.0,1.0,0.0});
		double[] control=new double[]{1,2,2.5,3,3.3,3.5,4,4.9,5,8};
		for (int i=0;i<control.length;i++){
			System.out.println(control[i]+" entspricht Fuzzywert "+f.getFuzzyValue(control[i]));
		}
	}

}
