package weka.core.fuzzy.norm;
import java.io.Serializable;

/**
* An implementation of interface TNorm (Algebraic Conjunction)
* @author Yasuyuki Yokoyama
*/
public class AlgebraicConjunction implements TNorm, Serializable{
	/** Constructs an AlgebraicConjunction object. */
	public AlgebraicConjunction(){}
	
	/** 
	* Application of AlgebraicConjunction to 2 arguments.
	* @param value1 Float-value of the first argument.
	* @param value2 Float-value of the second argument.
	* @return Float-value of the algebraic conjunction with value1 and value2 (i.e. value1*value2).
	*/
	public double invoke(double value1, double value2){
		return value1*value2;
	}
	
	/**
	* Successive application of AlgebraicConjunction.
	* @param values set of arguments.
	* @return Float-value of the algebraic conjunction with input arguments.
	*/
	public double invoke(double[] values){
		float result = 1;
		for(int i=0; i<values.length; i++){
			result *= values[i];
		}
		
		return result;
	}
}