package weka.core.fuzzy.norm;
import java.io.Serializable;

/**
* An implementation of interface TNorm (Bold Conjunction)
* @author Yasuyuki Yokoyama
*/
public class BoldConjunction implements TNorm, Serializable{
	
	/** Constructs a BoldConjunction object. */
	public BoldConjunction(){}
	
	/**
	* Returns max(0,x+y-1),where x,y are inputs. 
	* @param value1 first argument.
	* @param value2 second argument.
	* @return float value of max(0,value1+value2-1)
	*/		
	public double invoke(double value1, double value2){
		double value = value1 + value2 - 1; 
		return Math.max(0,value);
	}
	
	/**
	* Returns the result of successive application of Bold Conjunction.
	* @param values set of arguments.
	* @return float value of the result.
	*/
	public double invoke(double[] values){
		if(values.length==0){
			return 1;
		}
		
		else if(values.length==1){
			return values[0];
		}
		
		else{
			double[] val = new double[values.length-1];
			
			val[0]=invoke(values[0],values[1]); //first two arguments are calculated.
			
			for(int i=1; i<val.length; i++){ // rest of the arguments are saved in a new array.
				val[i]=values[i+1];
			}
			
			return invoke(val); // recursive!
		}
	}
}