package weka.core.fuzzy.norm;
import java.io.Serializable;
/**
 * An Implementation of NormFunction (Dual norm).
 * @author Jrgen Beringer / Yasuyuki Yokoyama
 */
public class DualNorm implements Aggregate, Serializable {
	
	/** NormFunction of DualNorm.*/
	Aggregate func;
	
	/** Creates a new instance of DualNorm. */
	DualNorm(){}
	
	/** Constructs a DualNorm object with NormFunction. */
	public DualNorm(Aggregate norm){
		func = norm;
	}
	
	/**
	* Returns the NormFunction of DualNorm.
	* @return Normfunction of DualNorm.
	*/
	public Aggregate getNormFunction(){
		return func;
	}
	
	/**
	* Returns the result of an application of DualNorm to 2 arguments.
	* @param value1 Float-value of the first argument.
	* @param value2 Float-value of the second argument.
	* @return Float-value of the DualNorm with arguments value1 and value2. 
	*/
	public double invoke(double value1, double value2){
		return 1 - func.invoke(1 - value1,1 - value2);	
	}
	
	/**
	* Returns the result of a successive application of DualNorm. 
	* @param values arguments of DualNorm.
	* @return Float-value of the DualNorm with arguments in values.
	*/
	public float invoke(float[] values){
		float[] dualVal = new float[values.length];
		
		for(int i=0; i< values.length; i++){
			dualVal[i]=1-values[i];
		}
		
		return	1 - invoke(dualVal);
	}

	
	public double invoke(double[] values) {
		throw new UnsupportedOperationException();
	}
}