package weka.core.fuzzy.norm;
import java.io.Serializable;

/**
* An implementation of interface TNorm (Yager class)
* @author Yasuyuki Yokoyama
*/
public class Frank implements TNorm, Serializable{
	
	/** Parameter for Yager class. */
	double s;
	double logs;
	
	/** Constructs a Yager object. */
	Frank(){}
	
	/** Constructs a Yager object with a parameter. */
	public Frank(double s){
		if (s==1 || !(s>0)) throw new IllegalArgumentException("s has to be >0 and !=1.");
		this.s = s;
		logs=Math.log(s);
	}
	
	/** 
	* Application of a Yager instance to 2 arguments.
	* @param value1 Float-value of the first argument.
	* @param value2 Float-value of the second argument.
	* @return Float-value of Yager with value1 and value2.
	*/		
	public double invoke(double value1, double value2){
		double value = 1+((Math.pow(s,value1)-1)*(Math.pow(s,value2)-1)/(s-1));
		if (value <=0) throw new RuntimeException("s="+s+"  x="+value1+" y="+value2+" val="+value);
		if (value==1.0) return 0.0;
		value=Math.log(value)/logs;
		return value;
	}
	
	/**
	* Successive application of a Yager instance.
	* @param values set of arguments.
	* @return Float-value of Yager with input arguments.
	*/
	public double invoke(double[] values){
		
		if(values.length==0){
			return 1;
		}
		
		else if(values.length==1){
			return invoke(values[0],1);
		}
		
		else{
			double[] val = new double[values.length-1];
			
			val[0]=invoke(values[0],values[1]);
			
			for(int i=1; i<val.length; i++){
				val[i]=values[i+1];
			}
			
			return invoke(val);
		}
	}
}