package weka.core.fuzzy.norm;
import java.io.Serializable;

/**
* An implementation of interface TNorm (Minimum Conjunction)
* @author Yasuyuki Yokoyama
*/
public class MinimumConjunction implements TNorm, Serializable{

	/** Constructs a MinimumConjunction object. */
	public MinimumConjunction(){}
	
	/**
	* Returns the result of an application of MinimumConjunction to 2 arguments. 
	* @param value1 Float-value of the first argument
	* @param value2 Float-value of the second argument
	* @return Float-value of MinimumConjunction with value1 and value2 (i.e. Min(value1, value2)).
	*/
	public double invoke(double value1, double value2){
		return Math.min(value1,value2);
	}
	
	/**
	* Returns the result of a successive application of Minimum Conjunction.
	* @param values set of arguments.
	* @return Float-value of the minimum conjunction with input arguments.
	*/
	public double invoke(double[] values){
		
		if(values.length==0){
			return 1;
		}
		
		else{
			double result = values[0];
			for(int i=1; i<values.length; i++){
				if(values[i] < result){
					result = values[i];
				}
			}
		
			return result;
		}
	}
}