package weka.core.fuzzy.norm;
import java.io.Serializable;

/**
* An implementation of interface TNorm (Yager class)
* @author Yasuyuki Yokoyama
*/
public class Yager implements TNorm, Serializable{
	
	/** Parameter for Yager class. */
	double w;
	
	/** Constructs a Yager object. */
	Yager(){}
	
	/** Constructs a Yager object with a parameter. */
	public Yager(double potenz){
		w = potenz;
	}
	
	/** 
	* Application of a Yager instance to 2 arguments.
	* @param value1 Float-value of the first argument.
	* @param value2 Float-value of the second argument.
	* @return Float-value of Yager with value1 and value2.
	*/		
	public double invoke(double value1, double value2){
		double value = (double)Math.pow((Math.pow((double)(1-value1),w) + Math.pow((double)(1-value2),w)), 1/w);
		
		return (1<value)? 0 : (1-value);
	}
	
	/**
	* Successive application of a Yager instance.
	* @param values set of arguments.
	* @return Float-value of Yager with input arguments.
	*/
	public double invoke(double[] values){
		
		if(values.length==0){
			return 1;
		}
		
		else if(values.length==1){
			return invoke(values[0],1);
		}
		
		else{
			double[] val = new double[values.length-1];
			
			val[0]=invoke(values[0],values[1]);
			
			for(int i=1; i<val.length; i++){
				val[i]=values[i+1];
			}
			
			return invoke(val);
		}
	}
}