package weka.filters;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import weka.core.Attribute;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.fuzzy.FuzzySet;
import weka.core.fuzzy.InterpolationFunction;
import weka.filters.supervised.attribute.Partitioning;

/**
 * @author beringer
 *
 */
public class FuzzyPartitioning2 {

	Instances input;

	Partitioning part;

	InterpolationFunction[][] funcs;

	public double[][] intervals;

	public double[][] splitValues;

	protected static double log2 = Math.log(2);

	public FuzzyPartitioning2() {
		part = new Partitioning();
	}

	public void setInputFormat(Instances input) {
		try {
			part.setInputFormat(input);
			this.input = part.getInputFormat();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void setInput(Instances input) {
		setInputFormat(input);
		Enumeration it = input.enumerateInstances();
		try {
			while (it.hasMoreElements()) {
				input((Instance) it.nextElement());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Input an instance for filtering. Ordinarily the instance is
	 * processed and made available for output immediately. Some filters
	 * require all instances be read before producing output, in which
	 * case output instances should be collected after calling
	 * batchFinished(). If the input marks the start of a new batch, the
	 * output queue is cleared. This default implementation assumes all
	 * instance conversion will occur when batchFinished() is called.
	 *
	 * @param instance the input instance
	 * @return true if the filtered instance may now be
	 * collected with output().
	 * @exception NullPointerException if the input format has not been
	 * defined.
	 * @exception Exception if the input instance was not of the correct 
	 * format or if there was a problem with the filtering.  
	 */
	public boolean input(Instance instance) throws Exception {

		return part.input(instance);
	}

	// nicht fuzzy partitionierer starten und fuzzyoptimieren
	public boolean batchFinished() {
		try {
			part.batchFinished();
		} catch (Exception e) {
			e.printStackTrace();
		}
		int classCount = input.numClasses();
		initialize();
		int attrCount = input.numAttributes();
		for (int attr = 0; attr < attrCount; attr++) {
			if (funcs[attr] == null) continue;
			double[][] sample = prepare(attr);
			optimize(attr, sample, classCount);
		}
		return true;

	}

	public FuzzySet[][] getFuzzySets() {
		int attrCount = input.numAttributes();
		FuzzySet[][] result = new FuzzySet[attrCount][];
		for (int i = 0; i < attrCount; i++) {
			if (funcs[i] == null) continue;
			int len = funcs[i].length;
			result[i] = new FuzzySet[len];
			Attribute att = input.attribute(i);
			for (int j = 0; j < len; j++) {
				result[i][j] = new FuzzySet(att, funcs[i][j], "Interval " + j);
			}
		}
		return result;
	}

	//Fuzzysets initialisieren
	protected void initialize() {
		int attrCount = input.numAttributes();
		funcs = new InterpolationFunction[attrCount][];
		intervals = new double[attrCount][];
		splitValues = new double[attrCount][];
		double[] min = part.getMinValues();
		double[] max = part.getMaxValues();
		for (int i = 0; i < attrCount; i++) {
			double[] splits = part.getSplitValues(i);
			splitValues[i] = splits;
			if (splits == null) continue;
			if (splits.length == 0) {
				funcs[i] = new InterpolationFunction[1];
				funcs[i][0] = new InterpolationFunction(new double[] { min[i],
						max[i]}, new double[] { 1.0, 1.0});
			} else {
				intervals[i] = new double[splits.length+1];
				funcs[i] = new InterpolationFunction[splits.length + 1];
				funcs[i][0] = new InterpolationFunction(new double[] {
						splits[0], splits[0]}, new double[] { 1.0, 0.0});
				funcs[i][splits.length] = new InterpolationFunction(
						new double[] { splits[splits.length - 1],
								splits[splits.length - 1]}, new double[] { 0.0,
								1.0});
				
				intervals[i][0] = (splits[0]-min[i]);
				intervals[i][splits.length] = max[i]-splits[splits.length - 1];
				for (int j= 1;j<splits.length;j++){
					intervals[i][j] = (splits[j] - splits[j-1]) / 2.0;
				}
				for (int j = 1; j < splits.length; j++) {
					funcs[i][j] = new InterpolationFunction(
							new double[] { splits[j - 1], splits[j - 1],
									splits[j], splits[j]}, new double[] { 0.0,
									1.0, 1.0, 0.0});
				}
			}
		}
	}

	//fuzzywerte fr ein attribut berechnen
	protected double[][] prepare(int attribute) {
		int c = input.numInstances();
		int pCount = funcs[attribute].length;
		Enumeration enum1 = input.enumerateInstances();
		double[][] insts = new double[c][pCount + 2];
		int pos = 0;
		while (enum1.hasMoreElements()) {
			Instance inst = (Instance) enum1.nextElement();
			insts[pos][0] = inst.classValue();
			insts[pos][1] = inst.value(attribute);
			for (int i = 0; i < pCount; i++) {
				insts[pos][i + 2] = funcs[attribute][i]
						.getFuzzyValue(insts[pos][0]);
			}
			pos++;
		}
		return insts;
	}

	//attribut diskretisierung optimieren
	protected void optimize(int attribute, double[][] sample, int classCount) {
		int c = input.numInstances();
		int pCount = funcs[attribute].length;
		for (int splitnr = 0; splitnr < pCount - 1; splitnr++) {
			double[][] splitSample = splitSample(sample, splitnr);
			int sCount = splitSample.length;
			double minL = 0;
			double minR = 0;
			double minVal = computeCriterium(splitSample, splitnr, classCount);
			for (double left=0;left<1.00001;left+=0.25){
				for (double right=0;right<1.00001;right+=0.25){
					if (left!=0 || right!=0) {
						setFuzzyFunction(attribute, splitnr, left,right, splitSample);
						double val = computeCriterium(splitSample, splitnr, classCount);
						if (val < minVal) {
							minVal = val;
							minL = left;
							minR = right;
						}
					}
				}
			}
			
			setFuzzyFunction(attribute, splitnr, minL,minR, splitSample);
		}
	}

	//Fuzzyset mit neuem Alpha anpassen
	protected void setFuzzyFunction(int attribute, int splitnr, double left,double right,
			double[][] sample) {
		double center = splitValues[attribute][splitnr];
		double maxDistL = intervals[attribute][splitnr];
		double maxDistR = intervals[attribute][splitnr+1];
		double distL = left * maxDistL;
		double distR = right * maxDistR;
		InterpolationFunction f1 = funcs[attribute][splitnr];
		InterpolationFunction f2 = funcs[attribute][splitnr + 1];
		double[] v1 = f1.getValues();
		v1[v1.length - 1] = center + distR;
		v1[v1.length - 2] = center - distL;
		double[] v2 = f2.getValues();
		v2[0] = center - distL;
		v2[1] = center + distR;
		int size = sample.length;
		for (int i = 0; i < size; i++) {
			sample[i][splitnr + 2] = f1.getFuzzyValue(sample[i][1]);
			sample[i][splitnr + 3] = f2.getFuzzyValue(sample[i][1]);
		}
	}

	//entropy Kriterium berechnen
	protected double computeCriterium(double[][] splitSample, int splitnr,
			int classCount) {
		double[][] result = splitEntropy(splitSample, splitnr, classCount);
		int sCount = splitSample.length;
		double entS1 = result[0][0];
		int ccS1 = (int) result[0][1];
		double sum1 = result[0][2];
		double entS2 = result[1][0];
		int ccS2 = (int) result[1][1];
		double sum2 = result[1][2];
		double val = sum1 / sCount * entS1 + sum2 / sCount * entS2;
		return val;
	}
	
//	entropy Kriterium berechnen
	protected double computeCriterium2(double[][] splitSample, int splitnr,
			int classCount) {
		double[][] result = splitGini(splitSample, splitnr, classCount);
		int sCount = splitSample.length;
		double entS1 = result[0][0];
		int ccS1 = (int) result[0][1];
		double sum1 = result[0][2];
		double entS2 = result[1][0];
		int ccS2 = (int) result[1][1];
		double sum2 = result[1][2];
		double val = entS1 + entS2;
		return val;
	}
	
	protected double computeCriterium3(double[][] splitSample, int splitnr,
			int classCount) {
		double[] counter1 = new double[classCount];
		double[] counter2 = new double[classCount];
		double sum1 = 0;
		double sum2 = 0;
		int size = splitSample.length;
		for (int i = 0; i < size; i++) {
			counter1[(int) splitSample[i][0]] += splitSample[i][splitnr + 2];
			counter2[(int) splitSample[i][0]] += splitSample[i][splitnr + 3];
			sum1 += splitSample[i][splitnr + 2];
			sum2 += splitSample[i][splitnr + 3];
		}
		double dist = 0.0;
		for (int i = 0; i < classCount; i++) {
			counter1[i] /= sum1;
			counter2[i] /= sum2;
			double dif=counter2[i] - counter1[i];
			dist += Math.abs(dif);
		}
		return dist;
	}

	//Daten heraussuchen, die zu den intervallen gehren
	protected double[][] splitSample(double[][] sample, int splitnr) {
		List list = new LinkedList();
		int c = sample.length;
		for (int i = 0; i < c; i++) {
			if (sample[i][splitnr + 2] > 0 || sample[i][splitnr + 3] > 0)
					list.add(sample[i]);
		}
		int size = list.size();
		double[][] result = new double[size][];
		int pos = 0;
		Iterator it = list.iterator();
		while (it.hasNext()) {
			result[pos] = (double[]) it.next();
			pos++;
		}
		return result;
	}

	//Entropywerte fr beide am Split beteiligten Fuzzysets berechnen
	protected double[][] splitEntropy(double[][] sample, int splitnr,
			int classCount) {
		double[] counter1 = new double[classCount];
		double[] counter2 = new double[classCount];
		double sum1 = 0;
		double sum2 = 0;
		int size = sample.length;
		for (int i = 0; i < size; i++) {
			counter1[(int) sample[i][0]] += sample[i][splitnr + 2];
			counter2[(int) sample[i][0]] += sample[i][splitnr + 3];
			sum1 += sample[i][splitnr + 2];
			sum2 += sample[i][splitnr + 3];
		}
		double entropy1 = 0.0;
		int diffClasses1 = 0;
		double entropy2 = 0.0;
		int diffClasses2 = 0;
		for (int i = 0; i < classCount; i++) {
			if (counter1[i] > 0) diffClasses1++;
			counter1[i] /= sum1;
			if (counter1[i] > 0) entropy1 -= counter1[i] * log2(counter1[i]);
			if (counter2[i] > 0) diffClasses2++;
			counter2[i] /= sum2;
			if (counter2[i] > 0) entropy2 -= counter2[i] * log2(counter2[i]);
		}
		return new double[][] { { entropy1, diffClasses1, sum1},
				{ entropy2, diffClasses2, sum2}};
	}
	
//	Entropywerte fr beide am Split beteiligten Fuzzysets berechnen
	protected double[][] splitGini(double[][] sample, int splitnr,
			int classCount) {
		double[] counter1 = new double[classCount];
		double[] counter2 = new double[classCount];
		double sum1 = 0;
		double sum2 = 0;
		int size = sample.length;
		for (int i = 0; i < size; i++) {
			counter1[(int) sample[i][0]] += sample[i][splitnr + 2];
			counter2[(int) sample[i][0]] += sample[i][splitnr + 3];
			sum1 += sample[i][splitnr + 2];
			sum2 += sample[i][splitnr + 3];
		}
		double gini1 = 0.0;
		int diffClasses1 = 0;
		double gini2 = 0.0;
		int diffClasses2 = 0;
		for (int i = 0; i < classCount; i++) {
			if (counter1[i] > 0) diffClasses1++;
			counter1[i] /= sum1;
			if (counter1[i] > 0) gini1 += counter1[i] * counter1[i];
			if (counter2[i] > 0) diffClasses2++;
			counter2[i] /= sum2;
			if (counter2[i] > 0) gini2 += counter2[i] * counter2[i];
		}
		return new double[][] { { gini1, diffClasses1, sum1},
				{ gini2, diffClasses2, sum2}};
	}

	protected double log2(double d) {
		return Math.log(d) / log2;
	}

}